@@ -33,18 +33,21 @@ from qgis.PyQt.QtWidgets import (QWidget,
                                  QSizePolicy,
                                  QDialogButtonBox)
 
-from qgis.core import (QgsProcessingFeedback,
-                       QgsProcessingParameterDefinition)
+from qgis.core import (
+    QgsProcessingException,
+    QgsProcessingFeedback,
+    QgsProcessingParameterDefinition
+)
 from qgis.gui import (QgsMessageBar,
                       QgsProjectionSelectionWidget,
-                      QgsProcessingAlgorithmDialogBase)
+                      QgsProcessingAlgorithmDialogBase,
+                      QgsProcessingLayerOutputDestinationWidget)
 
 from processing.gui.AlgorithmDialog import AlgorithmDialog
 from processing.gui.AlgorithmDialogBase import AlgorithmDialogBase
 from processing.gui.ParametersPanel import ParametersPanel
 from processing.gui.MultipleInputPanel import MultipleInputPanel
 from processing.gui.NumberInputPanel import NumberInputPanel
-from processing.gui.DestinationSelectionPanel import DestinationSelectionPanel
 from processing.gui.wrappers import WidgetWrapper
 from processing.tools.dataobjects import createContext
 
@@ -62,8 +65,9 @@ class GdalAlgorithmDialog(AlgorithmDialog):
 class GdalParametersPanel(ParametersPanel):
 
     def __init__(self, parent, alg):
-        ParametersPanel.__init__(self, parent, alg)
+        super().__init__(parent, alg)
 
+        self.dialog = parent
         w = QWidget()
         layout = QVBoxLayout()
         layout.setMargin(0)
@@ -75,7 +79,7 @@ class GdalParametersPanel(ParametersPanel):
         self.text.setReadOnly(True)
         layout.addWidget(self.text)
         w.setLayout(layout)
-        self.layoutMain.addWidget(w)
+        self.addExtraWidget(w)
 
         self.connectParameterSignals()
         self.parametersHaveChanged()
@@ -99,9 +103,6 @@ class GdalParametersPanel(ParametersPanel):
             for c in w.findChildren(QWidget):
                 self.connectWidgetChangedSignals(c)
 
-        for output_widget in self.outputWidgets.values():
-            self.connectWidgetChangedSignals(output_widget)
-
     def connectWidgetChangedSignals(self, w):
         if isinstance(w, QLineEdit):
             w.textChanged.connect(self.parametersHaveChanged)
@@ -115,30 +116,39 @@ class GdalParametersPanel(ParametersPanel):
             w.selectionChanged.connect(self.parametersHaveChanged)
         elif isinstance(w, NumberInputPanel):
             w.hasChanged.connect(self.parametersHaveChanged)
-        elif isinstance(w, DestinationSelectionPanel):
+        elif isinstance(w, QgsProcessingLayerOutputDestinationWidget):
             w.destinationChanged.connect(self.parametersHaveChanged)
 
     def parametersHaveChanged(self):
         context = createContext()
         feedback = QgsProcessingFeedback()
         try:
-            parameters = self.parent.getParameterValues()
-            for output in self.alg.destinationParameterDefinitions():
+            # messy as all heck, but we don't want to call the dialog's implementation of
+            # createProcessingParameters as we want to catch the exceptions raised by the
+            # parameter panel instead...
+            parameters = {} if self.dialog.mainWidget() is None else self.dialog.mainWidget().createProcessingParameters()
+            for output in self.algorithm().destinationParameterDefinitions():
                 if not output.name() in parameters or parameters[output.name()] is None:
-                    parameters[output.name()] = self.tr("[temporary file]")
-            for p in self.alg.parameterDefinitions():
+                    if not output.flags() & QgsProcessingParameterDefinition.FlagOptional:
+                        parameters[output.name()] = self.tr("[temporary file]")
+            for p in self.algorithm().parameterDefinitions():
                 if p.flags() & QgsProcessingParameterDefinition.FlagHidden:
                     continue
 
-                if (not p.name() in parameters and not p.flags() & QgsProcessingParameterDefinition.FlagOptional) \
-                        or (not p.checkValueIsAcceptable(parameters[p.name()])):
+                if p.flags() & QgsProcessingParameterDefinition.FlagOptional and p.name() not in parameters:
+                    continue
+
+                if p.name() not in parameters or not p.checkValueIsAcceptable(parameters[p.name()]):
                     # not ready yet
                     self.text.setPlainText('')
                     return
 
-            commands = self.alg.getConsoleCommands(parameters, context, feedback, executing=False)
-            commands = [c for c in commands if c not in ['cmd.exe', '/C ']]
-            self.text.setPlainText(" ".join(commands))
+            try:
+                commands = self.algorithm().getConsoleCommands(parameters, context, feedback, executing=False)
+                commands = [c for c in commands if c not in ['cmd.exe', '/C ']]
+                self.text.setPlainText(" ".join(commands))
+            except QgsProcessingException as e:
+                self.text.setPlainText(str(e))
         except AlgorithmDialogBase.InvalidParameterValue as e:
             self.text.setPlainText(self.tr("Invalid value for parameter '{0}'").format(e.parameter.description()))
         except AlgorithmDialogBase.InvalidOutputExtension as e:
