@@ -21,7 +21,6 @@ __author__ = 'Nyall Dawson'
 __date__ = 'May 2017'
 __copyright__ = '(C) 2017, Nyall Dawson'
 
-
 from qgis.core import (QgsApplication,
                        QgsProcessingAlgorithm,
                        QgsProcessingFeatureBasedAlgorithm,
@@ -30,7 +29,8 @@ from qgis.core import (QgsApplication,
                        QgsProcessing,
                        QgsWkbTypes,
                        QgsMapLayerType,
-                       QgsFields)
+                       QgsFields,
+                       QgsStringUtils)
 from processing.gui.MessageBarProgress import MessageBarProgress
 from processing.gui.MessageDialog import MessageDialog
 from processing.gui.AlgorithmDialog import AlgorithmDialog
@@ -72,17 +72,37 @@ class AlgorithmLocatorFilter(QgsLocatorFilter):
                     a.flags() & QgsProcessingAlgorithm.FlagKnownIssues:
                 continue
 
-            if QgsLocatorFilter.stringMatches(a.displayName(), string) or [t for t in a.tags() if QgsLocatorFilter.stringMatches(t, string)] or \
-                    (context.usingPrefix and not string):
-                result = QgsLocatorResult()
-                result.filter = self
-                result.displayString = a.displayName()
-                result.icon = a.icon()
-                result.userData = a.id()
-                if string and QgsLocatorFilter.stringMatches(a.displayName(), string):
-                    result.score = float(len(string)) / len(a.displayName())
-                else:
-                    result.score = 0
+            result = QgsLocatorResult()
+            result.filter = self
+            result.displayString = a.displayName()
+            result.icon = a.icon()
+            result.userData = a.id()
+            result.score = 0
+
+            if (context.usingPrefix and not string):
+                self.resultFetched.emit(result)
+
+            if not string:
+                return
+
+            string = string.lower()
+            tagScore = 0
+            if a.provider():
+                tags = [*a.tags(), a.provider().name()]
+            else:
+                tags = a.tags()
+
+            if a.group():
+                tags.append(a.group())
+
+            for t in tags:
+                if string in t.lower():
+                    tagScore = 1
+                    break
+
+            result.score = QgsStringUtils.fuzzyScore(result.displayString, string) * 0.5 + tagScore * 0.5
+
+            if result.score > 0:
                 self.resultFetched.emit(result)
 
     def triggerResult(self, result):
@@ -107,7 +127,10 @@ class AlgorithmLocatorFilter(QgsLocatorFilter):
                     canvas.mapTool().reset()
                 except:
                     pass
-                canvas.setMapTool(prevMapTool)
+                try:
+                    canvas.setMapTool(prevMapTool)
+                except RuntimeError:
+                    pass
 
 
 class InPlaceAlgorithmLocatorFilter(QgsLocatorFilter):
@@ -147,21 +170,42 @@ class InPlaceAlgorithmLocatorFilter(QgsLocatorFilter):
             if not a.supportInPlaceEdit(iface.activeLayer()):
                 continue
 
-            if QgsLocatorFilter.stringMatches(a.displayName(), string) or [t for t in a.tags() if QgsLocatorFilter.stringMatches(t, string)] or \
-                    (context.usingPrefix and not string):
-                result = QgsLocatorResult()
-                result.filter = self
-                result.displayString = a.displayName()
-                result.icon = a.icon()
-                result.userData = a.id()
-                if string and QgsLocatorFilter.stringMatches(a.displayName(), string):
-                    result.score = float(len(string)) / len(a.displayName())
-                else:
-                    result.score = 0
+            result = QgsLocatorResult()
+            result.filter = self
+            result.displayString = a.displayName()
+            result.icon = a.icon()
+            result.userData = a.id()
+            result.score = 0
+
+            if (context.usingPrefix and not string):
+                self.resultFetched.emit(result)
+
+            if not string:
+                return
+
+            string = string.lower()
+            tagScore = 0
+            if a.provider():
+                tags = [*a.tags(), a.provider().name()]
+            else:
+                tags = a.tags()
+
+            if a.group():
+                tags.append(a.group())
+
+            for t in tags:
+                if string in t.lower():
+                    tagScore = 1
+                    break
+
+            result.score = QgsStringUtils.fuzzyScore(result.displayString, string) * 0.5 + tagScore * 0.5
+
+            if result.score > 0:
                 self.resultFetched.emit(result)
 
     def triggerResult(self, result):
-        alg = QgsApplication.processingRegistry().createAlgorithmById(result.userData)
+        config = {'IN_PLACE': True}
+        alg = QgsApplication.processingRegistry().createAlgorithmById(result.userData, config)
         if alg:
             ok, message = alg.canExecute()
             if not ok:
@@ -171,8 +215,12 @@ class InPlaceAlgorithmLocatorFilter(QgsLocatorFilter):
                 dlg.exec_()
                 return
 
+            in_place_input_parameter_name = 'INPUT'
+            if hasattr(alg, 'inputParameterName'):
+                in_place_input_parameter_name = alg.inputParameterName()
+
             if [d for d in alg.parameterDefinitions() if
-                    d.name() not in ('INPUT', 'OUTPUT')]:
+                    d.name() not in (in_place_input_parameter_name, 'OUTPUT')]:
                 dlg = alg.createCustomParametersWidget(parent=iface.mainWindow())
                 if not dlg:
                     dlg = AlgorithmDialog(alg, True, parent=iface.mainWindow())
@@ -190,3 +238,4 @@ class InPlaceAlgorithmLocatorFilter(QgsLocatorFilter):
                 feedback = MessageBarProgress(algname=alg.displayName())
                 parameters = {}
                 execute_in_place(alg, parameters, feedback=feedback)
+                feedback.close()
