@@ -68,16 +68,21 @@ class TestGdalAlgorithms(unittest.TestCase):
         # Test that algorithms report a valid commandName
         p = QgsApplication.processingRegistry().providerById('gdal')
         for a in p.algorithms():
+            if a.id() in ('gdal:buildvirtualvector'):
+                # build virtual vector is an exception
+                continue
             self.assertTrue(a.commandName(), 'Algorithm {} has no commandName!'.format(a.id()))
 
     def testCommandNameInTags(self):
         # Test that algorithms commandName is present in provided tags
         p = QgsApplication.processingRegistry().providerById('gdal')
         for a in p.algorithms():
+            if not a.commandName():
+                continue
             self.assertTrue(a.commandName() in a.tags(), 'Algorithm {} commandName not found in tags!'.format(a.id()))
 
     def testNoParameters(self):
-        # Test that algorithms throw QgsProcessingExceptions and not base Python
+        # Test that algorithms throw QgsProcessingException and not base Python
         # exceptions when no parameters specified
         p = QgsApplication.processingRegistry().providerById('gdal')
         context = QgsProcessingContext()
@@ -159,6 +164,18 @@ class TestGdalAlgorithms(unittest.TestCase):
         path, layer = alg.getOgrCompatibleSource('INPUT', parameters, context, feedback, False)
         self.assertEqual(path, source)
 
+        # with subset string
+        vl.setSubsetString('x')
+        path, layer = alg.getOgrCompatibleSource('INPUT', parameters, context, feedback, False)
+        self.assertEqual(path, source)
+        # subset of layer must be exported
+        path, layer = alg.getOgrCompatibleSource('INPUT', parameters, context, feedback, True)
+        self.assertNotEqual(path, source)
+        self.assertTrue(path)
+        self.assertTrue(path.endswith('.gpkg'))
+        self.assertTrue(os.path.exists(path))
+        self.assertTrue(layer)
+
         # geopackage with layer
         source = os.path.join(testDataPath, 'custom', 'circular_strings.gpkg')
         vl2 = QgsVectorLayer(source + '|layername=circular_strings')
@@ -223,7 +240,7 @@ class TestGdalAlgorithms(unittest.TestCase):
     def testOgrLayerNameExtraction(self):
         with tempfile.TemporaryDirectory() as outdir:
             def _copyFile(dst):
-                shutil.copyfile(os.path.join(testDataPath, 'custom', 'grass7', 'weighted.csv'), dst)
+                shutil.copyfile(os.path.join(testDataPath, 'custom', 'weighted.csv'), dst)
 
             # OGR provider - single layer
             _copyFile(os.path.join(outdir, 'a.csv'))
@@ -315,24 +332,14 @@ class TestGdalAlgorithms(unittest.TestCase):
             '+proj=utm +zone=36 +south +a=600000 +b=70000 +towgs84=-143,-90,-294,0,0,0,0 +units=m +no_defs')
         self.assertTrue(crs.isValid())
 
-        if QgsProjUtils.projVersionMajor() >= 6:
-            # proj 6, WKT should be used
-            self.assertEqual(GdalUtils.gdal_crs_string(crs)[:40], 'BOUNDCRS[SOURCECRS[PROJCRS["unknown",BAS')
+        # proj 6, WKT should be used
+        self.assertEqual(GdalUtils.gdal_crs_string(crs)[:40], 'BOUNDCRS[SOURCECRS[PROJCRS["unknown",BAS')
 
-            self.assertEqual(GdalUtils.gdal_crs_string(QgsCoordinateReferenceSystem('ESRI:102003')), 'ESRI:102003')
-        else:
-            self.assertEqual(GdalUtils.gdal_crs_string(crs),
-                             '+proj=utm +zone=36 +south +a=600000 +b=70000 +towgs84=-143,-90,-294,0,0,0,0 +units=m +no_defs')
-            # check that newlines are stripped
-            crs = QgsCoordinateReferenceSystem()
-            crs.createFromProj(
-                '+proj=utm +zone=36 +south\n     +a=600000 +b=70000 \r\n    +towgs84=-143,-90,-294,0,0,0,0 +units=m\n+no_defs')
-            self.assertTrue(crs.isValid())
-            self.assertEqual(GdalUtils.gdal_crs_string(crs),
-                             '+proj=utm +zone=36 +south      +a=600000 +b=70000       +towgs84=-143,-90,-294,0,0,0,0 +units=m +no_defs')
+        self.assertEqual(GdalUtils.gdal_crs_string(QgsCoordinateReferenceSystem('ESRI:102003')), 'ESRI:102003')
 
     def testEscapeAndJoin(self):
-        self.assertEqual(GdalUtils.escapeAndJoin([1, "a", "a b", "a&b", "a(b)"]), '1 a "a b" "a&b" "a(b)"')
+        self.assertEqual(GdalUtils.escapeAndJoin([1, "a", "a b", "a&b", "a(b)", ";"]), '1 a "a b" "a&b" "a(b)" ";"')
+        self.assertEqual(GdalUtils.escapeAndJoin([1, "-srcnodata", "--srcnodata", "-9999 9999"]), '1 -srcnodata --srcnodata "-9999 9999"')
 
 
 if __name__ == '__main__':
