@@ -22,6 +22,8 @@
 #include "qgs3d.h"
 #include "qgsmaterialregistry.h"
 #include "qgs3dsceneexporter.h"
+#include "qgsvectorlayerelevationproperties.h"
+#include "qgsvectorlayer.h"
 
 QgsPolygon3DSymbol::QgsPolygon3DSymbol()
   : mMaterial( std::make_unique< QgsPhongMaterialSettings >() )
@@ -122,6 +124,32 @@ QList<QgsWkbTypes::GeometryType> QgsPolygon3DSymbol::compatibleGeometryTypes() c
   return QList< QgsWkbTypes::GeometryType >() << QgsWkbTypes::PolygonGeometry;
 }
 
+void QgsPolygon3DSymbol::setDefaultPropertiesFromLayer( const QgsVectorLayer *layer )
+{
+  const QgsVectorLayerElevationProperties *props = qgis::down_cast< const QgsVectorLayerElevationProperties * >( const_cast< QgsVectorLayer *>( layer )->elevationProperties() );
+
+  mAltClamping = props->clamping();
+  mAltBinding = props->binding();
+  mExtrusionHeight = props->extrusionEnabled() ? static_cast< float>( props->extrusionHeight() ) : 0.0f;
+  if ( props->dataDefinedProperties().isActive( QgsMapLayerElevationProperties::ExtrusionHeight ) )
+  {
+    mDataDefinedProperties.setProperty( PropertyExtrusionHeight, props->dataDefinedProperties().property( QgsMapLayerElevationProperties::ExtrusionHeight ) );
+  }
+  else
+  {
+    mDataDefinedProperties.setProperty( PropertyExtrusionHeight, QgsProperty() );
+  }
+  if ( props->dataDefinedProperties().isActive( QgsMapLayerElevationProperties::ZOffset ) )
+  {
+    mDataDefinedProperties.setProperty( PropertyHeight, props->dataDefinedProperties().property( QgsMapLayerElevationProperties::ZOffset ) );
+  }
+  else
+  {
+    mDataDefinedProperties.setProperty( PropertyHeight, QgsProperty() );
+  }
+  mHeight = static_cast< float >( props->zOffset() );
+}
+
 QgsAbstract3DSymbol *QgsPolygon3DSymbol::create()
 {
   return new QgsPolygon3DSymbol();
