@@ -20,6 +20,7 @@
 #include "qgisapp.h"
 #include "qgsmapcanvas.h"
 #include "qgscoordinateutils.h"
+#include "qgscoordinatereferencesystemutils.h"
 
 #include <QUrl>
 
@@ -38,13 +39,10 @@ void QgsGotoLocatorFilter::fetchResults( const QString &string, const QgsLocator
   if ( feedback->isCanceled() )
     return;
 
-  const QgsCoordinateReferenceSystem currentCrs = QgisApp::instance()->mapCanvas()->mapSettings().destinationCrs();
-  const QgsCoordinateReferenceSystem wgs84Crs( QStringLiteral( "EPSG:4326" ) );
-
-  bool okX = false;
-  bool okY = false;
-  double posX = 0.0;
-  double posY = 0.0;
+  bool firstOk = false;
+  bool secondOk = false;
+  double firstNumber = 0.0;
+  double secondNumber = 0.0;
   bool posIsWgs84 = false;
   const QLocale locale;
 
@@ -55,66 +53,87 @@ void QgsGotoLocatorFilter::fetchResults( const QString &string, const QgsLocator
   QRegularExpressionMatch match = separatorRx.match( string.trimmed() );
   if ( match.hasMatch() )
   {
-    posX = locale.toDouble( match.captured( 1 ), &okX );
-    posY = locale.toDouble( match.captured( 2 ), &okY );
+    firstNumber = locale.toDouble( match.captured( 1 ), &firstOk );
+    secondNumber = locale.toDouble( match.captured( 2 ), &secondOk );
   }
 
-  if ( !match.hasMatch() || !okX || !okY )
+  if ( !match.hasMatch() || !firstOk || !secondOk )
   {
     // Digit detection using user locale failed, use default C decimal separators
     separatorRx = QRegularExpression( QStringLiteral( "^([0-9\\-\\.]*)[\\s\\,]*([0-9\\-\\.]*)$" ) );
     match = separatorRx.match( string.trimmed() );
     if ( match.hasMatch() )
     {
-      posX = match.captured( 1 ).toDouble( &okX );
-      posY = match.captured( 2 ).toDouble( &okY );
+      firstNumber = match.captured( 1 ).toDouble( &firstOk );
+      secondNumber = match.captured( 2 ).toDouble( &secondOk );
     }
   }
 
   if ( !match.hasMatch() )
   {
     // Check if the string is a pair of decimal degrees with [N,S,E,W] suffixes
-    separatorRx = QRegularExpression( QStringLiteral( "^\\s*([0-9\\-\\.]*\\s*[NSEWnsew])[\\s\\,]*([0-9\\-\\.]*\\s*[NSEWnsew])\\s*$" ) );
+    separatorRx = QRegularExpression( QStringLiteral( "^\\s*([-]?\\d{1,3}(?:[\\.\\%1]\\d+)?\\s*[NSEWnsew])[\\s\\,]*([-]?\\d{1,3}(?:[\\.\\%1]\\d+)?\\s*[NSEWnsew])\\s*$" )
+                                      .arg( locale.decimalPoint() ) );
     match = separatorRx.match( string.trimmed() );
     if ( match.hasMatch() )
     {
       posIsWgs84 = true;
       bool isEasting = false;
-      posX = QgsCoordinateUtils::degreeToDecimal( match.captured( 1 ), &okX, &isEasting );
-      posY = QgsCoordinateUtils::degreeToDecimal( match.captured( 2 ), &okY );
-      if ( !isEasting )
-        std::swap( posX, posY );
+      firstNumber = QgsCoordinateUtils::degreeToDecimal( match.captured( 1 ), &firstOk, &isEasting );
+      secondNumber = QgsCoordinateUtils::degreeToDecimal( match.captured( 2 ), &secondOk );
+      // normalize to northing (i.e. Y) first
+      if ( isEasting )
+        std::swap( firstNumber, secondNumber );
     }
   }
 
   if ( !match.hasMatch() )
   {
     // Check if the string is a pair of degree minute second
-    separatorRx = QRegularExpression( QStringLiteral( "^((?:([-+nsew])\\s*)?\\d{1,3}(?:[^0-9.]+[0-5]?\\d)?[^0-9.]+[0-5]?\\d(?:\\.\\d+)?[^0-9.,]*[-+nsew]?)[,\\s]+((?:([-+nsew])\\s*)?\\d{1,3}(?:[^0-9.]+[0-5]?\\d)?[^0-9.]+[0-5]?\\d(?:\\.\\d+)?[^0-9.,]*[-+nsew]?)$" ) );
+    separatorRx = QRegularExpression( QStringLiteral( "^((?:([-+nsew])\\s*)?\\d{1,3}(?:[^0-9.]+[0-5]?\\d)?[^0-9.]+[0-5]?\\d(?:[\\.\\%1]\\d+)?[^0-9.,]*[-+nsew]?)[,\\s]+((?:([-+nsew])\\s*)?\\d{1,3}(?:[^0-9.]+[0-5]?\\d)?[^0-9.]+[0-5]?\\d(?:[\\.\\%1]\\d+)?[^0-9.,]*[-+nsew]?)$" )
+                                      .arg( locale.decimalPoint() ) );
     match = separatorRx.match( string.trimmed() );
     if ( match.hasMatch() )
     {
       posIsWgs84 = true;
       bool isEasting = false;
-      posX = QgsCoordinateUtils::dmsToDecimal( match.captured( 1 ), &okX, &isEasting );
-      posY = QgsCoordinateUtils::dmsToDecimal( match.captured( 3 ), &okY );
-      if ( !isEasting )
-        std::swap( posX, posY );
+      firstNumber = QgsCoordinateUtils::dmsToDecimal( match.captured( 1 ), &firstOk, &isEasting );
+      secondNumber = QgsCoordinateUtils::dmsToDecimal( match.captured( 3 ), &secondOk );
+      // normalize to northing (i.e. Y) first
+      if ( isEasting )
+        std::swap( firstNumber, secondNumber );
     }
   }
 
-  if ( okX && okY )
+  const QgsCoordinateReferenceSystem currentCrs = QgisApp::instance()->mapCanvas()->mapSettings().destinationCrs();
+  const QgsCoordinateReferenceSystem wgs84Crs( QStringLiteral( "EPSG:4326" ) );
+
+  if ( firstOk && secondOk )
   {
     QVariantMap data;
-    const QgsPointXY point( posX, posY );
-    data.insert( QStringLiteral( "point" ), point );
+    const bool currentCrsIsXY = QgsCoordinateReferenceSystemUtils::defaultCoordinateOrderForCrs( currentCrs ) == Qgis::CoordinateOrder::XY;
+    const bool withinWgs84 = wgs84Crs.bounds().contains( secondNumber, firstNumber );
 
-    const bool withinWgs84 = wgs84Crs.bounds().contains( point );
     if ( !posIsWgs84 && currentCrs != wgs84Crs )
     {
+      const QgsPointXY point( currentCrsIsXY ? firstNumber : secondNumber,
+                              currentCrsIsXY ? secondNumber : firstNumber );
+      data.insert( QStringLiteral( "point" ), point );
+
+      const QList< Qgis::CrsAxisDirection > axisList = currentCrs.axisOrdering();
+      QString firstSuffix;
+      QString secondSuffix;
+      if ( axisList.size() >= 2 )
+      {
+        firstSuffix = QgsCoordinateReferenceSystemUtils::axisDirectionToAbbreviatedString( axisList.at( 0 ) );
+        secondSuffix = QgsCoordinateReferenceSystemUtils::axisDirectionToAbbreviatedString( axisList.at( 1 ) );
+      }
+
       QgsLocatorResult result;
       result.filter = this;
-      result.displayString = tr( "Go to %1 %2 (Map CRS, %3)" ).arg( locale.toString( point.x(), 'g', 10 ), locale.toString( point.y(), 'g', 10 ), currentCrs.userFriendlyIdentifier() );
+      result.displayString = tr( "Go to %1%2 %3%4 (Map CRS, %5)" ).arg( locale.toString( firstNumber, 'g', 10 ), firstSuffix,
+                             locale.toString( secondNumber, 'g', 10 ), secondSuffix,
+                             currentCrs.userFriendlyIdentifier() );
       result.userData = data;
       result.score = 0.9;
       emit resultFetched( result );
@@ -122,6 +141,7 @@ void QgsGotoLocatorFilter::fetchResults( const QString &string, const QgsLocator
 
     if ( withinWgs84 )
     {
+      const QgsPointXY point( secondNumber, firstNumber );
       if ( currentCrs != wgs84Crs )
       {
         const QgsCoordinateTransform transform( wgs84Crs, currentCrs, QgsProject::instance()->transformContext() );
@@ -137,10 +157,14 @@ void QgsGotoLocatorFilter::fetchResults( const QString &string, const QgsLocator
         }
         data[QStringLiteral( "point" )] = transformedPoint;
       }
+      else
+      {
+        data[QStringLiteral( "point" )] = point;
+      }
 
       QgsLocatorResult result;
       result.filter = this;
-      result.displayString = tr( "Go to %1° %2° (%3)" ).arg( locale.toString( point.x(), 'g', 10 ), locale.toString( point.y(), 'g', 10 ), wgs84Crs.userFriendlyIdentifier() );
+      result.displayString = tr( "Go to %1°N %2°E (%3)" ).arg( locale.toString( point.y(), 'g', 10 ), locale.toString( point.x(), 'g', 10 ), wgs84Crs.userFriendlyIdentifier() );
       result.userData = data;
       result.score = 1.0;
       emit resultFetched( result );
@@ -177,10 +201,10 @@ void QgsGotoLocatorFilter::fetchResults( const QString &string, const QgsLocator
   {
     double scale = 0.0;
     int meters = 0;
-    okX = false;
-    okY = false;
-    posX = 0.0;
-    posY = 0.0;
+    bool okX = false;
+    bool okY = false;
+    double posX = 0.0;
+    double posY = 0.0;
     if ( url.hasFragment() )
     {
       // Check for OSM/Leaflet/OpenLayers pattern (e.g. http://www.openstreetmap.org/#map=6/46.423/4.746)
@@ -278,7 +302,7 @@ void QgsGotoLocatorFilter::fetchResults( const QString &string, const QgsLocator
 
       QgsLocatorResult result;
       result.filter = this;
-      result.displayString = tr( "Go to %1° %2° %3(%4)" ).arg( locale.toString( point.x(), 'g', 10 ), locale.toString( point.y(), 'g', 10 ),
+      result.displayString = tr( "Go to %1°N %2°E %3(%4)" ).arg( locale.toString( point.y(), 'g', 10 ), locale.toString( point.x(), 'g', 10 ),
                              scale > 0.0 ? tr( "at scale 1:%1 " ).arg( scale ) : QString(),
                              wgs84Crs.userFriendlyIdentifier() );
       result.userData = data;
