@@ -212,13 +212,13 @@ QgsMapToolEditMeshFrame::QgsMapToolEditMeshFrame( QgsMapCanvas *canvas )
   : QgsMapToolAdvancedDigitizing( canvas, QgisApp::instance()->cadDockWidget() )
   , mSnapIndicator( new QgsSnapIndicator( canvas ) )
 {
-  mActionDigitizing = new QAction( QgsApplication::getThemePixmap( QStringLiteral( "/mActionMeshDigitizing.svg" ) ), tr( "Digitize Mesh elements" ), this );
+  mActionDigitizing = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionMeshDigitizing.svg" ) ), tr( "Digitize Mesh Elements" ), this );
   mActionDigitizing->setCheckable( true );
 
-  mActionSelectByPolygon = new QAction( QgsApplication::getThemePixmap( QStringLiteral( "/mActionMeshSelectPolygon.svg" ) ), tr( "Select Mesh Elements by Polygon" ), this );
+  mActionSelectByPolygon = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionMeshSelectPolygon.svg" ) ), tr( "Select Mesh Elements by Polygon" ), this );
   mActionSelectByPolygon->setCheckable( true );
   mActionSelectByPolygon->setObjectName( QStringLiteral( "ActionMeshSelectByPolygon" ) );
-  mActionSelectByExpression = new QAction( QgsApplication::getThemePixmap( QStringLiteral( "/mActionMeshSelectExpression.svg" ) ), tr( "Select Mesh Elements by Expression" ), this );
+  mActionSelectByExpression = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionMeshSelectExpression.svg" ) ), tr( "Select Mesh Elements by Expression" ), this );
   mActionSelectByExpression->setObjectName( QStringLiteral( "ActionMeshSelectByExpression" ) );
 
   mSelectionHandler = std::make_unique<QgsMapToolSelectionHandler>( canvas, QgsMapToolSelectionHandler::SelectPolygon );
@@ -226,16 +226,16 @@ QgsMapToolEditMeshFrame::QgsMapToolEditMeshFrame( QgsMapCanvas *canvas )
   mSelectActions << mActionSelectByPolygon
                  << mActionSelectByExpression;
 
-  mActionTransformCoordinates = new QAction( QgsApplication::getThemePixmap( QStringLiteral( "/mActionMeshTransformByExpression.svg" ) ), tr( "Transform Vertices Coordinates" ), this );
+  mActionTransformCoordinates = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionMeshTransformByExpression.svg" ) ), tr( "Transform Vertices Coordinates" ), this );
   mActionTransformCoordinates->setCheckable( true );
 
-  mActionForceByLines = new QAction( QgsApplication::getThemePixmap( QStringLiteral( "/mActionMeshEditForceByVectorLines.svg" ) ), tr( "Force by Selected Geometries" ), this );
+  mActionForceByLines = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionMeshEditForceByVectorLines.svg" ) ), tr( "Force by Selected Geometries" ), this );
   mActionForceByLines->setCheckable( true );
 
   mWidgetActionForceByLine = new QgsMeshEditForceByLineAction( this );
   mWidgetActionForceByLine->setMapCanvas( canvas );
 
-  mActionReindexMesh = new QAction( QgsApplication::getThemePixmap( QStringLiteral( "/mActionMeshReindex.svg" ) ), tr( "Reindex Faces and Vertices" ), this );
+  mActionReindexMesh = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "/mActionMeshReindex.svg" ) ), tr( "Reindex Faces and Vertices" ), this );
 
   mActionRemoveVerticesFillingHole = new QAction( this );
   mActionDelaunayTriangulation = new QAction( tr( "Delaunay Triangulation with Selected Vertices" ), this );
@@ -397,7 +397,7 @@ QList<QAction *> QgsMapToolEditMeshFrame::selectActions() const
 
 QAction *QgsMapToolEditMeshFrame::defaultSelectActions() const
 {
-  QgsSettings settings;
+  const QgsSettings settings;
   bool ok = false;
   int defaultIndex = settings.value( QStringLiteral( "UI/Mesh/defaultSelection" ) ).toInt( &ok );
 
@@ -440,7 +440,7 @@ void QgsMapToolEditMeshFrame::initialize()
   mFaceRubberBand->setVisible( false );
   mFaceRubberBand->setZValue( 5 );
 
-  QColor color = digitizingStrokeColor();
+  const QColor color = digitizingStrokeColor();
   if ( !mFaceVerticesBand )
     mFaceVerticesBand = new QgsRubberBand( mCanvas );
   mFaceVerticesBand->setIcon( QgsRubberBand::ICON_CIRCLE );
@@ -568,7 +568,8 @@ void QgsMapToolEditMeshFrame::deactivate()
   deleteZValueWidget();
   qDeleteAll( mFreeVertexMarker );
   mFreeVertexMarker.clear();
-
+  mNewFaceCandidate.clear();
+  mNewVerticesForNewFaceCandidate.clear();
 }
 
 void QgsMapToolEditMeshFrame::clearAll()
@@ -699,69 +700,15 @@ QgsMapTool::Flags QgsMapToolEditMeshFrame::flags() const
     case SelectingByPolygon:
     case ForceByLines:
       return QgsMapTool::Flags();
-      break;
   }
 
   return QgsMapTool::Flags();
 }
 
-static QList<QgsMapToolIdentify::IdentifyResult> searchFeatureOnMap( QgsMapMouseEvent *e, QgsMapCanvas *canvas, const QList<QgsWkbTypes::GeometryType> &geomType )
-{
-  QList<QgsMapToolIdentify::IdentifyResult> results;
-  const QMap< QString, QString > derivedAttributes;
-
-  QgsPointXY mapPoint = e->mapPoint();
-  double x = mapPoint.x(), y = mapPoint.y();
-  const double sr = QgsMapTool::searchRadiusMU( canvas );
-
-  const QList<QgsMapLayer *> layers = canvas->layers();
-  for ( QgsMapLayer *layer : layers )
-  {
-    if ( layer->type() == QgsMapLayerType::VectorLayer )
-    {
-      QgsVectorLayer *vectorLayer = static_cast<QgsVectorLayer *>( layer );
-
-      bool typeIsSelectable = false;
-      for ( const QgsWkbTypes::GeometryType &type : geomType )
-        if ( vectorLayer->geometryType() == type )
-        {
-          typeIsSelectable = true;
-          break;
-        }
-      if ( typeIsSelectable )
-      {
-        QgsRectangle rect( x - sr, y - sr, x + sr, y + sr );
-        QgsCoordinateTransform transform = canvas->mapSettings().layerTransform( vectorLayer );
-        transform.setBallparkTransformsAreAppropriate( true );
-
-        try
-        {
-          rect = transform.transformBoundingBox( rect, Qgis::TransformDirection::Reverse );
-        }
-        catch ( QgsCsException & )
-        {
-          QgsDebugMsg( QStringLiteral( "Could not transform geometry to layer CRS" ) );
-        }
-
-        QgsFeatureIterator fit = vectorLayer->getFeatures( QgsFeatureRequest()
-                                 .setFilterRect( rect )
-                                 .setFlags( QgsFeatureRequest::ExactIntersect ) );
-        QgsFeature f;
-        while ( fit.nextFeature( f ) )
-        {
-          results << QgsMapToolIdentify::IdentifyResult( vectorLayer, f, derivedAttributes );
-        }
-      }
-    }
-  }
-
-  return results;
-}
-
 void QgsMapToolEditMeshFrame::forceByLineBySelectedFeature( QgsMapMouseEvent *e )
 {
   const QList<QgsMapToolIdentify::IdentifyResult> &results =
-    searchFeatureOnMap( e, mCanvas, QList<QgsWkbTypes::GeometryType>() << QgsWkbTypes::PolygonGeometry << QgsWkbTypes::LineGeometry );
+    QgsIdentifyMenu::findFeaturesOnCanvas( e, mCanvas, QList<QgsWkbTypes::GeometryType>() << QgsWkbTypes::PolygonGeometry << QgsWkbTypes::LineGeometry );
 
   QgsIdentifyMenu *menu = new QgsIdentifyMenu( mCanvas );
   menu->setExecWithSingleResult( true );
@@ -772,7 +719,7 @@ void QgsMapToolEditMeshFrame::forceByLineBySelectedFeature( QgsMapMouseEvent *e
 
   if ( !selectedFeatures.empty() && selectedFeatures[0].mFeature.hasGeometry() )
   {
-    QgsCoordinateTransform transform = mCanvas->mapSettings().layerTransform( selectedFeatures.at( 0 ).mLayer );
+    const QgsCoordinateTransform transform = mCanvas->mapSettings().layerTransform( selectedFeatures.at( 0 ).mLayer );
     QgsGeometry geom = selectedFeatures[0].mFeature.geometry();
     try
     {
@@ -795,7 +742,7 @@ void QgsMapToolEditMeshFrame::cadCanvasPressEvent( QgsMapMouseEvent *e )
 
   if ( e->button() == Qt::LeftButton &&
        ( !mCadDockWidget->cadEnabled() ||
-         mCadDockWidget->additionalConstraint() == QgsAdvancedDigitizingDockWidget::AdditionalConstraint::NoConstraint ) )
+         mCadDockWidget->betweenLineConstraint() == Qgis::BetweenLineConstraint::NoConstraint ) )
     mLeftButtonPressed = true;
 
   switch ( mCurrentState )
@@ -826,7 +773,7 @@ void QgsMapToolEditMeshFrame::cadCanvasPressEvent( QgsMapMouseEvent *e )
           // The workaround is to check if a feature exist under the mouse before sending the event to the selection handler.
           // This is not ideal because that leads to a double search but no better idea for now to allow the editing context menu with selecting by polygon
 
-          bool hasSelectableFeature = !searchFeatureOnMap( e, mCanvas, QList<QgsWkbTypes::GeometryType>() << QgsWkbTypes::PolygonGeometry ).isEmpty();
+          bool hasSelectableFeature = ! QgsIdentifyMenu::findFeaturesOnCanvas( e, mCanvas, QList<QgsWkbTypes::GeometryType>() << QgsWkbTypes::PolygonGeometry ).isEmpty();
 
           if ( hasSelectableFeature || mIsSelectingPolygonInProgress )
             mSelectionHandler->canvasPressEvent( e );
@@ -872,7 +819,7 @@ void QgsMapToolEditMeshFrame::cadCanvasMoveEvent( QgsMapMouseEvent *e )
     case AddingNewFace:
       mNewFaceBand->movePoint( mapPoint );
       highLight( mapPoint );
-      if ( testNewVertexInFaceCanditate( mCurrentVertexIndex ) )
+      if ( testNewVertexInFaceCanditate( true, mCurrentVertexIndex, mapPoint ) )
         mNewFaceBand->setColor( mValidFaceColor );
       else
         mNewFaceBand->setColor( mInvalidFaceColor );
@@ -978,7 +925,8 @@ void QgsMapToolEditMeshFrame::cadCanvasReleaseEvent( QgsMapMouseEvent *e )
                   mCurrentEdge.first != -1 && mCurrentEdge.second != -1 )  // flip edge
         {
           clearSelection();
-          QVector<int> edgeVert = edgeVertices( mCurrentEdge );
+          mCadDockWidget->clearPoints();
+          const QVector<int> edgeVert = edgeVertices( mCurrentEdge );
           mCurrentEditor->flipEdge( edgeVert.at( 0 ), edgeVert.at( 1 ) );
           mCurrentEdge = {-1, -1};
           highLight( mapPoint );
@@ -988,47 +936,67 @@ void QgsMapToolEditMeshFrame::cadCanvasReleaseEvent( QgsMapMouseEvent *e )
                   mCurrentEdge.first != -1 && mCurrentEdge.second != -1 ) // merge two faces
         {
           clearSelection();
-          QVector<int> edgeVert = edgeVertices( mCurrentEdge );
+          mCadDockWidget->clearPoints();
+          const QVector<int> edgeVert = edgeVertices( mCurrentEdge );
           mCurrentEditor->merge( edgeVert.at( 0 ), edgeVert.at( 1 ) );
           mCurrentEdge = {-1, -1};
           highLight( mapPoint );
         }
         else
+        {
           select( mapPoint, e->modifiers(), tolerance );
+          mCadDockWidget->clearPoints();
+        }
       }
       break;
     case AddingNewFace:
       if ( e->button() == Qt::LeftButton ) //eventually add a vertex to the face
       {
-        if ( mDoubleClicks )
-        {
-          addVertex( mFirstClickPoint, e->mapPointMatch() );
-          highlightCloseVertex( mFirstClickPoint );
-        }
 
         if ( mCurrentVertexIndex != -1 )
         {
           addVertexToFaceCanditate( mCurrentVertexIndex );
-          QgsPointXY currentPoint = mapVertexXY( mCurrentVertexIndex );
-          cadDockWidget()->setPoints( QList<QgsPointXY>() << currentPoint << currentPoint );
+          // Advanced digitizing base class adds a point at map point not at vertex position
+          // so we need to replace it by the position of the vertex
+          const QgsPoint &currentPoint = mapVertex( mCurrentVertexIndex );
+          cadDockWidget()->updateCurrentPoint( currentPoint );
+          cadDockWidget()->removePreviousPoint();
+          cadDockWidget()->addPoint( currentPoint );
+        }
+        else
+        {
+          bool acceptPoint = true;
+          if ( ! mNewFaceCandidate.isEmpty() &&
+               mNewFaceCandidate.last() == -1 &&
+               !mNewVerticesForNewFaceCandidate.isEmpty() ) //avoid dupplicate new vertex
+          {
+            acceptPoint = mapPoint.distance( mNewVerticesForNewFaceCandidate.last() ) > tolerance;
+          }
+
+          if ( acceptPoint )
+          {
+            addVertexToFaceCanditate( mapPoint );
+          }
         }
       }
       else if ( e->button() == Qt::RightButton ) //if possible validate and add the face to the mesh
       {
-        if ( testNewVertexInFaceCanditate( -1 ) )
+        if ( testNewVertexInFaceCanditate( false, -1, QgsPointXY() ) )
         {
-          mCurrentEditor->addFace( mNewFaceCandidate.toVector() );
+          mCurrentEditor->addFaceWithNewVertices( mNewFaceCandidate, mNewVerticesForNewFaceCandidate );
           mNewFaceBand->reset( QgsWkbTypes::PolygonGeometry );
           mNewFaceCandidate.clear();
+          mNewVerticesForNewFaceCandidate.clear();
           mCurrentState = Digitizing;
         }
       }
       break;
     case Selecting:
     {
-      QgsGeometry selectionGeom = mSelectionBand->asGeometry();
+      const QgsGeometry selectionGeom = mSelectionBand->asGeometry();
       selectByGeometry( selectionGeom, e->modifiers() );
       mSelectionBand->reset( QgsWkbTypes::PolygonGeometry );
+      mCadDockWidget->clearPoints();
       mCurrentState = Digitizing;
     }
     break;
@@ -1114,7 +1082,7 @@ void QgsMapToolEditMeshFrame::moveSelection( const QgsPointXY &destinationPoint
     for ( int i = 0; i < vertexData.meshFixedEdges.count(); ++i )
     {
       const QgsPointXY point2 = mapVertexXY( vertexData.meshFixedEdges.at( i ).second );
-      QgsGeometry edge( new QgsLineString( {point1, point2} ) );
+      const QgsGeometry edge( new QgsLineString( {point1, point2} ) );
       mMovingEdgesRubberband->addGeometry( edge );
       int associateFace = vertexData.meshFixedEdges.at( i ).first;
       if ( associateFace != -1 )
@@ -1213,7 +1181,7 @@ void QgsMapToolEditMeshFrame::select( const QgsPointXY &mapPoint, Qt::KeyboardMo
             mapPoint.distance( mSelectEdgeMarker->center() ) < tolerance &&
             mCurrentEdge.first != -1 && mCurrentEdge.second != -1 )
   {
-    QVector<int> edgeVert = edgeVertices( mCurrentEdge );
+    const QVector<int> edgeVert = edgeVertices( mCurrentEdge );
     setSelectedVertices( edgeVert.toList(), behavior );
     const QgsMeshVertex v1 = mCurrentLayer->triangularMesh()->vertices().at( edgeVert.at( 0 ) );
     const QgsMeshVertex v2 = mCurrentLayer->triangularMesh()->vertices().at( edgeVert.at( 1 ) );
@@ -1261,9 +1229,16 @@ void QgsMapToolEditMeshFrame::keyPressEvent( QKeyEvent *e )
       {
         mNewFaceBand->removePoint( -2, true );
         if ( !mNewFaceCandidate.isEmpty() )
+        {
+          if ( mNewFaceCandidate.last() == -1 && !mNewVerticesForNewFaceCandidate.isEmpty() )
+            mNewVerticesForNewFaceCandidate.removeLast();
           mNewFaceCandidate.removeLast();
+        }
         if ( mNewFaceCandidate.isEmpty() )
           mCurrentState = Digitizing;
+
+        mCadDockWidget->removePreviousPoint();
+
         consumned = true;
       }
 
@@ -1271,6 +1246,7 @@ void QgsMapToolEditMeshFrame::keyPressEvent( QKeyEvent *e )
       {
         mNewFaceBand->reset( QgsWkbTypes::PolygonGeometry );
         mNewFaceCandidate.clear();
+        mNewVerticesForNewFaceCandidate.clear();
         mCurrentState = Digitizing;
         consumned = true;
       }
@@ -1309,6 +1285,9 @@ void QgsMapToolEditMeshFrame::keyPressEvent( QKeyEvent *e )
       break;
   }
 
+  if ( e->key() == Qt::Key_Escape )
+    mCadDockWidget->clearPoints();
+
   if ( !consumned && mZValueWidget )
     QgsApplication::sendEvent( mZValueWidget->keyboardEntryWidget(), e );
   else
@@ -1429,11 +1408,11 @@ void QgsMapToolEditMeshFrame::searchEdge( const QgsPointXY &mapPoint )
       int iv1 = face.at( i );
       int iv2 = face.at( ( i + 1 ) % faceSize );
 
-      QgsPointXY pt1 = mapVertexXY( iv1 );
-      QgsPointXY pt2 = mapVertexXY( iv2 );
+      const QgsPointXY pt1 = mapVertexXY( iv1 );
+      const QgsPointXY pt2 = mapVertexXY( iv2 );
 
       QgsPointXY pointOneEdge;
-      double distance = sqrt( mapPoint.sqrDistToSegment( pt1.x(), pt1.y(), pt2.x(), pt2.y(), pointOneEdge ) );
+      double distance = sqrt( mapPoint.sqrDistToSegment( pt1.x(), pt1.y(), pt2.x(), pt2.y(), pointOneEdge, 0 ) );
       if ( distance < tolerance && distance < minimumDistance && edgeCanBeInteractive( iv1, iv2 ) )
       {
         mCurrentEdge = {faceIndex, iv2};
@@ -1445,9 +1424,8 @@ void QgsMapToolEditMeshFrame::searchEdge( const QgsPointXY &mapPoint )
 
 void QgsMapToolEditMeshFrame::highLight( const QgsPointXY &mapPoint )
 {
-  searchFace( mapPoint );
   highlightCurrentHoveredFace( mapPoint );
-  searchEdge( mapPoint );
+//  searchEdge( mapPoint );
   highlightCloseVertex( mapPoint );
   highlightCloseEdge( mapPoint );
 }
@@ -1462,7 +1440,11 @@ void QgsMapToolEditMeshFrame::setCurrentLayer( QgsMapLayer *layer )
   QgsMeshLayer *meshLayer = qobject_cast<QgsMeshLayer *>( layer );
 
   if ( mCurrentLayer == meshLayer && mCurrentEditor != nullptr )
+  {
+    activate();
+    updateFreeVertices();
     return;
+  }
 
   if ( mCurrentEditor )
     deactivate();
@@ -1474,6 +1456,7 @@ void QgsMapToolEditMeshFrame::setCurrentLayer( QgsMapLayer *layer )
   {
     disconnect( mCurrentLayer, &QgsMeshLayer::editingStarted, this, &QgsMapToolEditMeshFrame::onEditingStarted );
     disconnect( mCurrentLayer, &QgsMeshLayer::editingStopped, this, &QgsMapToolEditMeshFrame::onEditingStopped );
+    disconnect( mCurrentLayer->undoStack(), &QUndoStack::indexChanged, this, &QgsMapToolEditMeshFrame::onUndoRedo );
   }
 
   mCurrentLayer = meshLayer;
@@ -1490,6 +1473,7 @@ void QgsMapToolEditMeshFrame::setCurrentLayer( QgsMapLayer *layer )
   {
     connect( mCurrentLayer, &QgsMeshLayer::editingStarted, this, &QgsMapToolEditMeshFrame::onEditingStarted );
     connect( mCurrentLayer, &QgsMeshLayer::editingStopped, this, &QgsMapToolEditMeshFrame::onEditingStopped );
+    connect( mCurrentLayer->undoStack(), &QUndoStack::indexChanged, this, &QgsMapToolEditMeshFrame::onUndoRedo );
 
     if ( mCurrentLayer->isEditable() )
     {
@@ -1557,7 +1541,7 @@ QgsPointXY QgsMapToolEditMeshFrame::newFaceMarkerPosition( int vertexIndex )
   }
   else
   {
-    QgsMeshVertexCirculator circulator = mCurrentEditor->vertexCirculator( vertexIndex );
+    const QgsMeshVertexCirculator circulator = mCurrentEditor->vertexCirculator( vertexIndex );
     circulator.goBoundaryClockwise();
     int indexPt1 = circulator.oppositeVertexClockwise();
     circulator.goBoundaryCounterClockwise();
@@ -1596,12 +1580,35 @@ void QgsMapToolEditMeshFrame::addVertexToFaceCanditate( int vertexIndex )
   mNewFaceCandidate.append( vertexIndex );
 }
 
-bool QgsMapToolEditMeshFrame::testNewVertexInFaceCanditate( int vertexIndex )
+void QgsMapToolEditMeshFrame::addVertexToFaceCanditate( const QgsPointXY &vertexPosition )
 {
-  QgsMeshFace face = mNewFaceCandidate.toVector();
-  if ( vertexIndex != -1 && !face.empty() && vertexIndex != mNewFaceCandidate.last() )
-    face.append( vertexIndex );
-  return mCurrentEditor->faceCanBeAdded( face );
+  mNewFaceBand->movePoint( vertexPosition );
+  mNewFaceBand->addPoint( vertexPosition );
+  QgsMeshVertex vert = QgsMeshVertex( vertexPosition.x(), vertexPosition.y(), currentZValue() );
+  mNewVerticesForNewFaceCandidate.append( vert );
+  mNewFaceCandidate.append( -1 );
+}
+
+bool QgsMapToolEditMeshFrame::testNewVertexInFaceCanditate( bool testLast, int vertexIndex, const QgsPointXY &mapPoint ) const
+{
+  QList<int> faceToTest = mNewFaceCandidate;
+  QList<QgsMeshVertex> newVertices = mNewVerticesForNewFaceCandidate;
+
+  if ( testLast )
+  {
+    if ( vertexIndex != -1 &&
+         !mNewFaceCandidate.empty() &&
+         vertexIndex != mNewFaceCandidate.last() &&
+         vertexIndex != mNewFaceCandidate.first() )
+      faceToTest.append( vertexIndex );
+    else if ( vertexIndex == -1 )
+    {
+      faceToTest.append( -1 );
+      newVertices.append( QgsMeshVertex( mapPoint ) );
+    }
+  }
+
+  return mCurrentEditor->faceCanBeAddedWithNewVertices( faceToTest, newVertices );
 }
 
 
@@ -1637,7 +1644,7 @@ bool QgsMapToolEditMeshFrame::isFaceSelected( int faceIndex )
   return true;
 }
 
-void QgsMapToolEditMeshFrame::setSelectedVertices( const QList<int> newSelectedVertices, Qgis::SelectBehavior behavior )
+void QgsMapToolEditMeshFrame::setSelectedVertices( const QList<int> &newSelectedVertices, Qgis::SelectBehavior behavior )
 {
   if ( mSelectedVertices.isEmpty() )
   {
@@ -1673,7 +1680,7 @@ void QgsMapToolEditMeshFrame::setSelectedVertices( const QList<int> newSelectedV
   prepareSelection();
 }
 
-void QgsMapToolEditMeshFrame::setSelectedFaces( const QList<int> newSelectedFaces, Qgis::SelectBehavior behavior )
+void QgsMapToolEditMeshFrame::setSelectedFaces( const QList<int> &newSelectedFaces, Qgis::SelectBehavior behavior )
 {
   bool removeFaces = false;
 
@@ -1723,7 +1730,7 @@ void QgsMapToolEditMeshFrame::removeSelectedVerticesFromMesh( bool fillHole )
   if ( fillHole )
   {
 
-    QList<int> remainingVertex = mCurrentEditor->removeVerticesFillHoles( mSelectedVertices.keys() );
+    const QList<int> remainingVertex = mCurrentEditor->removeVerticesFillHoles( mSelectedVertices.keys() );
 
     if ( !remainingVertex.isEmpty() )
     {
@@ -1734,7 +1741,7 @@ void QgsMapToolEditMeshFrame::removeSelectedVerticesFromMesh( bool fillHole )
   }
   else
   {
-    QgsMeshEditingError error = mCurrentEditor->removeVerticesWithoutFillHoles( mSelectedVertices.keys() );
+    const QgsMeshEditingError error = mCurrentEditor->removeVerticesWithoutFillHoles( mSelectedVertices.keys() );
     if ( error != QgsMeshEditingError() )
     {
       QgisApp::instance()->messageBar()->pushWarning(
@@ -1810,7 +1817,7 @@ void QgsMapToolEditMeshFrame::triggerTransformCoordinatesDockWidget( bool checke
     if ( !mCurrentLayer || !mCurrentEditor )
       return;
 
-    QList<int> faceList = qgis::setToList( mSelectedFaces );
+    const QList<int> faceList = qgis::setToList( mSelectedFaces );
     QgsGeometry faceGeometrie;
     if ( faceList.count() == 1 )
     {
@@ -1861,7 +1868,7 @@ void QgsMapToolEditMeshFrame::triggerTransformCoordinatesDockWidget( bool checke
       }
     }
 
-    QgsCoordinateTransform coordinateTransform( mCurrentLayer->crs(), canvas()->mapSettings().destinationCrs(), QgsProject::instance() );
+    const QgsCoordinateTransform coordinateTransform( mCurrentLayer->crs(), canvas()->mapSettings().destinationCrs(), QgsProject::instance() );
 
     try
     {
@@ -1879,7 +1886,8 @@ void QgsMapToolEditMeshFrame::triggerTransformCoordinatesDockWidget( bool checke
 
     mMovingFacesRubberband->setToGeometry( faceGeometrie );
     mMovingEdgesRubberband->setToGeometry( edgesGeom );
-    for ( const int vertexIndex : mSelectedVertices.keys() )
+    const QList<int> vertexIndexes = mSelectedVertices.keys();
+    for ( const int vertexIndex : vertexIndexes )
       if ( mCurrentEditor->isVertexFree( vertexIndex ) )
       {
         QgsMeshVertex transformedVertex = mTransformDockWidget->transformedVertex( vertexIndex );
@@ -1925,19 +1933,47 @@ void QgsMapToolEditMeshFrame::reindexMesh()
   if ( !mCurrentLayer || !mCurrentLayer->isEditable() )
     return;
 
-  if ( QMessageBox::question( canvas(), tr( "Reindex the Mesh" ),
+  if ( QMessageBox::question( canvas(), tr( "Reindex Mesh" ),
                               tr( "Do you want to reindex the faces and vertices of the mesh layer %1?" ).arg( mCurrentLayer->name() ),
                               QMessageBox::Yes | QMessageBox::No, QMessageBox::No )
        == QMessageBox::No )
     return;
 
 
-  QgsCoordinateTransform transform( mCurrentLayer->crs(), canvas()->mapSettings().destinationCrs(), QgsProject::instance() );
+  const QgsCoordinateTransform transform( mCurrentLayer->crs(), canvas()->mapSettings().destinationCrs(), QgsProject::instance() );
 
   QgsTemporaryCursorOverride waitCursor( Qt::WaitCursor );
   mCurrentLayer->reindex( transform, true );
 }
 
+void QgsMapToolEditMeshFrame::onUndoRedo()
+{
+  switch ( mCurrentState )
+  {
+    case Digitizing:
+      break;
+    case AddingNewFace:
+      mNewFaceBand->reset( QgsWkbTypes::PolygonGeometry );
+      mNewFaceCandidate.clear();
+      mNewVerticesForNewFaceCandidate.clear();
+      mCurrentState = Digitizing;
+      mCadDockWidget->clearPoints();
+      break;
+    case MovingSelection:
+      mCurrentState = Digitizing;
+      mMovingEdgesRubberband->reset( QgsWkbTypes::LineGeometry );
+      mMovingFacesRubberband->reset( QgsWkbTypes::PolygonGeometry );
+      mMovingFreeVertexRubberband->reset( QgsWkbTypes::PointGeometry );
+      mCadDockWidget->setEnabledZ( mCadDockWidget->cadEnabled() );
+      mCadDockWidget->clearPoints();
+      break;
+    case ForceByLines:
+    case Selecting:
+    case SelectingByPolygon:
+      break;
+  }
+}
+
 void QgsMapToolEditMeshFrame::selectByGeometry( const QgsGeometry &geometry, Qt::KeyboardModifiers modifiers )
 {
   if ( mCurrentLayer.isNull() || !mCurrentLayer->triangularMesh() || mCurrentEditor.isNull() )
@@ -2057,8 +2093,6 @@ void QgsMapToolEditMeshFrame::prepareSelection()
   }
 
   mConcernedFaceBySelection.clear();
-  QMap<int, SelectedVertexData> movingVertices;
-
 
   double xMin = std::numeric_limits<double>::max();
   double xMax = -std::numeric_limits<double>::max();
@@ -2071,7 +2105,7 @@ void QgsMapToolEditMeshFrame::prepareSelection()
     SelectedVertexData &vertexData = it.value();
     int vertexIndex = it.key();
 
-    QgsPointXY vert = mapVertex( vertexIndex );
+    const QgsPointXY vert = mapVertex( vertexIndex );
     if ( vert.x() < xMin )
       xMin = vert.x();
     if ( vert.x() > xMax )
@@ -2083,7 +2117,7 @@ void QgsMapToolEditMeshFrame::prepareSelection()
 
     vertexData.borderEdges.clear();
     vertexData.meshFixedEdges.clear();
-    QgsMeshVertexCirculator circulator = mCurrentEditor->vertexCirculator( vertexIndex );
+    const QgsMeshVertexCirculator circulator = mCurrentEditor->vertexCirculator( vertexIndex );
 
     if ( !circulator.isValid() )
       continue;
@@ -2176,13 +2210,13 @@ void QgsMapToolEditMeshFrame::prepareSelection()
 
   if ( mSelectedVertices.count() == 1 )
   {
-    mActionRemoveVerticesFillingHole->setText( tr( "Remove selected vertex and fill hole" ) );
-    mActionRemoveVerticesWithoutFillingHole->setText( tr( "Remove selected vertex without filling hole" ) );
+    mActionRemoveVerticesFillingHole->setText( tr( "Remove Selected Vertex and Fill Hole" ) );
+    mActionRemoveVerticesWithoutFillingHole->setText( tr( "Remove Selected Vertex without Filling Hole" ) );
   }
   else if ( mSelectedVertices.count() > 1 )
   {
-    mActionRemoveVerticesFillingHole->setText( tr( "Remove selected vertices and fill hole(s)" ) );
-    mActionRemoveVerticesWithoutFillingHole->setText( tr( "Remove selected vertices without filling hole(s)" ) );
+    mActionRemoveVerticesFillingHole->setText( tr( "Remove Selected Vertices and Fill Hole(s)" ) );
+    mActionRemoveVerticesWithoutFillingHole->setText( tr( "Remove Selected Vertices without Filling Hole(s)" ) );
   }
 
   if ( facesList.count() == 1 )
@@ -2204,11 +2238,11 @@ void QgsMapToolEditMeshFrame::prepareSelection()
   }
 
   if ( mSplittableFaceCount == 1 )
-    mActionSplitFaces->setText( tr( "Split selected face" ) );
+    mActionSplitFaces->setText( tr( "Split Selected Face" ) );
   else if ( mSplittableFaceCount > 1 )
-    mActionSplitFaces->setText( tr( "Split %1 selected faces" ).arg( mSplittableFaceCount ) );
+    mActionSplitFaces->setText( tr( "Split %n Selected Face(s)", nullptr, mSplittableFaceCount ) );
   else
-    mActionSplitFaces->setText( tr( "Split current face" ) );
+    mActionSplitFaces->setText( tr( "Split Current Face" ) );
 
   if ( mRefinableFaceCount == 1 )
     mActionFacesRefinement->setText( tr( "Refine Selected Face" ) );
@@ -2256,7 +2290,7 @@ void QgsMapToolEditMeshFrame::setMovingRubberBandValidity( bool valid )
   }
 }
 
-bool QgsMapToolEditMeshFrame::isSelectionGrapped( QgsPointXY &grappedPoint )
+bool QgsMapToolEditMeshFrame::isSelectionGrapped( QgsPointXY &grappedPoint ) const
 {
   if ( mCurrentVertexIndex != -1 && mSelectedVertices.contains( mCurrentVertexIndex ) )
   {
@@ -2295,7 +2329,7 @@ bool QgsMapToolEditMeshFrame::isSelectionGrapped( QgsPointXY &grappedPoint )
 
 void QgsMapToolEditMeshFrame::forceByLineReleaseEvent( QgsMapMouseEvent *e )
 {
-  QgsPointXY mapPoint = e->mapPoint();
+  const QgsPointXY mapPoint = e->mapPoint();
 
   if ( e->button() == Qt::LeftButton )
   {
@@ -2312,7 +2346,7 @@ void QgsMapToolEditMeshFrame::forceByLineReleaseEvent( QgsMapMouseEvent *e )
     {
       if ( e->mapPointMatch().isValid() )
       {
-        QgsPoint layerPoint =  e->mapPointMatch().interpolatedPoint( mCanvas->mapSettings().destinationCrs() );
+        const QgsPoint layerPoint =  e->mapPointMatch().interpolatedPoint( mCanvas->mapSettings().destinationCrs() );
         zValue = layerPoint.z();
       }
 
@@ -2330,7 +2364,7 @@ void QgsMapToolEditMeshFrame::forceByLineReleaseEvent( QgsMapMouseEvent *e )
     }
 
     QVector<QgsPoint> points;
-    QgsPolylineXY rubbergandLines = mForceByLineRubberBand->asGeometry().asPolyline();
+    const QgsPolylineXY rubbergandLines = mForceByLineRubberBand->asGeometry().asPolyline();
     double defaultValue = currentZValue();
 
     if ( std::isnan( defaultValue ) )
@@ -2385,7 +2419,7 @@ void QgsMapToolEditMeshFrame::highlightCurrentHoveredFace( const QgsPointXY &map
     }
   }
 
-  QgsPointSequence faceGeometry = nativeFaceGeometry( mCurrentFaceIndex );
+  const QgsPointSequence faceGeometry = nativeFaceGeometry( mCurrentFaceIndex );
   mFaceRubberBand->reset( QgsWkbTypes::PolygonGeometry );
   mFaceVerticesBand->reset( QgsWkbTypes::PointGeometry );
   for ( const QgsPoint &pt : faceGeometry )
@@ -2401,8 +2435,6 @@ void QgsMapToolEditMeshFrame::highlightCurrentHoveredFace( const QgsPointXY &map
   }
   else
     mSelectFaceMarker->setVisible( false );
-
-  return;
 }
 
 void QgsMapToolEditMeshFrame::highlightCloseVertex( const QgsPointXY &mapPoint )
@@ -2410,7 +2442,6 @@ void QgsMapToolEditMeshFrame::highlightCloseVertex( const QgsPointXY &mapPoint )
   if ( !mCurrentEditor )
     return;
 
-
   if ( mCurrentState == Digitizing && mNewFaceMarker->isVisible() )
   {
     double tol = QgsTolerance::vertexSearchRadius( canvas()->mapSettings() );
@@ -2495,6 +2526,8 @@ void QgsMapToolEditMeshFrame::highlightCloseEdge( const QgsPointXY &mapPoint )
     }
   }
 
+  searchEdge( mapPoint );
+
   if ( mMergeFaceMarker->isVisible() )
   {
     if ( mapPoint.distance( mMergeFaceMarker->center() ) < tolerance )
@@ -2503,8 +2536,6 @@ void QgsMapToolEditMeshFrame::highlightCloseEdge( const QgsPointXY &mapPoint )
       mMergeFaceMarker->setColor( Qt::gray );
   }
 
-  searchEdge( mapPoint );
-
   mEdgeBand->reset();
   mFlipEdgeMarker->setVisible( false );
   mMergeFaceMarker->setVisible( false );
@@ -2578,7 +2609,7 @@ void QgsMapToolEditMeshFrame::highlightCloseEdge( const QgsPointXY &mapPoint )
 bool QgsMapToolEditMeshFrame::edgeCanBeInteractive( int vertexIndex1, int vertexIndex2 ) const
 {
   // If the edge is less than 90px width, the interactive marker will not be displayed to avoid too close marker and
-  // avoit the user to click on a marker if he doesn't want
+  // avoid the user to click on a marker if he doesn't want
   double mapUnitPerPixel = mCanvas->mapSettings().mapUnitsPerPixel();
   return mapVertexXY( vertexIndex1 ).distance( mapVertexXY( vertexIndex2 ) ) / mapUnitPerPixel > 90;
 }
@@ -2586,10 +2617,10 @@ bool QgsMapToolEditMeshFrame::edgeCanBeInteractive( int vertexIndex1, int vertex
 bool QgsMapToolEditMeshFrame::faceCanBeInteractive( int faceIndex ) const
 {
   // If both side of the face boundng box is less than 60px width, the interactive marker will not be displayed to avoid too close marker and
-  // avoit the user to click on a marker if he doesn't want
+  // avoid the user to click on a marker if he doesn't want
 
   double mapUnitPerPixel = mCanvas->mapSettings().mapUnitsPerPixel();
-  QgsGeometry faceGeom( new QgsLineString( nativeFaceGeometry( faceIndex ) ) );
+  const QgsGeometry faceGeom( new QgsLineString( nativeFaceGeometry( faceIndex ) ) );
   QgsRectangle bbox = faceGeom.boundingBox();
 
   return bbox.width() / mapUnitPerPixel > 60 || bbox.height() / mapUnitPerPixel > 60;
@@ -2637,6 +2668,7 @@ void QgsMapToolEditMeshFrame::clearCanvasHelpers()
   mFaceRubberBand->reset();
   mFaceVerticesBand->reset();
   mVertexBand->reset();
+  mNewFaceBand->reset();
 
   mSnapIndicator->setMatch( QgsPointLocator::Match() );
   mSelectFaceMarker->setVisible( false );
@@ -2667,7 +2699,7 @@ void QgsMapToolEditMeshFrame::addVertex(
             mapPointMatch.layer() &&
             QgsWkbTypes::hasZ( mapPointMatch.layer()->wkbType() ) )
   {
-    QgsPoint layerPoint = mapPointMatch.interpolatedPoint( mCanvas->mapSettings().destinationCrs() );
+    const QgsPoint layerPoint = mapPointMatch.interpolatedPoint( mCanvas->mapSettings().destinationCrs() );
     zValue = layerPoint.z();
   }
   else if ( mCurrentEdge.first != -1 && mCurrentEdge.second != -1 ) //we are on a edge -->interpolate the z value
@@ -2713,8 +2745,8 @@ void QgsMapToolEditMeshFrame::updateFreeVertices()
   int freeVertexCount = freeVertexIndexes.count();
   mFreeVertexMarker.reserve( freeVertexCount );
 
-  QColor freeVertexColor = digitizingStrokeColor();
-  QColor fillFreeVertexColor = freeVertexColor.lighter();
+  const QColor freeVertexColor = digitizingStrokeColor();
+  const QColor fillFreeVertexColor = freeVertexColor.lighter();
 
   for ( const int freeVertexIndex : freeVertexIndexes )
   {
