@@ -32,13 +32,12 @@
 #include <QDesktopServices>
 #include <QRegularExpression>
 #include <QUrl>
+#include <QCheckBox>
 
 #include "qgsmessagelog.h"
 
 #include "qgis.h"
-#include "qgisapp.h"
 #include "qgsapplication.h"
-#include "qgsconfig.h"
 #include "qgsmessagebar.h"
 #include "qgsproviderregistry.h"
 #include "qgspluginregistry.h"
@@ -49,6 +48,8 @@
 #include "qgslogger.h"
 #include "qgspluginitemdelegate.h"
 #include "qgssettings.h"
+#include "qgsgui.h"
+#include "qgssettingsregistrygui.h"
 #ifdef WITH_BINDINGS
 #include "qgspythonutils.h"
 #endif
@@ -239,11 +240,8 @@ void QgsPluginManager::setPythonUtils( QgsPythonUtils *pythonUtils )
   connect( mZipFileWidget, &QgsFileWidget::fileChanged, this, &QgsPluginManager::mZipFileWidget_fileChanged );
   connect( buttonInstallFromZip, &QPushButton::clicked, this, &QgsPluginManager::buttonInstallFromZip_clicked );
 
-  // Initialize list of allowed checking intervals
-  mCheckingOnStartIntervals << 0 << 1 << 3 << 7 << 14 << 30;
-
   // Initialize the "Settings" tab widgets
-  if ( settings.value( settingsGroup + "/checkOnStart", false ).toBool() )
+  if ( QgsSettingsRegistryGui::settingsAutomaticallyCheckForPluginUpdates.value() )
   {
     ckbCheckUpdates->setChecked( true );
   }
@@ -257,14 +255,8 @@ void QgsPluginManager::setPythonUtils( QgsPythonUtils *pythonUtils )
   {
     ckbDeprecated->setChecked( true );
   }
-
-  const int interval = settings.value( settingsGroup + "/checkOnStartInterval", "" ).toInt();
-  const int indx = mCheckingOnStartIntervals.indexOf( interval ); // if none found, just use -1 index.
-  comboInterval->setCurrentIndex( indx );
 }
 
-
-
 void QgsPluginManager::loadPlugin( const QString &id )
 {
   const QMap<QString, QString> *plugin = pluginMetadata( id );
@@ -281,12 +273,12 @@ void QgsPluginManager::loadPlugin( const QString &id )
   if ( plugin->value( QStringLiteral( "pythonic" ) ) == QLatin1String( "true" ) )
   {
     library = plugin->value( QStringLiteral( "id" ) );
-    QgsDebugMsg( "Loading Python plugin: " + library );
+    QgsDebugMsgLevel( "Loading Python plugin: " + library, 2 );
     pRegistry->loadPythonPlugin( library );
   }
   else // C++ plugin
   {
-    QgsDebugMsg( "Loading C++ plugin: " + library );
+    QgsDebugMsgLevel( "Loading C++ plugin: " + library, 2 );
     pRegistry->loadCppPlugin( library );
   }
 
@@ -363,10 +355,11 @@ void QgsPluginManager::getCppPluginsMetadata()
   QStringList myPathList( pr->libraryDirectory().path() );
 
   const QgsSettings settings;
-  const QString myPaths = settings.value( QStringLiteral( "plugins/searchPathsForPlugins" ), "" ).toString();
+  const QStringList myPaths = settings.value( QStringLiteral( "plugins/searchPathsForPlugins" ) ).toStringList();
   if ( !myPaths.isEmpty() )
   {
-    myPathList.append( myPaths.split( '|' ) );
+    myPathList.append( myPaths );
+    myPathList.removeDuplicates();
   }
 
   for ( int j = 0; j < myPathList.size(); ++j )
@@ -377,7 +370,7 @@ void QgsPluginManager::getCppPluginsMetadata()
     if ( pluginDir.count() == 0 )
     {
       QMessageBox::information( this, tr( "No Plugins" ), tr( "No QGIS plugins found in %1" ).arg( myPluginDir ) );
-      return;
+      continue;
     }
 
     for ( uint i = 0; i < pluginDir.count(); i++ )
@@ -403,7 +396,7 @@ void QgsPluginManager::getCppPluginsMetadata()
       }
       else
       {
-        QgsDebugMsg( "dlopen succeeded for " + lib );
+        QgsDebugMsgLevel( "dlopen succeeded for " + lib, 2 );
         dlclose( handle );
       }
 #endif //#ifndef Q_OS_WIN && Q_OS_MACX
@@ -600,12 +593,12 @@ void QgsPluginManager::reloadModelData()
       mypDetailItem->setEditable( false );
 
       // Set checkable if the plugin is installed and not disabled due to incompatibility.
-      // Broken plugins are checkable to to allow disabling them
+      // Broken plugins are checkable to allow disabling them
       mypDetailItem->setCheckable( it->value( QStringLiteral( "installed" ) ) == QLatin1String( "true" ) && it->value( QStringLiteral( "error" ) ) != QLatin1String( "incompatible" ) );
 
       // Set ckeckState depending on the plugin is loaded or not.
       // Initially mark all unchecked, then overwrite state of loaded ones with checked.
-      // Only do it with installed plugins, not not initialize checkboxes of not installed plugins at all.
+      // Only do it with installed plugins, do not initialize checkboxes of not installed plugins at all.
       if ( it->value( QStringLiteral( "installed" ) ) == QLatin1String( "true" ) )
       {
         mypDetailItem->setCheckState( Qt::Unchecked );
@@ -665,19 +658,19 @@ void QgsPluginManager::pluginItemChanged( QStandardItem *item )
   {
     if ( mPluginsAreEnabled && ! isPluginEnabled( id ) )
     {
-      QgsDebugMsg( " Loading plugin: " + id );
+      QgsDebugMsgLevel( " Loading plugin: " + id, 2 );
       loadPlugin( id );
     }
     else
     {
       // only enable the plugin, as we're in --noplugins mode
-      QgsDebugMsg( " Enabling plugin: " + id );
+      QgsDebugMsgLevel( " Enabling plugin: " + id, 2 );
       savePluginState( id, true );
     }
   }
   else if ( ! item->checkState() )
   {
-    QgsDebugMsg( " Unloading plugin: " + id );
+    QgsDebugMsgLevel( " Unloading plugin: " + id, 2 );
     unloadPlugin( id );
   }
 }
@@ -792,7 +785,7 @@ void QgsPluginManager::showPluginDetails( QStandardItem *item )
     voteLabel->show();
     voteSlider->show();
     voteSubmit->show();
-    QgsDebugMsg( QStringLiteral( "vote slider:%1" ).arg( std::round( metadata->value( "average_vote" ).toFloat() ) ) );
+    QgsDebugMsgLevel( QStringLiteral( "vote slider:%1" ).arg( std::round( metadata->value( "average_vote" ).toFloat() ) ), 2 );
     voteSlider->setValue( std::round( metadata->value( "average_vote" ).toFloat() ) );
     mCurrentPluginId = metadata->value( "plugin_id" ).toInt();
   }
@@ -917,7 +910,7 @@ void QgsPluginManager::showPluginDetails( QStandardItem *item )
   if ( ! metadata->value( QStringLiteral( "about" ) ).isEmpty() )
   {
     QString about = metadata->value( QStringLiteral( "about" ) );
-    // The regular expression insures that a new line will be present after the closure of a paragraph tag (i.e. </p>)
+    // The regular expression ensures that a new line will be present after the closure of a paragraph tag (i.e. </p>)
     about = about.replace( QRegularExpression( QStringLiteral( "&lt;\\/p&gt;([^\\n])" ) ), QStringLiteral( "&lt;/p&gt;\n\\1" ) ).remove( stripHtml );
     html += about.replace( '\n', QLatin1String( "<br/>" ) );
     html += QLatin1String( "<br/><br/>" );
@@ -1035,7 +1028,7 @@ void QgsPluginManager::showPluginDetails( QStandardItem *item )
     {
       const QDateTime dateUpdated = QDateTime::fromString( metadata->value( QStringLiteral( "update_date_stable" ) ).trimmed(), Qt::ISODate );
       if ( dateUpdated.isValid() )
-        dateUpdatedStr += QStringLiteral( "%1 %2" ).arg( tr( "updated at" ), dateUpdated.toString() );
+        dateUpdatedStr += tr( "updated at %1" ).arg( dateUpdated.toString() );
     }
 
     html += QStringLiteral( "<tr><td class='key'>%1 </td><td title='%2'><a href='%2'>%3</a> %4</td></tr>"
@@ -1059,7 +1052,7 @@ void QgsPluginManager::showPluginDetails( QStandardItem *item )
     {
       const QDateTime dateUpdated = QDateTime::fromString( metadata->value( QStringLiteral( "update_date_experimental" ) ).trimmed(), Qt::ISODate );
       if ( dateUpdated.isValid() )
-        dateUpdatedStr += QStringLiteral( "%1 %2" ).arg( tr( "updated at" ), dateUpdated.toString() );
+        dateUpdatedStr += tr( "updated at %1" ).arg( dateUpdated.toString() );
     }
 
     html += QStringLiteral( "<tr><td class='key'>%1 </td><td title='%2'><a href='%2'>%3</a> %4</td></tr>"
@@ -1094,20 +1087,24 @@ void QgsPluginManager::showPluginDetails( QStandardItem *item )
   if ( metadata->value( QStringLiteral( "status" ) ) == QLatin1String( "upgradeable" ) )
   {
     buttonInstall->setText( tr( "Upgrade Plugin" ) );
+    buttonInstall->setToolTip( tr( "Upgrades selected plugin to latest stable version" ) );
     buttonInstall->setDefault( true );
   }
   else if ( metadata->value( QStringLiteral( "status" ) ) == QLatin1String( "newer" ) )
   {
     buttonInstall->setText( tr( "Downgrade Plugin" ) );
+    buttonInstall->setToolTip( tr( "Downgrades selected plugin to latest stable version" ) );
   }
   else if ( metadata->value( QStringLiteral( "status" ) ) == QLatin1String( "not installed" ) || metadata->value( QStringLiteral( "status" ) ) == QLatin1String( "new" ) )
   {
     buttonInstall->setText( tr( "Install Plugin" ) );
+    buttonInstall->setToolTip( tr( "Installs latest stable version of the selected plugin" ) );
   }
   else
   {
     // Default (will be grayed out if not available for reinstallation)
     buttonInstall->setText( tr( "Reinstall Plugin" ) );
+    buttonInstall->setToolTip( tr( "Reinstalls latest stable version of the selected plugin" ) );
   }
 
   // Set buttonInstall text (and sometimes focus)
@@ -1115,19 +1112,23 @@ void QgsPluginManager::showPluginDetails( QStandardItem *item )
   if ( metadata->value( QStringLiteral( "status_exp" ) ) == QLatin1String( "upgradeable" ) )
   {
     buttonInstallExperimental->setText( tr( "Upgrade Experimental Plugin" ) );
+    buttonInstallExperimental->setToolTip( tr( "Upgrades selected plugin to the experimental version" ) );
   }
   else if ( metadata->value( QStringLiteral( "status_exp" ) ) == QLatin1String( "newer" ) )
   {
     buttonInstallExperimental->setText( tr( "Downgrade Experimental Plugin" ) );
+    buttonInstallExperimental->setToolTip( tr( "Downgrades selected plugin to the experimental version" ) );
   }
   else if ( metadata->value( QStringLiteral( "status_exp" ) ) == QLatin1String( "not installed" ) || metadata->value( QStringLiteral( "status" ) ) == QLatin1String( "new" ) )
   {
     buttonInstallExperimental->setText( tr( "Install Experimental Plugin" ) );
+    buttonInstallExperimental->setToolTip( tr( "Installs experimental version of the selected plugin" ) );
   }
   else
   {
     // Default (will be grayed out if not available for reinstallation)
     buttonInstallExperimental->setText( tr( "Reinstall Experimental Plugin" ) );
+    buttonInstallExperimental->setToolTip( tr( "Reinstalls experimental version of the selected plugin" ) );
   }
 
   // Enable/disable buttons
@@ -1147,9 +1148,7 @@ void QgsPluginManager::showPluginDetails( QStandardItem *item )
 
   buttonUninstall->setEnabled( metadata->value( QStringLiteral( "pythonic" ) ).toUpper() == QLatin1String( "TRUE" ) && metadata->value( QStringLiteral( "readonly" ) ) != QLatin1String( "true" ) && ! metadata->value( QStringLiteral( "version_installed" ) ).isEmpty() );
 
-  buttonUninstall->setHidden(
-    metadata->value( QStringLiteral( "version_installed" ) ).isEmpty()
-  );
+  buttonUninstall->setHidden( metadata->value( QStringLiteral( "version_installed" ) ).isEmpty() );
 
   // Store the id of the currently displayed plugin
   mCurrentlyDisplayedPlugin = metadata->value( QStringLiteral( "id" ) );
@@ -1290,9 +1289,7 @@ void QgsPluginManager::reject()
     // get the QgsSettings group from the installer
     QString settingsGroup;
     QgsPythonRunner::eval( QStringLiteral( "pyplugin_installer.instance().exportSettingsGroup()" ), settingsGroup );
-    QgsSettings settings;
-    settings.setValue( settingsGroup + "/checkOnStart", QVariant( ckbCheckUpdates->isChecked() ) );
-    settings.setValue( settingsGroup + "/checkOnStartInterval", QVariant( mCheckingOnStartIntervals.value( comboInterval->currentIndex() ) ) );
+    QgsSettingsRegistryGui::settingsAutomaticallyCheckForPluginUpdates.setValue( ckbCheckUpdates->isChecked() );
     QgsPythonRunner::run( QStringLiteral( "pyplugin_installer.instance().onManagerClose()" ) );
   }
 #endif
@@ -1469,29 +1466,23 @@ void QgsPluginManager::leFilter_textChanged( QString text )
   {
     text = text.remove( QStringLiteral( "tag:" ) );
     mModelProxy->setFilterRole( PLUGIN_TAGS_ROLE );
-    QgsDebugMsg( "PluginManager TAG filter changed to :" + text );
+    QgsDebugMsgLevel( "PluginManager TAG filter changed to :" + text, 3 );
   }
   else
   {
     mModelProxy->setFilterRole( 0 );
-    QgsDebugMsg( "PluginManager filter changed to :" + text );
+    QgsDebugMsgLevel( "PluginManager filter changed to :" + text, 3 );
   }
 
-  const QRegExp::PatternSyntax mySyntax = QRegExp::PatternSyntax( QRegExp::RegExp );
-  const Qt::CaseSensitivity myCaseSensitivity = Qt::CaseInsensitive;
-  const QRegExp myRegExp( text, myCaseSensitivity, mySyntax );
-  mModelProxy->setFilterRegExp( myRegExp );
+  const QRegularExpression filterRegExp( text, QRegularExpression::CaseInsensitiveOption );
+  mModelProxy->setFilterRegularExpression( filterRegExp );
 }
 
-
-
 void QgsPluginManager::buttonUpgradeAll_clicked()
 {
   QgsPythonRunner::run( QStringLiteral( "pyplugin_installer.instance().upgradeAllUpgradeable()" ) );
 }
 
-
-
 void QgsPluginManager::buttonInstall_clicked()
 {
   QgsPythonRunner::run( QStringLiteral( "pyplugin_installer.instance().installPlugin('%1', stable=True)" ).arg( mCurrentlyDisplayedPlugin ) );
@@ -1570,7 +1561,7 @@ void QgsPluginManager::setRepositoryFilter()
   {
     QString key = current->text( 1 );
     key = key.replace( '\'', QLatin1String( "\\\'" ) ).replace( '\"', QLatin1String( "\\\"" ) );
-    QgsDebugMsg( "Disabling all repositories but selected: " + key );
+    QgsDebugMsgLevel( "Disabling all repositories but selected: " + key, 2 );
     QgsPythonRunner::run( QStringLiteral( "pyplugin_installer.instance().setRepositoryInspectionFilter('%1')" ).arg( key ) );
   }
 }
@@ -1579,7 +1570,7 @@ void QgsPluginManager::setRepositoryFilter()
 
 void QgsPluginManager::clearRepositoryFilter()
 {
-  QgsDebugMsg( QStringLiteral( "Enabling all repositories back" ) );
+  QgsDebugMsgLevel( QStringLiteral( "Enabling all repositories back" ), 2 );
   QgsPythonRunner::run( QStringLiteral( "pyplugin_installer.instance().setRepositoryInspectionFilter()" ) );
 }
 
@@ -1587,7 +1578,7 @@ void QgsPluginManager::clearRepositoryFilter()
 
 void QgsPluginManager::buttonRefreshRepos_clicked()
 {
-  QgsDebugMsg( QStringLiteral( "Refreshing repositories..." ) );
+  QgsDebugMsgLevel( QStringLiteral( "Refreshing repositories..." ), 2 );
   QgsPythonRunner::run( QStringLiteral( "pyplugin_installer.instance().reloadAndExportData()" ) );
 }
 
@@ -1595,7 +1586,7 @@ void QgsPluginManager::buttonRefreshRepos_clicked()
 
 void QgsPluginManager::buttonAddRep_clicked()
 {
-  QgsDebugMsg( QStringLiteral( "Adding repository connection..." ) );
+  QgsDebugMsgLevel( QStringLiteral( "Adding repository connection..." ), 2 );
   QgsPythonRunner::run( QStringLiteral( "pyplugin_installer.instance().addRepository()" ) );
 }
 
@@ -1608,7 +1599,7 @@ void QgsPluginManager::buttonEditRep_clicked()
   {
     QString key = current->text( 1 );
     key = key.replace( '\'', QLatin1String( "\\\'" ) ).replace( '\"', QLatin1String( "\\\"" ) );
-    QgsDebugMsg( "Editing repository connection: " + key );
+    QgsDebugMsgLevel( "Editing repository connection: " + key, 2 );
     QgsPythonRunner::run( QStringLiteral( "pyplugin_installer.instance().editRepository('%1')" ).arg( key ) );
   }
 }
@@ -1622,7 +1613,7 @@ void QgsPluginManager::buttonDeleteRep_clicked()
   {
     QString key = current->text( 1 );
     key = key.replace( '\'', QLatin1String( "\\\'" ) ).replace( '\"', QLatin1String( "\\\"" ) );
-    QgsDebugMsg( "Deleting repository connection: " + key );
+    QgsDebugMsgLevel( "Deleting repository connection: " + key, 2 );
     QgsPythonRunner::run( QStringLiteral( "pyplugin_installer.instance().deleteRepository('%1')" ).arg( key ) );
   }
 }
