@@ -58,6 +58,7 @@
 #include "qgsorganizetablecolumnsdialog.h"
 #include "qgsvectorlayereditbuffer.h"
 #include "qgstransactiongroup.h"
+#include "qgsdockablewidgethelper.h"
 
 QgsExpressionContext QgsAttributeTableDialog::createExpressionContext() const
 {
@@ -75,6 +76,25 @@ QgsExpressionContext QgsAttributeTableDialog::createExpressionContext() const
   return expContext;
 }
 
+QDomElement QgsAttributeTableDialog::writeXml( QDomDocument &document )
+{
+  QDomElement element = document.createElement( QStringLiteral( "attributeTable" ) );
+  mDockableWidgetHelper->writeXml( element );
+
+  element.setAttribute( QStringLiteral( "layer" ), mLayer ? mLayer->id() : QString() );
+  element.setAttribute( QStringLiteral( "filterMode" ), qgsEnumValueToKey( mMainView->filterMode() ) );
+  element.setAttribute( QStringLiteral( "view" ), qgsEnumValueToKey( mMainView->view() ) );
+
+  return element;
+}
+
+void QgsAttributeTableDialog::readXml( const QDomElement &element )
+{
+  mDockableWidgetHelper->readXml( element );
+
+  mMainView->setView( qgsEnumKeyToValue( element.attribute( QStringLiteral( "view" ) ), QgsDualView::ViewMode::AttributeTable ) );
+}
+
 void QgsAttributeTableDialog::updateMultiEditButtonState()
 {
   if ( ! mLayer || ( mLayer->editFormConfig().layout() == QgsEditFormConfig::EditorLayout::UiFileLayout ) )
@@ -88,13 +108,21 @@ void QgsAttributeTableDialog::updateMultiEditButtonState()
   }
 }
 
-QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *layer, QgsAttributeTableFilterModel::FilterMode initialMode, QWidget *parent, Qt::WindowFlags flags )
+QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *layer, QgsAttributeTableFilterModel::FilterMode initialMode, QWidget *parent, Qt::WindowFlags flags, bool *initiallyDocked, const QString &filterExpression )
   : QDialog( parent, flags )
   , mLayer( layer )
-
 {
   setObjectName( QStringLiteral( "QgsAttributeTableDialog/" ) + layer->id() );
   setupUi( this );
+
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+  connect( mMainViewButtonGroup, qOverload< int >( &QButtonGroup::buttonClicked ), mMainView, &QgsDualView::setCurrentIndex );
+#else
+  connect( mMainViewButtonGroup, &QButtonGroup::idClicked, mMainView, &QgsDualView::setCurrentIndex );
+#endif
+  connect( mActionToggleEditing, &QAction::toggled, mActionSaveEdits, &QAction::setEnabled );
+  connect( mActionToggleEditing, &QAction::toggled, mActionReload, &QAction::setDisabled );
+
   connect( mActionCutSelectedRows, &QAction::triggered, this, &QgsAttributeTableDialog::mActionCutSelectedRows_triggered );
   connect( mActionCopySelectedRows, &QAction::triggered, this, &QgsAttributeTableDialog::mActionCopySelectedRows_triggered );
   connect( mActionPasteFeatures, &QAction::triggered, this, &QgsAttributeTableDialog::mActionPasteFeatures_triggered );
@@ -142,12 +170,6 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *layer, QgsAttr
     ? mActionAddFeatureViaAttributeForm->icon()
     : mActionAddFeatureViaAttributeTable->icon() );
 
-  const QgsFields fields = mLayer->fields();
-  for ( const QgsField &field : fields )
-  {
-    mVisibleFields.append( field.name() );
-  }
-
   // Fix selection color on losing focus (Windows)
   setStyleSheet( QgisApp::instance()->styleSheet() );
 
@@ -199,6 +221,10 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *layer, QgsAttr
   {
     r.setFilterFids( layer->editBuffer() ? layer->editBuffer()->allAddedOrEditedFeatures() : QgsFeatureIds() );
   }
+  else if ( !filterExpression.isEmpty() )
+  {
+    r.setFilterExpression( filterExpression );
+  }
   if ( !needsGeom )
     r.setFlags( QgsFeatureRequest::NoGeometry );
 
@@ -255,17 +281,40 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *layer, QgsAttr
   // info from table to application
   connect( this, &QgsAttributeTableDialog::saveEdits, this, [ = ] { QgisApp::instance()->saveEdits(); } );
 
-  const bool dockTable = settings.value( QStringLiteral( "qgis/dockAttributeTable" ), false ).toBool();
-  if ( dockTable )
+  const bool isDocked = initiallyDocked ? *initiallyDocked : settings.value( QStringLiteral( "qgis/dockAttributeTable" ), false ).toBool();
+  mDockableWidgetHelper = new QgsDockableWidgetHelper( isDocked, windowTitle(), this, QgisApp::instance(),
+      Qt::BottomDockWidgetArea,  QStringList(), !initiallyDocked, QStringLiteral( "Windows/BetterAttributeTable/geometry" ) );
+  connect( mDockableWidgetHelper, &QgsDockableWidgetHelper::closed, this, [ = ]()
   {
-    mDock = new QgsAttributeTableDock( QString(), QgisApp::instance() );
-    mDock->setWidget( this );
-    connect( this, &QObject::destroyed, mDock, &QWidget::close );
-    QgisApp::instance()->addTabifiedDockWidget( Qt::BottomDockWidgetArea, mDock, QStringList(), true );
-  }
-  mActionDockUndock->setChecked( dockTable );
-  connect( mActionDockUndock, &QAction::toggled, this, &QgsAttributeTableDialog::toggleDockMode );
-  installEventFilter( this );
+    close();
+  } );
+  connect( mDockableWidgetHelper, &QgsDockableWidgetHelper::dockModeToggled, this, [ = ]( bool docked )
+  {
+    if ( docked )
+    {
+      // To prevent "QAction::event: Ambiguous shortcut overload"
+      QgsDebugMsgLevel( QStringLiteral( "Remove shortcuts from attribute table already defined in main window" ), 2 );
+      mActionZoomMapToSelectedRows->setShortcut( QKeySequence() );
+      mActionRemoveSelection->setShortcut( QKeySequence() );
+      // duplicated on Main Window, with different semantics
+      mActionPanMapToSelectedRows->setShortcut( QKeySequence() );
+      mActionSearchForm->setShortcut( QKeySequence() );
+    }
+    else
+    {
+      // restore attribute table shortcuts in window mode
+      QgsDebugMsgLevel( QStringLiteral( "Restore attribute table dialog shortcuts in window mode" ), 2 );
+      // duplicated on Main Window
+      mActionZoomMapToSelectedRows->setShortcut( QStringLiteral( "Ctrl+J" ) );
+      mActionRemoveSelection->setShortcut( QStringLiteral( "Ctrl+Shift+A" ) );
+      // duplicated on Main Window, with different semantics
+      mActionPanMapToSelectedRows->setShortcut( QStringLiteral( "Ctrl+P" ) );
+      mActionSearchForm->setShortcut( QStringLiteral( "Ctrl+F" ) );
+    }
+  } );
+
+  mActionDockUndock = mDockableWidgetHelper->createDockUndockAction( tr( "Dock Attribute Table" ), this );
+  mToolbar->addAction( mActionDockUndock );
 
   updateTitle();
 
@@ -340,6 +389,10 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *layer, QgsAttr
       mFeatureFilterWidget->filterEdited();
       break;
 
+    case QgsAttributeTableFilterModel::ShowFilteredList:
+      mFeatureFilterWidget->setFilterExpression( filterExpression );
+      break;
+
     case QgsAttributeTableFilterModel::ShowAll:
     default:
       mFeatureFilterWidget->filterShowAll();
@@ -392,21 +445,23 @@ QgsAttributeTableDialog::QgsAttributeTableDialog( QgsVectorLayer *layer, QgsAttr
   }
 }
 
+QgsAttributeTableDialog::~QgsAttributeTableDialog()
+{
+  delete mDockableWidgetHelper;
+}
+
 void QgsAttributeTableDialog::updateTitle()
 {
   if ( ! mLayer )
   {
     return;
   }
-  QWidget *w = mDock ? qobject_cast<QWidget *>( mDock )
-               : mDialog ? qobject_cast<QWidget *>( mDialog )
-               : qobject_cast<QWidget *>( this );
-  w->setWindowTitle( tr( " %1 — Features Total: %2, Filtered: %3, Selected: %4" )
-                     .arg( mLayer->name() )
-                     .arg( std::max( static_cast< long long >( mMainView->featureCount() ), mLayer->featureCount() ) ) // layer count may be estimated, so use larger of the two
-                     .arg( mMainView->filteredFeatureCount() )
-                     .arg( mLayer->selectedFeatureCount() )
-                   );
+  const QString title = tr( " %1 — Features Total: %L2, Filtered: %L3, Selected: %L4" )
+                        .arg( mLayer->name() )
+                        .arg( std::max( static_cast< long long >( mMainView->featureCount() ), mLayer->featureCount() ) ) // layer count may be estimated, so use larger of the two
+                        .arg( mMainView->filteredFeatureCount() )
+                        .arg( mLayer->selectedFeatureCount() );
+  mDockableWidgetHelper->setWindowTitle( title );
 
   if ( mMainView->filterMode() == QgsAttributeTableFilterModel::ShowAll )
     mRunFieldCalc->setText( tr( "Update All" ) );
@@ -437,20 +492,6 @@ void QgsAttributeTableDialog::keyPressEvent( QKeyEvent *event )
   }
 }
 
-bool QgsAttributeTableDialog::eventFilter( QObject *object, QEvent *ev )
-{
-  if ( ev->type() == QEvent::Close && !mDock && ( !mDialog || mDialog == object ) )
-  {
-    if ( QWidget *w = qobject_cast< QWidget * >( object ) )
-    {
-      QgsSettings settings;
-      settings.setValue( QStringLiteral( "Windows/BetterAttributeTable/geometry" ), w->saveGeometry() );
-    }
-  }
-
-  return QDialog::eventFilter( object, ev );
-}
-
 void QgsAttributeTableDialog::updateFieldFromExpression()
 {
   bool filtered = mMainView->filterMode() != QgsAttributeTableFilterModel::ShowAll;
@@ -658,7 +699,7 @@ void QgsAttributeTableDialog::mActionAddFeatureViaAttributeTable_triggered()
   QgsAttributeTableModel *masterModel = mMainView->masterModel();
 
   QgsFeature f;
-  QgsFeatureAction action( tr( "Geometryless feature added" ), f, mLayer, QString(), -1, this );
+  QgsFeatureAction action( tr( "Geometryless feature added" ), f, mLayer, QUuid(), -1, this );
   action.setForceSuppressFormPopup( true ); // we're already showing the table, allowing users to enter the new feature's attributes directly
   if ( action.addFeature() )
   {
@@ -679,7 +720,7 @@ void QgsAttributeTableDialog::mActionAddFeatureViaAttributeForm_triggered()
 
   QgsFeature f;
 
-  QgsFeatureAction action( tr( "Feature Added" ), f, mLayer, QString(), -1, this );
+  QgsFeatureAction action( tr( "Feature Added" ), f, mLayer, QUuid(), -1, this );
   QgsAttributeTableModel *masterModel = mMainView->masterModel();
 
   if ( action.addFeature() )
@@ -890,6 +931,11 @@ void QgsAttributeTableDialog::mActionAddAttribute_triggered()
     if ( mLayer->addAttribute( dialog.field() ) )
     {
       mLayer->endEditCommand();
+
+      if ( mLayer->displayExpression().isEmpty() )
+      {
+        mLayer->setDisplayExpression( dialog.field().name() );
+      }
     }
     else
     {
@@ -897,7 +943,6 @@ void QgsAttributeTableDialog::mActionAddAttribute_triggered()
       QMessageBox::critical( this, tr( "Add Field" ), tr( "Failed to add field '%1' of type '%2'. Is the field name unique?" ).arg( dialog.field().name(), dialog.field().typeName() ) );
     }
 
-
     // update model - a field has been added or updated
     masterModel->reload( masterModel->index( 0, 0 ), masterModel->index( masterModel->rowCount() - 1, masterModel->columnCount() - 1 ) );
   }
@@ -919,12 +964,17 @@ void QgsAttributeTableDialog::mActionRemoveAttribute_triggered()
       return;
     }
 
+    // check whether display expression is a single field
+    int fieldIdx = QgsExpression::expressionToLayerFieldIndex( mLayer->displayExpression(), mLayer );
     QgsAttributeTableModel *masterModel = mMainView->masterModel();
 
     mLayer->beginEditCommand( tr( "Deleted attribute" ) );
     if ( mLayer->deleteAttributes( attributes ) )
     {
       mLayer->endEditCommand();
+
+      if ( fieldIdx != -1 && attributes.contains( fieldIdx ) )
+        mLayer->setDisplayExpression( mLayer->fields().count() > 0 ? mLayer->fields().at( 0 ).name() : QString() );
     }
     else
     {
@@ -962,6 +1012,11 @@ void QgsAttributeTableDialog::setFilterExpression( const QString &filterString,
   mFeatureFilterWidget->setFilterExpression( filterString, type, alwaysShowFilter );
 }
 
+void QgsAttributeTableDialog::setView( QgsDualView::ViewMode mode )
+{
+  mMainView->setView( mode );
+}
+
 void QgsAttributeTableDialog::deleteFeature( const QgsFeatureId fid )
 {
   QgsDebugMsg( QStringLiteral( "Delete %1" ).arg( fid ) );
@@ -975,7 +1030,7 @@ void QgsAttributeTableDialog::deleteFeature( const QgsFeatureId fid )
     for ( QgsVectorLayer *chl : infoContextLayers )
     {
       childrenCount += infoContext.duplicatedFeatures( chl ).size();
-      childrenInfo += ( tr( "%1 feature(s) on layer \"%2\", " ).arg( infoContext.duplicatedFeatures( chl ).size() ).arg( chl->name() ) );
+      childrenInfo += ( tr( "%n feature(s) on layer \"%1\", ", nullptr, infoContext.duplicatedFeatures( chl ).size() ).arg( chl->name() ) );
     }
 
     // for extra safety to make sure we know that the delete can have impact on children and joins
@@ -999,7 +1054,7 @@ void QgsAttributeTableDialog::deleteFeature( const QgsFeatureId fid )
       feedbackMessage += tr( "%1 on layer %2. " ).arg( context.handledFeatures( contextLayer ).size() ).arg( contextLayer->name() );
       deletedCount += context.handledFeatures( contextLayer ).size();
     }
-    QgisApp::instance()->messageBar()->pushMessage( tr( "%1 features deleted: %2" ).arg( deletedCount ).arg( feedbackMessage ), Qgis::MessageLevel::Success );
+    QgisApp::instance()->messageBar()->pushMessage( tr( "%n feature(s) deleted: %1", nullptr, deletedCount ).arg( feedbackMessage ), Qgis::MessageLevel::Success );
   }
 }
 
@@ -1012,72 +1067,6 @@ void QgsAttributeTableDialog::showContextMenu( QgsActionMenu *menu, const QgsFea
   }
 }
 
-void QgsAttributeTableDialog::toggleDockMode( bool docked )
-{
-  if ( docked )
-  {
-    // going from window -> dock, so save current window geometry
-    QgsSettings().setValue( QStringLiteral( "Windows/BetterAttributeTable/geometry" ), mDialog ? mDialog->saveGeometry() : saveGeometry() );
-    if ( mDialog )
-    {
-      mDialog->removeEventFilter( this );
-      mDialog->setLayout( nullptr );
-      mDialog->deleteLater();
-      mDialog = nullptr;
-    }
-
-    mDock = new QgsAttributeTableDock( QString(), QgisApp::instance() );
-    mDock->setWidget( this );
-    connect( this, &QObject::destroyed, mDock, &QWidget::close );
-    QgisApp::instance()->addTabifiedDockWidget( Qt::BottomDockWidgetArea, mDock, QStringList(), true );
-    updateTitle();
-
-    // To prevent "QAction::event: Ambiguous shortcut overload"
-    QgsDebugMsgLevel( QStringLiteral( "Remove shortcuts from attribute table already defined in main window" ), 2 );
-    mActionZoomMapToSelectedRows->setShortcut( QKeySequence() );
-    mActionRemoveSelection->setShortcut( QKeySequence() );
-    // duplicated on Main Window, with different semantics
-    mActionPanMapToSelectedRows->setShortcut( QKeySequence() );
-    mActionSearchForm->setShortcut( QKeySequence() );
-  }
-  else
-  {
-    // going from dock -> window
-
-    mDialog = new QDialog( QgisApp::instance(), Qt::Window );
-    mDialog->setAttribute( Qt::WA_DeleteOnClose );
-
-    QVBoxLayout *vl = new QVBoxLayout();
-    vl->setContentsMargins( 0, 0, 0, 0 );
-    vl->addWidget( this );
-    mDialog->setLayout( vl );
-
-    if ( mDock )
-    {
-      mDock->setWidget( nullptr );
-      disconnect( this, &QObject::destroyed, mDock, &QWidget::close );
-      mDock->deleteLater();
-      mDock = nullptr;
-    }
-
-    // subscribe to close events, so that we can save window geometry
-    mDialog->installEventFilter( this );
-
-    updateTitle();
-    mDialog->restoreGeometry( QgsSettings().value( QStringLiteral( "Windows/BetterAttributeTable/geometry" ) ).toByteArray() );
-    mDialog->show();
-
-    // restore attribute table shortcuts in window mode
-    QgsDebugMsgLevel( QStringLiteral( "Restore attribute table dialog shortcuts in window mode" ), 2 );
-    // duplicated on Main Window
-    mActionZoomMapToSelectedRows->setShortcut( QStringLiteral( "Ctrl+J" ) );
-    mActionRemoveSelection->setShortcut( QStringLiteral( "Ctrl+Shift+A" ) );
-    // duplicated on Main Window, with different semantics
-    mActionPanMapToSelectedRows->setShortcut( QStringLiteral( "Ctrl+P" ) );
-    mActionSearchForm->setShortcut( QStringLiteral( "Ctrl+F" ) );
-  }
-}
-
 void QgsAttributeTableDialog::updateLayerModifiedActions()
 {
   bool saveEnabled { mActionToggleEditing->isEnabled() &&mLayer->isEditable() &&mLayer->isModified() };
@@ -1094,19 +1083,3 @@ void QgsAttributeTableDialog::updateLayerModifiedActions()
   }
   mActionSaveEdits->setEnabled( saveEnabled );
 }
-
-//
-// QgsAttributeTableDock
-//
-
-QgsAttributeTableDock::QgsAttributeTableDock( const QString &title, QWidget *parent, Qt::WindowFlags flags )
-  : QgsDockWidget( title, parent, flags )
-{
-  setObjectName( QStringLiteral( "AttributeTable" ) ); // set object name so the position can be saved
-}
-
-void QgsAttributeTableDock::closeEvent( QCloseEvent *ev )
-{
-  Q_UNUSED( ev )
-  deleteLater();
-}
