@@ -23,7 +23,6 @@
 #include <QPixmap>
 #include <QMenu>
 #include <QClipboard>
-#include <QDesktopWidget>
 #include <QMenuBar>
 #include <QPushButton>
 #include <QPrinter>
@@ -39,6 +38,7 @@
 #include <QRegExp>
 #include <QScreen>
 #include <QFont>
+#include <QActionGroup>
 
 //graph
 #include <qwt_plot.h>
@@ -50,16 +50,15 @@
 #include "qgisapp.h"
 #include "qgsapplication.h"
 #include "qgsactionmanager.h"
-#include "qgsattributedialog.h"
 #include "qgsdockwidget.h"
 #include "qgseditorwidgetregistry.h"
 #include "qgsfeatureaction.h"
 #include "qgsfeatureiterator.h"
 #include "qgsfeaturestore.h"
+#include "qgsfileutils.h"
 #include "qgsgeometry.h"
 #include "qgsguiutils.h"
 #include "qgshighlight.h"
-#include "qgsmaptoolidentifyaction.h"
 #include "qgsidentifyresultsdialog.h"
 #include "qgslogger.h"
 #include "qgsmapcanvas.h"
@@ -89,6 +88,7 @@
 #include "qgsjsonutils.h"
 #include "qgsjsoneditwidget.h"
 #include "qgspointcloudlayer.h"
+#include "qgscolorrampimpl.h"
 
 #include <nlohmann/json.hpp>
 
@@ -134,7 +134,7 @@ void QgsIdentifyResultsWebView::handleDownload( QUrl url )
     // Try to get some information from the URL
     const QFileInfo info( url.toString() );
     QString savePath = settings.value( DOWNLOADER_LAST_DIR_KEY ).toString();
-    const QString fileName = info.fileName().replace( QRegExp( "[^A-z0-9\\-_\\.]" ), QStringLiteral( "_" ) );
+    const QString fileName = QgsFileUtils::stringToSafeFilename( info.fileName() );
     if ( ! savePath.isEmpty() && ! fileName.isEmpty() )
     {
       savePath = QDir::cleanPath( savePath + QDir::separator() + fileName );
@@ -339,6 +339,7 @@ QgsIdentifyResultsDialog::QgsIdentifyResultsDialog( QgsMapCanvas *canvas, QWidge
   connect( mActionCopy, &QAction::triggered, this, &QgsIdentifyResultsDialog::mActionCopy_triggered );
   connect( mActionAutoFeatureForm, &QAction::toggled, this, &QgsIdentifyResultsDialog::mActionAutoFeatureForm_toggled );
   connect( mActionHideDerivedAttributes, &QAction::toggled, this, &QgsIdentifyResultsDialog::mActionHideDerivedAttributes_toggled );
+  connect( mActionHideNullValues, &QAction::toggled, this, &QgsIdentifyResultsDialog::mActionHideNullValues_toggled );
 
   mOpenFormAction->setDisabled( true );
 
@@ -443,6 +444,8 @@ QgsIdentifyResultsDialog::QgsIdentifyResultsDialog( QgsMapCanvas *canvas, QWidge
   mActionAutoFeatureForm->setChecked( mySettings.value( QStringLiteral( "Map/identifyAutoFeatureForm" ), false ).toBool() );
   settingsMenu->addAction( mActionHideDerivedAttributes );
   mActionHideDerivedAttributes->setChecked( mySettings.value( QStringLiteral( "Map/hideDerivedAttributes" ), false ).toBool() );
+  settingsMenu->addAction( mActionHideNullValues );
+  mActionHideNullValues->setChecked( QgsIdentifyResultsDialog::settingHideNullValues.value() );
 
 }
 
@@ -520,7 +523,7 @@ void QgsIdentifyResultsDialog::addFeature( const QgsMapToolIdentify::IdentifyRes
       break;
     case QgsMapLayerType::PluginLayer:
     case QgsMapLayerType::AnnotationLayer:
-
+    case QgsMapLayerType::GroupLayer:
       break;
   }
 }
@@ -703,11 +706,16 @@ QgsIdentifyResultsFeatureItem *QgsIdentifyResultsDialog::createFeatureItem( QgsV
       continue;
     }
 
+    if ( QgsVariantUtils::isNull( attrs.at( i ) ) && QgsIdentifyResultsDialog::settingHideNullValues.value() )
+    {
+      continue;
+    }
     QString defVal;
     if ( fields.fieldOrigin( i ) == QgsFields::OriginProvider && vlayer->dataProvider() )
       defVal = vlayer->dataProvider()->defaultValueClause( fields.fieldOriginIndex( i ) );
 
     const QString originalValue = defVal == attrs.at( i ) ? defVal : fields.at( i ).displayString( attrs.at( i ) );
+
     QgsTreeWidgetItem *attrItem = new QgsTreeWidgetItem( QStringList() << QString::number( i ) << originalValue );
     featItem->addChild( attrItem );
 
@@ -844,7 +852,12 @@ QgsIdentifyPlotCurve::QgsIdentifyPlotCurve( const QMap<QString, QString> &attrib
   for ( QMap<QString, QString>::const_iterator it = attributes.begin();
         it != attributes.end(); ++it )
   {
-    myData << QPointF( double( i++ ), it.value().toDouble() );
+    bool ok;
+    const double val {it.value().toDouble( &ok )};
+    if ( ok && std::isfinite( val ) )
+    {
+      myData << QPointF( double( i++ ), val );
+    }
   }
   mPlotCurve->setSamples( myData );
 
@@ -986,8 +999,8 @@ void QgsIdentifyResultsDialog::addFeature( QgsRasterLayer *layer,
           {
             // Precision is not set, let's guess it from the
             // standard conversion to string
-            const auto strVal { value.toString() };
-            const int dotPosition { strVal.indexOf( '.' ) };
+            const QString strVal = value.toString();
+            const auto dotPosition = strVal.indexOf( '.' );
             int precision;
             if ( dotPosition < 0 )
             {
@@ -1246,7 +1259,7 @@ void QgsIdentifyResultsDialog::addFeature( QgsVectorTileLayer *layer,
     if ( i >= fields.count() )
       break;
 
-    if ( attrs.at( i ).isNull() || !attrs.at( i ).isValid() )
+    if ( QgsVariantUtils::isNull( attrs.at( i ) ) )
       continue;  // skip attributes that are not present (there can be many of them)
 
     const QString value = fields.at( i ).displayString( attrs.at( i ) );
@@ -1425,8 +1438,7 @@ void QgsIdentifyResultsDialog::show()
   }
   else
   {
-    mDock->show();
-    mDock->raise();
+    mDock->setUserVisible( true );
   }
 }
 
@@ -1440,7 +1452,7 @@ void QgsIdentifyResultsDialog::itemClicked( QTreeWidgetItem *item, int column )
   }
   else if ( item->data( 0, Qt::UserRole ).toString() == QLatin1String( "action" ) )
   {
-    doAction( item, item->data( 0, Qt::UserRole + 1 ).toString() );
+    doAction( item, item->data( 0, Qt::UserRole + 1 ).toUuid() );
   }
   else if ( item->data( 0, Qt::UserRole ).toString() == QLatin1String( "map_layer_action" ) )
   {
@@ -1505,6 +1517,8 @@ void QgsIdentifyResultsDialog::contextMenuEvent( QContextMenuEvent *event )
     mActionPopup->addAction( tr( "Copy Attribute Value" ), this, &QgsIdentifyResultsDialog::copyAttributeValue );
     mActionPopup->addAction( tr( "Copy Feature Attributes" ), this, &QgsIdentifyResultsDialog::copyFeatureAttributes );
 
+    mActionPopup->addAction( tr( "Select Features by Attribute Value" ), this, &QgsIdentifyResultsDialog::selectFeatureByAttribute );
+
     if ( item->parent() == featItem && item->childCount() == 0 )
     {
       idx = item->data( 0, Qt::UserRole + 1 ).toInt();
@@ -1688,7 +1702,7 @@ void QgsIdentifyResultsDialog::deactivate()
   }
 }
 
-void QgsIdentifyResultsDialog::doAction( QTreeWidgetItem *item, const QString &action )
+void QgsIdentifyResultsDialog::doAction( QTreeWidgetItem *item, const QUuid &action )
 {
   QTreeWidgetItem *featItem = featureItem( item );
   if ( !featItem )
@@ -2109,6 +2123,7 @@ void QgsIdentifyResultsDialog::highlightFeature( QTreeWidgetItem *item )
     case QgsMapLayerType::PluginLayer:
     case QgsMapLayerType::MeshLayer:
     case QgsMapLayerType::AnnotationLayer:
+    case QgsMapLayerType::GroupLayer:
       return; // not supported
   }
 
@@ -2183,7 +2198,7 @@ void QgsIdentifyResultsDialog::featureForm()
   if ( !vlayer->getFeatures( QgsFeatureRequest().setFilterFid( fid ) ).nextFeature( f ) )
     return;
 
-  const QString actionId = featItem->data( 0, Qt::UserRole + 1 ).toString();
+  const QUuid actionId = featItem->data( 0, Qt::UserRole + 1 ).toUuid();
 
   QgsFeatureAction action( tr( "Attributes changed" ), f, vlayer, actionId, -1, this );
   if ( vlayer->isEditable() )
@@ -2317,6 +2332,27 @@ void QgsIdentifyResultsDialog::copyFeatureAttributes()
   clipboard->setText( text );
 }
 
+void QgsIdentifyResultsDialog::selectFeatureByAttribute()
+{
+  QTreeWidgetItem *item = lstResults->currentItem();
+  if ( !item ) // should not happen
+  {
+    QgsDebugMsg( QStringLiteral( "Selected item is not feature" ) );
+    return;
+  }
+
+  const QString attribute = item->data( 0, Qt::DisplayRole ).toString();
+  const QVariant value = item->data( 1, Qt::UserRole );
+
+  QgsVectorLayer *vlayer = vectorLayer( item );
+
+  if ( !vlayer )
+    return;
+
+  QString expression = QgsExpression::createFieldEqualityExpression( attribute, value );
+  vlayer->selectByExpression( expression );
+}
+
 void QgsIdentifyResultsDialog::copyGetFeatureInfoUrl()
 {
   QClipboard *clipboard = QApplication::clipboard();
@@ -2372,6 +2408,11 @@ void QgsIdentifyResultsDialog::mActionHideDerivedAttributes_toggled( bool checke
   settings.setValue( QStringLiteral( "Map/hideDerivedAttributes" ), checked );
 }
 
+void QgsIdentifyResultsDialog::mActionHideNullValues_toggled( bool checked )
+{
+  QgsIdentifyResultsDialog::settingHideNullValues.setValue( checked );
+}
+
 void QgsIdentifyResultsDialog::mExpandNewAction_triggered( bool checked )
 {
   QgsSettings settings;
