@@ -233,6 +233,7 @@ void QgsMergeAttributesDialog::createTableWidgetContents()
   {
     int idx = mTableWidget->horizontalHeaderItem( j )->data( FieldIndex ).toInt();
     bool setToManual = false;
+
     if ( !mVectorLayer->dataProvider()->defaultValueClause( idx ).isEmpty() )
     {
       mTableWidget->item( mTableWidget->rowCount() - 1, j )->setData( Qt::DisplayRole, mVectorLayer->dataProvider()->defaultValueClause( idx ) );
@@ -465,6 +466,8 @@ void QgsMergeAttributesDialog::setAllAttributesFromFeature( QgsFeatureId feature
       currentComboBox->setCurrentIndex( currentComboBox->findData( QStringLiteral( "f%1" ).arg( FID_TO_STRING( featureId ) ) ) );
     }
   }
+
+  mTargetFeatureId = featureId;
 }
 
 QVariant QgsMergeAttributesDialog::calcStatistic( int col, QgsStatisticalSummary::Statistic stat )
@@ -666,12 +669,18 @@ void QgsMergeAttributesDialog::mRemoveFeatureFromSelectionButton_clicked()
         f_it != mFeatureList.end();
         ++f_it )
   {
+    if ( f_it->id() == mTargetFeatureId )
+      mTargetFeatureId = FID_NULL;
+
     if ( f_it->id() == featureId )
     {
       mFeatureList.erase( f_it );
       break;
     }
   }
+
+  if ( mTargetFeatureId == FID_NULL && !mFeatureList.isEmpty() )
+    mTargetFeatureId = mFeatureList.first().id();
 }
 
 void QgsMergeAttributesDialog::tableWidgetCellChanged( int row, int column )
@@ -745,6 +754,11 @@ QgsAttributes QgsMergeAttributesDialog::mergedAttributes() const
   return results;
 }
 
+QgsFeatureId QgsMergeAttributesDialog::targetFeatureId() const
+{
+  return mTargetFeatureId;
+}
+
 QSet<int> QgsMergeAttributesDialog::skippedAttributeIndexes() const
 {
   QSet<int> skipped;
