@@ -79,6 +79,9 @@ QgsStatisticalSummaryDockWidget::QgsStatisticalSummaryDockWidget( QWidget *paren
 
   mStatisticsMenu = new QMenu( mOptionsToolButton );
   mOptionsToolButton->setMenu( mStatisticsMenu );
+  mSyncAction = new QAction( tr( "Follow Selected Layer" ), this );
+  mSyncAction->setCheckable( true );
+  connect( mSyncAction, &QAction::toggled, this, &QgsStatisticalSummaryDockWidget::manageSyncLayer );
 
   mFieldType = DataType::Numeric;
   mPreviousFieldType = DataType::Numeric;
@@ -104,6 +107,7 @@ void QgsStatisticalSummaryDockWidget::fieldChanged()
   if ( mFieldExpressionWidget->expression() != mExpression )
   {
     mExpression = mFieldExpressionWidget->expression();
+    mLastExpression.insert( mLayerComboBox->currentLayer()->id(), mFieldExpressionWidget->currentText() );
     refreshStatistics();
   }
 }
@@ -136,6 +140,20 @@ void QgsStatisticalSummaryDockWidget::copyStatistics()
   }
 }
 
+void QgsStatisticalSummaryDockWidget::manageSyncLayer( bool checked )
+{
+  mLayerComboBox->setEnabled( !checked );
+  if ( checked )
+  {
+    connect( QgisApp::instance(), &QgisApp::activeLayerChanged, mLayerComboBox, &QgsMapLayerComboBox::setLayer );
+    mLayerComboBox->setLayer( QgisApp::instance()->activeLayer() );
+  }
+  else
+  {
+    disconnect( QgisApp::instance(), &QgisApp::activeLayerChanged, mLayerComboBox, &QgsMapLayerComboBox::setLayer );
+  }
+}
+
 void QgsStatisticalSummaryDockWidget::refreshStatistics()
 {
   if ( !mLayer || mFieldExpressionWidget->currentField().isEmpty() || ( mFieldExpressionWidget->isExpression() && !mFieldExpressionWidget->isValidExpression() ) )
@@ -269,7 +287,7 @@ void QgsStatisticalSummaryDockWidget::updateNumericStatistics()
     const double val = value.toDouble( &convertOk );
     if ( convertOk )
       values << val;
-    else if ( value.isNull() )
+    else if ( QgsVariantUtils::isNull( value ) )
     {
       missingValues += 1;
     }
@@ -376,12 +394,12 @@ void QgsStatisticalSummaryDockWidget::layerChanged( QgsMapLayer *layer )
 
   mLayer = newLayer;
 
-  // clear expression, so that we don't force an unwanted recalculation
-  mFieldExpressionWidget->setExpression( QString() );
   mFieldExpressionWidget->setLayer( mLayer );
 
   if ( mLayer )
   {
+    // Get last expression
+    mFieldExpressionWidget->setExpression( mLastExpression.value( mLayer->id(), QString() ) );
     connect( mLayer, &QgsVectorLayer::selectionChanged, this, &QgsStatisticalSummaryDockWidget::layerSelectionChanged );
   }
 
@@ -439,6 +457,10 @@ void QgsStatisticalSummaryDockWidget::layersRemoved( const QStringList &layers )
     disconnect( mLayer, &QgsVectorLayer::selectionChanged, this, &QgsStatisticalSummaryDockWidget::layerSelectionChanged );
     mLayer = nullptr;
   }
+  for ( QString layerId : layers )
+  {
+    mLastExpression.remove( layerId );
+  }
 }
 
 void QgsStatisticalSummaryDockWidget::layerSelectionChanged()
@@ -482,7 +504,7 @@ void QgsStatisticalSummaryDockWidget::updateDateTimeStatistics()
   for ( const QgsDateTimeStatisticalSummary::Statistic stat : constStatsToDisplay )
   {
     const QString value = ( stat == QgsDateTimeStatisticalSummary::Range
-                            ? tr( "%1 seconds" ).arg( stats.range().seconds() )
+                            ? tr( "%n second(s)", nullptr, stats.range().seconds() )
                             : stats.statistic( stat ).toString() );
 
     addRow( row, QgsDateTimeStatisticalSummary::displayName( stat ),
@@ -582,6 +604,9 @@ void QgsStatisticalSummaryDockWidget::refreshStatisticsMenu()
       break;
     }
   }
+
+  mStatisticsMenu->addSeparator();
+  mStatisticsMenu->addAction( mSyncAction );
 }
 
 QgsStatisticalSummaryDockWidget::DataType QgsStatisticalSummaryDockWidget::fieldType( const QString &fieldName )
