@@ -18,10 +18,12 @@
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QLineEdit>
-#include <QRegExpValidator>
 #include <QSpacerItem>
 #include <QTimer>
 #include <QToolButton>
+#include <QRegularExpression>
+#include <QRegularExpressionValidator>
+#include <QRandomGenerator>
 
 #include "qgsstatusbarcoordinateswidget.h"
 #include "qgsapplication.h"
@@ -30,7 +32,7 @@
 #include "qgscoordinateutils.h"
 #include "qgsvectorlayer.h"
 #include "qgsvectorlayerjoininfo.h"
-
+#include "qgscoordinatereferencesystemutils.h"
 
 QgsStatusBarCoordinatesWidget::QgsStatusBarCoordinatesWidget( QWidget *parent )
   : QWidget( parent )
@@ -58,8 +60,8 @@ QgsStatusBarCoordinatesWidget::QgsStatusBarCoordinatesWidget( QWidget *parent )
   mLineEdit->setAlignment( Qt::AlignCenter );
   connect( mLineEdit, &QLineEdit::returnPressed, this, &QgsStatusBarCoordinatesWidget::validateCoordinates );
 
-  const QRegExp coordValidator( "[+-]?\\d+\\.?\\d*\\s*,\\s*[+-]?\\d+\\.?\\d*" );
-  mCoordsEditValidator = new QRegExpValidator( coordValidator, this );
+  const QRegularExpression coordValidator( "[+-]?\\d+\\.?\\d*\\s*,\\s*[+-]?\\d+\\.?\\d*" );
+  mCoordsEditValidator = new QRegularExpressionValidator( coordValidator, this );
   mLineEdit->setToolTip( tr( "Current map coordinate (longitude,latitude or east,north)" ) );
 
   //toggle to switch between mouse pos and extents display in status bar widget
@@ -83,6 +85,12 @@ QgsStatusBarCoordinatesWidget::QgsStatusBarCoordinatesWidget( QWidget *parent )
   // When you feel dizzy
   mDizzyTimer = new QTimer( this );
   connect( mDizzyTimer, &QTimer::timeout, this, &QgsStatusBarCoordinatesWidget::dizzy );
+
+  connect( QgsProject::instance()->displaySettings(), &QgsProjectDisplaySettings::coordinateCrsChanged, this, &QgsStatusBarCoordinatesWidget::coordinateDisplaySettingsChanged );
+  connect( QgsProject::instance()->displaySettings(), &QgsProjectDisplaySettings::geographicCoordinateFormatChanged, this, &QgsStatusBarCoordinatesWidget::coordinateDisplaySettingsChanged );
+  connect( QgsProject::instance()->displaySettings(), &QgsProjectDisplaySettings::coordinateTypeChanged, this, &QgsStatusBarCoordinatesWidget::coordinateDisplaySettingsChanged );
+
+  coordinateDisplaySettingsChanged();
 }
 
 void QgsStatusBarCoordinatesWidget::setMapCanvas( QgsMapCanvas *mapCanvas )
@@ -109,7 +117,6 @@ void QgsStatusBarCoordinatesWidget::setMouseCoordinatesPrecision( unsigned int p
   mMousePrecisionDecimalPlaces = precision;
 }
 
-
 void QgsStatusBarCoordinatesWidget::validateCoordinates()
 {
   if ( !mMapCanvas )
@@ -161,15 +168,17 @@ void QgsStatusBarCoordinatesWidget::validateCoordinates()
 
   bool xOk = false;
   bool  yOk = false;
-  double x = 0., y = 0.;
+  double first = 0;
+  double second = 0;
   QString coordText = mLineEdit->text();
-  coordText.replace( QRegExp( " {2,}" ), QStringLiteral( " " ) );
+  const thread_local QRegularExpression sMultipleWhitespaceRx( QStringLiteral( " {2,}" ) );
+  coordText.replace( sMultipleWhitespaceRx, QStringLiteral( " " ) );
 
   QStringList parts = coordText.split( ',' );
   if ( parts.size() == 2 )
   {
-    x = parts.at( 0 ).toDouble( &xOk );
-    y = parts.at( 1 ).toDouble( &yOk );
+    first = parts.at( 0 ).toDouble( &xOk );
+    second = parts.at( 1 ).toDouble( &yOk );
   }
 
   if ( !xOk || !yOk )
@@ -177,15 +186,29 @@ void QgsStatusBarCoordinatesWidget::validateCoordinates()
     parts = coordText.split( ' ' );
     if ( parts.size() == 2 )
     {
-      x = parts.at( 0 ).toDouble( &xOk );
-      y = parts.at( 1 ).toDouble( &yOk );
+      first = parts.at( 0 ).toDouble( &xOk );
+      second = parts.at( 1 ).toDouble( &yOk );
     }
   }
 
   if ( !xOk || !yOk )
     return;
 
-  mMapCanvas->setCenter( QgsPointXY( x, y ) );
+  const Qgis::CoordinateOrder projectAxisOrder = QgsProject::instance()->displaySettings()->coordinateAxisOrder();
+
+  const Qgis::CoordinateOrder coordinateOrder = projectAxisOrder == Qgis::CoordinateOrder::Default ? QgsCoordinateReferenceSystemUtils::defaultCoordinateOrderForCrs( mMapCanvas->mapSettings().destinationCrs() ) : projectAxisOrder;
+  // we may need to flip coordinates depending on crs axis ordering
+  switch ( coordinateOrder )
+  {
+    case Qgis::CoordinateOrder::Default:
+    case Qgis::CoordinateOrder::XY:
+      mMapCanvas->setCenter( QgsPointXY( first, second ) );
+      break;
+    case Qgis::CoordinateOrder::YX:
+      mMapCanvas->setCenter( QgsPointXY( second, first ) );
+      break;
+  }
+
   mMapCanvas->refresh();
 }
 
@@ -202,10 +225,11 @@ void QgsStatusBarCoordinatesWidget::dizzy()
   QRectF rect = mMapCanvas->sceneRect();
   if ( rect.x() < -d || rect.x() > d || rect.y() < -d || rect.y() > d )
     return; // do not affect panning
-  rect.moveTo( ( qrand() % ( 2 * d ) ) - d, ( qrand() % ( 2 * d ) ) - d );
+
+  rect.moveTo( static_cast< int >( QRandomGenerator::global()->generate() % ( 2 * d ) ) - d, static_cast< int >( QRandomGenerator::global()->generate() % ( 2 * d ) ) - d );
   mMapCanvas->setSceneRect( rect );
   QTransform matrix;
-  matrix.rotate( ( qrand() % ( 2 * r ) ) - r );
+  matrix.rotate( static_cast<int >( QRandomGenerator::global()->generate() % ( 2 * r ) ) - r );
   mMapCanvas->setTransform( matrix );
 }
 
@@ -234,7 +258,8 @@ void QgsStatusBarCoordinatesWidget::world()
   }
   const QString fileName = QgsApplication::pkgDataPath() + QStringLiteral( "/resources/data/world_map.gpkg|layername=countries" );
   const QFileInfo fileInfo = QFileInfo( fileName );
-  const QgsVectorLayer::LayerOptions options { QgsProject::instance()->transformContext() };
+  QgsVectorLayer::LayerOptions options { QgsProject::instance()->transformContext() };
+  options.forceReadOnly = true;
   QgsVectorLayer *layer = new QgsVectorLayer( fileInfo.absoluteFilePath(),
       tr( "World Map" ), QStringLiteral( "ogr" ), options );
   // Register this layer with the layers registry
@@ -326,18 +351,10 @@ void QgsStatusBarCoordinatesWidget::refreshMapCanvas()
 
 void QgsStatusBarCoordinatesWidget::showMouseCoordinates( const QgsPointXY &p )
 {
-  if ( !mMapCanvas || mToggleExtentsViewButton->isChecked() )
-  {
-    return;
-  }
-
-  mLineEdit->setText( QgsCoordinateUtils::formatCoordinateForProject( QgsProject::instance(), p, mMapCanvas->mapSettings().destinationCrs(),
-                      mMousePrecisionDecimalPlaces ) );
-
-  ensureCoordinatesVisible();
+  mLastCoordinate = p;
+  updateCoordinateDisplay();
 }
 
-
 void QgsStatusBarCoordinatesWidget::showExtent()
 {
   if ( !mToggleExtentsViewButton->isChecked() )
@@ -364,3 +381,48 @@ void QgsStatusBarCoordinatesWidget::ensureCoordinatesVisible()
   }
 }
 
+void QgsStatusBarCoordinatesWidget::updateCoordinateDisplay()
+{
+  if ( mToggleExtentsViewButton->isChecked() )
+  {
+    return;
+  }
+
+  if ( mLastCoordinate.isEmpty() )
+    mLineEdit->clear();
+  else
+    mLineEdit->setText( QgsCoordinateUtils::formatCoordinateForProject( QgsProject::instance(), mLastCoordinate, mMapCanvas->mapSettings().destinationCrs(),
+                        static_cast< int >( mMousePrecisionDecimalPlaces ) ) );
+
+  ensureCoordinatesVisible();
+}
+
+void QgsStatusBarCoordinatesWidget::coordinateDisplaySettingsChanged()
+{
+  const QgsCoordinateReferenceSystem coordinateCrs = QgsProject::instance()->displaySettings()->coordinateCrs();
+
+  const Qgis::CoordinateOrder projectOrder = QgsProject::instance()->displaySettings()->coordinateAxisOrder();
+  const Qgis::CoordinateOrder order = projectOrder == Qgis::CoordinateOrder::Default
+                                      ? QgsCoordinateReferenceSystemUtils::defaultCoordinateOrderForCrs( coordinateCrs )
+                                      : projectOrder;
+
+  switch ( order )
+  {
+    case Qgis::CoordinateOrder::XY:
+      if ( coordinateCrs.isGeographic() )
+        mLineEdit->setToolTip( tr( "Current map coordinate (Longitude, Latitude)" ) );
+      else
+        mLineEdit->setToolTip( tr( "Current map coordinate (Easting, Northing)" ) );
+      break;
+    case Qgis::CoordinateOrder::YX:
+      if ( coordinateCrs.isGeographic() )
+        mLineEdit->setToolTip( tr( "Current map coordinate (Latitude, Longitude)" ) );
+      else
+        mLineEdit->setToolTip( tr( "Current map coordinate (Northing, Easting)" ) );
+      break;
+    case Qgis::CoordinateOrder::Default:
+      break;
+  }
+
+  updateCoordinateDisplay();
+}
