@@ -176,5 +176,5 @@ void QgsTextAnnotationDialog::deleteItem()
 
 void QgsTextAnnotationDialog::showHelp()
 {
-  QgsHelp::openHelp( QStringLiteral( "introduction/general_tools.html#annotation-tools" ) );
+  QgsHelp::openHelp( QStringLiteral( "map_views/map_view.html#sec-annotations" ) );
 }
