@@ -27,6 +27,7 @@
 #include "qgsproject.h"
 #include "qgscoordinatetransform.h"
 #include "qgsdoublevalidator.h"
+#include "qgspanelwidgetstack.h"
 
 #include <QClipboard>
 #include <QLabel>
@@ -37,6 +38,9 @@
 #include <QKeyEvent>
 #include <QLineEdit>
 #include <QVector2D>
+#include <QCheckBox>
+#include <QStackedWidget>
+#include <QMenu>
 
 static const int MIN_RADIUS_ROLE = Qt::UserRole + 1;
 
@@ -62,14 +66,11 @@ void QgsVertexEditorModel::setFeature( QgsLockedFeature *lockedFeature )
     mHasZ = QgsWkbTypes::hasZ( layerWKBType );
     mHasM = QgsWkbTypes::hasM( layerWKBType );
 
-    if ( mHasZ )
-      mZCol = 2;
+    mZCol = mHasZ ? 2 : -1;
 
-    if ( mHasM )
-      mMCol = 2 + ( mHasZ ? 1 : 0 );
+    mMCol = mHasM ? ( 2 + ( mHasZ ? 1 : 0 ) ) : -1;
 
-    if ( mHasR )
-      mRCol = 2 + ( mHasZ ? 1 : 0 ) + ( mHasM ? 1 : 0 );
+    mRCol = mHasR ? ( 2 + ( mHasZ ? 1 : 0 ) + ( mHasM ? 1 : 0 ) ) : -1;
   }
 
   endResetModel();
@@ -314,41 +315,69 @@ bool QgsVertexEditorModel::calcR( int row, double &r, double &minRadius ) const
   return true;
 }
 
+//
+// QgsVertexEditorWidget
+//
 
-QgsVertexEditor::QgsVertexEditor( QgsMapCanvas *canvas )
-  : mCanvas( canvas )
+QgsVertexEditorWidget::QgsVertexEditorWidget( QgsMapCanvas *canvas )
+  : QgsPanelWidget()
+  , mCanvas( canvas )
   , mVertexModel( new QgsVertexEditorModel( mCanvas, this ) )
 {
-  setWindowTitle( tr( "Vertex Editor" ) );
+  setPanelTitle( tr( "Vertex Editor" ) );
   setObjectName( QStringLiteral( "VertexEditor" ) );
 
-  QWidget *content = new QWidget( this );
-  content->setMinimumHeight( 160 );
-  QVBoxLayout *layout = new QVBoxLayout( content );
+  QVBoxLayout *layout = new QVBoxLayout();
   layout->setContentsMargins( 0, 0, 0, 0 );
 
-  mHintLabel = new QLabel( this );
+  mStackedWidget = new QStackedWidget();
+  mPageHint = new QWidget();
+  mStackedWidget->addWidget( mPageHint );
+
+  QVBoxLayout *pageHintLayout = new QVBoxLayout();
+  mHintLabel = new QLabel();
   mHintLabel->setText( QStringLiteral( "%1\n\n%2" ).arg( tr( "Right click on an editable feature to show its table of vertices." ),
                        tr( "When a feature is bound to this panel, dragging a rectangle to select vertices on the canvas will only select those of the bound feature." ) ) );
   mHintLabel->setWordWrap( true );
-  mHintLabel->setAlignment( Qt::AlignHCenter | Qt::AlignVCenter );
 
-  mTableView = new QTableView( this );
+  pageHintLayout->addStretch();
+  pageHintLayout->addWidget( mHintLabel );
+  pageHintLayout->addStretch();
+  mPageHint->setLayout( pageHintLayout );
+
+  mPageTable = new QWidget();
+  mStackedWidget->addWidget( mPageTable );
+
+  QVBoxLayout *pageTableLayout = new QVBoxLayout();
+  pageTableLayout->setContentsMargins( 0, 0, 0, 0 );
+
+  mTableView = new QTableView();
   mTableView->setSelectionMode( QTableWidget::ExtendedSelection );
   mTableView->setSelectionBehavior( QTableWidget::SelectRows );
-  mTableView->setVisible( false );
   mTableView->setModel( mVertexModel );
-  connect( mTableView->selectionModel(), &QItemSelectionModel::selectionChanged, this, &QgsVertexEditor::updateVertexSelection );
+  connect( mTableView->selectionModel(), &QItemSelectionModel::selectionChanged, this, &QgsVertexEditorWidget::updateVertexSelection );
 
-  layout->addWidget( mTableView );
-  layout->addWidget( mHintLabel );
+  pageTableLayout->addWidget( mTableView );
+  mPageTable->setLayout( pageTableLayout );
 
-  setWidget( content );
+  mStackedWidget->setCurrentWidget( mPageHint );
+  layout->addWidget( mStackedWidget );
+
+  setLayout( layout );
+
+  mWidgetMenu = new QMenu( this );
+  QAction *autoPopupAction = new QAction( tr( "Auto-open Table" ), this );
+  autoPopupAction->setCheckable( true );
+  autoPopupAction->setChecked( QgsVertexEditor::settingAutoPopupVertexEditorDock.value() );
+  connect( autoPopupAction, &QAction::toggled, this, [ = ]( bool checked )
+  {
+    QgsVertexEditor::settingAutoPopupVertexEditorDock.setValue( checked );
+  } );
+  mWidgetMenu->addAction( autoPopupAction );
 }
 
-void QgsVertexEditor::updateEditor( QgsLockedFeature *lockedFeature )
+void QgsVertexEditorWidget::updateEditor( QgsLockedFeature *lockedFeature )
 {
-
   mLockedFeature = lockedFeature;
 
   mVertexModel->setFeature( mLockedFeature );
@@ -357,10 +386,9 @@ void QgsVertexEditor::updateEditor( QgsLockedFeature *lockedFeature )
 
   if ( mLockedFeature )
   {
-    mHintLabel->setVisible( false );
-    mTableView->setVisible( true );
+    mStackedWidget->setCurrentWidget( mPageTable );
 
-    connect( mLockedFeature, &QgsLockedFeature::selectionChanged, this, &QgsVertexEditor::updateTableSelection );
+    connect( mLockedFeature, &QgsLockedFeature::selectionChanged, this, &QgsVertexEditorWidget::updateTableSelection );
 
     if ( mLockedFeature->layer() )
     {
@@ -374,12 +402,21 @@ void QgsVertexEditor::updateEditor( QgsLockedFeature *lockedFeature )
   }
   else
   {
-    mHintLabel->setVisible( true );
-    mTableView->setVisible( false );
+    mStackedWidget->setCurrentWidget( mPageHint );
   }
 }
 
-void QgsVertexEditor::updateTableSelection()
+QMenu *QgsVertexEditorWidget::menuButtonMenu()
+{
+  return mWidgetMenu;
+}
+
+QString QgsVertexEditorWidget::menuButtonTooltip() const
+{
+  return tr( "Options" );
+}
+
+void QgsVertexEditorWidget::updateTableSelection()
 {
   if ( !mLockedFeature || mUpdatingVertexSelection || mUpdatingTableSelection )
     return;
@@ -405,7 +442,7 @@ void QgsVertexEditor::updateTableSelection()
   mUpdatingTableSelection = false;
 }
 
-void QgsVertexEditor::updateVertexSelection( const QItemSelection &, const QItemSelection & )
+void QgsVertexEditorWidget::updateVertexSelection( const QItemSelection &, const QItemSelection & )
 {
   if ( !mLockedFeature || mUpdatingVertexSelection || mUpdatingTableSelection )
     return;
@@ -450,7 +487,7 @@ void QgsVertexEditor::updateVertexSelection( const QItemSelection &, const QItem
   mUpdatingVertexSelection = false;
 }
 
-void QgsVertexEditor::keyPressEvent( QKeyEvent *e )
+void QgsVertexEditorWidget::keyPressEvent( QKeyEvent *e )
 {
   if ( e->key() == Qt::Key_Backspace || e->key() == Qt::Key_Delete )
   {
@@ -477,6 +514,30 @@ void QgsVertexEditor::keyPressEvent( QKeyEvent *e )
   }
 }
 
+
+//
+// QgsVertexEditor
+//
+
+QgsVertexEditor::QgsVertexEditor( QgsMapCanvas *canvas )
+{
+  setWindowTitle( tr( "Vertex Editor" ) );
+  setObjectName( QStringLiteral( "VertexEditor" ) );
+
+  QgsPanelWidgetStack *stack = new QgsPanelWidgetStack();
+  setWidget( stack );
+
+  mWidget = new QgsVertexEditorWidget( canvas );
+  stack->setMainPanel( mWidget );
+
+  connect( mWidget, &QgsVertexEditorWidget::deleteSelectedRequested, this, &QgsVertexEditor::deleteSelectedRequested );
+}
+
+void QgsVertexEditor::updateEditor( QgsLockedFeature *lockedFeature )
+{
+  mWidget->updateEditor( lockedFeature );
+}
+
 void QgsVertexEditor::closeEvent( QCloseEvent *event )
 {
   QgsDockWidget::closeEvent( event );
@@ -494,16 +555,16 @@ CoordinateItemDelegate::CoordinateItemDelegate( const QgsCoordinateReferenceSyst
 
 }
 
-QString CoordinateItemDelegate::displayText( const QVariant &value, const QLocale &locale ) const
+QString CoordinateItemDelegate::displayText( const QVariant &value, const QLocale & ) const
 {
-  return locale.toString( value.toDouble(), 'f', displayDecimalPlaces() );
+  return QLocale().toString( value.toDouble(), 'f', displayDecimalPlaces() );
 }
 
 QWidget *CoordinateItemDelegate::createEditor( QWidget *parent, const QStyleOptionViewItem &, const QModelIndex &index ) const
 {
   QLineEdit *lineEdit = new QLineEdit( parent );
   QgsDoubleValidator *validator = new QgsDoubleValidator( lineEdit );
-  if ( !index.data( MIN_RADIUS_ROLE ).isNull() )
+  if ( index.data( MIN_RADIUS_ROLE ).isValid() )
     validator->setBottom( index.data( MIN_RADIUS_ROLE ).toDouble() );
   lineEdit->setValidator( validator );
   return lineEdit;
@@ -523,7 +584,7 @@ void CoordinateItemDelegate::setEditorData( QWidget *editor, const QModelIndex &
   QLineEdit *lineEdit = qobject_cast<QLineEdit *>( editor );
   if ( lineEdit && index.isValid() )
   {
-    lineEdit->setText( displayText( index.data( ).toDouble( ), QLocale() ) );
+    lineEdit->setText( displayText( index.data( ).toDouble( ), QLocale() ).replace( QLocale().groupSeparator(), QString( ) ) );
   }
 }
 
