@@ -166,7 +166,7 @@ QgsEditFormConfig &QgsEditFormConfig::operator=( const QgsEditFormConfig &o )  /
   return *this;
 }
 
-bool QgsEditFormConfig::operator==( const QgsEditFormConfig &o )
+bool QgsEditFormConfig::operator==( const QgsEditFormConfig &o ) const
 {
   return d == o.d;
 }
@@ -217,7 +217,7 @@ void QgsEditFormConfig::setUiForm( const QString &ui )
   if ( !ui.isEmpty() && !QUrl::fromUserInput( ui ).isLocalFile() )
   {
     // any existing download will not be restarted!
-    QgsApplication::instance()->networkContentFetcherRegistry()->fetch( ui, Qgis::ActionStart::Immediate );
+    QgsApplication::networkContentFetcherRegistry()->fetch( ui, Qgis::ActionStart::Immediate );
   }
 
   if ( ui.isEmpty() )
@@ -323,7 +323,7 @@ void QgsEditFormConfig::setInitFilePath( const QString &filePath )
   if ( !filePath.isEmpty() && !QUrl::fromUserInput( filePath ).isLocalFile() )
   {
     // any existing download will not be restarted!
-    QgsApplication::instance()->networkContentFetcherRegistry()->fetch( filePath, Qgis::ActionStart::Immediate );
+    QgsApplication::networkContentFetcherRegistry()->fetch( filePath, Qgis::ActionStart::Immediate );
   }
 }
 
