@@ -273,6 +273,69 @@ class CORE_EXPORT QgsExpressionNodeIndexOperator : public QgsExpressionNode
 
 };
 
+/**
+ * \brief SQL-like BETWEEN and NOT BETWEEN predicates.
+ * \ingroup core
+ * \since QGIS 3.26
+ */
+class CORE_EXPORT QgsExpressionNodeBetweenOperator: public QgsExpressionNode
+{
+  public:
+
+    /**
+     * This node tests if the result of \a node is between the result of \a nodeLowerBound and \a nodeHigherBound nodes. Optionally it can be inverted with \a negate which by default is FALSE.
+     */
+    QgsExpressionNodeBetweenOperator( QgsExpressionNode *node SIP_TRANSFER, QgsExpressionNode *nodeLowerBound SIP_TRANSFER, QgsExpressionNode *nodeHigherBound SIP_TRANSFER, bool negate = false )
+      : mNode( node )
+      , mLowerBound( nodeLowerBound )
+      , mHigherBound( nodeHigherBound )
+      , mNegate( negate )
+    {}
+
+    ~QgsExpressionNodeBetweenOperator() override;
+
+    /**
+     * Returns the expression node.
+     */
+    QgsExpressionNode *node() const { return mNode; }
+
+
+    QgsExpressionNode::NodeType nodeType() const override;
+    bool prepareNode( QgsExpression *parent, const QgsExpressionContext *context ) override;
+    QVariant evalNode( QgsExpression *parent, const QgsExpressionContext *context ) override;
+    QString dump() const override;
+
+    QSet<QString> referencedColumns() const override;
+    QSet<QString> referencedVariables() const override;
+    QSet<QString> referencedFunctions() const override;
+    QList<const QgsExpressionNode *> nodes() const override; SIP_SKIP
+    bool needsGeometry() const override;
+    QgsExpressionNode *clone() const override SIP_FACTORY;
+    bool isStatic( QgsExpression *parent, const QgsExpressionContext *context ) const override;
+
+    /**
+     * Returns the lower bound expression node of the range.
+     */
+    QgsExpressionNode *lowerBound() const;
+
+    /**
+     * Returns the higher bound expression node of the range.
+     */
+    QgsExpressionNode *higherBound() const;
+
+    /**
+     * Returns TRUE if the predicate is an exclusion test (NOT BETWEEN).
+     */
+    bool negate() const;
+
+  private:
+    QgsExpressionNode *mNode = nullptr;
+    QgsExpressionNode *mLowerBound = nullptr;
+    QgsExpressionNode *mHigherBound = nullptr;
+    bool mNegate = false;
+
+};
+
 /**
  * \brief An expression node for value IN or NOT IN clauses.
  * \ingroup core
