@@ -156,6 +156,11 @@ bool QgsGeometryUtils::verticesAtDistance( const QgsAbstractGeometry &geometry,
   previousVertex = QgsVertexId();
   nextVertex = QgsVertexId();
 
+  if ( geometry.isEmpty() )
+  {
+    return false;
+  }
+
   QgsPoint point;
   QgsPoint previousPoint;
 
@@ -239,6 +244,22 @@ double QgsGeometryUtils::sqrDistToLine( double ptX, double ptY, double x1, doubl
   return dist;
 }
 
+double QgsGeometryUtils::distToInfiniteLine( const QgsPoint &point, const QgsPoint &linePoint1, const QgsPoint &linePoint2, double epsilon )
+{
+  const double area = std::abs(
+                        ( linePoint1.x() - linePoint2.x() ) * ( point.y() - linePoint2.y() ) -
+                        ( linePoint1.y() - linePoint2.y() ) * ( point.x() - linePoint2.x() )
+                      );
+
+  const double length = std::sqrt(
+                          std::pow( linePoint1.x() - linePoint2.x(), 2 ) +
+                          std::pow( linePoint1.y() - linePoint2.y(), 2 )
+                        );
+
+  const double distance = area / length;
+  return qgsDoubleNear( distance, 0.0, epsilon ) ? 0.0 : distance;
+}
+
 bool QgsGeometryUtils::lineIntersection( const QgsPoint &p1, QgsVector v1, const QgsPoint &p2, QgsVector v2, QgsPoint &intersection )
 {
   const double d = v1.y() * v2.x() - v1.x() * v2.y();
@@ -1200,7 +1221,7 @@ QgsPointSequence QgsGeometryUtils::pointsFromWKT( const QString &wktCoordinateLi
   return points;
 }
 
-void QgsGeometryUtils::pointsToWKB( QgsWkbPtr &wkb, const QgsPointSequence &points, bool is3D, bool isMeasure )
+void QgsGeometryUtils::pointsToWKB( QgsWkbPtr &wkb, const QgsPointSequence &points, bool is3D, bool isMeasure, QgsAbstractGeometry::WkbFlags flags )
 {
   wkb << static_cast<quint32>( points.size() );
   for ( const QgsPoint &point : points )
@@ -1208,11 +1229,21 @@ void QgsGeometryUtils::pointsToWKB( QgsWkbPtr &wkb, const QgsPointSequence &poin
     wkb << point.x() << point.y();
     if ( is3D )
     {
-      wkb << point.z();
+      double z = point.z();
+      if ( flags & QgsAbstractGeometry::FlagExportNanAsDoubleMin
+           && std::isnan( z ) )
+        z = -std::numeric_limits<double>::max();
+
+      wkb << z;
     }
     if ( isMeasure )
     {
-      wkb << point.m();
+      double m = point.m();
+      if ( flags & QgsAbstractGeometry::FlagExportNanAsDoubleMin
+           && std::isnan( m ) )
+        m = -std::numeric_limits<double>::max();
+
+      wkb << m;
     }
   }
 }
@@ -1605,6 +1636,23 @@ QgsLineString QgsGeometryUtils::perpendicularSegment( const QgsPoint &p, const Q
   return line;
 }
 
+void QgsGeometryUtils::perpendicularCenterSegment( double pointx, double pointy, double segmentPoint1x, double segmentPoint1y, double segmentPoint2x, double segmentPoint2y, double &perpendicularSegmentPoint1x, double &perpendicularSegmentPoint1y, double &perpendicularSegmentPoint2x, double &perpendicularSegmentPoint2y, double desiredSegmentLength )
+{
+  QgsVector segmentVector =  QgsVector( segmentPoint2x - segmentPoint1x, segmentPoint2y - segmentPoint1y );
+  QgsVector perpendicularVector = segmentVector.perpVector();
+  if ( desiredSegmentLength )
+  {
+    if ( desiredSegmentLength != 0 )
+    {
+      perpendicularVector = perpendicularVector.normalized() * ( desiredSegmentLength ) / 2;
+    }
+  }
+  perpendicularSegmentPoint1x = pointx - perpendicularVector.x();
+  perpendicularSegmentPoint1y = pointy - perpendicularVector.y();
+  perpendicularSegmentPoint2x = pointx + perpendicularVector.x();
+  perpendicularSegmentPoint2y = pointy + perpendicularVector.y();
+}
+
 double QgsGeometryUtils::lineAngle( double x1, double y1, double x2, double y2 )
 {
   const double at = std::atan2( y2 - y1, x2 - x1 );
