@@ -16,6 +16,7 @@
  ***************************************************************************/
 
 #include "qgslocator.h"
+#include "qgsmessagelog.h"
 #include "qgssettings.h"
 #include <QtConcurrent>
 #include <functional>
@@ -29,7 +30,9 @@ const QList<QString> QgsLocator::CORE_FILTERS = QList<QString>() << QStringLiter
     <<  QStringLiteral( "calculator" )
     <<  QStringLiteral( "bookmarks" )
     <<  QStringLiteral( "optionpages" )
-    <<  QStringLiteral( "edit_features" );
+    <<  QStringLiteral( "edit_features" )
+    <<  QStringLiteral( "goto" )
+    <<  QStringLiteral( "nominatimgeocoder" ) ;
 
 QgsLocator::QgsLocator( QObject *parent )
   : QObject( parent )
@@ -57,7 +60,7 @@ QList<QgsLocatorFilter *> QgsLocator::filters( const QString &prefix )
     QList<QgsLocatorFilter *> filters =  QList<QgsLocatorFilter *>();
     for ( QgsLocatorFilter *filter : mFilters )
     {
-      if ( !filter->activePrefix().isEmpty() && filter->activePrefix() == prefix )
+      if ( !filter->activePrefix().isEmpty() && filter->activePrefix().compare( prefix, Qt::CaseInsensitive ) == 0 )
       {
         filters << filter;
       }
@@ -77,7 +80,11 @@ QMap<QString, QgsLocatorFilter *> QgsLocator::prefixedFilters() const
   {
     if ( !filter->activePrefix().isEmpty() && filter->enabled() )
     {
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
       filters.insertMulti( filter->activePrefix(), filter );
+#else
+      filters.insert( filter->activePrefix(), filter );
+#endif
     }
   }
   return filters;
@@ -89,10 +96,9 @@ void QgsLocator::registerFilter( QgsLocatorFilter *filter )
   filter->setParent( this );
 
   // restore settings
-  QgsSettings settings;
-  bool enabled = settings.value( QStringLiteral( "locator_filters/enabled_%1" ).arg( filter->name() ), true, QgsSettings::Section::Gui ).toBool();
-  bool byDefault = settings.value( QStringLiteral( "locator_filters/default_%1" ).arg( filter->name() ), filter->useWithoutPrefix(), QgsSettings::Section::Gui ).toBool();
-  QString prefix = settings.value( QStringLiteral( "locator_filters/prefix_%1" ).arg( filter->name() ), filter->prefix(), QgsSettings::Section::Gui ).toString();
+  bool enabled = QgsLocator::settingsLocatorFilterEnabled.value( filter->name() );
+  bool byDefault = QgsLocator::settingsLocatorFilterDefault.valueWithDefaultOverride( filter->useWithoutPrefix(), filter->name() );
+  QString prefix = QgsLocator::settingsLocatorFilterPrefix.valueWithDefaultOverride( filter->prefix(), filter->name() );
   if ( prefix.isEmpty() )
   {
     prefix = filter->prefix();
@@ -123,6 +129,8 @@ void QgsLocator::registerFilter( QgsLocatorFilter *filter )
 
 void QgsLocator::fetchResults( const QString &string, const QgsLocatorContext &c, QgsFeedback *feedback )
 {
+  mAutocompletionList.clear();
+
   QgsLocatorContext context( c );
   // ideally this should not be required, as well behaved callers
   // will NOT fire up a new fetchResults call while an existing one is
@@ -144,12 +152,12 @@ void QgsLocator::fetchResults( const QString &string, const QgsLocatorContext &c
 
   QList< QgsLocatorFilter * > activeFilters;
   QString searchString = string;
-  QString prefix = searchString.left( std::max( searchString.indexOf( ' ' ), 0 ) );
+  QString prefix = searchString.left( std::max( static_cast<int>( searchString.indexOf( ' ' ) ), 0 ) );
   if ( !prefix.isEmpty() )
   {
-    for ( QgsLocatorFilter *filter : qgis::as_const( mFilters ) )
+    for ( QgsLocatorFilter *filter : std::as_const( mFilters ) )
     {
-      if ( filter->activePrefix() == prefix && filter->enabled() )
+      if ( filter->activePrefix().compare( prefix, Qt::CaseInsensitive ) == 0 && filter->enabled() )
       {
         activeFilters << filter;
       }
@@ -162,7 +170,7 @@ void QgsLocator::fetchResults( const QString &string, const QgsLocatorContext &c
   }
   else
   {
-    for ( QgsLocatorFilter *filter : qgis::as_const( mFilters ) )
+    for ( QgsLocatorFilter *filter : std::as_const( mFilters ) )
     {
       if ( filter->useWithoutPrefix() && filter->enabled() )
       {
@@ -172,16 +180,29 @@ void QgsLocator::fetchResults( const QString &string, const QgsLocatorContext &c
   }
 
   QList< QgsLocatorFilter *> threadedFilters;
-  for ( QgsLocatorFilter *filter : qgis::as_const( activeFilters ) )
+  for ( QgsLocatorFilter *filter : std::as_const( activeFilters ) )
   {
     filter->clearPreviousResults();
     std::unique_ptr< QgsLocatorFilter > clone( filter->clone() );
+    if ( ! clone )
+    {
+      QgsMessageLog::logMessage( tr( "QgsLocatorFilter '%1' could not provide a valid clone" ).arg( filter->name() ), QString(), Qgis::MessageLevel::Critical );
+      continue;
+    }
     connect( clone.get(), &QgsLocatorFilter::resultFetched, clone.get(), [this, filter]( QgsLocatorResult result )
     {
       result.filter = filter;
       filterSentResult( result );
     } );
-    clone->prepare( searchString, context );
+    QStringList autoCompleteList = clone->prepare( searchString, context );
+    if ( context.usingPrefix )
+    {
+      for ( int i = 0; i < autoCompleteList.length(); i++ )
+      {
+        autoCompleteList[i].prepend( QStringLiteral( "%1 " ).arg( prefix ) );
+      }
+    }
+    mAutocompletionList.append( autoCompleteList );
 
     if ( clone->flags() & QgsLocatorFilter::FlagFast )
     {
@@ -190,19 +211,26 @@ void QgsLocator::fetchResults( const QString &string, const QgsLocatorContext &c
     }
     else
     {
-      // run filter in background
       threadedFilters.append( clone.release() );
     }
   }
 
   mActiveThreads.clear();
-  for ( QgsLocatorFilter *filter : qgis::as_const( threadedFilters ) )
+  for ( QgsLocatorFilter *filter : std::as_const( threadedFilters ) )
   {
     QThread *thread = new QThread();
     mActiveThreads.append( thread );
     filter->moveToThread( thread );
     connect( thread, &QThread::started, filter, [filter, searchString, context, feedback]
     {
+      int delay = filter->fetchResultsDelay();
+      while ( delay > 0 )
+      {
+        if ( feedback->isCanceled() )
+          break;
+        QThread::msleep( 50 );
+        delay -= 50;
+      }
       if ( !feedback->isCanceled() )
         filter->fetchResults( searchString, context, feedback );
       filter->emit finished();
@@ -219,6 +247,8 @@ void QgsLocator::fetchResults( const QString &string, const QgsLocatorContext &c
     thread->start();
   }
 
+  emit searchPrepared();
+
   if ( mActiveThreads.empty() )
     emit finished();
 }
@@ -241,7 +271,7 @@ bool QgsLocator::isRunning() const
 
 void QgsLocator::clearPreviousResults()
 {
-  for ( QgsLocatorFilter *filter : qgis::as_const( mFilters ) )
+  for ( QgsLocatorFilter *filter : std::as_const( mFilters ) )
   {
     if ( filter->enabled() )
     {
