@@ -19,11 +19,25 @@
 #include "qgslabelingengine.h"
 #include "qgslogger.h"
 #include "qgsmaplayerrenderer.h"
-#include "qgsmaplayerlistutils.h"
+#include "qgsmaplayerlistutils_p.h"
 #include "qgsvectorlayerlabeling.h"
+#include "qgsrasterlayerrenderer.h"
 
 #include <QtConcurrentRun>
 
+Q_GUI_EXPORT extern int qt_defaultDpiX();
+Q_GUI_EXPORT extern int qt_defaultDpiY();
+
+static void _fixQPictureDPI( QPainter *p )
+{
+  // QPicture makes an assumption that we drawing to it with system DPI.
+  // Then when being drawn, it scales the painter. The following call
+  // negates the effect. There is no way of setting QPicture's DPI.
+  // See QTBUG-20361
+  p->scale( static_cast< double >( qt_defaultDpiX() ) / p->device()->logicalDpiX(),
+            static_cast< double >( qt_defaultDpiY() ) / p->device()->logicalDpiY() );
+}
+
 //
 // QgsMapRendererAbstractCustomPainterJob
 //
@@ -40,9 +54,8 @@ void QgsMapRendererAbstractCustomPainterJob::preparePainter( QPainter *painter,
   painter->fillRect( 0, 0, mSettings.deviceOutputSize().width(), mSettings.deviceOutputSize().height(), backgroundColor );
 
   painter->setRenderHint( QPainter::Antialiasing, mSettings.testFlag( Qgis::MapSettingsFlag::Antialiasing ) );
-#if QT_VERSION >= QT_VERSION_CHECK(5, 13, 0)
+  painter->setRenderHint( QPainter::SmoothPixmapTransform, mSettings.testFlag( Qgis::MapSettingsFlag::HighQualityImageTransforms ) );
   painter->setRenderHint( QPainter::LosslessImageRendering, mSettings.testFlag( Qgis::MapSettingsFlag::LosslessImageRendering ) );
-#endif
 
 #ifndef QT_NO_DEBUG
   QPaintDevice *paintDevice = painter->device();
@@ -294,6 +307,8 @@ void QgsMapRendererCustomPainterJob::doRender()
     if ( job.context()->renderingStopped() )
       break;
 
+    emit layerRenderingStarted( job.layerId );
+
     if ( ! hasSecondPass && job.context()->useAdvancedEffects() )
     {
       // Set the QPainter composition mode so that this layer is rendered using
@@ -314,6 +329,11 @@ void QgsMapRendererCustomPainterJob::doRender()
 
       job.completed = job.renderer->render();
 
+      if ( job.picture )
+      {
+        job.renderer->renderContext()->painter()->end();
+      }
+
       job.renderingTime += layerTime.elapsed();
     }
 
@@ -325,8 +345,10 @@ void QgsMapRendererCustomPainterJob::doRender()
       mPainter->setOpacity( 1.0 );
     }
 
+    emit layerRendered( job.layerId );
   }
 
+  emit renderingLayersFinished();
   QgsDebugMsgLevel( QStringLiteral( "Done rendering map layers" ), 5 );
 
   if ( mSettings.testFlag( Qgis::MapSettingsFlag::DrawLabeling ) && !mLabelJob.context.renderingStopped() )
@@ -345,6 +367,14 @@ void QgsMapRendererCustomPainterJob::doRender()
         drawLabeling( mLabelJob.context, mLabelingEngineV2.get(), &painter );
         painter.end();
       }
+      else if ( mLabelJob.picture )
+      {
+        QPainter painter;
+        painter.begin( mLabelJob.picture.get() );
+        mLabelJob.context.setPainter( &painter );
+        drawLabeling( mLabelJob.context, mLabelingEngineV2.get(), &painter );
+        painter.end();
+      }
       else
       {
         drawLabeling( mLabelJob.context, mLabelingEngineV2.get(), mPainter );
@@ -367,6 +397,8 @@ void QgsMapRendererCustomPainterJob::doRender()
   }
   else
   {
+    initSecondPassJobs( mSecondPassLayerJobs, mLabelJob );
+
     for ( LayerRenderJob &job : mSecondPassLayerJobs )
     {
       if ( job.context()->renderingStopped() )
@@ -385,20 +417,52 @@ void QgsMapRendererCustomPainterJob::doRender()
 
         job.completed = job.renderer->render();
 
+        if ( job.picture )
+        {
+          job.renderer->renderContext()->painter()->end();
+        }
+
         job.renderingTime += layerTime.elapsed();
       }
     }
 
-    composeSecondPass( mSecondPassLayerJobs, mLabelJob );
+    bool forceVector = mSettings.testFlag( Qgis::MapSettingsFlag::ForceVectorOutput ) && !mSettings.testFlag( Qgis::MapSettingsFlag::ForceRasterMasks );
+    composeSecondPass( mSecondPassLayerJobs, mLabelJob, forceVector );
 
-    const QImage finalImage = composeImage( mSettings, mLayerJobs, mLabelJob );
+    if ( !forceVector )
+    {
+      const QImage finalImage = composeImage( mSettings, mLayerJobs, mLabelJob );
 
-    mPainter->setCompositionMode( QPainter::CompositionMode_SourceOver );
-    mPainter->setOpacity( 1.0 );
-    mPainter->drawImage( 0, 0, finalImage );
+      mPainter->setCompositionMode( QPainter::CompositionMode_SourceOver );
+      mPainter->setOpacity( 1.0 );
+      mPainter->drawImage( 0, 0, finalImage );
+    }
+    else
+    {
+      //Vector composition is simply draw the saved picture on the painter
+      for ( LayerRenderJob &job : mLayerJobs )
+      {
+        // if there is vector rendering we use it, else we use the raster rendering
+        if ( job.picture )
+        {
+          mPainter->save();
+          _fixQPictureDPI( mPainter );
+          mPainter->drawPicture( 0, 0, *job.picture );
+          mPainter->restore();
+        }
+        else
+          mPainter->drawImage( 0, 0, *job.img );
+      }
+
+      if ( mLabelJob.picture )
+      {
+        mPainter->save();
+        _fixQPictureDPI( mPainter );
+        mPainter->drawPicture( 0, 0, *mLabelJob.picture );
+        mPainter->restore();
+      }
+    }
   }
 
   QgsDebugMsgLevel( QStringLiteral( "Rendering completed in (seconds): %1" ).arg( renderTime.elapsed() / 1000.0 ), 2 );
 }
-
-
