@@ -24,11 +24,13 @@
 #include "qgis_sip.h"
 #include "qgsvector3d.h"
 #include "qgspointcloudattribute.h"
+#include "qgselevationmap.h"
 
 class QgsPointCloudBlock;
 class QgsLayerTreeLayer;
 class QgsLayerTreeModelLegendNode;
 class QgsPointCloudLayer;
+class QgsElevationMap;
 
 /**
  * \ingroup core
@@ -173,6 +175,24 @@ class CORE_EXPORT QgsPointCloudRenderContext
      */
     QgsFeedback *feedback() const { return mFeedback; }
 
+#ifndef SIP_RUN   // intentionally left out from SIP to avoid API breaks in future when we move elevation post-processing elsewhere
+
+    /**
+     * Sets elevation map that will be used to record elevation of rendered points.
+     * \note Takes ownership of the passed object
+     *
+     * \since QGIS 3.28
+     */
+    void setElevationMap( QgsElevationMap *elevationMap SIP_TRANSFER );
+
+    /**
+     * Returns elevation map. It may be a null pointer if elevation map is not needed in rendering.
+     *
+     * \since QGIS 3.28
+     */
+    QgsElevationMap *elevationMap() { return mElevationMap.get(); }
+#endif
+
 #ifndef SIP_RUN
 
     /**
@@ -180,18 +200,31 @@ class CORE_EXPORT QgsPointCloudRenderContext
      * \a type indicates the original data type for the attribute.
      */
     template <typename T>
-    void getAttribute( const char *data, std::size_t offset, QgsPointCloudAttribute::DataType type, T &value ) const
+    static void getAttribute( const char *data, std::size_t offset, QgsPointCloudAttribute::DataType type, T &value )
     {
       switch ( type )
       {
+        case QgsPointCloudAttribute::UChar:
+          value = *reinterpret_cast< const unsigned char * >( data + offset );
+          return;
         case QgsPointCloudAttribute::Char:
           value = *( data + offset );
           return;
 
+        case QgsPointCloudAttribute::UInt32:
+          value = *reinterpret_cast< const quint32 * >( data + offset );
+          return;
         case QgsPointCloudAttribute::Int32:
           value = *reinterpret_cast< const qint32 * >( data + offset );
           return;
 
+        case QgsPointCloudAttribute::UInt64:
+          value = *reinterpret_cast< const quint64 * >( data + offset );
+          return;
+        case QgsPointCloudAttribute::Int64:
+          value = *reinterpret_cast< const qint64 * >( data + offset );
+          return;
+
         case QgsPointCloudAttribute::Short:
           value = *reinterpret_cast< const short * >( data + offset );
           return;
@@ -227,10 +260,53 @@ class CORE_EXPORT QgsPointCloudRenderContext
     int mZOffset = 0;
     double mZValueScale = 1.0;
     double mZValueFixedOffset = 0;
+    std::unique_ptr<QgsElevationMap> mElevationMap;
 
     QgsFeedback *mFeedback = nullptr;
 };
 
+#ifndef SIP_RUN
+
+/**
+ * \ingroup core
+ * \class QgsPreparedPointCloudRendererData
+ *
+ * \brief Base class for 2d point cloud renderer prepared data containers.
+ * \note Not available in Python bindings
+ *
+ * \since QGIS 3.26
+ */
+class CORE_EXPORT QgsPreparedPointCloudRendererData
+{
+  public:
+
+    virtual ~QgsPreparedPointCloudRendererData();
+
+    /**
+     * Returns the set of attributes used by the prepared point cloud renderer.
+     */
+    virtual QSet< QString > usedAttributes() const = 0;
+
+    /**
+     * Prepares the renderer for using the specified \a block.
+     *
+     * Returns FALSE if preparation failed.
+     */
+    virtual bool prepareBlock( const QgsPointCloudBlock *block ) = 0;
+
+    /**
+     * An optimised method of retrieving the color of a point from a point cloud block.
+     *
+     * Before calling this method prepareBlock() must be called for each incoming point cloud block.
+     *
+     * \since QGIS 3.26
+     */
+    virtual QColor pointColor( const QgsPointCloudBlock *block, int i, double z ) = 0;
+
+};
+
+#endif
+
 
 /**
  * \ingroup core
@@ -263,15 +339,6 @@ class CORE_EXPORT QgsPointCloudRenderer
 
   public:
 
-    /**
-     * Rendering symbols for points.
-     */
-    enum PointSymbol
-    {
-      Square, //!< Renders points as squares
-      Circle, //!< Renders points as circles
-    };
-
     /**
      * Constructor for QgsPointCloudRenderer.
      */
@@ -347,6 +414,14 @@ class CORE_EXPORT QgsPointCloudRenderer
      */
     virtual QSet< QString > usedAttributes( const QgsPointCloudRenderContext &context ) const;
 
+    /**
+     * Returns prepared data container for bulk point color retrieval.
+     *
+     * \note Not available in Python bindings.
+     * \since QGIS 3.26
+     */
+    virtual std::unique_ptr< QgsPreparedPointCloudRendererData > prepare() SIP_SKIP;
+
     /**
      * Must be called when a new render cycle is started. A call to startRender() must always
      * be followed by a corresponding call to stopRender() after all features have been rendered.
@@ -436,19 +511,35 @@ class CORE_EXPORT QgsPointCloudRenderer
      */
     const QgsMapUnitScale &pointSizeMapUnitScale() const { return mPointSizeMapUnitScale; }
 
+    /**
+     * Returns the drawing order used by the renderer for drawing points.
+     *
+     * \see setDrawOrder2d()
+     * \since QGIS 3.24
+     */
+    Qgis::PointCloudDrawOrder drawOrder2d() const;
+
+    /**
+     * Sets the drawing \a order used by the renderer for drawing points.
+     *
+     * \see drawOrder2d()
+     * \since QGIS 3.24
+     */
+    void setDrawOrder2d( Qgis::PointCloudDrawOrder order );
+
     /**
      * Returns the symbol used by the renderer for drawing points.
      *
      * \see setPointSymbol()
      */
-    PointSymbol pointSymbol() const;
+    Qgis::PointCloudSymbol pointSymbol() const;
 
     /**
      * Sets the \a symbol used by the renderer for drawing points.
      *
      * \see pointSymbol()
      */
-    void setPointSymbol( PointSymbol symbol );
+    void setPointSymbol( Qgis::PointCloudSymbol symbol );
 
     /**
      * Returns the maximum screen error allowed when rendering the point cloud.
@@ -500,6 +591,62 @@ class CORE_EXPORT QgsPointCloudRenderer
      */
     virtual QStringList legendRuleKeys() const;
 
+    /**
+     * Returns whether eye dome lighting effect will be used
+     * \note This is not a part of stable API - this function may be removed in a future release
+     * \since QGIS 3.28
+     */
+    bool eyeDomeLightingEnabled() const { return mEyeDomeLightingEnabled; }
+
+    /**
+     * Sets whether eye dome lighting effect will be used
+     * \note This is not a part of stable API - this function may be removed in a future release
+     * \since QGIS 3.28
+     */
+    void setEyeDomeLightingEnabled( bool enabled ) { mEyeDomeLightingEnabled = enabled; }
+
+    /**
+     * Returns the eye dome lighting strength value
+     * \note This is not a part of stable API - this function may be removed in a future release
+     * \since QGIS 3.28
+     */
+    double eyeDomeLightingStrength() const { return mEyeDomeLightingStrength; }
+
+    /**
+     * Sets the eye dome lighting strength value
+     * \note This is not a part of stable API - this function may be removed in a future release
+     * \since QGIS 3.28
+     */
+    void setEyeDomeLightingStrength( double strength ) { mEyeDomeLightingStrength = strength; }
+
+    /**
+     * Returns the eye dome lighting distance
+     * \note This is not a part of stable API - this function may be removed in a future release
+     * \since QGIS 3.28
+     */
+    double eyeDomeLightingDistance() const { return mEyeDomeLightingDistance; }
+
+    /**
+     * Sets the eye dome lighting distance
+     * \note This is not a part of stable API - this function may be removed in a future release
+     * \since QGIS 3.28
+     */
+    void setEyeDomeLightingDistance( double distance ) { mEyeDomeLightingDistance = distance; }
+
+    /**
+     * Returns unit for the eye dome lighting distance
+     * \note This is not a part of stable API - this function may be removed in a future release
+     * \since QGIS 3.28
+     */
+    QgsUnitTypes::RenderUnit eyeDomeLightingDistanceUnit() const { return mEyeDomeLightingDistanceUnit; }
+
+    /**
+     * Sets unit for the eye dome lighting distance
+     * \note This is not a part of stable API - this function may be removed in a future release
+     * \since QGIS 3.28
+     */
+    void setEyeDomeLightingDistanceUnit( QgsUnitTypes::RenderUnit unit ) { mEyeDomeLightingDistanceUnit = unit; }
+
   protected:
 
     /**
@@ -536,13 +683,13 @@ class CORE_EXPORT QgsPointCloudRenderer
       QPainter *painter = context.renderContext().painter();
       switch ( mPointSymbol )
       {
-        case Square:
+        case Qgis::PointCloudSymbol::Square:
           painter->fillRect( QRectF( x - mPainterPenWidth * 0.5,
                                      y - mPainterPenWidth * 0.5,
                                      mPainterPenWidth, mPainterPenWidth ), color );
           break;
 
-        case Circle:
+        case Qgis::PointCloudSymbol::Circle:
           painter->setBrush( QBrush( color ) );
           painter->setPen( Qt::NoPen );
           painter->drawEllipse( QRectF( x - mPainterPenWidth * 0.5,
@@ -552,6 +699,15 @@ class CORE_EXPORT QgsPointCloudRenderer
       };
     }
 
+#ifndef SIP_RUN   // intentionally left out from SIP to avoid API breaks in future when we move elevation post-processing elsewhere
+
+    /**
+     * Draws a point at the elevation \a z using at the specified \a x and \a y (in map coordinates) on the elevation map.
+     * \since QGIS 3.28
+     */
+    void drawPointToElevationMap( double x, double y, double z, QgsPointCloudRenderContext &context ) const;
+#endif
+
     /**
      * Copies common point cloud properties (such as point size and screen error) to the \a destination renderer.
      */
@@ -590,8 +746,14 @@ class CORE_EXPORT QgsPointCloudRenderer
     QgsUnitTypes::RenderUnit mPointSizeUnit = QgsUnitTypes::RenderMillimeters;
     QgsMapUnitScale mPointSizeMapUnitScale;
 
-    PointSymbol mPointSymbol = Square;
+    Qgis::PointCloudSymbol mPointSymbol = Qgis::PointCloudSymbol::Square;
     int mPainterPenWidth = 1;
+    Qgis::PointCloudDrawOrder mDrawOrder2d = Qgis::PointCloudDrawOrder::Default;
+
+    bool mEyeDomeLightingEnabled = false;
+    double mEyeDomeLightingStrength = 1000.0;
+    double mEyeDomeLightingDistance = 0.5;
+    QgsUnitTypes::RenderUnit mEyeDomeLightingDistanceUnit = QgsUnitTypes::RenderMillimeters;
 };
 
 #endif // QGSPOINTCLOUDRENDERER_H
