@@ -66,6 +66,11 @@ QStringList QgsProcessingProvider::supportedOutputRasterLayerExtensions() const
   return QgsRasterFileWriter::supportedFormatExtensions();
 }
 
+QStringList QgsProcessingProvider::supportedOutputPointCloudLayerExtensions() const
+{
+  return QStringList();
+}
+
 void QgsProcessingProvider::refreshAlgorithms()
 {
   qDeleteAll( mAlgorithms );
@@ -185,6 +190,17 @@ bool QgsProcessingProvider::isSupportedOutputValue( const QVariant &outputValue,
     }
     return true;
   }
+  else if ( parameter->type() == QgsProcessingParameterPointCloudDestination::typeName() )
+  {
+    const QFileInfo fi( outputPath );
+    const QString extension = fi.completeSuffix();
+    if ( !supportedOutputPointCloudLayerExtensions().contains( extension, Qt::CaseInsensitive ) )
+    {
+      error = tr( "“.%1” files are not supported as outputs for this algorithm" ).arg( extension );
+      return false;
+    }
+    return true;
+  }
   else
   {
     return true;
@@ -234,6 +250,27 @@ QString QgsProcessingProvider::defaultRasterFileExtension() const
   }
 }
 
+QString QgsProcessingProvider::defaultPointCloudFileExtension() const
+{
+  const QString userDefault = QgsProcessingUtils::defaultPointCloudExtension();
+
+  const QStringList supportedExtensions = supportedOutputPointCloudLayerExtensions();
+  if ( supportedExtensions.contains( userDefault, Qt::CaseInsensitive ) )
+  {
+    // user set default is supported by provider, use that
+    return userDefault;
+  }
+  else if ( !supportedExtensions.empty() )
+  {
+    return supportedExtensions.at( 0 );
+  }
+  else
+  {
+    // who knows? provider says it has no file support at all...
+    return QStringLiteral( "las" );
+  }
+}
+
 bool QgsProcessingProvider::supportsNonFileBasedOutput() const
 {
   return true;
