@@ -18,7 +18,7 @@
 #include "qgscoordinatereferencesystem.h"
 #include "qgscoordinatereferencesystem_p.h"
 
-#include "qgscoordinatereferencesystem_legacy.h"
+#include "qgscoordinatereferencesystem_legacy_p.h"
 #include "qgscoordinatereferencesystemregistry.h"
 #include "qgsreadwritelocker.h"
 
@@ -125,6 +125,7 @@ QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const long id, CrsTy
 QgsCoordinateReferenceSystem::QgsCoordinateReferenceSystem( const QgsCoordinateReferenceSystem &srs )  //NOLINT
   : d( srs.d )
   , mValidationHint( srs.mValidationHint )
+  , mNativeFormat( srs.mNativeFormat )
 {
 }
 
@@ -132,6 +133,7 @@ QgsCoordinateReferenceSystem &QgsCoordinateReferenceSystem::operator=( const Qgs
 {
   d = srs.d;
   mValidationHint = srs.mValidationHint;
+  mNativeFormat = srs.mNativeFormat;
   return *this;
 }
 
@@ -794,6 +796,101 @@ bool QgsCoordinateReferenceSystem::hasAxisInverted() const
   return d->mAxisInverted;
 }
 
+QList<Qgis::CrsAxisDirection> QgsCoordinateReferenceSystem::axisOrdering() const
+{
+  const PJ *projObject = d->threadLocalProjObject();
+  if ( !projObject )
+    return {};
+
+  PJ_CONTEXT *context = QgsProjContext::get();
+  QgsProjUtils::proj_pj_unique_ptr pjCs( proj_crs_get_coordinate_system( context, projObject ) );
+  if ( !pjCs )
+    return {};
+
+  const thread_local QMap< Qgis::CrsAxisDirection, QString > mapping =
+  {
+    { Qgis::CrsAxisDirection::North, QStringLiteral( "north" ) },
+    { Qgis::CrsAxisDirection::NorthNorthEast, QStringLiteral( "northNorthEast" ) },
+    { Qgis::CrsAxisDirection::NorthEast, QStringLiteral( "northEast" ) },
+    { Qgis::CrsAxisDirection::EastNorthEast, QStringLiteral( "eastNorthEast" ) },
+    { Qgis::CrsAxisDirection::East, QStringLiteral( "east" ) },
+    { Qgis::CrsAxisDirection::EastSouthEast, QStringLiteral( "eastSouthEast" ) },
+    { Qgis::CrsAxisDirection::SouthEast, QStringLiteral( "southEast" ) },
+    { Qgis::CrsAxisDirection::SouthSouthEast, QStringLiteral( "southSouthEast" ) },
+    { Qgis::CrsAxisDirection::South, QStringLiteral( "south" ) },
+    { Qgis::CrsAxisDirection::SouthSouthWest, QStringLiteral( "southSouthWest" ) },
+    { Qgis::CrsAxisDirection::SouthWest, QStringLiteral( "southWest" ) },
+    { Qgis::CrsAxisDirection::WestSouthWest, QStringLiteral( "westSouthWest" ) },
+    { Qgis::CrsAxisDirection::West, QStringLiteral( "west" ) },
+    { Qgis::CrsAxisDirection::WestNorthWest, QStringLiteral( "westNorthWest" ) },
+    { Qgis::CrsAxisDirection::NorthWest, QStringLiteral( "northWest" ) },
+    { Qgis::CrsAxisDirection::NorthNorthWest, QStringLiteral( "northNorthWest" ) },
+    { Qgis::CrsAxisDirection::GeocentricX, QStringLiteral( "geocentricX" ) },
+    { Qgis::CrsAxisDirection::GeocentricY, QStringLiteral( "geocentricY" ) },
+    { Qgis::CrsAxisDirection::GeocentricZ, QStringLiteral( "geocentricZ" ) },
+    { Qgis::CrsAxisDirection::Up, QStringLiteral( "up" ) },
+    { Qgis::CrsAxisDirection::Down, QStringLiteral( "down" ) },
+    { Qgis::CrsAxisDirection::Forward, QStringLiteral( "forward" ) },
+    { Qgis::CrsAxisDirection::Aft, QStringLiteral( "aft" ) },
+    { Qgis::CrsAxisDirection::Port, QStringLiteral( "port" ) },
+    { Qgis::CrsAxisDirection::Starboard, QStringLiteral( "starboard" ) },
+    { Qgis::CrsAxisDirection::Clockwise, QStringLiteral( "clockwise" ) },
+    { Qgis::CrsAxisDirection::CounterClockwise, QStringLiteral( "counterClockwise" ) },
+    { Qgis::CrsAxisDirection::ColumnPositive, QStringLiteral( "columnPositive" ) },
+    { Qgis::CrsAxisDirection::ColumnNegative, QStringLiteral( "columnNegative" ) },
+    { Qgis::CrsAxisDirection::RowPositive, QStringLiteral( "rowPositive" ) },
+    { Qgis::CrsAxisDirection::RowNegative, QStringLiteral( "rowNegative" ) },
+    { Qgis::CrsAxisDirection::DisplayRight, QStringLiteral( "displayRight" ) },
+    { Qgis::CrsAxisDirection::DisplayLeft, QStringLiteral( "displayLeft" ) },
+    { Qgis::CrsAxisDirection::DisplayUp, QStringLiteral( "displayUp" ) },
+    { Qgis::CrsAxisDirection::DisplayDown, QStringLiteral( "displayDown" ) },
+    { Qgis::CrsAxisDirection::Future, QStringLiteral( "future" ) },
+    { Qgis::CrsAxisDirection::Past, QStringLiteral( "past" ) },
+    { Qgis::CrsAxisDirection::Towards, QStringLiteral( "towards" ) },
+    { Qgis::CrsAxisDirection::AwayFrom, QStringLiteral( "awayFrom" ) },
+  };
+
+  QList< Qgis::CrsAxisDirection > res;
+  const int axisCount = proj_cs_get_axis_count( context, pjCs.get() );
+  if ( axisCount > 0 )
+  {
+    res.reserve( axisCount );
+
+    for ( int i = 0; i < axisCount; ++i )
+    {
+      const char *outDirection = nullptr;
+      proj_cs_get_axis_info( context, pjCs.get(), i,
+                             nullptr,
+                             nullptr,
+                             &outDirection,
+                             nullptr,
+                             nullptr,
+                             nullptr,
+                             nullptr
+                           );
+      // get first word of direction only
+      const thread_local QRegularExpression rx( QStringLiteral( "([^\\s]+).*" ) );
+      const QRegularExpressionMatch match = rx.match( QString( outDirection ) );
+      if ( !match.hasMatch() )
+        continue;
+
+      const QString direction = match.captured( 1 );
+      Qgis::CrsAxisDirection dir = Qgis::CrsAxisDirection::Unspecified;
+      for ( auto it = mapping.constBegin(); it != mapping.constEnd(); ++it )
+      {
+        if ( it.value().compare( direction, Qt::CaseInsensitive ) == 0 )
+        {
+          dir = it.key();
+          break;
+        }
+      }
+
+      res.append( dir );
+    }
+  }
+  return res;
+}
+
 bool QgsCoordinateReferenceSystem::createFromWkt( const QString &wkt )
 {
   return createFromWktInternal( wkt, QString() );
@@ -1337,11 +1434,16 @@ QgsProjectionFactors QgsCoordinateReferenceSystem::factors( const QgsPoint &poin
 
 QgsProjOperation QgsCoordinateReferenceSystem::operation() const
 {
+  if ( !d->mIsValid )
+    return QgsProjOperation();
+
   QgsProjOperation res;
 
   // we have to make a transformation object corresponding to the crs
   QString projString = toProj();
   projString.replace( QLatin1String( "+type=crs" ), QString() );
+  if ( projString.isEmpty() )
+    return QgsProjOperation();
 
   QgsProjUtils::proj_pj_unique_ptr transformation( proj_create( QgsProjContext::get(), projString.toUtf8().constData() ) );
   if ( !transformation )
@@ -1393,6 +1495,29 @@ QgsRectangle QgsCoordinateReferenceSystem::bounds() const
   return rect;
 }
 
+QString QgsCoordinateReferenceSystem::toOgcUri() const
+{
+  const auto parts { authid().split( ':' ) };
+  if ( parts.length() == 2 )
+  {
+    if ( parts[0] == QLatin1String( "EPSG" ) )
+      return  QStringLiteral( "http://www.opengis.net/def/crs/EPSG/0/%1" ).arg( parts[1] ) ;
+    else if ( parts[0] == QLatin1String( "OGC" ) )
+    {
+      return  QStringLiteral( "http://www.opengis.net/def/crs/OGC/1.3/%1" ).arg( parts[1] ) ;
+    }
+    else
+    {
+      QgsMessageLog::logMessage( QStringLiteral( "Error converting published CRS to URI %1: (not OGC or EPSG)" ).arg( authid() ), QStringLiteral( "CRS" ), Qgis::MessageLevel::Critical );
+    }
+  }
+  else
+  {
+    QgsMessageLog::logMessage( QStringLiteral( "Error converting published CRS to URI: %1" ).arg( authid() ), QStringLiteral( "CRS" ), Qgis::MessageLevel::Critical );
+  }
+  return QString();
+}
+
 void QgsCoordinateReferenceSystem::updateDefinition()
 {
   if ( !d->mIsValid )
@@ -1863,6 +1988,8 @@ bool QgsCoordinateReferenceSystem::readXml( const QDomNode &node )
     {
       d->mCoordinateEpoch = std::numeric_limits< double >::quiet_NaN();
     }
+
+    mNativeFormat = qgsEnumKeyToValue<Qgis::CrsDefinitionFormat>( srsNode.toElement().attribute( QStringLiteral( "nativeFormat" ) ), Qgis::CrsDefinitionFormat::Wkt );
   }
   else
   {
@@ -1878,6 +2005,8 @@ bool QgsCoordinateReferenceSystem::writeXml( QDomNode &node, QDomDocument &doc )
   QDomElement layerNode = node.toElement();
   QDomElement srsElement = doc.createElement( QStringLiteral( "spatialrefsys" ) );
 
+  srsElement.setAttribute( QStringLiteral( "nativeFormat" ), qgsEnumValueToKey<Qgis::CrsDefinitionFormat>( mNativeFormat ) );
+
   if ( std::isfinite( d->mCoordinateEpoch ) )
   {
     srsElement.setAttribute( QStringLiteral( "coordinateEpoch" ), d->mCoordinateEpoch );
@@ -2045,16 +2174,26 @@ void QgsCoordinateReferenceSystem::setValidationHint( const QString &html )
   mValidationHint = html;
 }
 
-QString QgsCoordinateReferenceSystem::validationHint()
+QString QgsCoordinateReferenceSystem::validationHint() const
 {
   return mValidationHint;
 }
 
-long QgsCoordinateReferenceSystem::saveAsUserCrs( const QString &name, Format nativeFormat )
+long QgsCoordinateReferenceSystem::saveAsUserCrs( const QString &name, Qgis::CrsDefinitionFormat nativeFormat )
 {
   return QgsApplication::coordinateReferenceSystemRegistry()->addUserCrs( *this, name, nativeFormat );
 }
 
+void QgsCoordinateReferenceSystem::setNativeFormat( Qgis::CrsDefinitionFormat format )
+{
+  mNativeFormat = format;
+}
+
+Qgis::CrsDefinitionFormat QgsCoordinateReferenceSystem::nativeFormat() const
+{
+  return mNativeFormat;
+}
+
 long QgsCoordinateReferenceSystem::getRecordCount()
 {
   sqlite3_database_unique_ptr database;
@@ -2128,7 +2267,6 @@ void getOperationAndEllipsoidFromProjString( const QString &proj, QString &opera
   }
   operation = projMatch.captured( 1 );
 
-  thread_local const QRegularExpression ellipseRegExp( QStringLiteral( "\\+(?:ellps|datum)=(\\S+)" ) );
   const QRegularExpressionMatch ellipseMatch = projRegExp.match( proj );
   if ( ellipseMatch.hasMatch() )
   {
@@ -2189,8 +2327,6 @@ bool QgsCoordinateReferenceSystem::loadFromAuthCode( const QString &auth, const
   d->setPj( std::move( crs ) );
 
   const QString dbVals = sAuthIdToQgisSrsIdMap.value( QStringLiteral( "%1:%2" ).arg( auth, code ).toUpper() );
-  QString srsId;
-  QString srId;
   if ( !dbVals.isEmpty() )
   {
     const QStringList parts = dbVals.split( ',' );
@@ -2614,6 +2750,35 @@ const QHash<long, QgsCoordinateReferenceSystem> &QgsCoordinateReferenceSystem::s
   return *sSrsIdCache();
 }
 
+QgsCoordinateReferenceSystem QgsCoordinateReferenceSystem::toGeographicCrs() const
+{
+  if ( isGeographic() )
+  {
+    return *this;
+  }
+
+  if ( PJ *obj = d->threadLocalProjObject() )
+  {
+    PJ_CONTEXT *pjContext = QgsProjContext::get();
+    QgsProjUtils::proj_pj_unique_ptr geoCrs( proj_crs_get_geodetic_crs( pjContext, obj ) );
+    if ( !geoCrs )
+      return QgsCoordinateReferenceSystem();
+
+    if ( !testIsGeographic( geoCrs.get() ) )
+      return QgsCoordinateReferenceSystem();
+
+    QgsProjUtils::proj_pj_unique_ptr normalized( proj_normalize_for_visualization( pjContext, geoCrs.get() ) );
+    if ( !normalized )
+      return QgsCoordinateReferenceSystem();
+
+    return QgsCoordinateReferenceSystem::fromProjObject( normalized.get() );
+  }
+  else
+  {
+    return QgsCoordinateReferenceSystem();
+  }
+}
+
 QString QgsCoordinateReferenceSystem::geographicCrsAuthId() const
 {
   if ( isGeographic() )
@@ -2636,6 +2801,73 @@ PJ *QgsCoordinateReferenceSystem::projObject() const
   return d->threadLocalProjObject();
 }
 
+QgsCoordinateReferenceSystem QgsCoordinateReferenceSystem::fromProjObject( PJ *object )
+{
+  QgsCoordinateReferenceSystem crs;
+  crs.createFromProjObject( object );
+  return crs;
+}
+
+bool QgsCoordinateReferenceSystem::createFromProjObject( PJ *object )
+{
+  d.detach();
+  d->mIsValid = false;
+  d->mProj4.clear();
+  d->mWktPreferred.clear();
+
+  if ( !object )
+  {
+    return false;
+  }
+
+  switch ( proj_get_type( object ) )
+  {
+    case PJ_TYPE_GEODETIC_CRS:
+    case PJ_TYPE_GEOCENTRIC_CRS:
+    case PJ_TYPE_GEOGRAPHIC_CRS:
+    case PJ_TYPE_GEOGRAPHIC_2D_CRS:
+    case PJ_TYPE_GEOGRAPHIC_3D_CRS:
+    case PJ_TYPE_VERTICAL_CRS:
+    case PJ_TYPE_PROJECTED_CRS:
+    case PJ_TYPE_COMPOUND_CRS:
+    case PJ_TYPE_TEMPORAL_CRS:
+    case PJ_TYPE_ENGINEERING_CRS:
+    case PJ_TYPE_BOUND_CRS:
+    case PJ_TYPE_OTHER_CRS:
+      break;
+
+    default:
+      return false;
+  }
+
+  d->setPj( QgsProjUtils::crsToSingleCrs( object ) );
+
+  if ( !d->hasPj() )
+  {
+    return d->mIsValid;
+  }
+  else
+  {
+    // maybe we can directly grab the auth name and code from the crs
+    const QString authName( proj_get_id_auth_name( d->threadLocalProjObject(), 0 ) );
+    const QString authCode( proj_get_id_code( d->threadLocalProjObject(), 0 ) );
+    if ( !authName.isEmpty() && !authCode.isEmpty() && loadFromAuthCode( authName, authCode ) )
+    {
+      return d->mIsValid;
+    }
+    else
+    {
+      // Still a valid CRS, just not a known one
+      d->mIsValid = true;
+      d->mDescription = QString( proj_get_name( d->threadLocalProjObject() ) );
+      setMapUnits();
+      d->mIsGeographic = testIsGeographic( d->threadLocalProjObject() );
+    }
+  }
+
+  return d->mIsValid;
+}
+
 QStringList QgsCoordinateReferenceSystem::recentProjections()
 {
   QStringList projections;
