@@ -16,8 +16,9 @@
 #include "qgsabstractdatabaseproviderconnection.h"
 #include "qgsvectorlayer.h"
 #include "qgsexception.h"
-#include "qgslogger.h"
+#include "qgsweakrelation.h"
 #include "qgsfeedback.h"
+#include "qgsprovidersqlquerybuilder.h"
 
 #include <QVariant>
 #include <QObject>
@@ -1014,6 +1015,16 @@ QMultiMap<Qgis::SqlKeywordCategory, QStringList> QgsAbstractDatabaseProviderConn
   };
 }
 
+QList<Qgis::FieldDomainType> QgsAbstractDatabaseProviderConnection::supportedFieldDomainTypes() const
+{
+  return {};
+}
+
+QgsProviderSqlQueryBuilder *QgsAbstractDatabaseProviderConnection::queryBuilder() const
+{
+  return new QgsProviderSqlQueryBuilder();
+}
+
 void QgsAbstractDatabaseProviderConnection::createVectorTable( const QString &schema,
     const QString &name,
     const QgsFields &fields,
@@ -1069,6 +1080,16 @@ bool QgsAbstractDatabaseProviderConnection::tableExists( const QString &schema,
   return false;
 }
 
+
+QList<QgsLayerMetadataProviderResult> QgsAbstractDatabaseProviderConnection::searchLayerMetadata( const QgsMetadataSearchContext &searchContext, const QString &searchString, const QgsRectangle &geographicExtent, QgsFeedback *feedback ) const
+{
+  Q_UNUSED( feedback );
+  Q_UNUSED( searchContext );
+  Q_UNUSED( searchString );
+  Q_UNUSED( geographicExtent );
+  throw QgsNotSupportedException( QObject::tr( "Provider %1 has no %2 method" ).arg( providerKey(), QStringLiteral( "searchLayerMetadata" ) ) );
+}
+
 void QgsAbstractDatabaseProviderConnection::dropRasterTable( const QString &, const QString & ) const
 {
   checkCapability( Capability::DropRasterTable );
@@ -1179,6 +1200,38 @@ void QgsAbstractDatabaseProviderConnection::addField( const QgsField &field, con
   }
 }
 
+void QgsAbstractDatabaseProviderConnection::renameField( const QString &schema, const QString &tableName, const QString &name, const QString &newName ) const
+{
+  checkCapability( Capability::RenameField );
+
+  QgsVectorLayer::LayerOptions options { false, false };
+  options.skipCrsValidation = true;
+  std::unique_ptr<QgsVectorLayer> vl( std::make_unique<QgsVectorLayer>( tableUri( schema, tableName ), QStringLiteral( "temp_layer" ), mProviderKey, options ) );
+  if ( ! vl->isValid() )
+  {
+    throw QgsProviderConnectionException( QObject::tr( "Could not create a vector layer for table '%1' in schema '%2'" )
+                                          .arg( tableName, schema ) );
+  }
+  int existingIndex = vl->fields().lookupField( name );
+  if ( existingIndex == -1 )
+  {
+    throw QgsProviderConnectionException( QObject::tr( "Field '%1' in table '%2' in does not exist" )
+                                          .arg( name, tableName ) );
+
+  }
+  if ( vl->fields().lookupField( newName ) != -1 )
+  {
+    throw QgsProviderConnectionException( QObject::tr( "A field with name '%1' already exists in table '%2'" )
+                                          .arg( newName, tableName ) );
+
+  }
+  if ( ! vl->dataProvider()->renameAttributes( {{existingIndex, newName}} ) )
+  {
+    throw QgsProviderConnectionException( QObject::tr( "Unknown error renaming field '%1' in table '%2' to '%3'" )
+                                          .arg( name, tableName, newName ) );
+  }
+}
+
 QList<QgsAbstractDatabaseProviderConnection::TableProperty> QgsAbstractDatabaseProviderConnection::tables( const QString &, const QgsAbstractDatabaseProviderConnection::TableFlags & ) const
 {
   checkCapability( Capability::Tables );
@@ -1259,6 +1312,34 @@ QgsFields QgsAbstractDatabaseProviderConnection::fields( const QString &schema,
   }
 }
 
+QStringList QgsAbstractDatabaseProviderConnection::fieldDomainNames() const
+{
+  checkCapability( Capability::ListFieldDomains );
+  return QStringList();
+}
+
+QgsFieldDomain *QgsAbstractDatabaseProviderConnection::fieldDomain( const QString & ) const
+{
+  checkCapability( Capability::RetrieveFieldDomain );
+  return nullptr;
+}
+
+void QgsAbstractDatabaseProviderConnection::setFieldDomainName( const QString &, const QString &, const QString &, const QString & ) const
+{
+  checkCapability( Capability::SetFieldDomain );
+}
+
+void QgsAbstractDatabaseProviderConnection::addFieldDomain( const QgsFieldDomain &, const QString & ) const
+{
+  checkCapability( Capability::AddFieldDomain );
+}
+
+QList< QgsWeakRelation > QgsAbstractDatabaseProviderConnection::relationships( const QString &, const QString & ) const
+{
+  checkCapability( Capability::RetrieveRelationships );
+  return {};
+}
+
 QString QgsAbstractDatabaseProviderConnection::TableProperty::defaultName() const
 {
   QString n = mTableName;
@@ -1476,7 +1557,7 @@ QgsAbstractDatabaseProviderConnection::QueryResult::QueryResult( std::shared_ptr
   : mResultIterator( iterator )
 {}
 
-double QgsAbstractDatabaseProviderConnection::QueryResult::queryExecutionTime()
+double QgsAbstractDatabaseProviderConnection::QueryResult::queryExecutionTime() const
 {
   return mQueryExecutionTime;
 }
