@@ -16,15 +16,12 @@
 
 #include "qgsabstractgeopdfexporter.h"
 #include "qgscoordinatetransformcontext.h"
-#include "qgsrenderedfeaturehandlerinterface.h"
-#include "qgsfeaturerequest.h"
 #include "qgslogger.h"
 #include "qgsgeometry.h"
-#include "qgsvectorlayer.h"
 #include "qgsvectorfilewriter.h"
+#include "qgsfileutils.h"
 
 #include <gdal.h>
-#include "qgsgdalutils.h"
 #include "cpl_string.h"
 
 #include <QMutex>
@@ -123,7 +120,7 @@ bool QgsAbstractGeoPdfExporter::finalize( const QList<ComponentLayerDetail> &com
 
 QString QgsAbstractGeoPdfExporter::generateTemporaryFilepath( const QString &filename ) const
 {
-  return mTemporaryDir.filePath( filename );
+  return mTemporaryDir.filePath( QgsFileUtils::stringToSafeFilename( filename ) );
 }
 
 bool QgsAbstractGeoPdfExporter::compositionModeSupported( QPainter::CompositionMode mode )
