@@ -47,6 +47,7 @@ QgsExpressionContextScope::QgsExpressionContextScope( const QgsExpressionContext
   , mFeature( other.mFeature )
   , mHasGeometry( other.mHasGeometry )
   , mGeometry( other.mGeometry )
+  , mHiddenVariables( other.mHiddenVariables )
 {
   QHash<QString, QgsScopedExpressionFunction * >::const_iterator it = other.mFunctions.constBegin();
   for ( ; it != other.mFunctions.constEnd(); ++it )
@@ -63,6 +64,7 @@ QgsExpressionContextScope &QgsExpressionContextScope::operator=( const QgsExpres
   mFeature = other.mFeature;
   mHasGeometry = other.mHasGeometry;
   mGeometry = other.mGeometry;
+  mHiddenVariables = other.mHiddenVariables;
 
   qDeleteAll( mFunctions );
   mFunctions.clear();
@@ -82,12 +84,11 @@ QgsExpressionContextScope::~QgsExpressionContextScope()
 
 void QgsExpressionContextScope::setVariable( const QString &name, const QVariant &value, bool isStatic )
 {
-  if ( mVariables.contains( name ) )
+  auto it = mVariables.find( name );
+  if ( it != mVariables.end() )
   {
-    StaticVariable existing = mVariables.value( name );
-    existing.value = value;
-    existing.isStatic = isStatic;
-    addVariable( existing );
+    it->value = value;
+    it->isStatic = isStatic;
   }
   else
   {
@@ -118,9 +119,40 @@ QVariant QgsExpressionContextScope::variable( const QString &name ) const
 QStringList QgsExpressionContextScope::variableNames() const
 {
   QStringList names = mVariables.keys();
+
+  if ( hasFeature() )
+  {
+    names.append( QStringLiteral( "feature" ) );
+    names.append( QStringLiteral( "id" ) );
+    names.append( QStringLiteral( "geometry" ) );
+  }
+
   return names;
 }
 
+QStringList QgsExpressionContextScope::hiddenVariables() const
+{
+  return mHiddenVariables;
+}
+
+void QgsExpressionContextScope::setHiddenVariables( const QStringList &hiddenVariables )
+{
+  mHiddenVariables = hiddenVariables;
+}
+
+void QgsExpressionContextScope::addHiddenVariable( const QString &hiddenVariable )
+{
+  if ( !mHiddenVariables.contains( hiddenVariable ) )
+    mHiddenVariables << hiddenVariable;
+}
+
+void QgsExpressionContextScope::removeHiddenVariable( const QString &hiddenVariable )
+{
+  if ( mHiddenVariables.contains( hiddenVariable ) )
+    mHiddenVariables.removeAt( mHiddenVariables.indexOf( hiddenVariable ) );
+}
+
+
 /// @cond PRIVATE
 class QgsExpressionContextVariableCompare
 {
@@ -209,8 +241,14 @@ void QgsExpressionContextScope::readXml( const QDomElement &element, const QgsRe
   {
     const QDomElement variableElement = variablesNodeList.at( i ).toElement();
     const QString key = variableElement.attribute( QStringLiteral( "name" ) );
-    const QVariant value = QgsXmlUtils::readVariant( variableElement.firstChildElement( QStringLiteral( "Option" ) ) );
-    setVariable( key, value );
+    if ( variableElement.tagName() == QLatin1String( "Variable" ) )
+    {
+      const QVariant value = QgsXmlUtils::readVariant( variableElement.firstChildElement( QStringLiteral( "Option" ) ) );
+      setVariable( key, value );
+    }
+    else
+      addHiddenVariable( key );
+
   }
 }
 
@@ -224,6 +262,13 @@ bool QgsExpressionContextScope::writeXml( QDomElement &element, QDomDocument &do
     varElem.appendChild( valueElem );
     element.appendChild( varElem );
   }
+
+  for ( QString hiddenVariable : mHiddenVariables )
+  {
+    QDomElement varElem = document.createElement( QStringLiteral( "HiddenVariable" ) );
+    varElem.setAttribute( QStringLiteral( "name" ), hiddenVariable );
+    element.appendChild( varElem );
+  }
   return true;
 }
 
@@ -410,13 +455,14 @@ int QgsExpressionContext::indexOfScope( const QString &scopeName ) const
 
 QStringList QgsExpressionContext::variableNames() const
 {
-  QStringList names;
-  const auto constMStack = mStack;
-  for ( const QgsExpressionContextScope *scope : constMStack )
+  QSet< QString> names;
+  for ( const QgsExpressionContextScope *scope : mStack )
   {
-    names << scope->variableNames();
+    const QStringList variableNames = scope->variableNames();
+    for ( const QString &name : variableNames )
+      names.insert( name );
   }
-  return qgis::setToList( qgis::listToSet( names ) );
+  return QStringList( names.constBegin(), names.constEnd() );
 }
 
 QStringList QgsExpressionContext::filteredVariableNames() const
@@ -424,9 +470,20 @@ QStringList QgsExpressionContext::filteredVariableNames() const
   QStringList allVariables = variableNames();
   QStringList filtered;
   const auto constAllVariables = allVariables;
+
+  QStringList hiddenVariables;
+
+  for ( const QgsExpressionContextScope *scope : mStack )
+  {
+    const QStringList scopeHiddenVariables = scope->hiddenVariables();
+    for ( const QString &name : scopeHiddenVariables )
+      hiddenVariables << name ;
+  }
+
   for ( const QString &variable : constAllVariables )
   {
-    if ( variable.startsWith( '_' ) )
+    if ( variable.startsWith( '_' ) ||
+         hiddenVariables.contains( variable ) )
       continue;
 
     filtered << variable;
@@ -466,15 +523,16 @@ bool QgsExpressionContext::hasFunction( const QString &name ) const
 
 QStringList QgsExpressionContext::functionNames() const
 {
-  QStringList result;
-  const auto constMStack = mStack;
-  for ( const QgsExpressionContextScope *scope : constMStack )
+  QSet< QString > result;
+  for ( const QgsExpressionContextScope *scope : mStack )
   {
-    result << scope->functionNames();
+    const QStringList functionNames = scope->functionNames();
+    for ( const QString &name : functionNames )
+      result.insert( name );
   }
-  result = qgis::setToList( qgis::listToSet( result ) );
-  result.sort();
-  return result;
+  QStringList listResult( result.constBegin(), result.constEnd() );
+  listResult.sort();
+  return listResult;
 }
 
 QgsExpressionFunction *QgsExpressionContext::function( const QString &name ) const
