@@ -20,7 +20,7 @@
 #include "qgsconditionalstyle.h"
 #include "qgsapplication.h"
 #include "qgssettings.h"
-
+#include "qgsexpressioncontextutils.h"
 
 QgsFeaturePickerModelBase::QgsFeaturePickerModelBase( QObject *parent )
   : QAbstractItemModel( parent )
@@ -177,13 +177,8 @@ QVariant QgsFeaturePickerModelBase::data( const QModelIndex &index, int role ) c
     case IdentifierValuesRole:
       return mEntries.value( index.row() ).identifierFields;
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
-    case Qt::BackgroundColorRole:
-    case Qt::TextColorRole:
-#else
     case Qt::BackgroundRole:
     case Qt::ForegroundRole:
-#endif
     case Qt::DecorationRole:
     case Qt::FontRole:
     {
@@ -191,11 +186,7 @@ QVariant QgsFeaturePickerModelBase::data( const QModelIndex &index, int role ) c
       if ( isNull )
       {
         // Representation for NULL value
-#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
-        if ( role == Qt::TextColorRole )
-#else
         if ( role == Qt::ForegroundRole )
-#endif
         {
           return QBrush( QColor( Qt::gray ) );
         }
@@ -216,17 +207,9 @@ QVariant QgsFeaturePickerModelBase::data( const QModelIndex &index, int role ) c
 
         if ( style.isValid() )
         {
-#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
-          if ( role == Qt::BackgroundColorRole && style.validBackgroundColor() )
-#else
           if ( role == Qt::BackgroundRole && style.validBackgroundColor() )
-#endif
             return style.backgroundColor();
-#if QT_VERSION < QT_VERSION_CHECK(5, 13, 0)
-          if ( role == Qt::TextColorRole && style.validTextColor() )
-#else
           if ( role == Qt::ForegroundRole && style.validTextColor() )
-#endif
             return style.textColor();
           if ( role == Qt::DecorationRole )
             return style.icon();
@@ -421,7 +404,10 @@ void QgsFeaturePickerModelBase::scheduledReload()
       filterClause = QStringLiteral( "(%1) AND ((%2) ILIKE '%%3%')" ).arg( mFilterExpression, mDisplayExpression, mFilterValue );
 
     if ( !filterClause.isEmpty() )
+    {
       request.setFilterExpression( filterClause );
+      request.expressionContext()->appendScopes( QgsExpressionContextUtils::globalProjectLayerScopes( sourceLayer() ) );
+    }
   }
   QSet<QString> attributes = requestedAttributes();
   if ( !attributes.isEmpty() )
