@@ -32,7 +32,7 @@
 #include <sys/time.h>
 #endif
 
-#ifdef MSVC
+#ifdef _MSC_VER
 #include <Windows.h>
 #include <ShlObj.h>
 #pragma comment(lib,"Shell32.lib")
@@ -57,7 +57,7 @@ QString QgsFileUtils::representFileSize( qint64 bytes )
 
 QStringList QgsFileUtils::extensionsFromFilter( const QString &filter )
 {
-  const QRegularExpression rx( QStringLiteral( "\\*\\.([a-zA-Z0-9]+)" ) );
+  const thread_local QRegularExpression rx( QStringLiteral( "\\*\\.([a-zA-Z0-9]+)" ) );
   QStringList extensions;
   QRegularExpressionMatchIterator matches = rx.globalMatch( filter );
 
@@ -76,7 +76,7 @@ QStringList QgsFileUtils::extensionsFromFilter( const QString &filter )
 
 QString QgsFileUtils::wildcardsFromFilter( const QString &filter )
 {
-  const QRegularExpression globPatternsRx( QStringLiteral( ".*\\((.*?)\\)$" ) );
+  const thread_local QRegularExpression globPatternsRx( QStringLiteral( ".*\\((.*?)\\)$" ) );
   const QRegularExpressionMatch matches = globPatternsRx.match( filter );
   if ( matches.hasMatch() )
     return matches.captured( 1 );
@@ -143,7 +143,7 @@ QString QgsFileUtils::addExtensionFromFilter( const QString &fileName, const QSt
 
 QString QgsFileUtils::stringToSafeFilename( const QString &string )
 {
-  QRegularExpression rx( QStringLiteral( "[/\\\\\\?%\\*\\:\\|\"<>]" ) );
+  const thread_local QRegularExpression rx( QStringLiteral( "[/\\\\\\?%\\*\\:\\|\"<>]" ) );
   QString s = string;
   s.replace( rx, QStringLiteral( "_" ) );
   return s;
@@ -280,7 +280,7 @@ QStringList QgsFileUtils::findFile( const QString &file, const QString &basePath
   return foundFiles;
 }
 
-#ifdef MSVC
+#ifdef _MSC_VER
 std::unique_ptr< wchar_t[] > pathToWChar( const QString &path )
 {
   const QString nativePath = QDir::toNativeSeparators( path );
@@ -294,8 +294,8 @@ std::unique_ptr< wchar_t[] > pathToWChar( const QString &path )
 
 Qgis::DriveType QgsFileUtils::driveType( const QString &path )
 {
-#ifdef MSVC
-  auto pathType = [ = ]( const QString & path ) -> DriveType
+#ifdef _MSC_VER
+  auto pathType = [ = ]( const QString & path ) -> Qgis::DriveType
   {
     std::unique_ptr< wchar_t[] > pathArray = pathToWChar( path );
     const UINT type = GetDriveTypeW( pathArray.get() );
@@ -323,7 +323,7 @@ Qgis::DriveType QgsFileUtils::driveType( const QString &path )
         return Qgis::DriveType::RamDisk;
     }
 
-    return Unknown;
+    return Qgis::DriveType::Unknown;
 
   };
 
@@ -334,11 +334,11 @@ Qgis::DriveType QgsFileUtils::driveType( const QString &path )
   {
     prevPath = currentPath;
     currentPath = QFileInfo( currentPath ).path();
-    const DriveType type = pathType( currentPath );
-    if ( type != Unknown && type != Invalid )
+    const Qgis::DriveType type = pathType( currentPath );
+    if ( type != Qgis::DriveType::Unknown && type != Qgis::DriveType::Invalid )
       return type;
   }
-  return Unknown;
+  return Qgis::DriveType::Unknown;
 
 #else
   ( void )path;
@@ -501,3 +501,31 @@ bool QgsFileUtils::isCloseToLimitOfOpenedFiles( int filesToBeOpened )
   constexpr int SOME_MARGIN = 20;
   return nFileCount > 0 && nFileLimit > 0 && nFileCount + filesToBeOpened > nFileLimit - SOME_MARGIN;
 }
+
+QStringList QgsFileUtils::splitPathToComponents( const QString &input )
+{
+  QStringList result;
+  QString path = QDir::cleanPath( input );
+  if ( path.isEmpty() )
+    return result;
+
+  const QString fileName = QFileInfo( path ).fileName();
+  if ( !fileName.isEmpty() )
+    result << fileName;
+  else if ( QFileInfo( path ).path() == path )
+    result << path;
+
+  QString prevPath = path;
+  while ( ( path = QFileInfo( path ).path() ).length() < prevPath.length() )
+  {
+    const QString dirName = QDir( path ).dirName();
+    if ( dirName == QLatin1String( "." ) )
+      break;
+
+    result << ( !dirName.isEmpty() ? dirName : path );
+    prevPath = path;
+  }
+
+  std::reverse( result.begin(), result.end() );
+  return result;
+}
