@@ -212,15 +212,24 @@ class CORE_EXPORT QgsOgcUtils
         QString *errorMessage = nullptr ) SIP_SKIP;
 
     /**
-     * Creates an OGC expression XML element.
-     * \returns valid OGC expression QDomElement on success,
+     * Creates an OGC expression XML element from the \a exp expression
+     * with default values for the geometry name, srs name, honourAsisOrientation and invertAxisOrientation.
+     * \returns valid OGC expression QDomElement on success or a valid \verbatim <Filter> \endverbatim QDomElement when \a requiresFilterElement is set.
      * otherwise null QDomElement
      */
-    static QDomElement expressionToOgcExpression( const QgsExpression &exp, QDomDocument &doc, QString *errorMessage = nullptr );
+    static QDomElement expressionToOgcExpression( const QgsExpression &exp, QDomDocument &doc, QString *errorMessage = nullptr,
+        bool requiresFilterElement = false );
 
     /**
-     * Creates an OGC expression XML element.
-     * \returns valid OGC expression QDomElement on success,
+     * Creates an ElseFilter from \a doc
+     * \returns valid OGC ElseFilter QDomElement
+     * \since QGIS 3.28
+     */
+    static QDomElement elseFilterExpression( QDomDocument &doc );
+
+    /**
+     * Creates an OGC expression XML element from the \a exp expression.
+     * \returns valid OGC expression QDomElement on success or a valid \verbatim <Filter> \endverbatim QDomElement when \a requiresFilterElement is set.
      * otherwise null QDomElement
      */
     static QDomElement expressionToOgcExpression( const QgsExpression &exp,
@@ -231,7 +240,8 @@ class CORE_EXPORT QgsOgcUtils
         const QString &srsName,
         bool honourAxisOrientation,
         bool invertAxisOrientation,
-        QString *errorMessage = nullptr );
+        QString *errorMessage = nullptr,
+        bool requiresFilterElement = false );
 
 #ifndef SIP_RUN
 
@@ -303,6 +313,16 @@ class CORE_EXPORT QgsOgcUtils
     //! Static method that creates geometry from GML MultiPolygon
     static QgsGeometry geometryFromGMLMultiPolygon( const QDomElement &geometryElement );
 
+    /**
+     * Creates an empty \verbatim <Filter> \endverbatim QDomElement
+     * \returns valid \verbatim <Filter> \endverbatim QDomElement
+     */
+    static QDomElement filterElement(
+      QDomDocument &doc,
+      QgsOgcUtils::GMLVersion gmlVersion,
+      FilterVersion filterVersion,
+      bool GMLUsed );
+
     /**
      * Reads the \verbatim <gml:coordinates> \endverbatim element and extracts the coordinates as points
      * \param coords list where the found coordinates are appended
