@@ -96,7 +96,7 @@ QgsRelation QgsRelation::createFromXml( const QDomNode &node, QgsReadWriteContex
   relation.d->mReferencedLayer = qobject_cast<QgsVectorLayer *>( referencedLayer );
   relation.d->mRelationId = id;
   relation.d->mRelationName = name;
-  relation.d->mRelationStrength = qgsEnumKeyToValue<QgsRelation::RelationStrength>( strength, RelationStrength::Association );
+  relation.d->mRelationStrength = qgsEnumKeyToValue<Qgis::RelationshipStrength>( strength, Qgis::RelationshipStrength::Association );
 
   QDomNodeList references = elem.elementsByTagName( QStringLiteral( "fieldRef" ) );
   for ( int i = 0; i < references.size(); ++i )
@@ -121,7 +121,7 @@ void QgsRelation::writeXml( QDomNode &node, QDomDocument &doc ) const
   elem.setAttribute( QStringLiteral( "name" ), d->mRelationName );
   elem.setAttribute( QStringLiteral( "referencingLayer" ), d->mReferencingLayerId );
   elem.setAttribute( QStringLiteral( "referencedLayer" ), d->mReferencedLayerId );
-  elem.setAttribute( QStringLiteral( "strength" ), qgsEnumValueToKey<RelationStrength>( d->mRelationStrength ) );
+  elem.setAttribute( QStringLiteral( "strength" ), qgsEnumValueToKey<Qgis::RelationshipStrength>( d->mRelationStrength ) );
 
   for ( const FieldPair &pair : std::as_const( d->mFieldPairs ) )
   {
@@ -149,7 +149,7 @@ void QgsRelation::setName( const QString &name )
 }
 
 
-void QgsRelation::setStrength( RelationStrength strength )
+void QgsRelation::setStrength( Qgis::RelationshipStrength strength )
 {
   d.detach();
   d->mRelationStrength = strength;
@@ -221,7 +221,16 @@ QString QgsRelation::getRelatedFeaturesFilter( const QgsFeature &feature ) const
   for ( const FieldPair &pair : std::as_const( d->mFieldPairs ) )
   {
     QVariant val( feature.attribute( pair.referencedField() ) );
-    conditions << QgsExpression::createFieldEqualityExpression( pair.referencingField(), val );
+    int referencingIdx = referencingLayer()->fields().lookupField( pair.referencingField() );
+    if ( referencingIdx >= 0 )
+    {
+      QVariant::Type fieldType = referencingLayer()->fields().at( referencingIdx ).type();
+      conditions << QgsExpression::createFieldEqualityExpression( pair.referencingField(), val, fieldType );
+    }
+    else
+    {
+      conditions << QgsExpression::createFieldEqualityExpression( pair.referencingField(), val );
+    }
   }
 
   return conditions.join( QLatin1String( " AND " ) );
@@ -233,8 +242,17 @@ QgsFeatureRequest QgsRelation::getReferencedFeatureRequest( const QgsAttributes
 
   for ( const FieldPair &pair : std::as_const( d->mFieldPairs ) )
   {
+    int referencedIdx = referencedLayer()->fields().lookupField( pair.referencedField() );
     int referencingIdx = referencingLayer()->fields().lookupField( pair.referencingField() );
-    conditions << QgsExpression::createFieldEqualityExpression( pair.referencedField(), attributes.at( referencingIdx ) );
+    if ( referencedIdx >= 0 )
+    {
+      QVariant::Type fieldType = referencedLayer()->fields().at( referencedIdx ).type();
+      conditions << QgsExpression::createFieldEqualityExpression( pair.referencedField(), attributes.at( referencingIdx ), fieldType );
+    }
+    else
+    {
+      conditions << QgsExpression::createFieldEqualityExpression( pair.referencedField(), attributes.at( referencingIdx ) );
+    }
   }
 
   QgsFeatureRequest myRequest;
@@ -265,7 +283,7 @@ QString QgsRelation::name() const
   return d->mRelationName;
 }
 
-QgsRelation::RelationStrength QgsRelation::strength() const
+Qgis::RelationshipStrength QgsRelation::strength() const
 {
   return d->mRelationStrength;
 }
@@ -342,6 +360,33 @@ bool QgsRelation::isValid() const
   return d->mValid && !d->mReferencingLayer.isNull() && !d->mReferencedLayer.isNull() && d->mReferencingLayer.data()->isValid() && d->mReferencedLayer.data()->isValid();
 }
 
+QString QgsRelation::validationError() const
+{
+  if ( isValid() )
+    return QString();
+
+  if ( d->mReferencingLayer.isNull() )
+  {
+    if ( d->mReferencingLayerId.isEmpty() )
+      return QObject::tr( "Referencing layer not set" );
+    else
+      return QObject::tr( "Referencing layer %1 does not exist" ).arg( d->mReferencingLayerId );
+  }
+  else if ( !d->mReferencingLayer.data()->isValid() )
+    return QObject::tr( "Referencing layer %1 is not valid" ).arg( d->mReferencingLayerId );
+  else if ( d->mReferencedLayer.isNull() )
+  {
+    if ( d->mReferencedLayerId.isEmpty() )
+      return QObject::tr( "Referenced layer not set" );
+    else
+      return QObject::tr( "Referenced layer %1 does not exist" ).arg( d->mReferencedLayerId );
+  }
+  else if ( !d->mReferencedLayer.data()->isValid() )
+    return QObject::tr( "Referenced layer %1 is not valid" ).arg( d->mReferencedLayerId );
+  else
+    return d->mValidationError;
+}
+
 bool QgsRelation::hasEqualDefinition( const QgsRelation &other ) const
 {
   return d->mReferencedLayerId == other.d->mReferencedLayerId && d->mReferencingLayerId == other.d->mReferencingLayerId && d->mFieldPairs == other.d->mFieldPairs;
@@ -379,6 +424,7 @@ void QgsRelation::updateRelationStatus()
   if ( d->mRelationId.isEmpty() )
   {
     QgsDebugMsg( QStringLiteral( "Invalid relation: no ID" ) );
+    d->mValidationError = QObject::tr( "Relationship has no ID" );
     d->mValid = false;
   }
   else
@@ -386,11 +432,13 @@ void QgsRelation::updateRelationStatus()
     if ( !d->mReferencedLayer )
     {
       QgsDebugMsgLevel( QStringLiteral( "Invalid relation: referenced layer does not exist. ID: %1" ).arg( d->mReferencedLayerId ), 4 );
+      d->mValidationError = QObject::tr( "Referenced layer %1 does not exist" ).arg( d->mReferencedLayerId );
       d->mValid = false;
     }
     else if ( !d->mReferencingLayer )
     {
       QgsDebugMsgLevel( QStringLiteral( "Invalid relation: referencing layer does not exist. ID: %2" ).arg( d->mReferencingLayerId ), 4 );
+      d->mValidationError = QObject::tr( "Referencing layer %1 does not exist" ).arg( d->mReferencingLayerId );
       d->mValid = false;
     }
     else
@@ -398,6 +446,7 @@ void QgsRelation::updateRelationStatus()
       if ( d->mFieldPairs.count() < 1 )
       {
         QgsDebugMsgLevel( QStringLiteral( "Invalid relation: no pair of field is specified." ), 4 );
+        d->mValidationError = QObject::tr( "No fields specified for relationship" );
         d->mValid = false;
       }
 
@@ -406,12 +455,14 @@ void QgsRelation::updateRelationStatus()
         if ( -1 == d->mReferencingLayer->fields().lookupField( pair.first ) )
         {
           QgsDebugMsg( QStringLiteral( "Invalid relation: field %1 does not exist in referencing layer %2" ).arg( pair.first, d->mReferencingLayer->name() ) );
+          d->mValidationError = QObject::tr( "Field %1 does not exist in referencing layer %2" ).arg( pair.first, d->mReferencingLayer->name() );
           d->mValid = false;
           break;
         }
         else if ( -1 == d->mReferencedLayer->fields().lookupField( pair.second ) )
         {
           QgsDebugMsg( QStringLiteral( "Invalid relation: field %1 does not exist in referenced layer %2" ).arg( pair.second, d->mReferencedLayer->name() ) );
+          d->mValidationError = QObject::tr( "Field %1 does not exist in referenced layer %2" ).arg( pair.second, d->mReferencedLayer->name() );
           d->mValid = false;
           break;
         }
@@ -440,10 +491,38 @@ QgsPolymorphicRelation QgsRelation::polymorphicRelation() const
   return mContext.project()->relationManager()->polymorphicRelation( d->mPolymorphicRelationId );
 }
 
-QgsRelation::RelationType QgsRelation::type() const
+Qgis::RelationshipType QgsRelation::type() const
 {
   if ( d->mPolymorphicRelationId.isNull() )
-    return QgsRelation::Normal;
+    return Qgis::RelationshipType::Normal;
   else
-    return QgsRelation::Generated;
+    return Qgis::RelationshipType::Generated;
+}
+
+QString QgsRelation::cardinalityToDisplayString( Qgis::RelationshipCardinality cardinality )
+{
+  switch ( cardinality )
+  {
+    case Qgis::RelationshipCardinality::OneToOne:
+      return QObject::tr( "One-to-one" );
+    case Qgis::RelationshipCardinality::OneToMany:
+      return QObject::tr( "One-to-many" );
+    case Qgis::RelationshipCardinality::ManyToOne:
+      return QObject::tr( "Many-to-one" );
+    case Qgis::RelationshipCardinality::ManyToMany:
+      return QObject::tr( "Many-to-many" );
+  }
+  BUILTIN_UNREACHABLE
+}
+
+QString QgsRelation::strengthToDisplayString( Qgis::RelationshipStrength strength )
+{
+  switch ( strength )
+  {
+    case Qgis::RelationshipStrength::Association:
+      return QObject::tr( "Association" );
+    case Qgis::RelationshipStrength::Composition:
+      return QObject::tr( "Composition" );
+  }
+  BUILTIN_UNREACHABLE
 }
