@@ -15,7 +15,6 @@
 
 #include "qgsrelationmanager.h"
 
-#include "qgsapplication.h"
 #include "qgslogger.h"
 #include "qgsproject.h"
 #include "qgsvectordataprovider.h"
@@ -27,8 +26,11 @@ QgsRelationManager::QgsRelationManager( QgsProject *project )
 {
   if ( project )
   {
+    // TODO: QGIS 4 remove: relations are now stored with the layer style
     connect( project, &QgsProject::readProjectWithContext, this, &QgsRelationManager::readProject );
+    // TODO: QGIS 4 remove: relations are now stored with the layer style
     connect( project, &QgsProject::writeProject, this, &QgsRelationManager::writeProject );
+
     connect( project, &QgsProject::layersRemoved, this, &QgsRelationManager::layersRemoved );
   }
 }
@@ -41,8 +43,7 @@ QgsRelationContext QgsRelationManager::context() const
 void QgsRelationManager::setRelations( const QList<QgsRelation> &relations )
 {
   mRelations.clear();
-  const auto constRelations = relations;
-  for ( const QgsRelation &rel : constRelations )
+  for ( const QgsRelation &rel : std::as_const( relations ) )
   {
     addRelation( rel );
   }
@@ -71,9 +72,9 @@ void QgsRelationManager::addRelation( const QgsRelation &relation )
 
 void QgsRelationManager::updateRelationsStatus()
 {
-  for ( auto relation : mRelations )
+  for ( auto relationIt = mRelations.begin(); relationIt != mRelations.end(); ++relationIt )
   {
-    relation.updateRelationStatus();
+    relationIt->updateRelationStatus();
   }
 }
 
@@ -99,8 +100,7 @@ QList<QgsRelation> QgsRelationManager::relationsByName( const QString &name ) co
 {
   QList<QgsRelation> relations;
 
-  const auto constMRelations = mRelations;
-  for ( const QgsRelation &rel : constMRelations )
+  for ( const QgsRelation &rel : std::as_const( mRelations ) )
   {
     if ( QString::compare( rel.name(), name, Qt::CaseInsensitive ) == 0 )
       relations << rel;
@@ -124,8 +124,7 @@ QList<QgsRelation> QgsRelationManager::referencingRelations( const QgsVectorLaye
 
   QList<QgsRelation> relations;
 
-  const auto constMRelations = mRelations;
-  for ( const QgsRelation &rel : constMRelations )
+  for ( const QgsRelation &rel : std::as_const( mRelations ) )
   {
     if ( rel.referencingLayer() == layer )
     {
@@ -154,7 +153,7 @@ QList<QgsRelation> QgsRelationManager::referencingRelations( const QgsVectorLaye
   return relations;
 }
 
-QList<QgsRelation> QgsRelationManager::referencedRelations( QgsVectorLayer *layer ) const
+QList<QgsRelation> QgsRelationManager::referencedRelations( const QgsVectorLayer *layer ) const
 {
   if ( !layer )
   {
@@ -163,8 +162,7 @@ QList<QgsRelation> QgsRelationManager::referencedRelations( QgsVectorLayer *laye
 
   QList<QgsRelation> relations;
 
-  const auto constMRelations = mRelations;
-  for ( const QgsRelation &rel : constMRelations )
+  for ( const QgsRelation &rel : std::as_const( mRelations ) )
   {
     if ( rel.referencedLayer() == layer )
     {
@@ -178,13 +176,14 @@ QList<QgsRelation> QgsRelationManager::referencedRelations( QgsVectorLayer *laye
 void QgsRelationManager::readProject( const QDomDocument &doc, QgsReadWriteContext &context )
 {
   mRelations.clear();
+  mPolymorphicRelations.clear();
 
-  QDomNodeList nodes = doc.elementsByTagName( QStringLiteral( "relations" ) );
-  if ( nodes.count() )
+  QDomNodeList relationNodes = doc.elementsByTagName( QStringLiteral( "relations" ) );
+  if ( relationNodes.count() )
   {
     QgsRelationContext relcontext( mProject );
 
-    QDomNode node = nodes.item( 0 );
+    QDomNode node = relationNodes.item( 0 );
     QDomNodeList relationNodes = node.childNodes();
     int relCount = relationNodes.count();
     for ( int i = 0; i < relCount; ++i )
@@ -197,6 +196,24 @@ void QgsRelationManager::readProject( const QDomDocument &doc, QgsReadWriteConte
     QgsDebugMsg( QStringLiteral( "No relations data present in this document" ) );
   }
 
+  QDomNodeList polymorphicRelationNodes = doc.elementsByTagName( QStringLiteral( "polymorphicRelations" ) );
+  if ( polymorphicRelationNodes.count() )
+  {
+    QgsRelationContext relcontext( mProject );
+
+    QDomNode node = polymorphicRelationNodes.item( 0 );
+    QDomNodeList relationNodes = node.childNodes();
+    int relCount = relationNodes.count();
+    for ( int i = 0; i < relCount; ++i )
+    {
+      addPolymorphicRelation( QgsPolymorphicRelation::createFromXml( relationNodes.at( i ), context, relcontext ) );
+    }
+  }
+  else
+  {
+    QgsDebugMsgLevel( QStringLiteral( "No polymorphic relations data present in this document" ), 3 );
+  }
+
   emit relationsLoaded();
   emit changed();
 }
@@ -214,18 +231,34 @@ void QgsRelationManager::writeProject( QDomDocument &doc )
   QDomElement relationsNode = doc.createElement( QStringLiteral( "relations" ) );
   qgisNode.appendChild( relationsNode );
 
-  const auto constMRelations = mRelations;
-  for ( const QgsRelation &relation : constMRelations )
+  for ( const QgsRelation &relation : std::as_const( mRelations ) )
   {
+    // the generated relations for polymorphic relations should be ignored,
+    // they are generated every time when a polymorphic relation is added
+    switch ( relation.type() )
+    {
+      case Qgis::RelationshipType::Generated:
+        continue;
+      case Qgis::RelationshipType::Normal:
+        break;
+    }
+
     relation.writeXml( relationsNode, doc );
   }
+
+  QDomElement polymorphicRelationsNode = doc.createElement( QStringLiteral( "polymorphicRelations" ) );
+  qgisNode.appendChild( polymorphicRelationsNode );
+
+  for ( const QgsPolymorphicRelation &relation : std::as_const( mPolymorphicRelations ) )
+  {
+    relation.writeXml( polymorphicRelationsNode, doc );
+  }
 }
 
 void QgsRelationManager::layersRemoved( const QStringList &layers )
 {
   bool relationsChanged = false;
-  const auto constLayers = layers;
-  for ( const QString &layer : constLayers )
+  for ( const QString &layer : std::as_const( layers ) )
   {
     QMapIterator<QString, QgsRelation> it( mRelations );
 
@@ -249,8 +282,7 @@ void QgsRelationManager::layersRemoved( const QStringList &layers )
 
 static bool hasRelationWithEqualDefinition( const QList<QgsRelation> &existingRelations, const QgsRelation &relation )
 {
-  const auto constExistingRelations = existingRelations;
-  for ( const QgsRelation &cur : constExistingRelations )
+  for ( const QgsRelation &cur : std::as_const( existingRelations ) )
   {
     if ( cur.hasEqualDefinition( relation ) ) return true;
   }
@@ -260,17 +292,60 @@ static bool hasRelationWithEqualDefinition( const QList<QgsRelation> &existingRe
 QList<QgsRelation> QgsRelationManager::discoverRelations( const QList<QgsRelation> &existingRelations, const QList<QgsVectorLayer *> &layers )
 {
   QList<QgsRelation> result;
-  const auto constLayers = layers;
-  for ( const QgsVectorLayer *layer : constLayers )
+  for ( const QgsVectorLayer *layer : std::as_const( layers ) )
   {
-    const auto constDiscoverRelations = layer->dataProvider()->discoverRelations( layer, layers );
-    for ( const QgsRelation &relation : constDiscoverRelations )
+    if ( const QgsVectorDataProvider *provider = layer->dataProvider() )
     {
-      if ( !hasRelationWithEqualDefinition( existingRelations, relation ) )
+      const auto constDiscoverRelations = provider->discoverRelations( layer, layers );
+      for ( const QgsRelation &relation : constDiscoverRelations )
       {
-        result.append( relation );
+        if ( !hasRelationWithEqualDefinition( existingRelations, relation ) )
+        {
+          result.append( relation );
+        }
       }
     }
   }
   return result;
 }
+
+QMap<QString, QgsPolymorphicRelation> QgsRelationManager::polymorphicRelations() const
+{
+  return mPolymorphicRelations;
+}
+
+QgsPolymorphicRelation QgsRelationManager::polymorphicRelation( const QString &polymorphicRelationId ) const
+{
+  return mPolymorphicRelations.value( polymorphicRelationId );
+}
+
+void QgsRelationManager::addPolymorphicRelation( const QgsPolymorphicRelation &polymorphicRelation )
+{
+  if ( !polymorphicRelation.referencingLayer() || polymorphicRelation.id().isNull() )
+    return;
+
+  mPolymorphicRelations.insert( polymorphicRelation.id(), polymorphicRelation );
+
+  const QList<QgsRelation> generatedRelations = polymorphicRelation.generateRelations();
+  for ( const QgsRelation &generatedRelation : generatedRelations )
+    addRelation( generatedRelation );
+}
+
+void QgsRelationManager::removePolymorphicRelation( const QString &polymorphicRelationId )
+{
+  QgsPolymorphicRelation relation = mPolymorphicRelations.take( polymorphicRelationId );
+
+  const QList<QgsRelation> generatedRelations = relation.generateRelations();
+  for ( const QgsRelation &generatedRelation : generatedRelations )
+    removeRelation( generatedRelation.id() );
+}
+
+void QgsRelationManager::setPolymorphicRelations( const QList<QgsPolymorphicRelation> &relations )
+{
+  const QList<QgsPolymorphicRelation> oldRelations = polymorphicRelations().values();
+  for ( const QgsPolymorphicRelation &oldRelation : oldRelations )
+    removePolymorphicRelation( oldRelation.id() );
+
+  for ( const QgsPolymorphicRelation &newRelation : relations )
+    addPolymorphicRelation( newRelation );
+}
