@@ -21,23 +21,25 @@
 #include <QRegularExpression>
 #include <cstdlib> // for std::abs
 
-QString QgsStringUtils::capitalize( const QString &string, QgsStringUtils::Capitalization capitalization )
+QString QgsStringUtils::capitalize( const QString &string, Qgis::Capitalization capitalization )
 {
   if ( string.isEmpty() )
     return QString();
 
   switch ( capitalization )
   {
-    case MixedCase:
+    case Qgis::Capitalization::MixedCase:
+    case Qgis::Capitalization::SmallCaps:
       return string;
 
-    case AllUppercase:
+    case Qgis::Capitalization::AllUppercase:
       return string.toUpper();
 
-    case AllLowercase:
+    case Qgis::Capitalization::AllLowercase:
+    case Qgis::Capitalization::AllSmallCaps:
       return string.toLower();
 
-    case ForceFirstLetterToCapital:
+    case Qgis::Capitalization::ForceFirstLetterToCapital:
     {
       QString temp = string;
 
@@ -58,7 +60,7 @@ QString QgsStringUtils::capitalize( const QString &string, QgsStringUtils::Capit
       return temp;
     }
 
-    case TitleCase:
+    case Qgis::Capitalization::TitleCase:
     {
       // yes, this is MASSIVELY simplifying the problem!!
 
@@ -103,8 +105,8 @@ QString QgsStringUtils::capitalize( const QString &string, QgsStringUtils::Capit
       return result;
     }
 
-    case UpperCamelCase:
-      QString result = QgsStringUtils::capitalize( string.toLower(), QgsStringUtils::ForceFirstLetterToCapital ).simplified();
+    case Qgis::Capitalization::UpperCamelCase:
+      QString result = QgsStringUtils::capitalize( string.toLower(), Qgis::Capitalization::ForceFirstLetterToCapital ).simplified();
       result.remove( ' ' );
       return result;
   }
@@ -523,9 +525,9 @@ QString QgsStringUtils::insertLinks( const QString &string, bool *foundLinks )
 
   // http://alanstorm.com/url_regex_explained
   // note - there's more robust implementations available
-  static thread_local QRegularExpression urlRegEx( "(\\b(([\\w-]+://?|www[.])[^\\s()<>]+(?:\\([\\w\\d]+\\)|([^!\"#$%&'()*+,\\-./:;<=>?@[\\\\\\]^_`{|}~\\s]|/))))" );
-  static thread_local QRegularExpression protoRegEx( "^(?:f|ht)tps?://|file://" );
-  static thread_local QRegularExpression emailRegEx( "([\\w._%+-]+@[\\w.-]+\\.[A-Za-z]+)" );
+  const thread_local QRegularExpression urlRegEx( QStringLiteral( "(\\b(([\\w-]+://?|www[.])[^\\s()<>]+(?:\\([\\w\\d]+\\)|([^!\"#$%&'()*+,\\-./:;<=>?@[\\\\\\]^_`{|}~\\s]|/))))" ) );
+  const thread_local QRegularExpression protoRegEx( QStringLiteral( "^(?:f|ht)tps?://|file://" ) );
+  const thread_local QRegularExpression emailRegEx( QStringLiteral( "([\\w._%+-]+@[\\w.-]+\\.[A-Za-z]+)" ) );
 
   int offset = 0;
   bool found = false;
@@ -565,7 +567,7 @@ QString QgsStringUtils::insertLinks( const QString &string, bool *foundLinks )
 
 bool QgsStringUtils::isUrl( const QString &string )
 {
-  const thread_local QRegularExpression rxUrl( "^(http|https|ftp|file)://\\S+$" );
+  const thread_local QRegularExpression rxUrl( QStringLiteral( "^(http|https|ftp|file)://\\S+$" ) );
   return rxUrl.match( string ).hasMatch();
 }
 
@@ -576,8 +578,10 @@ QString QgsStringUtils::htmlToMarkdown( const QString &html )
   converted.replace( QLatin1String( "<br>" ), QLatin1String( "\n" ) );
   converted.replace( QLatin1String( "<b>" ), QLatin1String( "**" ) );
   converted.replace( QLatin1String( "</b>" ), QLatin1String( "**" ) );
+  converted.replace( QLatin1String( "<pre>" ), QLatin1String( "\n```\n" ) );
+  converted.replace( QLatin1String( "</pre>" ), QLatin1String( "```\n" ) );
 
-  static thread_local QRegularExpression hrefRegEx( "<a\\s+href\\s*=\\s*([^<>]*)\\s*>([^<>]*)</a>" );
+  const thread_local QRegularExpression hrefRegEx( QStringLiteral( "<a\\s+href\\s*=\\s*([^<>]*)\\s*>([^<>]*)</a>" ) );
 
   int offset = 0;
   QRegularExpressionMatch match = hrefRegEx.match( converted );
@@ -669,7 +673,7 @@ QString QgsStringUtils::wordWrap( const QString &string, const int length, const
 #if QT_VERSION < QT_VERSION_CHECK(5, 15, 2)
         newstr.append( line.midRef( strCurrent ) );
 #else
-        newstr.append( QStringView {line}.mid( strCurrent ) );
+        newstr.append( QStringView {line} .mid( strCurrent ) );
 #endif
         strCurrent = strLength;
       }
@@ -760,7 +764,7 @@ QgsStringReplacement::QgsStringReplacement( const QString &match, const QString
 {
   if ( mWholeWordOnly )
   {
-    mRx.setPattern( QString( "\\b%1\\b" ).arg( mMatch ) );
+    mRx.setPattern( QStringLiteral( "\\b%1\\b" ).arg( mMatch ) );
     mRx.setPatternOptions( mCaseSensitive ? QRegularExpression::NoPatternOption : QRegularExpression::CaseInsensitiveOption );
   }
 }
@@ -783,8 +787,8 @@ QgsStringMap QgsStringReplacement::properties() const
   QgsStringMap map;
   map.insert( QStringLiteral( "match" ), mMatch );
   map.insert( QStringLiteral( "replace" ), mReplacement );
-  map.insert( QStringLiteral( "caseSensitive" ), mCaseSensitive ? "1" : "0" );
-  map.insert( QStringLiteral( "wholeWord" ), mWholeWordOnly ? "1" : "0" );
+  map.insert( QStringLiteral( "caseSensitive" ), mCaseSensitive ? QStringLiteral( "1" ) : QStringLiteral( "0" ) );
+  map.insert( QStringLiteral( "wholeWord" ), mWholeWordOnly ? QStringLiteral( "1" ) : QStringLiteral( "0" ) );
   return map;
 }
 
@@ -799,8 +803,7 @@ QgsStringReplacement QgsStringReplacement::fromProperties( const QgsStringMap &p
 QString QgsStringReplacementCollection::process( const QString &input ) const
 {
   QString result = input;
-  const auto constMReplacements = mReplacements;
-  for ( const QgsStringReplacement &r : constMReplacements )
+  for ( const QgsStringReplacement &r : mReplacements )
   {
     result = r.process( result );
   }
@@ -809,8 +812,7 @@ QString QgsStringReplacementCollection::process( const QString &input ) const
 
 void QgsStringReplacementCollection::writeXml( QDomElement &elem, QDomDocument &doc ) const
 {
-  const auto constMReplacements = mReplacements;
-  for ( const QgsStringReplacement &r : constMReplacements )
+  for ( const QgsStringReplacement &r : mReplacements )
   {
     QgsStringMap props = r.properties();
     QDomElement propEl = doc.createElement( QStringLiteral( "replacement" ) );
