@@ -20,18 +20,21 @@
 
 #include "zip.h"
 
+#include <zlib.h>
+
 #include "qgsmessagelog.h"
 #include "qgsziputils.h"
 #include "qgslogger.h"
 
 #include <iostream>
 
+
 bool QgsZipUtils::isZipFile( const QString &filename )
 {
   return QFileInfo( filename ).suffix().compare( QLatin1String( "qgz" ), Qt::CaseInsensitive ) == 0;
 }
 
-bool QgsZipUtils::unzip( const QString &zipFilename, const QString &dir, QStringList &files )
+bool QgsZipUtils::unzip( const QString &zipFilename, const QString &dir, QStringList &files, bool checkConsistency )
 {
   files.clear();
 
@@ -63,7 +66,7 @@ bool QgsZipUtils::unzip( const QString &zipFilename, const QString &dir, QString
 
   int rc = 0;
   const QByteArray fileNamePtr = zipFilename.toUtf8();
-  struct zip *z = zip_open( fileNamePtr.constData(), ZIP_CHECKCONS, &rc );
+  struct zip *z = zip_open( fileNamePtr.constData(), checkConsistency ? ZIP_CHECKCONS : 0, &rc );
 
   if ( rc == ZIP_ER_OK && z )
   {
@@ -188,3 +191,137 @@ bool QgsZipUtils::zip( const QString &zipFilename, const QStringList &files )
 
   return true;
 }
+
+bool QgsZipUtils::decodeGzip( const QByteArray &bytesIn, QByteArray &bytesOut )
+{
+  return decodeGzip( bytesIn.constData(), bytesIn.count(), bytesOut );
+}
+
+bool QgsZipUtils::decodeGzip( const char *bytesIn, std::size_t size, QByteArray &bytesOut )
+{
+  unsigned char *bytesInPtr = reinterpret_cast<unsigned char *>( const_cast<char *>( bytesIn ) );
+  uint bytesInLeft = static_cast<uint>( size );
+
+  const uint CHUNK = 16384;
+  unsigned char out[CHUNK];
+  const int DEC_MAGIC_NUM_FOR_GZIP = 16;
+
+  // allocate inflate state
+  z_stream strm;
+  strm.zalloc = Z_NULL;
+  strm.zfree = Z_NULL;
+  strm.opaque = Z_NULL;
+  strm.avail_in = 0;
+  strm.next_in = Z_NULL;
+
+  int ret = inflateInit2( &strm, MAX_WBITS + DEC_MAGIC_NUM_FOR_GZIP );
+  if ( ret != Z_OK )
+    return false;
+
+  while ( ret != Z_STREAM_END ) // done when inflate() says it's done
+  {
+    // prepare next chunk
+    const uint bytesToProcess = std::min( CHUNK, bytesInLeft );
+    strm.next_in = bytesInPtr;
+    strm.avail_in = bytesToProcess;
+    bytesInPtr += bytesToProcess;
+    bytesInLeft -= bytesToProcess;
+
+    if ( bytesToProcess == 0 )
+      break;  // we end with an error - no more data but inflate() wants more data
+
+    // run inflate() on input until output buffer not full
+    do
+    {
+      strm.avail_out = CHUNK;
+      strm.next_out = out;
+      ret = inflate( &strm, Z_NO_FLUSH );
+      Q_ASSERT( ret != Z_STREAM_ERROR ); // state not clobbered
+      if ( ret == Z_NEED_DICT || ret == Z_DATA_ERROR || ret == Z_MEM_ERROR )
+      {
+        inflateEnd( &strm );
+        return false;
+      }
+      const unsigned have = CHUNK - strm.avail_out;
+      bytesOut.append( QByteArray::fromRawData( reinterpret_cast<const char *>( out ), static_cast<int>( have ) ) );
+    }
+    while ( strm.avail_out == 0 );
+  }
+
+  inflateEnd( &strm );
+  return ret == Z_STREAM_END;
+}
+
+bool QgsZipUtils::encodeGzip( const QByteArray &bytesIn, QByteArray &bytesOut )
+{
+  unsigned char *bytesInPtr = reinterpret_cast<unsigned char *>( const_cast<char *>( bytesIn.constData() ) );
+  const uint bytesInLeft = static_cast<uint>( bytesIn.count() );
+
+  const uint CHUNK = 16384;
+  unsigned char out[CHUNK];
+  const int DEC_MAGIC_NUM_FOR_GZIP = 16;
+
+  // allocate deflate state
+  z_stream strm;
+  strm.zalloc = Z_NULL;
+  strm.zfree = Z_NULL;
+  strm.opaque = Z_NULL;
+
+  int ret = deflateInit2( &strm, Z_DEFAULT_COMPRESSION, Z_DEFLATED, MAX_WBITS + DEC_MAGIC_NUM_FOR_GZIP, 8, Z_DEFAULT_STRATEGY );
+  if ( ret != Z_OK )
+    return false;
+
+  strm.avail_in = bytesInLeft;
+  strm.next_in = bytesInPtr;
+
+  // run deflate() on input until output buffer not full, finish
+  // compression if all of source has been read in
+  do
+  {
+    strm.avail_out = CHUNK;
+    strm.next_out = out;
+    ret = deflate( &strm, Z_FINISH );  // no bad return value
+    Q_ASSERT( ret != Z_STREAM_ERROR ); // state not clobbered
+
+    const unsigned have = CHUNK - strm.avail_out;
+    bytesOut.append( QByteArray::fromRawData( reinterpret_cast<const char *>( out ), static_cast<int>( have ) ) );
+  }
+  while ( strm.avail_out == 0 );
+  Q_ASSERT( ret == Z_STREAM_END );      // stream will be complete
+
+  // clean up and return
+  deflateEnd( &strm );
+  return true;
+}
+
+const QStringList QgsZipUtils::files( const QString &zip )
+{
+  if ( zip.isEmpty() && !QFileInfo::exists( zip ) )
+  {
+    return QStringList();
+  }
+  QStringList files;
+
+  int rc = 0;
+  const QByteArray fileNamePtr = zip.toUtf8();
+  struct zip *z = zip_open( fileNamePtr.constData(), 0, &rc );
+
+  if ( rc == ZIP_ER_OK && z )
+  {
+    const int count = zip_get_num_files( z );
+    if ( count != -1 )
+    {
+      struct zip_stat stat;
+
+      for ( int i = 0; i < count; i++ )
+      {
+        zip_stat_index( z, i, 0, &stat );
+        files << QString( stat.name );
+      }
+    }
+
+    zip_close( z );
+  }
+
+  return files;
+}
