@@ -147,7 +147,7 @@ bool QgsPointDistanceRenderer::renderFeature( const QgsFeature &feature, QgsRend
   return true;
 }
 
-void QgsPointDistanceRenderer::drawGroup( const ClusteredGroup &group, QgsRenderContext &context )
+void QgsPointDistanceRenderer::drawGroup( const ClusteredGroup &group, QgsRenderContext &context ) const
 {
   //calculate centroid of all points, this will be center of group
   QgsMultiPoint *groupMultiPoint = new QgsMultiPoint();
@@ -300,6 +300,14 @@ QSet< QString > QgsPointDistanceRenderer::legendKeysForFeature( const QgsFeature
   return mRenderer->legendKeysForFeature( feature, context );
 }
 
+QString QgsPointDistanceRenderer::legendKeyToExpression( const QString &key, QgsVectorLayer *layer, bool &ok ) const
+{
+  ok = false;
+  if ( !mRenderer )
+    return QString();
+  return mRenderer->legendKeyToExpression( key, layer, ok );
+}
+
 bool QgsPointDistanceRenderer::willRenderFeature( const QgsFeature &feature, QgsRenderContext &context ) const
 {
   if ( !mRenderer )
@@ -406,7 +414,7 @@ QString QgsPointDistanceRenderer::getLabel( const QgsFeature &feature ) const
   return attribute;
 }
 
-void QgsPointDistanceRenderer::drawLabels( QPointF centerPoint, QgsSymbolRenderContext &context, const QList<QPointF> &labelShifts, const ClusteredGroup &group )
+void QgsPointDistanceRenderer::drawLabels( QPointF centerPoint, QgsSymbolRenderContext &context, const QList<QPointF> &labelShifts, const ClusteredGroup &group ) const
 {
   QPainter *p = context.renderContext().painter();
   if ( !p )
