@@ -26,17 +26,14 @@
 #include "qgssymbol.h"
 #include "qgssymbollayer.h"
 
-#include "qgslinesymbollayer.h"
-#include "qgsmarkersymbollayer.h"
-#include "qgsfillsymbollayer.h"
 #include "qgsgeometrygeneratorsymbollayer.h"
 #include "qgsmaptopixelgeometrysimplifier.h"
 #include "qgslogger.h"
 #include "qgsrendercontext.h" // for bigSymbolPreview
 #include "qgsproject.h"
+#include "qgsprojectstylesettings.h"
 #include "qgsstyle.h"
 #include "qgspainteffect.h"
-#include "qgseffectstack.h"
 #include "qgsvectorlayer.h"
 #include "qgsfeature.h"
 #include "qgsgeometry.h"
@@ -81,6 +78,165 @@ QgsSymbol::QgsSymbol( Qgis::SymbolType type, const QgsSymbolLayerList &layers )
 Q_NOWARN_DEPRECATED_POP
 
 QPolygonF QgsSymbol::_getLineString( QgsRenderContext &context, const QgsCurve &curve, bool clipToExtent )
+{
+  if ( curve.is3D() )
+    return _getLineString3d( context, curve, clipToExtent );
+  else
+    return _getLineString2d( context, curve, clipToExtent );
+}
+
+QPolygonF QgsSymbol::_getLineString3d( QgsRenderContext &context, const QgsCurve &curve, bool clipToExtent )
+{
+  const unsigned int nPoints = curve.numPoints();
+
+  QgsCoordinateTransform ct = context.coordinateTransform();
+  const QgsMapToPixel &mtp = context.mapToPixel();
+  QVector< double > pointsX;
+  QVector< double > pointsY;
+  QVector< double > pointsZ;
+
+  // apply clipping for large lines to achieve a better rendering performance
+  if ( clipToExtent && nPoints > 1 && !( context.flags() & Qgis::RenderContextFlag::ApplyClipAfterReprojection ) )
+  {
+    const QgsRectangle e = context.extent();
+    const double cw = e.width() / 10;
+    const double ch = e.height() / 10;
+    const QgsBox3d clipRect( e.xMinimum() - cw, e.yMinimum() - ch, -HUGE_VAL, e.xMaximum() + cw, e.yMaximum() + ch, HUGE_VAL ); // TODO also need to be clipped according to z axis
+
+    const QgsLineString *lineString = nullptr;
+    std::unique_ptr< QgsLineString > segmentized;
+    if ( const QgsLineString *ls = qgsgeometry_cast< const QgsLineString * >( &curve ) )
+    {
+      lineString = ls;
+    }
+    else
+    {
+      segmentized.reset( qgsgeometry_cast< QgsLineString * >( curve.segmentize( ) ) );
+      lineString = segmentized.get();
+    }
+
+    QgsClipper::clipped3dLine( lineString->xVector(), lineString->yVector(), lineString->zVector(), pointsX, pointsY, pointsZ, clipRect );
+  }
+  else
+  {
+    // clone...
+    if ( const QgsLineString *ls = qgsgeometry_cast<const QgsLineString *>( &curve ) )
+    {
+      pointsX = ls->xVector();
+      pointsY = ls->yVector();
+      pointsZ = ls->zVector();
+    }
+    else
+    {
+      std::unique_ptr< QgsLineString > segmentized;
+      segmentized.reset( qgsgeometry_cast< QgsLineString * >( curve.segmentize( ) ) );
+
+      pointsX = segmentized->xVector();
+      pointsY = segmentized->yVector();
+      pointsZ = segmentized->zVector();
+    }
+  }
+
+  // transform the points to screen coordinates
+  const QVector< double > preTransformPointsZ = pointsZ;
+  bool wasTransformed = false;
+  if ( ct.isValid() )
+  {
+    //create x, y arrays
+    const int nVertices = pointsX.size();
+    wasTransformed = true;
+
+    try
+    {
+      ct.transformCoords( nVertices, pointsX.data(), pointsY.data(), pointsZ.data(), Qgis::TransformDirection::Forward );
+    }
+    catch ( QgsCsException & )
+    {
+      // we don't abort the rendering here, instead we remove any invalid points and just plot those which ARE valid
+    }
+  }
+
+  // remove non-finite points, e.g. infinite or NaN points caused by reprojecting errors
+  {
+    const int size = pointsX.size();
+
+    const double *xIn = pointsX.data();
+    const double *yIn = pointsY.data();
+    const double *zIn = pointsZ.data();
+
+    const double *preTransformZIn = wasTransformed ? preTransformPointsZ.constData() : nullptr;
+
+    double *xOut = pointsX.data();
+    double *yOut = pointsY.data();
+    double *zOut = pointsZ.data();
+    int outSize = 0;
+    for ( int i = 0; i < size; ++i )
+    {
+      bool pointOk = std::isfinite( *xIn ) && std::isfinite( *yIn );
+
+      // skip z points which have been made non-finite during transformations only. Ie if:
+      // - we did no transformation, then always render even if non-finite z
+      // - we did transformation and z is finite then render
+      // - we did transformation and z is non-finite BUT input z was also non finite then render
+      // - we did transformation and z is non-finite AND input z WAS finite then skip
+      pointOk &= !wasTransformed || std::isfinite( *zIn ) || !std::isfinite( *preTransformZIn );
+
+      if ( pointOk )
+      {
+        *xOut++ = *xIn++;
+        *yOut++ = *yIn++;
+        *zOut++ = *zIn++;
+        outSize++;
+      }
+      else
+      {
+        xIn++;
+        yIn++;
+        zIn++;
+      }
+
+      if ( preTransformZIn )
+        preTransformZIn++;
+    }
+    pointsX.resize( outSize );
+    pointsY.resize( outSize );
+    pointsZ.resize( outSize );
+  }
+
+  if ( clipToExtent && nPoints > 1 && context.flags() & Qgis::RenderContextFlag::ApplyClipAfterReprojection )
+  {
+    // early clipping was not possible, so we have to apply it here after transformation
+    const QgsRectangle e = context.mapExtent();
+    const double cw = e.width() / 10;
+    const double ch = e.height() / 10;
+    const QgsBox3d clipRect( e.xMinimum() - cw, e.yMinimum() - ch, -HUGE_VAL, e.xMaximum() + cw, e.yMaximum() + ch, HUGE_VAL ); // TODO also need to be clipped according to z axis
+
+    QVector< double > tempX;
+    QVector< double > tempY;
+    QVector< double > tempZ;
+    QgsClipper::clipped3dLine( pointsX, pointsY, pointsZ, tempX, tempY, tempZ, clipRect );
+    pointsX = tempX;
+    pointsY = tempY;
+    pointsZ = tempZ;
+  }
+
+  const int polygonSize = pointsX.size();
+  QPolygonF out( polygonSize );
+  const double *x = pointsX.constData();
+  const double *y = pointsY.constData();
+  QPointF *dest = out.data();
+  for ( int i = 0; i < polygonSize; ++i )
+  {
+    double screenX = *x++;
+    double screenY = *y++;
+    mtp.transformInPlace( screenX, screenY );
+    *dest++ = QPointF( screenX, screenY );
+  }
+
+  return out;
+}
+
+QPolygonF QgsSymbol::_getLineString2d( QgsRenderContext &context, const QgsCurve &curve, bool clipToExtent )
 {
   const unsigned int nPoints = curve.numPoints();
 
@@ -88,7 +244,7 @@ QPolygonF QgsSymbol::_getLineString( QgsRenderContext &context, const QgsCurve &
   const QgsMapToPixel &mtp = context.mapToPixel();
   QPolygonF pts;
 
-  //apply clipping for large lines to achieve a better rendering performance
+  // apply clipping for large lines to achieve a better rendering performance
   if ( clipToExtent && nPoints > 1 && !( context.flags() & Qgis::RenderContextFlag::ApplyClipAfterReprojection ) )
   {
     const QgsRectangle e = context.extent();
@@ -102,7 +258,7 @@ QPolygonF QgsSymbol::_getLineString( QgsRenderContext &context, const QgsCurve &
     pts = curve.asQPolygonF();
   }
 
-  //transform the QPolygonF to screen coordinates
+  // transform the QPolygonF to screen coordinates
   if ( ct.isValid() )
   {
     try
@@ -141,7 +297,185 @@ QPolygonF QgsSymbol::_getLineString( QgsRenderContext &context, const QgsCurve &
   return pts;
 }
 
+
 QPolygonF QgsSymbol::_getPolygonRing( QgsRenderContext &context, const QgsCurve &curve, const bool clipToExtent, const bool isExteriorRing, const bool correctRingOrientation )
+{
+  if ( curve.is3D() )
+    return _getPolygonRing3d( context, curve, clipToExtent, isExteriorRing, correctRingOrientation );
+  else
+    return _getPolygonRing2d( context, curve, clipToExtent, isExteriorRing, correctRingOrientation );
+}
+
+QPolygonF QgsSymbol::_getPolygonRing3d( QgsRenderContext &context, const QgsCurve &curve, const bool clipToExtent, const bool isExteriorRing, const bool correctRingOrientation )
+{
+  const QgsCoordinateTransform ct = context.coordinateTransform();
+  const QgsMapToPixel &mtp = context.mapToPixel();
+
+  QVector< double > pointsX;
+  QVector< double > pointsY;
+  QVector< double > pointsZ;
+
+  if ( curve.numPoints() < 1 )
+    return QPolygonF();
+
+  bool reverseRing = false;
+  if ( correctRingOrientation )
+  {
+    // ensure consistent polygon ring orientation
+    if ( ( isExteriorRing && curve.orientation() != Qgis::AngularDirection::Clockwise ) || ( !isExteriorRing && curve.orientation() != Qgis::AngularDirection::CounterClockwise ) )
+    {
+      reverseRing = true;
+    }
+  }
+
+  //clip close to view extent, if needed
+  if ( clipToExtent && !( context.flags() & Qgis::RenderContextFlag::ApplyClipAfterReprojection ) && !context.extent().contains( curve.boundingBox() ) )
+  {
+    const QgsRectangle e = context.extent();
+    const double cw = e.width() / 10;
+    const double ch = e.height() / 10;
+    const QgsBox3d clipRect( e.xMinimum() - cw, e.yMinimum() - ch, -HUGE_VAL, e.xMaximum() + cw, e.yMaximum() + ch, HUGE_VAL ); // TODO also need to be clipped according to z axis
+
+    const QgsLineString *lineString = nullptr;
+    std::unique_ptr< QgsLineString > segmentized;
+    if ( const QgsLineString *ls = qgsgeometry_cast< const QgsLineString * >( &curve ) )
+    {
+      lineString = ls;
+    }
+    else
+    {
+      segmentized.reset( qgsgeometry_cast< QgsLineString * >( curve.segmentize( ) ) );
+      lineString = segmentized.get();
+    }
+
+    pointsX = lineString->xVector();
+    pointsY = lineString->yVector();
+    pointsZ = lineString->zVector();
+
+    QgsClipper::trimPolygon( pointsX, pointsY, pointsZ, clipRect );
+  }
+  else
+  {
+    // clone...
+    if ( const QgsLineString *ls = qgsgeometry_cast<const QgsLineString *>( &curve ) )
+    {
+      pointsX = ls->xVector();
+      pointsY = ls->yVector();
+      pointsZ = ls->zVector();
+    }
+    else
+    {
+      std::unique_ptr< QgsLineString > segmentized;
+      segmentized.reset( qgsgeometry_cast< QgsLineString * >( curve.segmentize( ) ) );
+
+      pointsX = segmentized->xVector();
+      pointsY = segmentized->yVector();
+      pointsZ = segmentized->zVector();
+    }
+  }
+
+  if ( reverseRing )
+  {
+    std::reverse( pointsX.begin(), pointsX.end() );
+    std::reverse( pointsY.begin(), pointsY.end() );
+    std::reverse( pointsZ.begin(), pointsZ.end() );
+  }
+
+  //transform the QPolygonF to screen coordinates
+  const QVector< double > preTransformPointsZ = pointsZ;
+  bool wasTransformed = false;
+  if ( ct.isValid() )
+  {
+    const int nVertices = pointsX.size();
+    wasTransformed = true;
+    try
+    {
+      ct.transformCoords( nVertices, pointsX.data(), pointsY.data(), pointsZ.data(), Qgis::TransformDirection::Forward );
+    }
+    catch ( QgsCsException & )
+    {
+      // we don't abort the rendering here, instead we remove any invalid points and just plot those which ARE valid
+    }
+  }
+
+  // remove non-finite points, e.g. infinite or NaN points caused by reprojecting errors
+  {
+    const int size = pointsX.size();
+
+    const double *xIn = pointsX.data();
+    const double *yIn = pointsY.data();
+    const double *zIn = pointsZ.data();
+
+    const double *preTransformZIn = wasTransformed ? preTransformPointsZ.constData() : nullptr;
+
+    double *xOut = pointsX.data();
+    double *yOut = pointsY.data();
+    double *zOut = pointsZ.data();
+    int outSize = 0;
+    for ( int i = 0; i < size; ++i )
+    {
+      bool pointOk = std::isfinite( *xIn ) && std::isfinite( *yIn );
+      // skip z points which have been made non-finite during transformations only. Ie if:
+      // - we did no transformation, then always render even if non-finite z
+      // - we did transformation and z is finite then render
+      // - we did transformation and z is non-finite BUT input z was also non finite then render
+      // - we did transformation and z is non-finite AND input z WAS finite then skip
+      pointOk &= !wasTransformed || std::isfinite( *zIn ) || !std::isfinite( *preTransformZIn );
+
+      if ( pointOk )
+      {
+        *xOut++ = *xIn++;
+        *yOut++ = *yIn++;
+        *zOut++ = *zIn++;
+        outSize++;
+      }
+      else
+      {
+        xIn++;
+        yIn++;
+        zIn++;
+      }
+
+      if ( preTransformZIn )
+        preTransformZIn++;
+    }
+    pointsX.resize( outSize );
+    pointsY.resize( outSize );
+    pointsZ.resize( outSize );
+  }
+
+  if ( clipToExtent && context.flags() & Qgis::RenderContextFlag::ApplyClipAfterReprojection && !context.mapExtent().contains( curve.boundingBox() ) )
+  {
+    // early clipping was not possible, so we have to apply it here after transformation
+    const QgsRectangle e = context.mapExtent();
+    const double cw = e.width() / 10;
+    const double ch = e.height() / 10;
+    const QgsBox3d clipRect( e.xMinimum() - cw, e.yMinimum() - ch, -HUGE_VAL, e.xMaximum() + cw, e.yMaximum() + ch, HUGE_VAL ); // TODO also need to be clipped according to z axis
+
+    QgsClipper::trimPolygon( pointsX, pointsY, pointsZ, clipRect );
+  }
+
+  const int polygonSize = pointsX.size();
+  QPolygonF out( polygonSize );
+  const double *x = pointsX.constData();
+  const double *y = pointsY.constData();
+  QPointF *dest = out.data();
+  for ( int i = 0; i < polygonSize; ++i )
+  {
+    double screenX = *x++;
+    double screenY = *y++;
+    mtp.transformInPlace( screenX, screenY );
+    *dest++ = QPointF( screenX, screenY );
+  }
+
+  if ( !out.empty() && !out.isClosed() )
+    out << out.at( 0 );
+
+  return out;
+}
+
+
+QPolygonF QgsSymbol::_getPolygonRing2d( QgsRenderContext &context, const QgsCurve &curve, const bool clipToExtent, const bool isExteriorRing, const bool correctRingOrientation )
 {
   const QgsCoordinateTransform ct = context.coordinateTransform();
   const QgsMapToPixel &mtp = context.mapToPixel();
@@ -154,9 +488,9 @@ QPolygonF QgsSymbol::_getPolygonRing( QgsRenderContext &context, const QgsCurve
   if ( correctRingOrientation )
   {
     // ensure consistent polygon ring orientation
-    if ( isExteriorRing && curve.orientation() != QgsCurve::Clockwise )
+    if ( isExteriorRing && curve.orientation() != Qgis::AngularDirection::Clockwise )
       std::reverse( poly.begin(), poly.end() );
-    else if ( !isExteriorRing && curve.orientation() != QgsCurve::CounterClockwise )
+    else if ( !isExteriorRing && curve.orientation() != Qgis::AngularDirection::CounterClockwise )
       std::reverse( poly.begin(), poly.end() );
   }
 
@@ -334,7 +668,7 @@ QgsMapUnitScale QgsSymbol::mapUnitScale() const
   return scale;
 }
 
-void QgsSymbol::setOutputUnit( QgsUnitTypes::RenderUnit u )
+void QgsSymbol::setOutputUnit( QgsUnitTypes::RenderUnit u ) const
 {
   const auto constMLayers = mLayers;
   for ( QgsSymbolLayer *layer : constMLayers )
@@ -343,7 +677,7 @@ void QgsSymbol::setOutputUnit( QgsUnitTypes::RenderUnit u )
   }
 }
 
-void QgsSymbol::setMapUnitScale( const QgsMapUnitScale &scale )
+void QgsSymbol::setMapUnitScale( const QgsMapUnitScale &scale ) const
 {
   const auto constMLayers = mLayers;
   for ( QgsSymbolLayer *layer : constMLayers )
@@ -352,28 +686,40 @@ void QgsSymbol::setMapUnitScale( const QgsMapUnitScale &scale )
   }
 }
 
+QgsSymbolAnimationSettings &QgsSymbol::animationSettings()
+{
+  return mAnimationSettings;
+}
+
+const QgsSymbolAnimationSettings &QgsSymbol::animationSettings() const
+{
+  return mAnimationSettings;
+}
+
+void QgsSymbol::setAnimationSettings( const QgsSymbolAnimationSettings &settings )
+{
+  mAnimationSettings = settings;
+}
+
 QgsSymbol *QgsSymbol::defaultSymbol( QgsWkbTypes::GeometryType geomType )
 {
   std::unique_ptr< QgsSymbol > s;
 
   // override global default if project has a default for this type
-  QString defaultSymbol;
   switch ( geomType )
   {
-    case QgsWkbTypes::PointGeometry :
-      defaultSymbol = QgsProject::instance()->readEntry( QStringLiteral( "DefaultStyles" ), QStringLiteral( "/Marker" ) );
+    case QgsWkbTypes::PointGeometry:
+      s.reset( QgsProject::instance()->styleSettings()->defaultSymbol( Qgis::SymbolType::Marker ) );
       break;
-    case QgsWkbTypes::LineGeometry :
-      defaultSymbol = QgsProject::instance()->readEntry( QStringLiteral( "DefaultStyles" ), QStringLiteral( "/Line" ) );
+    case QgsWkbTypes::LineGeometry:
+      s.reset( QgsProject::instance()->styleSettings()->defaultSymbol( Qgis::SymbolType::Line ) );
       break;
-    case QgsWkbTypes::PolygonGeometry :
-      defaultSymbol = QgsProject::instance()->readEntry( QStringLiteral( "DefaultStyles" ), QStringLiteral( "/Fill" ) );
+    case QgsWkbTypes::PolygonGeometry:
+      s.reset( QgsProject::instance()->styleSettings()->defaultSymbol( Qgis::SymbolType::Fill ) );
       break;
     default:
       break;
   }
-  if ( !defaultSymbol.isEmpty() )
-    s.reset( QgsStyle::defaultStyle()->symbol( defaultSymbol ) );
 
   // if no default found for this type, get global default (as previously)
   if ( !s )
@@ -396,20 +742,10 @@ QgsSymbol *QgsSymbol::defaultSymbol( QgsWkbTypes::GeometryType geomType )
   }
 
   // set opacity
-  double opacity = 1.0;
-  bool ok = false;
-  // upgrade old setting
-  double alpha = QgsProject::instance()->readDoubleEntry( QStringLiteral( "DefaultStyles" ), QStringLiteral( "/AlphaInt" ), 255, &ok );
-  if ( ok )
-    opacity = alpha / 255.0;
-  double newOpacity = QgsProject::instance()->readDoubleEntry( QStringLiteral( "DefaultStyles" ), QStringLiteral( "/Opacity" ), 1.0, &ok );
-  if ( ok )
-    opacity = newOpacity;
-  s->setOpacity( opacity );
+  s->setOpacity( QgsProject::instance()->styleSettings()->defaultSymbolOpacity() );
 
   // set random color, it project prefs allow
-  if ( defaultSymbol.isEmpty() ||
-       QgsProject::instance()->readBoolEntry( QStringLiteral( "DefaultStyles" ), QStringLiteral( "/RandomColors" ), true ) )
+  if ( QgsProject::instance()->styleSettings()->randomizeDefaultSymbolColor() )
   {
     s->setColor( QgsApplication::colorSchemeRegistry()->fetchRandomStyleColor() );
   }
@@ -498,6 +834,26 @@ void QgsSymbol::startRender( QgsRenderContext &context, const QgsFields &fields
   QgsSymbolRenderContext symbolContext( context, QgsUnitTypes::RenderUnknownUnit, mOpacity, false, mRenderHints, nullptr, fields );
 
   std::unique_ptr< QgsExpressionContextScope > scope( QgsExpressionContextUtils::updateSymbolScope( this, new QgsExpressionContextScope() ) );
+
+  if ( mAnimationSettings.isAnimated() )
+  {
+    const long long mapFrameNumber = context.currentFrame();
+    double animationTimeSeconds = 0;
+    if ( mapFrameNumber >= 0 && context.frameRate() > 0 )
+    {
+      // render is part of an animation, so we base the calculated frame on that
+      animationTimeSeconds = mapFrameNumber / context.frameRate();
+    }
+    else
+    {
+      // render is outside of animation, so base the calculated frame on the current epoch
+      animationTimeSeconds = QDateTime::currentMSecsSinceEpoch() / 1000.0;
+    }
+
+    const long long symbolFrame = static_cast< long long >( std::floor( animationTimeSeconds * mAnimationSettings.frameRate() ) );
+    scope->setVariable( QStringLiteral( "symbol_frame" ), symbolFrame, true );
+  }
+
   mSymbolRenderContext->setExpressionContextScope( scope.release() );
 
   mDataDefinedProperties.prepare( context.expressionContext() );
@@ -509,6 +865,7 @@ void QgsSymbol::startRender( QgsRenderContext &context, const QgsFields &fields
       continue;
 
     layer->prepareExpressions( symbolContext );
+    layer->prepareMasks( symbolContext );
     layer->startRender( symbolContext );
   }
 }
@@ -538,7 +895,7 @@ void QgsSymbol::stopRender( QgsRenderContext &context )
   Q_NOWARN_DEPRECATED_POP
 }
 
-void QgsSymbol::setColor( const QColor &color )
+void QgsSymbol::setColor( const QColor &color ) const
 {
   const auto constMLayers = mLayers;
   for ( QgsSymbolLayer *layer : constMLayers )
@@ -687,6 +1044,7 @@ QImage QgsSymbol::asImage( QSize size, QgsRenderContext *customContext )
 
   QPainter p( &image );
   p.setRenderHint( QPainter::Antialiasing );
+  p.setRenderHint( QPainter::SmoothPixmapTransform );
 
   drawPreviewIcon( &p, size, customContext );
 
@@ -712,6 +1070,9 @@ QImage QgsSymbol::bigSymbolPreviewImage( QgsExpressionContext *expressionContext
 
   QgsRenderContext context = QgsRenderContext::fromQPainter( &p );
   context.setFlag( Qgis::RenderContextFlag::RenderSymbolPreview );
+  context.setFlag( Qgis::RenderContextFlag::Antialiasing );
+  context.setFlag( Qgis::RenderContextFlag::HighQualityImageTransforms );
+  context.setPainterFlagsUsingContext( &p );
   if ( expressionContext )
     context.setExpressionContext( *expressionContext );
 
