@@ -91,6 +91,27 @@ class CORE_EXPORT QgsSymbolLayerUtils
     static QString encodeSldBrushStyle( Qt::BrushStyle style );
     static Qt::BrushStyle decodeSldBrushStyle( const QString &str );
 
+    /**
+     * Decodes a \a string representing a symbol coordinate reference mode.
+     *
+     * \param string string to decode
+     * \param ok will be set to TRUE if \a string was successfully decoded
+     * \returns decoded marker clip mode
+     *
+     * \see encodeCoordinateReference()
+     * \since QGIS 3.24
+     */
+    static Qgis::SymbolCoordinateReference decodeCoordinateReference( const QString &string, bool *ok SIP_OUT = nullptr );
+
+    /**
+     * Encodes a symbol coordinate reference mode to a string.
+     *
+     * \param coordinateReference coordinate reference mode
+     * \see decodeCoordinateReference()
+     * \since QGIS 3.24
+     */
+    static QString encodeCoordinateReference( Qgis::SymbolCoordinateReference coordinateReference );
+
     /**
      * Decodes a \a value representing an arrow head type.
      * \since QGIS 3.2
@@ -103,6 +124,46 @@ class CORE_EXPORT QgsSymbolLayerUtils
      */
     static QgsArrowSymbolLayer::ArrowType decodeArrowType( const QVariant &value, bool *ok SIP_OUT = nullptr );
 
+    /**
+     * Decodes a \a string representing a marker clip mode.
+     *
+     * \param string string to decode
+     * \param ok will be set to TRUE if \a string was successfully decoded
+     * \returns decoded marker clip mode
+     *
+     * \see encodeMarkerClipMode()
+     * \since QGIS 3.24
+     */
+    static Qgis::MarkerClipMode decodeMarkerClipMode( const QString &string, bool *ok SIP_OUT = nullptr );
+
+    /**
+     * Encodes a marker clip \a mode to a string.
+     *
+     * \see decodeMarkerClipMode()
+     * \since QGIS 3.24
+     */
+    static QString encodeMarkerClipMode( Qgis::MarkerClipMode mode );
+
+    /**
+     * Decodes a \a string representing a line clip mode.
+     *
+     * \param string string to decode
+     * \param ok will be set to TRUE if \a string was successfully decoded
+     * \returns decoded line clip mode
+     *
+     * \see encodeLineClipMode()
+     * \since QGIS 3.24
+     */
+    static Qgis::LineClipMode decodeLineClipMode( const QString &string, bool *ok SIP_OUT = nullptr );
+
+    /**
+     * Encodes a line clip \a mode to a string.
+     *
+     * \see decodeLineClipMode()
+     * \since QGIS 3.24
+     */
+    static QString encodeLineClipMode( Qgis::LineClipMode mode );
+
     /**
      * Encodes a QPointF to a string.
      * \see decodePoint()
@@ -262,10 +323,11 @@ class CORE_EXPORT QgsSymbolLayerUtils
      * \param size target size of preview icon
      * \param scale map unit scale for preview
      * \param parentSymbolType since QGIS 3.22, can be used to specify the parent symbol type so that geometry generator preview icons are correctly calculated
+     * \param mapLayer since QGIS 3.28, can be used to specify the associated map layer so that layer related expressions are correctly calculated
      * \returns icon containing symbol layer preview
      * \see symbolLayerPreviewPicture()
      */
-    static QIcon symbolLayerPreviewIcon( const QgsSymbolLayer *layer, QgsUnitTypes::RenderUnit u, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale(), Qgis::SymbolType parentSymbolType = Qgis::SymbolType::Hybrid );
+    static QIcon symbolLayerPreviewIcon( const QgsSymbolLayer *layer, QgsUnitTypes::RenderUnit u, QSize size, const QgsMapUnitScale &scale = QgsMapUnitScale(), Qgis::SymbolType parentSymbolType = Qgis::SymbolType::Hybrid, QgsMapLayer *mapLayer = nullptr );
 
     /**
      * Returns an icon preview for a color ramp.
@@ -804,6 +866,15 @@ class CORE_EXPORT QgsSymbolLayerUtils
      */
     static QSet<const QgsSymbolLayer *> toSymbolLayerPointers( QgsFeatureRenderer *renderer, const QSet<QgsSymbolLayerId> &symbolLayerIds );
 
+    /**
+     * Calculates the frame rate (in frames per second) at which the given \a renderer must be redrawn.
+     *
+     * Returns -1 if the \a renderer is not animated.
+     *
+     * \since QGIS 3.26
+     */
+    static double rendererFrameRate( const QgsFeatureRenderer *renderer );
+
     /**
      * \brief Creates a new symbol with size restricted to min/max size if original size is out of min/max range
      * \param s the original symbol
