@@ -16,6 +16,7 @@
 
 #include "qgsgeometry.h"
 #include "qgslogger.h"
+#include "qgsvectorlayereditbuffergroup.h"
 #include "qgsvectorlayerundocommand.h"
 #include "qgsvectordataprovider.h"
 #include "qgsvectorlayer.h"
@@ -52,8 +53,12 @@ bool QgsVectorLayerEditBuffer::isModified() const
 
 void QgsVectorLayerEditBuffer::undoIndexChanged( int index )
 {
+  if ( mBlockModifiedSignals )
+    return;
+
   QgsDebugMsgLevel( QStringLiteral( "undo index changed %1" ).arg( index ), 4 );
   Q_UNUSED( index )
+  L->triggerRepaint();
   emit layerModified();
 }
 
@@ -80,6 +85,15 @@ void QgsVectorLayerEditBuffer::updateFields( QgsFields &fields )
   }
 }
 
+QgsVectorLayerEditBufferGroup *QgsVectorLayerEditBuffer::editBufferGroup() const
+{
+  return mEditBufferGroup;
+}
+
+void QgsVectorLayerEditBuffer::setEditBufferGroup( QgsVectorLayerEditBufferGroup *editBufferGroup )
+{
+  mEditBufferGroup = editBufferGroup;
+}
 
 void QgsVectorLayerEditBuffer::updateFeatureGeometry( QgsFeature &f )
 {
@@ -139,13 +153,27 @@ bool QgsVectorLayerEditBuffer::addFeatures( QgsFeatureList &features )
   if ( !( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures ) )
     return false;
 
+  // we don't want to emit layerModified for every added feature, rather just once for the batch lot
+  mBlockModifiedSignals++;
+
   bool result = true;
+  bool anyAdded = false;
   for ( QgsFeatureList::iterator iter = features.begin(); iter != features.end(); ++iter )
   {
-    result = result && addFeature( *iter );
+    const bool thisFeatureResult = addFeature( *iter );
+    result = result && thisFeatureResult;
+    anyAdded |= thisFeatureResult;
+  }
+
+  mBlockModifiedSignals--;
+
+  if ( anyAdded )
+  {
+    emit layerModified();
+    L->triggerRepaint();
+    L->updateExtents();
   }
 
-  L->updateExtents();
   return result;
 }
 
@@ -188,11 +216,17 @@ bool QgsVectorLayerEditBuffer::deleteFeatures( const QgsFeatureIds &fids )
     return false;
   }
 
+  // we don't want to emit layerModified for every deleted feature, rather just once for the batch lot
+  mBlockModifiedSignals++;
+
   bool ok = true;
-  const auto constFids = fids;
-  for ( QgsFeatureId fid : constFids )
+  for ( QgsFeatureId fid : fids )
     ok = deleteFeature( fid ) && ok;
 
+  mBlockModifiedSignals--;
+  L->triggerRepaint();
+  emit layerModified();
+
   return ok;
 }
 
@@ -221,6 +255,10 @@ bool QgsVectorLayerEditBuffer::changeGeometry( QgsFeatureId fid, const QgsGeomet
 bool QgsVectorLayerEditBuffer::changeAttributeValues( QgsFeatureId fid, const QgsAttributeMap &newValues, const QgsAttributeMap &oldValues )
 {
   bool success = true;
+
+  // we don't want to emit layerModified for every changed attribute, rather just once for the batch lot
+  mBlockModifiedSignals++;
+
   for ( auto it = newValues.constBegin() ; it != newValues.constEnd(); ++it )
   {
     const int field = it.key();
@@ -233,6 +271,10 @@ bool QgsVectorLayerEditBuffer::changeAttributeValues( QgsFeatureId fid, const Qg
     success &= changeAttributeValue( fid, field, newValue, oldValue );
   }
 
+  mBlockModifiedSignals--;
+  L->triggerRepaint();
+  emit layerModified();
+
   return success;
 }
 
@@ -328,10 +370,8 @@ bool QgsVectorLayerEditBuffer::renameAttribute( int index, const QString &newNam
 
 bool QgsVectorLayerEditBuffer::commitChanges( QStringList &commitErrors )
 {
-  QgsVectorDataProvider *provider = L->dataProvider();
   commitErrors.clear();
 
-  int cap = provider->capabilities();
   bool success = true;
 
   // geometry updates   attribute updates
@@ -339,55 +379,12 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList &commitErrors )
   // no                 yes                   => changeAttributeValues
   // yes                yes                   => changeFeatures
 
+  // Check geometry types
   // to fix https://github.com/qgis/QGIS/issues/23663
   if ( !mAddedFeatures.isEmpty() )
-  {
-    if ( cap & QgsVectorDataProvider::AddFeatures )
-    {
-      if ( provider->doesStrictFeatureTypeCheck() )
-      {
-        for ( const QgsFeature &f : std::as_const( mAddedFeatures ) )
-        {
-          if ( ( ! f.hasGeometry() ) ||
-               ( f.geometry().wkbType() == provider->wkbType() ) )
-            continue;
+    success &= commitChangesCheckGeometryTypeCompatibility( commitErrors );
 
-          if ( provider->convertToProviderType( f.geometry() ).isNull() )
-          {
-            commitErrors << tr( "ERROR: %n feature(s) not added - geometry type is not compatible with the current layer.", "not added features count", mAddedFeatures.size() );
-            success = false;
-            break;
-          }
-        }
-      }
-    }
-    else
-    {
-      commitErrors << tr( "ERROR: %n feature(s) not added - provider doesn't support adding features.", "not added features count", mAddedFeatures.size() );
-      success = false;
-    }
-  }
-
-  //
-  // update geometries
-  //
-  if ( success && !mChangedGeometries.isEmpty() && ( ( cap & QgsVectorDataProvider::ChangeFeatures ) == 0 || mChangedAttributeValues.isEmpty() ) )
-  {
-    if ( provider->changeGeometryValues( mChangedGeometries ) )
-    {
-      commitErrors << tr( "SUCCESS: %n geometries were changed.", "changed geometries count", mChangedGeometries.size() );
-
-      emit committedGeometriesChanges( L->id(), mChangedGeometries );
-      mChangedGeometries.clear();
-    }
-    else
-    {
-      commitErrors << tr( "ERROR: %n geometries not changed.", "not changed geometries count", mChangedGeometries.size() );
-      success = false;
-    }
-  }
-
-  QgsFields oldFields = L->fields();
+  const QgsFields oldFields = L->fields();
 
   //
   // delete attributes
@@ -395,292 +392,63 @@ bool QgsVectorLayerEditBuffer::commitChanges( QStringList &commitErrors )
   bool attributesChanged = false;
   if ( success && !mDeletedAttributeIds.isEmpty() )
   {
-    if ( ( cap & QgsVectorDataProvider::DeleteAttributes ) && provider->deleteAttributes( qgis::listToSet( mDeletedAttributeIds ) ) )
-    {
-      commitErrors << tr( "SUCCESS: %n attribute(s) deleted.", "deleted attributes count", mDeletedAttributeIds.size() );
-
-      emit committedAttributesDeleted( L->id(), mDeletedAttributeIds );
-
-      mDeletedAttributeIds.clear();
-      attributesChanged = true;
-    }
-    else
-    {
-      commitErrors << tr( "ERROR: %n attribute(s) not deleted.", "not deleted attributes count", mDeletedAttributeIds.size() );
-#if 0
-      QString list = "ERROR: Pending attribute deletes:";
-      const auto constMDeletedAttributeIds = mDeletedAttributeIds;
-      for ( int idx : constMDeletedAttributeIds )
-      {
-        list.append( ' ' + L->fields().at( idx ).name() );
-      }
-      commitErrors << list;
-#endif
-      success = false;
-    }
+    bool attributesDeleted = false;
+    success &= commitChangesDeleteAttributes( attributesDeleted, commitErrors );
+    attributesChanged |= attributesDeleted;
   }
 
   // rename attributes
   if ( success && !mRenamedAttributes.isEmpty() )
   {
-    if ( ( cap & QgsVectorDataProvider::RenameAttributes ) && provider->renameAttributes( mRenamedAttributes ) )
-    {
-      commitErrors << tr( "SUCCESS: %n attribute(s) renamed.", "renamed attributes count", mRenamedAttributes.size() );
-
-      emit committedAttributesRenamed( L->id(), mRenamedAttributes );
-
-      mRenamedAttributes.clear();
-      attributesChanged = true;
-    }
-    else
-    {
-      commitErrors << tr( "ERROR: %n attribute(s) not renamed", "not renamed attributes count", mRenamedAttributes.size() );
-      success = false;
-    }
+    bool attributesRenamed = false;
+    success &= commitChangesRenameAttributes( attributesRenamed, commitErrors );
+    attributesChanged |= attributesRenamed;
   }
 
   //
   // add attributes
   //
-  if ( !mAddedAttributes.isEmpty() )
+  if ( success && !mAddedAttributes.isEmpty() )
   {
-    if ( ( cap & QgsVectorDataProvider::AddAttributes ) && provider->addAttributes( mAddedAttributes ) )
-    {
-      commitErrors << tr( "SUCCESS: %n attribute(s) added.", "added attributes count", mAddedAttributes.size() );
-
-      emit committedAttributesAdded( L->id(), mAddedAttributes );
-
-      mAddedAttributes.clear();
-      attributesChanged = true;
-    }
-    else
-    {
-      commitErrors << tr( "ERROR: %n new attribute(s) not added", "not added attributes count", mAddedAttributes.size() );
-#if 0
-      QString list = "ERROR: Pending adds:";
-      const auto constMAddedAttributes = mAddedAttributes;
-      for ( QgsField f : constMAddedAttributes )
-      {
-        list.append( ' ' + f.name() );
-      }
-      commitErrors << list;
-#endif
-      success = false;
-    }
+    bool attributesAdded = false;
+    success &= commitChangesAddAttributes( attributesAdded, commitErrors );
+    attributesChanged |= attributesAdded;
   }
 
   //
   // check that addition/removal went as expected
   //
-  bool attributeChangesOk = true;
   if ( success && attributesChanged )
+    success &= commitChangesCheckAttributesModifications( oldFields, commitErrors );
+
+  //
+  // change attributes
+  //
+  if ( success && ( !mChangedAttributeValues.isEmpty() || !mChangedGeometries.isEmpty() ) )
   {
-    L->updateFields();
-    QgsFields newFields = L->fields();
-
-    if ( oldFields.count() != newFields.count() )
-    {
-      commitErrors << tr( "ERROR: the count of fields is incorrect after addition/removal of fields!" );
-      attributeChangesOk = false;   // don't try attribute updates - they'll fail.
-    }
-
-    for ( int i = 0; i < std::min( oldFields.count(), newFields.count() ); ++i )
-    {
-      QgsField oldField = oldFields.at( i );
-      QgsField newField = newFields.at( i );
-      if ( attributeChangesOk && oldField != newField )
-      {
-        commitErrors
-            << tr( "ERROR: field with index %1 is not the same!" ).arg( i )
-            << tr( "Provider: %1" ).arg( L->providerType() )
-            << tr( "Storage: %1" ).arg( L->storageType() )
-            << QStringLiteral( "%1: name=%2 type=%3 typeName=%4 len=%5 precision=%6" )
-            .arg( tr( "expected field" ),
-                  oldField.name(),
-                  QVariant::typeToName( oldField.type() ),
-                  oldField.typeName() )
-            .arg( oldField.length() )
-            .arg( oldField.precision() )
-            << QStringLiteral( "%1: name=%2 type=%3 typeName=%4 len=%5 precision=%6" )
-            .arg( tr( "retrieved field" ),
-                  newField.name(),
-                  QVariant::typeToName( newField.type() ),
-                  newField.typeName() )
-            .arg( newField.length() )
-            .arg( newField.precision() );
-        attributeChangesOk = false;   // don't try attribute updates - they'll fail.
-      }
-    }
+    bool attributesChanged;
+    success &= commitChangesChangeAttributes( attributesChanged, commitErrors );
   }
 
-  if ( success && attributeChangesOk )
+  //
+  // delete features
+  //
+  if ( success && !mDeletedFeatureIds.isEmpty() )
   {
-    if ( cap & QgsVectorDataProvider::ChangeFeatures && !mChangedGeometries.isEmpty() && !mChangedAttributeValues.isEmpty() )
-    {
-      Q_ASSERT( ( cap & ( QgsVectorDataProvider::ChangeAttributeValues | QgsVectorDataProvider::ChangeGeometries ) ) == ( QgsVectorDataProvider::ChangeAttributeValues | QgsVectorDataProvider::ChangeGeometries ) );
-
-      if ( provider->changeFeatures( mChangedAttributeValues, mChangedGeometries ) )
-      {
-        commitErrors << tr( "SUCCESS: %1 attribute value(s) and %2 geometries changed." ).arg( mChangedAttributeValues.size(), mChangedGeometries.size() );
-        emit committedAttributeValuesChanges( L->id(), mChangedAttributeValues );
-        mChangedAttributeValues.clear();
-
-        emit committedGeometriesChanges( L->id(), mChangedGeometries );
-        mChangedGeometries.clear();
-      }
-      else
-      {
-        success = false;
-      }
-    }
-    else
-    {
-      //
-      // change attributes
-      //
-      if ( !mChangedAttributeValues.isEmpty() && ( ( cap & QgsVectorDataProvider::ChangeFeatures ) == 0 || mChangedGeometries.isEmpty() ) )
-      {
-        if ( ( cap & QgsVectorDataProvider::ChangeAttributeValues ) && provider->changeAttributeValues( mChangedAttributeValues ) )
-        {
-          commitErrors << tr( "SUCCESS: %n attribute value(s) changed.", "changed attribute values count", mChangedAttributeValues.size() );
-
-          emit committedAttributeValuesChanges( L->id(), mChangedAttributeValues );
-          mChangedAttributeValues.clear();
-        }
-        else
-        {
-          commitErrors << tr( "ERROR: %n attribute value change(s) not applied.", "not changed attribute values count", mChangedAttributeValues.size() );
-#if 0
-          QString list = "ERROR: pending changes:";
-          const auto constKeys = mChangedAttributeValues.keys();
-          for ( QgsFeatureId id : constKeys )
-          {
-            list.append( "\n  " + FID_TO_STRING( id ) + '[' );
-            const auto constKeys = mChangedAttributeValues[ id ].keys();
-            for ( int idx : constKeys )
-            {
-              list.append( QString( " %1:%2" ).arg( L->fields().at( idx ).name() ).arg( mChangedAttributeValues[id][idx].toString() ) );
-            }
-            list.append( " ]" );
-          }
-          commitErrors << list;
-#endif
-          success = false;
-        }
-      }
-    }
-
-    //
-    // delete features
-    //
-    if ( success && !mDeletedFeatureIds.isEmpty() )
-    {
-      if ( ( cap & QgsVectorDataProvider::DeleteFeatures ) && provider->deleteFeatures( mDeletedFeatureIds ) )
-      {
-        commitErrors << tr( "SUCCESS: %n feature(s) deleted.", "deleted features count", mDeletedFeatureIds.size() );
-        // TODO[MD]: we should not need this here
-        for ( QgsFeatureId id : std::as_const( mDeletedFeatureIds ) )
-        {
-          mChangedAttributeValues.remove( id );
-          mChangedGeometries.remove( id );
-        }
-
-        emit committedFeaturesRemoved( L->id(), mDeletedFeatureIds );
-
-        mDeletedFeatureIds.clear();
-      }
-      else
-      {
-        commitErrors << tr( "ERROR: %n feature(s) not deleted.", "not deleted features count", mDeletedFeatureIds.size() );
-#if 0
-        QString list = "ERROR: pending deletes:";
-        const auto constMDeletedFeatureIds = mDeletedFeatureIds;
-        for ( QgsFeatureId id : constMDeletedFeatureIds )
-        {
-          list.append( ' ' + FID_TO_STRING( id ) );
-        }
-        commitErrors << list;
-#endif
-        success = false;
-      }
-    }
-
-    //
-    //  add features
-    //
-    if ( success && !mAddedFeatures.isEmpty() )
-    {
-      if ( cap & QgsVectorDataProvider::AddFeatures )
-      {
-        QList<QgsFeatureId> ids;
-        QgsFeatureList featuresToAdd;
-        // get the list of added features in reversed order
-        // this will preserve the order how they have been added e.g. (-1, -2, -3) while in the map they are ordered (-3, -2, -1)
-        mapToReversedLists( mAddedFeatures, ids, featuresToAdd );
-
-        // we need to strip any extra attributes here -- e.g. virtual fields, which should
-        // not be sent to the data provider. Refs #18784
-        for ( int i = 0; i < featuresToAdd.count(); ++i )
-        {
-          QgsVectorLayerUtils::matchAttributesToFields( featuresToAdd[i], provider->fields() );
-        }
-
-        if ( provider->addFeatures( featuresToAdd, QgsFeatureSink::Flag::RollBackOnErrors ) )
-        {
-          commitErrors << tr( "SUCCESS: %n feature(s) added.", "added features count", featuresToAdd.size() );
-
-          emit committedFeaturesAdded( L->id(), featuresToAdd );
-
-          // notify everyone that the features with temporary ids were updated with permanent ids
-          for ( int i = 0; i < featuresToAdd.count(); ++i )
-          {
-            if ( featuresToAdd[i].id() != ids[i] )
-            {
-              //update selection
-              if ( L->mSelectedFeatureIds.contains( ids[i] ) )
-              {
-                L->mSelectedFeatureIds.remove( ids[i] );
-                L->mSelectedFeatureIds.insert( featuresToAdd[i].id() );
-              }
-              emit featureDeleted( ids[i] );
-              emit featureAdded( featuresToAdd[i].id() );
-            }
-          }
-
-          mAddedFeatures.clear();
-        }
-        else
-        {
-          commitErrors << tr( "ERROR: %n feature(s) not added.", "not added features count", mAddedFeatures.size() );
-#if 0
-          QString list = "ERROR: pending adds:";
-          const auto constMAddedFeatures = mAddedFeatures;
-          for ( QgsFeature f : constMAddedFeatures )
-          {
-            list.append( ' ' + FID_TO_STRING( f.id() ) + '[' );
-            for ( int i = 0; i < L->fields().size(); i++ )
-            {
-              list.append( QString( " %1:%2" ).arg( L->fields().at( i ).name() ).arg( f.attributes()[i].toString() ) );
-            }
-            list.append( " ]" );
-          }
-          commitErrors << list;
-#endif
-          success = false;
-        }
-      }
-      else
-      {
-        commitErrors << tr( "ERROR: %n feature(s) not added - provider doesn't support adding features.", "not added features count", mAddedFeatures.size() );
-        success = false;
-      }
-    }
-  }
-  else
-  {
-    success = false;
+    bool featuresDeleted;
+    success &= commitChangesDeleteFeatures( featuresDeleted, commitErrors );
   }
 
+  //
+  //  add features
+  //
+  if ( success && !mAddedFeatures.isEmpty() )
+  {
+    bool featuresAdded;
+    success &= commitChangesAddFeatures( featuresAdded, commitErrors );
+  }
+
+  QgsVectorDataProvider *provider = L->dataProvider();
   if ( !success && provider->hasErrors() )
   {
     commitErrors << tr( "\n  Provider errors:" );
@@ -831,3 +599,376 @@ void QgsVectorLayerEditBuffer::updateLayerFields()
 {
   L->updateFields();
 }
+
+bool QgsVectorLayerEditBuffer::commitChangesCheckGeometryTypeCompatibility( QStringList &commitErrors )
+{
+  if ( mAddedFeatures.isEmpty() )
+    return true;
+
+  if ( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures )
+  {
+    if ( L->dataProvider()->doesStrictFeatureTypeCheck() )
+    {
+      for ( const QgsFeature &f : std::as_const( mAddedFeatures ) )
+      {
+        if ( ( ! f.hasGeometry() ) ||
+             ( f.geometry().wkbType() == L->dataProvider()->wkbType() ) )
+          continue;
+
+        if ( L->dataProvider()->convertToProviderType( f.geometry() ).isNull() )
+        {
+          commitErrors << tr( "ERROR: %n feature(s) not added - geometry type is not compatible with the current layer.", "not added features count", mAddedFeatures.size() );
+          return false;
+        }
+      }
+    }
+  }
+  else
+  {
+    commitErrors << tr( "ERROR: %n feature(s) not added - provider doesn't support adding features.", "not added features count", mAddedFeatures.size() );
+    return false;
+  }
+
+  return true;
+}
+
+bool QgsVectorLayerEditBuffer::commitChangesDeleteAttributes( bool &attributesDeleted, QStringList &commitErrors )
+{
+  attributesDeleted = false;
+
+  if ( mDeletedAttributeIds.isEmpty() )
+    return true;
+
+  if ( ( L->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteAttributes ) && L->dataProvider()->deleteAttributes( qgis::listToSet( mDeletedAttributeIds ) ) )
+  {
+    commitErrors << tr( "SUCCESS: %n attribute(s) deleted.", "deleted attributes count", mDeletedAttributeIds.size() );
+
+    emit committedAttributesDeleted( L->id(), mDeletedAttributeIds );
+
+    mDeletedAttributeIds.clear();
+    attributesDeleted = true;
+  }
+  else
+  {
+    commitErrors << tr( "ERROR: %n attribute(s) not deleted.", "not deleted attributes count", mDeletedAttributeIds.size() );
+#if 0
+    QString list = "ERROR: Pending attribute deletes:";
+    const auto constMDeletedAttributeIds = mDeletedAttributeIds;
+    for ( int idx : constMDeletedAttributeIds )
+    {
+      list.append( ' ' + L->fields().at( idx ).name() );
+    }
+    commitErrors << list;
+#endif
+    return false;
+  }
+
+  return true;
+}
+
+bool QgsVectorLayerEditBuffer::commitChangesRenameAttributes( bool &attributesRenamed, QStringList &commitErrors )
+{
+  attributesRenamed = false;
+
+  if ( mRenamedAttributes.isEmpty() )
+    return true;
+
+  if ( ( L->dataProvider()->capabilities() & QgsVectorDataProvider::RenameAttributes ) && L->dataProvider()->renameAttributes( mRenamedAttributes ) )
+  {
+    commitErrors << tr( "SUCCESS: %n attribute(s) renamed.", "renamed attributes count", mRenamedAttributes.size() );
+
+    emit committedAttributesRenamed( L->id(), mRenamedAttributes );
+
+    mRenamedAttributes.clear();
+    attributesRenamed = true;
+  }
+  else
+  {
+    commitErrors << tr( "ERROR: %n attribute(s) not renamed", "not renamed attributes count", mRenamedAttributes.size() );
+    return false;
+  }
+
+  return true;
+}
+
+bool QgsVectorLayerEditBuffer::commitChangesAddAttributes( bool &attributesAdded, QStringList &commitErrors )
+{
+  attributesAdded = false;
+
+  if ( mAddedAttributes.isEmpty() )
+    return true;
+
+  if ( ( L->dataProvider()->capabilities()  & QgsVectorDataProvider::AddAttributes ) && L->dataProvider()->addAttributes( mAddedAttributes ) )
+  {
+    commitErrors << tr( "SUCCESS: %n attribute(s) added.", "added attributes count", mAddedAttributes.size() );
+    emit committedAttributesAdded( L->id(), mAddedAttributes );
+    mAddedAttributes.clear();
+    attributesAdded = true;
+  }
+  else
+  {
+    commitErrors << tr( "ERROR: %n new attribute(s) not added", "not added attributes count", mAddedAttributes.size() );
+#if 0
+    QString list = "ERROR: Pending adds:";
+    const auto constMAddedAttributes = mAddedAttributes;
+    for ( QgsField f : constMAddedAttributes )
+    {
+      list.append( ' ' + f.name() );
+    }
+    commitErrors << list;
+#endif
+    return false;
+  }
+
+  return true;
+}
+
+bool QgsVectorLayerEditBuffer::commitChangesCheckAttributesModifications( const QgsFields oldFields, QStringList &commitErrors )
+{
+  L->updateFields();
+  QgsFields newFields = L->fields();
+
+  if ( oldFields.count() != newFields.count() )
+  {
+    commitErrors << tr( "ERROR: the count of fields is incorrect after addition/removal of fields!" );
+    return false;   // don't try attribute updates - they'll fail.
+  }
+
+  for ( int i = 0; i < std::min( oldFields.count(), newFields.count() ); ++i )
+  {
+    QgsField oldField = oldFields.at( i );
+    QgsField newField = newFields.at( i );
+    if ( oldField != newField )
+    {
+      commitErrors
+          << tr( "ERROR: field with index %1 is not the same!" ).arg( i )
+          << tr( "Provider: %1" ).arg( L->providerType() )
+          << tr( "Storage: %1" ).arg( L->storageType() )
+          << QStringLiteral( "%1: name=%2 type=%3 typeName=%4 len=%5 precision=%6" )
+          .arg( tr( "expected field" ),
+                oldField.name(),
+                QVariant::typeToName( oldField.type() ),
+                oldField.typeName() )
+          .arg( oldField.length() )
+          .arg( oldField.precision() )
+          << QStringLiteral( "%1: name=%2 type=%3 typeName=%4 len=%5 precision=%6" )
+          .arg( tr( "retrieved field" ),
+                newField.name(),
+                QVariant::typeToName( newField.type() ),
+                newField.typeName() )
+          .arg( newField.length() )
+          .arg( newField.precision() );
+      return false;   // don't try attribute updates - they'll fail.
+    }
+  }
+
+  return true;
+}
+
+bool QgsVectorLayerEditBuffer::commitChangesChangeAttributes( bool &attributesChanged, QStringList &commitErrors )
+{
+  attributesChanged = false;
+
+  if ( L->dataProvider()->capabilities() & QgsVectorDataProvider::ChangeFeatures && !mChangedGeometries.isEmpty() && !mChangedAttributeValues.isEmpty() )
+  {
+    Q_ASSERT( ( L->dataProvider()->capabilities() & ( QgsVectorDataProvider::ChangeAttributeValues | QgsVectorDataProvider::ChangeGeometries ) ) == ( QgsVectorDataProvider::ChangeAttributeValues | QgsVectorDataProvider::ChangeGeometries ) );
+
+    if ( L->dataProvider()->changeFeatures( mChangedAttributeValues, mChangedGeometries ) )
+    {
+      commitErrors << tr( "SUCCESS: %1 attribute value(s) and %2 geometries changed." ).arg( mChangedAttributeValues.size(), mChangedGeometries.size() );
+      attributesChanged = true;
+      emit committedAttributeValuesChanges( L->id(), mChangedAttributeValues );
+      mChangedAttributeValues.clear();
+
+      emit committedGeometriesChanges( L->id(), mChangedGeometries );
+      mChangedGeometries.clear();
+    }
+    else
+    {
+      commitErrors << tr( "ERROR: %1 attributes and %2 geometries not changed.", "not changed attributes and geometries count" ).arg( mChangedAttributeValues.size() ).arg( mChangedGeometries.size() );
+      return false;
+    }
+  }
+
+  if ( !mChangedGeometries.isEmpty() )
+  {
+    if ( ! L->dataProvider()->capabilities().testFlag( QgsVectorDataProvider::ChangeFeatures )
+         && ! L->dataProvider()->capabilities().testFlag( QgsVectorDataProvider::ChangeGeometries ) )
+    {
+      commitErrors << tr( "ERROR: %1 geometries not changed. Data provider '%2' does not have ChangeFeatures or ChangeGeometries capabilities", "not changed geometries count" )
+                   .arg( mChangedGeometries.size() )
+                   .arg( L->dataProvider()->name() );
+      return false;
+    }
+
+    if ( L->dataProvider()->changeGeometryValues( mChangedGeometries ) )
+    {
+      commitErrors << tr( "SUCCESS: %n geometries were changed.", "changed geometries count", mChangedGeometries.size() );
+      attributesChanged = true;
+      emit committedGeometriesChanges( L->id(), mChangedGeometries );
+      mChangedGeometries.clear();
+    }
+    else
+    {
+      commitErrors << tr( "ERROR: %n geometries not changed.", "not changed geometries count", mChangedGeometries.size() );
+      return false;
+    }
+  }
+
+  if ( !mChangedAttributeValues.isEmpty() )
+  {
+    if ( ! L->dataProvider()->capabilities().testFlag( QgsVectorDataProvider::ChangeFeatures )
+         && ! L->dataProvider()->capabilities().testFlag( QgsVectorDataProvider::ChangeAttributeValues ) )
+    {
+      commitErrors << tr( "ERROR: %1 attribute value change(s) not applied. Data provider '%2' does not have ChangeFeatures or ChangeAttributeValues capabilities", "not changed attribute values count" )
+                   .arg( mChangedAttributeValues.size() )
+                   .arg( L->dataProvider()->name() );
+      return false;
+    }
+
+    if ( L->dataProvider()->changeAttributeValues( mChangedAttributeValues ) )
+    {
+      commitErrors << tr( "SUCCESS: %n attribute value(s) changed.", "changed attribute values count", mChangedAttributeValues.size() );
+      attributesChanged = true;
+      emit committedAttributeValuesChanges( L->id(), mChangedAttributeValues );
+      mChangedAttributeValues.clear();
+    }
+    else
+    {
+      commitErrors << tr( "ERROR: %n attribute value change(s) not applied.", "not changed attribute values count", mChangedAttributeValues.size() );
+#if 0
+      QString list = "ERROR: pending changes:";
+      const auto constKeys = mChangedAttributeValues.keys();
+      for ( QgsFeatureId id : constKeys )
+      {
+        list.append( "\n  " + FID_TO_STRING( id ) + '[' );
+        const auto constKeys = mChangedAttributeValues[ id ].keys();
+        for ( int idx : constKeys )
+        {
+          list.append( QString( " %1:%2" ).arg( L->fields().at( idx ).name() ).arg( mChangedAttributeValues[id][idx].toString() ) );
+        }
+        list.append( " ]" );
+      }
+      commitErrors << list;
+#endif
+      return false;
+    }
+  }
+
+  return true;
+}
+
+bool QgsVectorLayerEditBuffer::commitChangesDeleteFeatures( bool &featuresDeleted, QStringList &commitErrors )
+{
+  featuresDeleted = false;
+
+  if ( mDeletedFeatureIds.isEmpty() )
+    return true;
+
+  if ( ( L->dataProvider()->capabilities() & QgsVectorDataProvider::DeleteFeatures ) && L->dataProvider()->deleteFeatures( mDeletedFeatureIds ) )
+  {
+    commitErrors << tr( "SUCCESS: %n feature(s) deleted.", "deleted features count", mDeletedFeatureIds.size() );
+    featuresDeleted = true;
+    // TODO[MD]: we should not need this here
+    for ( QgsFeatureId id : std::as_const( mDeletedFeatureIds ) )
+    {
+      mChangedAttributeValues.remove( id );
+      mChangedGeometries.remove( id );
+    }
+
+    emit committedFeaturesRemoved( L->id(), mDeletedFeatureIds );
+
+    mDeletedFeatureIds.clear();
+  }
+  else
+  {
+    commitErrors << tr( "ERROR: %n feature(s) not deleted.", "not deleted features count", mDeletedFeatureIds.size() );
+#if 0
+    QString list = "ERROR: pending deletes:";
+    const auto constMDeletedFeatureIds = mDeletedFeatureIds;
+    for ( QgsFeatureId id : constMDeletedFeatureIds )
+    {
+      list.append( ' ' + FID_TO_STRING( id ) );
+    }
+    commitErrors << list;
+#endif
+    return false;
+  }
+
+  return true;
+}
+
+bool QgsVectorLayerEditBuffer::commitChangesAddFeatures( bool &featuresAdded, QStringList &commitErrors )
+{
+  featuresAdded = false;
+
+  if ( mAddedFeatures.isEmpty() )
+    return true;
+
+  if ( L->dataProvider()->capabilities() & QgsVectorDataProvider::AddFeatures )
+  {
+    QList<QgsFeatureId> ids;
+    QgsFeatureList featuresToAdd;
+    // get the list of added features in reversed order
+    // this will preserve the order how they have been added e.g. (-1, -2, -3) while in the map they are ordered (-3, -2, -1)
+    mapToReversedLists( mAddedFeatures, ids, featuresToAdd );
+
+    // we need to strip any extra attributes here -- e.g. virtual fields, which should
+    // not be sent to the data provider. Refs #18784
+    for ( int i = 0; i < featuresToAdd.count(); ++i )
+    {
+      QgsVectorLayerUtils::matchAttributesToFields( featuresToAdd[i], L->dataProvider()->fields() );
+    }
+
+    if ( L->dataProvider()->addFeatures( featuresToAdd, QgsFeatureSink::Flag::RollBackOnErrors ) )
+    {
+      commitErrors << tr( "SUCCESS: %n feature(s) added.", "added features count", featuresToAdd.size() );
+      featuresAdded = true;
+      emit committedFeaturesAdded( L->id(), featuresToAdd );
+
+      // notify everyone that the features with temporary ids were updated with permanent ids
+      for ( int i = 0; i < featuresToAdd.count(); ++i )
+      {
+        if ( featuresToAdd[i].id() != ids[i] )
+        {
+          //update selection
+          if ( L->mSelectedFeatureIds.contains( ids[i] ) )
+          {
+            L->mSelectedFeatureIds.remove( ids[i] );
+            L->mSelectedFeatureIds.insert( featuresToAdd[i].id() );
+          }
+          emit featureDeleted( ids[i] );
+          emit featureAdded( featuresToAdd[i].id() );
+        }
+      }
+
+      mAddedFeatures.clear();
+    }
+    else
+    {
+      commitErrors << tr( "ERROR: %n feature(s) not added.", "not added features count", mAddedFeatures.size() );
+#if 0
+      QString list = "ERROR: pending adds:";
+      const auto constMAddedFeatures = mAddedFeatures;
+      for ( QgsFeature f : constMAddedFeatures )
+      {
+        list.append( ' ' + FID_TO_STRING( f.id() ) + '[' );
+        for ( int i = 0; i < L->fields().size(); i++ )
+        {
+          list.append( QString( " %1:%2" ).arg( L->fields().at( i ).name() ).arg( f.attributes()[i].toString() ) );
+        }
+        list.append( " ]" );
+      }
+      commitErrors << list;
+#endif
+      return false;
+    }
+  }
+  else
+  {
+    commitErrors << tr( "ERROR: %n feature(s) not added - provider doesn't support adding features.", "not added features count", mAddedFeatures.size() );
+    return false;
+  }
+
+  return true;
+}
