@@ -43,6 +43,9 @@
 #include "qgsexpressioncontextutils.h"
 #include "qgsshortcutsmanager.h"
 #include "qgsfieldconditionalformatwidget.h"
+#include "qgsmapcanvasutils.h"
+#include "qgsmessagebar.h"
+#include "qgsvectorlayereditbuffer.h"
 
 
 QgsDualView::QgsDualView( QWidget *parent )
@@ -64,7 +67,7 @@ QgsDualView::QgsDualView( QWidget *parent )
   mConditionalFormatWidgetStack->setMainPanel( mConditionalFormatWidget );
   mConditionalFormatWidget->setDockMode( true );
 
-  QgsSettings settings;
+  const QgsSettings settings;
   mConditionalSplitter->restoreState( settings.value( QStringLiteral( "/qgis/attributeTable/splitterState" ), QByteArray() ).toByteArray() );
 
   mPreviewColumnsMenu = new QMenu( this );
@@ -100,11 +103,11 @@ QgsDualView::QgsDualView( QWidget *parent )
   buttonGroup->setExclusive( false );
   buttonGroup->addButton( mAutoPanButton, PanToFeature );
   buttonGroup->addButton( mAutoZoomButton, ZoomToFeature );
-  FeatureListBrowsingAction action = QgsSettings().enumValue( QStringLiteral( "/qgis/attributeTable/featureListBrowsingAction" ), NoAction );
+  const FeatureListBrowsingAction action = QgsSettings().enumValue( QStringLiteral( "/qgis/attributeTable/featureListBrowsingAction" ), NoAction );
   QAbstractButton *bt = buttonGroup->button( static_cast<int>( action ) );
   if ( bt )
     bt->setChecked( true );
-  connect( buttonGroup, qgis::overload< QAbstractButton *, bool >::of( &QButtonGroup::buttonToggled ), this, &QgsDualView::panZoomGroupButtonToggled );
+  connect( buttonGroup, qOverload< QAbstractButton *, bool >( &QButtonGroup::buttonToggled ), this, &QgsDualView::panZoomGroupButtonToggled );
   mFlashButton->setChecked( QgsSettings().value( QStringLiteral( "/qgis/attributeTable/featureListHighlightFeature" ), true ).toBool() );
   connect( mFlashButton, &QToolButton::clicked, this, &QgsDualView::flashButtonClicked );
 }
@@ -116,25 +119,60 @@ QgsDualView::~QgsDualView()
 }
 
 void QgsDualView::init( QgsVectorLayer *layer, QgsMapCanvas *mapCanvas, const QgsFeatureRequest &request,
-                        const QgsAttributeEditorContext &context, bool loadFeatures )
+                        const QgsAttributeEditorContext &context, bool loadFeatures, bool showFirstFeature )
 {
   if ( !layer )
     return;
 
+  delete mAttributeForm;
+  mAttributeForm = nullptr;
+
   mLayer = layer;
+
+  // Keep fields order in sync: force config reset
+  connect( mLayer, &QgsVectorLayer::updatedFields, this, [ = ]
+  {
+    mFilterModel->setAttributeTableConfig( attributeTableConfig(), /* force */ true );
+  } );
+
   mEditorContext = context;
 
-  initLayerCache( !( request.flags() & QgsFeatureRequest::NoGeometry ) || !request.filterRect().isNull() );
+  // create an empty form to find out if it needs geometry or not
+  const QgsAttributeForm emptyForm( mLayer, QgsFeature(), mEditorContext );
+
+  const bool needsGeometry = mLayer->conditionalStyles()->rulesNeedGeometry() ||
+                             !( request.flags() & QgsFeatureRequest::NoGeometry )
+                             || ( request.spatialFilterType() != Qgis::SpatialFilterType::NoFilter )
+                             || emptyForm.needsGeometry();
+
+  initLayerCache( needsGeometry );
   initModels( mapCanvas, request, loadFeatures );
 
   mConditionalFormatWidget->setLayer( mLayer );
 
   mTableView->setModel( mFilterModel );
   mFeatureListView->setModel( mFeatureListModel );
-  delete mAttributeForm;
-  mAttributeForm = new QgsAttributeForm( mLayer, mTempAttributeFormFeature, mEditorContext );
-  mTempAttributeFormFeature = QgsFeature();
-  if ( !context.parentContext() )
+
+  connect( mFilterModel, &QgsAttributeTableFilterModel::sortColumnChanged, this, &QgsDualView::onSortColumnChanged );
+
+  if ( mFeatureListPreviewButton->defaultAction() )
+    mFeatureListView->setDisplayExpression( mDisplayExpression );
+  else
+    columnBoxInit();
+
+  // This slows down load of the attribute table heaps and uses loads of memory.
+  //mTableView->resizeColumnsToContents();
+
+  if ( showFirstFeature && mFeatureListModel->rowCount( ) > 0 )
+    mFeatureListView->setEditSelection( QgsFeatureIds() << mFeatureListModel->data( mFeatureListModel->index( 0, 0 ), QgsFeatureListModel::Role::FeatureRole ).value<QgsFeature>().id() );
+}
+
+void QgsDualView::initAttributeForm( const QgsFeature &feature )
+{
+  Q_ASSERT( !mAttributeForm );
+
+  mAttributeForm = new QgsAttributeForm( mLayer, feature, mEditorContext );
+  if ( !mEditorContext.parentContext() )
   {
     mAttributeEditorScrollArea = new QgsScrollArea();
     mAttributeEditorScrollArea->setWidgetResizable( true );
@@ -146,36 +184,39 @@ void QgsDualView::init( QgsVectorLayer *layer, QgsMapCanvas *mapCanvas, const Qg
     mAttributeEditor->layout()->addWidget( mAttributeForm );
   }
 
+  // This is an arbitrary yet small value to fix issue GH #50181
+  // the default value is 0.
+  mAttributeForm->setMinimumWidth( 200 );
+
+  setAttributeTableConfig( mLayer->attributeTableConfig() );
+
   connect( mAttributeForm, &QgsAttributeForm::widgetValueChanged, this, &QgsDualView::featureFormAttributeChanged );
   connect( mAttributeForm, &QgsAttributeForm::modeChanged, this, &QgsDualView::formModeChanged );
   connect( mAttributeForm, &QgsAttributeForm::filterExpressionSet, this, &QgsDualView::filterExpressionSet );
+  connect( mAttributeForm, &QgsAttributeForm::flashFeatures, this, [ = ]( const QString & filter )
+  {
+    if ( QgsMapCanvas *canvas = mFilterModel->mapCanvas() )
+    {
+      QgsMapCanvasUtils::flashMatchingFeatures( canvas, mLayer, filter );
+    }
+  } );
+  connect( mAttributeForm, &QgsAttributeForm::zoomToFeatures, this, [ = ]( const QString & filter )
+  {
+    if ( QgsMapCanvas *canvas = mFilterModel->mapCanvas() )
+    {
+      QgsMapCanvasUtils::zoomToMatchingFeatures( canvas, mLayer, filter );
+    }
+  } );
+
   connect( mMasterModel, &QgsAttributeTableModel::modelChanged, mAttributeForm, &QgsAttributeForm::refreshFeature );
-  connect( mFilterModel, &QgsAttributeTableFilterModel::sortColumnChanged, this, &QgsDualView::onSortColumnChanged );
-
-  if ( mFeatureListPreviewButton->defaultAction() )
-    mFeatureListView->setDisplayExpression( mDisplayExpression );
-  else
-    columnBoxInit();
-
-  // This slows down load of the attribute table heaps and uses loads of memory.
-  //mTableView->resizeColumnsToContents();
 }
 
 void QgsDualView::columnBoxInit()
 {
   // load fields
-  QList<QgsField> fields = mLayer->fields().toList();
+  const QList<QgsField> fields = mLayer->fields().toList();
 
-  QString defaultField;
-
-  // default expression: saved value
-  QString displayExpression = mLayer->displayExpression();
-
-  if ( displayExpression.isEmpty() )
-  {
-    // ... there isn't really much to display
-    displayExpression = QStringLiteral( "'[Please define preview text]'" );
-  }
+  const QString defaultField;
 
   mFeatureListPreviewButton->addAction( mActionExpressionPreview );
   mFeatureListPreviewButton->addAction( mActionPreviewColumnsMenu );
@@ -183,15 +224,15 @@ void QgsDualView::columnBoxInit()
   const auto constFields = fields;
   for ( const QgsField &field : constFields )
   {
-    int fieldIndex = mLayer->fields().lookupField( field.name() );
+    const int fieldIndex = mLayer->fields().lookupField( field.name() );
     if ( fieldIndex == -1 )
       continue;
 
-    QString fieldName = field.name();
+    const QString fieldName = field.name();
     if ( QgsGui::editorWidgetRegistry()->findBest( mLayer, fieldName ).type() != QLatin1String( "Hidden" ) )
     {
-      QIcon icon = mLayer->fields().iconForField( fieldIndex );
-      QString text = mLayer->attributeDisplayName( fieldIndex );
+      const QIcon icon = mLayer->fields().iconForField( fieldIndex );
+      const QString text = mLayer->attributeDisplayName( fieldIndex );
 
       // Generate action for the preview popup button of the feature list
       QAction *previewAction = new QAction( icon, text, mFeatureListPreviewButton );
@@ -205,9 +246,31 @@ void QgsDualView::columnBoxInit()
     }
   }
 
-  QAction *sortByPreviewExpression = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "sort.svg" ) ), tr( "Sort by preview expression" ), this );
-  connect( sortByPreviewExpression, &QAction::triggered, this, &QgsDualView::sortByPreviewExpression );
-  mFeatureListPreviewButton->addAction( sortByPreviewExpression );
+  QMenu *sortMenu = new QMenu( this );
+  QAction *sortMenuAction = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "sort.svg" ) ), tr( "Sort…" ), this );
+  sortMenuAction->setMenu( sortMenu );
+
+  QAction *sortByPreviewExpressionAsc = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "sort.svg" ) ), tr( "By Preview Expression (ascending)" ), this );
+  connect( sortByPreviewExpressionAsc, &QAction::triggered, this, [ = ]()
+  {
+    mFeatureListModel->setSortByDisplayExpression( true, Qt::AscendingOrder );
+  } );
+  sortMenu->addAction( sortByPreviewExpressionAsc );
+  QAction *sortByPreviewExpressionDesc = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "sort-reverse.svg" ) ), tr( "By Preview Expression (descending)" ), this );
+  connect( sortByPreviewExpressionDesc, &QAction::triggered, this, [ = ]()
+  {
+    mFeatureListModel->setSortByDisplayExpression( true, Qt::DescendingOrder );
+  } );
+  sortMenu->addAction( sortByPreviewExpressionDesc );
+  QAction *sortByPreviewExpressionCustom = new QAction( QgsApplication::getThemeIcon( QStringLiteral( "mIconExpressionPreview.svg" ) ), tr( "By Custom Expression" ), this );
+  connect( sortByPreviewExpressionCustom, &QAction::triggered, this, [ = ]()
+  {
+    if ( modifySort() )
+      mFeatureListModel->setSortByDisplayExpression( false );
+  } );
+  sortMenu->addAction( sortByPreviewExpressionCustom );
+
+  mFeatureListPreviewButton->addAction( sortMenuAction );
 
   QAction *separator = new QAction( mFeatureListPreviewButton );
   separator->setSeparator( true );
@@ -217,9 +280,10 @@ void QgsDualView::columnBoxInit()
   // If there is no single field found as preview
   if ( !mFeatureListPreviewButton->defaultAction() )
   {
-    mFeatureListView->setDisplayExpression( displayExpression );
+    mFeatureListView->setDisplayExpression( mLayer->displayExpression() );
     mFeatureListPreviewButton->setDefaultAction( mActionExpressionPreview );
-    setDisplayExpression( mFeatureListView->displayExpression() );
+    const QString displayExpression = mFeatureListView->displayExpression();
+    setDisplayExpression( displayExpression.isEmpty() ? tr( "'[Please define preview text]'" ) : displayExpression );
   }
   else
   {
@@ -244,11 +308,19 @@ void QgsDualView::setFilterMode( QgsAttributeTableFilterModel::FilterMode filter
   {
     case QgsAttributeTableFilterModel::ShowVisible:
       disconnect( mFilterModel->mapCanvas(), &QgsMapCanvas::extentsChanged, this, &QgsDualView::extentChanged );
+      disconnect( mFilterModel, &QgsAttributeTableFilterModel::visibleReloaded, this, &QgsDualView::filterChanged );
       break;
 
     case QgsAttributeTableFilterModel::ShowAll:
-    case QgsAttributeTableFilterModel::ShowEdited:
     case QgsAttributeTableFilterModel::ShowFilteredList:
+      disconnect( mFilterModel, &QgsAttributeTableFilterModel::featuresFiltered, this, &QgsDualView::filterChanged );
+      disconnect( mFilterModel, &QgsAttributeTableFilterModel::filterError, this, &QgsDualView::filterError );
+      break;
+
+    case QgsAttributeTableFilterModel::ShowEdited:
+      disconnect( mFilterModel, &QgsAttributeTableFilterModel::featuresFiltered, this, &QgsDualView::filterChanged );
+      disconnect( mFilterModel, &QgsAttributeTableFilterModel::filterError, this, &QgsDualView::filterError );
+      disconnect( masterModel()->layer(), &QgsVectorLayer::layerModified, this, &QgsDualView::updateEditedAddedFeatures );
       break;
 
     case QgsAttributeTableFilterModel::ShowSelected:
@@ -257,11 +329,11 @@ void QgsDualView::setFilterMode( QgsAttributeTableFilterModel::FilterMode filter
   }
 
   QgsFeatureRequest r = mMasterModel->request();
-  bool needsGeometry = filterMode == QgsAttributeTableFilterModel::ShowVisible;
+  const bool needsGeometry = filterMode == QgsAttributeTableFilterModel::ShowVisible;
 
-  bool requiresTableReload = ( r.filterType() != QgsFeatureRequest::FilterNone || !r.filterRect().isNull() ) // previous request was subset
-                             || ( needsGeometry && r.flags() & QgsFeatureRequest::NoGeometry ) // no geometry for last request
-                             || ( mMasterModel->rowCount() == 0 ); // no features
+  const bool requiresTableReload = ( r.filterType() != QgsFeatureRequest::FilterNone || r.spatialFilterType() != Qgis::SpatialFilterType::NoFilter ) // previous request was subset
+                                   || ( needsGeometry && r.flags() & QgsFeatureRequest::NoGeometry ) // no geometry for last request
+                                   || ( mMasterModel->rowCount() == 0 ); // no features
 
   if ( !needsGeometry )
     r.setFlags( r.flags() | QgsFeatureRequest::NoGeometry );
@@ -278,15 +350,29 @@ void QgsDualView::setFilterMode( QgsAttributeTableFilterModel::FilterMode filter
       connect( mFilterModel->mapCanvas(), &QgsMapCanvas::extentsChanged, this, &QgsDualView::extentChanged );
       if ( mFilterModel->mapCanvas() )
       {
-        QgsRectangle rect = mFilterModel->mapCanvas()->mapSettings().mapToLayerCoordinates( mLayer, mFilterModel->mapCanvas()->extent() );
+        const QgsRectangle rect = mFilterModel->mapCanvas()->mapSettings().mapToLayerCoordinates( mLayer, mFilterModel->mapCanvas()->extent() );
         r.setFilterRect( rect );
       }
+      connect( mFilterModel, &QgsAttributeTableFilterModel::visibleReloaded, this, &QgsDualView::filterChanged );
+      break;
+
+    case QgsAttributeTableFilterModel::ShowEdited:
+      r.setFilterFids( masterModel()->layer()->editBuffer() ? masterModel()->layer()->editBuffer()->allAddedOrEditedFeatures() : QgsFeatureIds() );
+      connect( mFilterModel, &QgsAttributeTableFilterModel::featuresFiltered, this, &QgsDualView::filterChanged );
+      connect( mFilterModel, &QgsAttributeTableFilterModel::filterError, this, &QgsDualView::filterError );
+      connect( masterModel()->layer(), &QgsVectorLayer::layerModified, this, &QgsDualView::updateEditedAddedFeatures );
       break;
 
     case QgsAttributeTableFilterModel::ShowAll:
-    case QgsAttributeTableFilterModel::ShowEdited:
     case QgsAttributeTableFilterModel::ShowFilteredList:
+    {
+      const QString filterExpression = filterMode == QgsAttributeTableFilterModel::ShowFilteredList ? mFilterModel->filterExpression() : QString();
+      if ( !filterExpression.isEmpty() )
+        r.setFilterExpression( mFilterModel->filterExpression() );
+      connect( mFilterModel, &QgsAttributeTableFilterModel::featuresFiltered, this, &QgsDualView::filterChanged );
+      connect( mFilterModel, &QgsAttributeTableFilterModel::filterError, this, &QgsDualView::filterError );
       break;
+    }
 
     case QgsAttributeTableFilterModel::ShowSelected:
       connect( masterModel()->layer(), &QgsVectorLayer::selectionChanged, this, &QgsDualView::updateSelectedFeatures );
@@ -294,14 +380,6 @@ void QgsDualView::setFilterMode( QgsAttributeTableFilterModel::FilterMode filter
       break;
   }
 
-  if ( requiresTableReload )
-  {
-    mMasterModel->setRequest( r );
-    whileBlocking( mLayerCache )->setCacheGeometry( needsGeometry );
-    mMasterModel->loadLayer();
-  }
-
-
   // disable the browsing auto pan/scale if the list only shows visible items
   switch ( filterMode )
   {
@@ -317,6 +395,16 @@ void QgsDualView::setFilterMode( QgsAttributeTableFilterModel::FilterMode filter
       break;
   }
 
+  if ( requiresTableReload )
+  {
+    //disconnect the connections of the current (old) filtermode before reload
+    mFilterModel->disconnectFilterModeConnections();
+
+    mMasterModel->setRequest( r );
+    whileBlocking( mLayerCache )->setCacheGeometry( needsGeometry );
+    mMasterModel->loadLayer();
+  }
+
   //update filter model
   mFilterModel->setFilterMode( filterMode );
   emit filterChanged();
@@ -330,8 +418,8 @@ void QgsDualView::setSelectedOnTop( bool selectedOnTop )
 void QgsDualView::initLayerCache( bool cacheGeometry )
 {
   // Initialize the cache
-  QgsSettings settings;
-  int cacheSize = settings.value( QStringLiteral( "qgis/attributeTableRowCache" ), "10000" ).toInt();
+  const QgsSettings settings;
+  const int cacheSize = settings.value( QStringLiteral( "qgis/attributeTableRowCache" ), "10000" ).toInt();
   mLayerCache = new QgsVectorLayerCache( mLayer, cacheSize, this );
   mLayerCache->setCacheGeometry( cacheGeometry );
   if ( 0 == cacheSize || 0 == ( QgsVectorDataProvider::SelectAtId & mLayer->dataProvider()->capabilities() ) )
@@ -387,7 +475,7 @@ void QgsDualView::saveRecentDisplayExpressions() const
   {
     return;
   }
-  QList<QAction *> actions = mFeatureListPreviewButton->actions();
+  const QList<QAction *> actions = mFeatureListPreviewButton->actions();
 
   // Remove existing same action
   int index = actions.indexOf( mLastDisplayExpressionAction );
@@ -412,10 +500,10 @@ void QgsDualView::setDisplayExpression( const QString &expression )
 
 void QgsDualView::insertRecentlyUsedDisplayExpression( const QString &expression )
 {
-  QList<QAction *> actions = mFeatureListPreviewButton->actions();
+  const QList<QAction *> actions = mFeatureListPreviewButton->actions();
 
   // Remove existing same action
-  int index = actions.indexOf( mLastDisplayExpressionAction );
+  const int index = actions.indexOf( mLastDisplayExpressionAction );
   if ( index != -1 )
   {
     for ( int i = 0; index + i < actions.length(); ++i )
@@ -441,7 +529,7 @@ void QgsDualView::insertRecentlyUsedDisplayExpression( const QString &expression
   {
     name = expression.mid( 11, expression.length() - 24 ); // Numbers calculated from the COALESCE / <NULL> parts
 
-    int fieldIndex = mLayer->fields().indexOf( name );
+    const int fieldIndex = mLayer->fields().indexOf( name );
     if ( fieldIndex != -1 )
     {
       name = mLayer->attributeDisplayName( fieldIndex );
@@ -473,6 +561,10 @@ void QgsDualView::updateEditSelectionProgress( int progress, int count )
   mNextFeatureButton->setEnabled( progress + 1 < count );
   mFirstFeatureButton->setEnabled( progress > 0 );
   mLastFeatureButton->setEnabled( progress + 1 < count );
+  if ( mAttributeForm )
+  {
+    mAttributeForm->setVisible( count > 0 );
+  }
 }
 
 void QgsDualView::panOrZoomToFeature( const QgsFeatureIds &featureset )
@@ -512,7 +604,7 @@ void QgsDualView::setBrowsingAutoPanScaleAllowed( bool allowed )
   mAutoPanButton->setEnabled( allowed );
   mAutoZoomButton->setEnabled( allowed );
 
-  QString disabledHint = tr( "(disabled when attribute table only shows features visible in the current map canvas extent)" );
+  const QString disabledHint = tr( "(disabled when attribute table only shows features visible in the current map canvas extent)" );
 
   mAutoPanButton->setToolTip( tr( "Automatically pan to the current feature" ) + ( allowed ? QString() : QString( ' ' ) + disabledHint ) );
   mAutoZoomButton->setToolTip( tr( "Automatically zoom to the current feature" ) + ( allowed ? QString() : QString( ' ' ) + disabledHint ) );
@@ -535,7 +627,7 @@ void QgsDualView::panZoomGroupButtonToggled( QAbstractButton *button, bool check
     QgsSettings().setEnumValue( QStringLiteral( "/qgis/attributeTable/featureListBrowsingAction" ), NoAction );
   }
 
-  if ( checked )
+  if ( checked && mLayer->isSpatial() )
     panOrZoomToFeature( mFeatureListView->currentEditSelection() );
 }
 
@@ -551,8 +643,19 @@ void QgsDualView::flashButtonClicked( bool clicked )
     canvas->flashFeatureIds( mLayer, mFeatureListView->currentEditSelection() );
 }
 
+void QgsDualView::filterError( const QString &errorMessage )
+{
+  if ( mEditorContext.mainMessageBar() )
+  {
+    mEditorContext.mainMessageBar()->pushWarning( tr( "An error occurred while filtering features" ), errorMessage );
+  }
+}
+
 void QgsDualView::featureListAboutToChangeEditSelection( bool &ok )
 {
+  if ( !mAttributeForm )
+    return;
+
   if ( mLayer->isEditable() && !mAttributeForm->save() )
     ok = false;
 }
@@ -561,7 +664,7 @@ void QgsDualView::featureListCurrentEditSelectionChanged( const QgsFeature &feat
 {
   if ( !mAttributeForm )
   {
-    mTempAttributeFormFeature = feat;
+    initAttributeForm( feat );
   }
   else if ( !mLayer->isEditable() || mAttributeForm->save() )
   {
@@ -570,7 +673,8 @@ void QgsDualView::featureListCurrentEditSelectionChanged( const QgsFeature &feat
     featureset << feat.id();
     setCurrentEditSelection( featureset );
 
-    panOrZoomToFeature( featureset );
+    if ( mLayer->isSpatial() )
+      panOrZoomToFeature( featureset );
 
   }
   else
@@ -587,7 +691,7 @@ void QgsDualView::setCurrentEditSelection( const QgsFeatureIds &fids )
 
 bool QgsDualView::saveEditChanges()
 {
-  return mAttributeForm->save();
+  return mAttributeForm ? mAttributeForm->save() : false;
 }
 
 void QgsDualView::openConditionalStyles()
@@ -597,29 +701,45 @@ void QgsDualView::openConditionalStyles()
 
 void QgsDualView::setMultiEditEnabled( bool enabled )
 {
+  if ( !mAttributeForm )
+    return;
+
   if ( enabled )
+  {
+    mPreviousView = view();
     setView( AttributeEditor );
+  }
+  else
+  {
+    setView( mPreviousView );
+  }
 
   mAttributeForm->setMode( enabled ? QgsAttributeEditorContext::MultiEditMode : QgsAttributeEditorContext::SingleEditMode );
 }
 
 void QgsDualView::toggleSearchMode( bool enabled )
 {
+  if ( !mAttributeForm )
+    return;
+
   if ( enabled )
   {
     setView( AttributeEditor );
     mAttributeForm->setMode( QgsAttributeEditorContext::SearchMode );
+    mAttributeForm->setVisible( true );
   }
   else
   {
     mAttributeForm->setMode( QgsAttributeEditorContext::SingleEditMode );
+    mAttributeForm->setVisible( mFilterModel->rowCount( ) > 0 );
   }
+
 }
 
 void QgsDualView::previewExpressionBuilder()
 {
   // Show expression builder
-  QgsExpressionContext context( QgsExpressionContextUtils::globalProjectLayerScopes( mLayer ) );
+  const QgsExpressionContext context( QgsExpressionContextUtils::globalProjectLayerScopes( mLayer ) );
 
   QgsExpressionBuilderDialog dlg( mLayer, mFeatureListView->displayExpression(), this, QStringLiteral( "generic" ), context );
   dlg.setWindowTitle( tr( "Expression Based Preview" ) );
@@ -673,7 +793,7 @@ void QgsDualView::copyCellContent() const
     return;
   }
 
-  QVariant var = mMasterModel->data( currentIndex, Qt::DisplayRole );
+  const QVariant var = mMasterModel->data( currentIndex, Qt::DisplayRole );
   QApplication::clipboard()->setText( var.toString() );
 }
 
@@ -683,6 +803,14 @@ void QgsDualView::cancelProgress()
     mProgressDlg->cancel();
 }
 
+void QgsDualView::parentFormValueChanged( const QString &attribute, const QVariant &newValue )
+{
+  if ( mAttributeForm )
+  {
+    mAttributeForm->parentFormValueChanged( attribute, newValue );
+  }
+}
+
 void QgsDualView::hideEvent( QHideEvent *event )
 {
   Q_UNUSED( event )
@@ -700,7 +828,7 @@ void QgsDualView::viewWillShowContextMenu( QMenu *menu, const QModelIndex &maste
   menu->addAction( copyContentAction );
   connect( copyContentAction, &QAction::triggered, this, [masterIndex, this]
   {
-    QVariant var = mMasterModel->data( masterIndex, Qt::DisplayRole );
+    const QVariant var = mMasterModel->data( masterIndex, Qt::DisplayRole );
     QApplication::clipboard()->setText( var.toString() );
   } );
 
@@ -737,7 +865,7 @@ void QgsDualView::viewWillShowContextMenu( QMenu *menu, const QModelIndex &maste
       menu->addAction( action.name(), a, &QgsAttributeTableAction::execute );
     }
   }
-  QModelIndex rowSourceIndex = mMasterModel->index( masterIndex.row(), 0 );
+  const QModelIndex rowSourceIndex = mMasterModel->index( masterIndex.row(), 0 );
   if ( ! rowSourceIndex.isValid() )
   {
     return;
@@ -777,8 +905,8 @@ void QgsDualView::viewWillShowContextMenu( QMenu *menu, const QModelIndex &maste
   }
 
   menu->addSeparator();
-  QgsAttributeTableAction *a = new QgsAttributeTableAction( tr( "Open Form" ), this, QString(), rowSourceIndex );
-  menu->addAction( tr( "Open Form" ), a, &QgsAttributeTableAction::featureForm );
+  QgsAttributeTableAction *a = new QgsAttributeTableAction( tr( "Open Form" ), this, QUuid(), rowSourceIndex );
+  menu->addAction( tr( "Open Form…" ), a, &QgsAttributeTableAction::featureForm );
 }
 
 
@@ -790,7 +918,7 @@ void QgsDualView::widgetWillShowContextMenu( QgsActionMenu *menu, const QModelIn
 
 void QgsDualView::showViewHeaderMenu( QPoint point )
 {
-  int col = mTableView->columnAt( point.x() );
+  const int col = mTableView->columnAt( point.x() );
 
   delete mHorizontalHeaderMenu;
   mHorizontalHeaderMenu = new QMenu( this );
@@ -803,17 +931,28 @@ void QgsDualView::showViewHeaderMenu( QPoint point )
   connect( setWidth, &QAction::triggered, this, &QgsDualView::resizeColumn );
   setWidth->setData( col );
   mHorizontalHeaderMenu->addAction( setWidth );
+
+  QAction *setWidthAllColumns = new QAction( tr( "&Set All Column Widths…" ), mHorizontalHeaderMenu );
+  connect( setWidthAllColumns, &QAction::triggered, this, &QgsDualView::resizeAllColumns );
+  setWidthAllColumns->setData( col );
+  mHorizontalHeaderMenu->addAction( setWidthAllColumns );
+
   QAction *optimizeWidth = new QAction( tr( "&Autosize" ), mHorizontalHeaderMenu );
   connect( optimizeWidth, &QAction::triggered, this, &QgsDualView::autosizeColumn );
   optimizeWidth->setData( col );
   mHorizontalHeaderMenu->addAction( optimizeWidth );
 
+  QAction *optimizeWidthAllColumns = new QAction( tr( "&Autosize All Columns" ), mHorizontalHeaderMenu );
+  connect( optimizeWidthAllColumns, &QAction::triggered, this, &QgsDualView::autosizeAllColumns );
+  mHorizontalHeaderMenu->addAction( optimizeWidthAllColumns );
+
+
   mHorizontalHeaderMenu->addSeparator();
   QAction *organize = new QAction( tr( "&Organize Columns…" ), mHorizontalHeaderMenu );
   connect( organize, &QAction::triggered, this, &QgsDualView::organizeColumns );
   mHorizontalHeaderMenu->addAction( organize );
   QAction *sort = new QAction( tr( "&Sort…" ), mHorizontalHeaderMenu );
-  connect( sort, &QAction::triggered, this, &QgsDualView::modifySort );
+  connect( sort, &QAction::triggered, this, [ = ]() {modifySort();} );
   mHorizontalHeaderMenu->addAction( sort );
 
   mHorizontalHeaderMenu->popup( mTableView->horizontalHeader()->mapToGlobal( point ) );
@@ -829,7 +968,7 @@ void QgsDualView::organizeColumns()
   QgsOrganizeTableColumnsDialog dialog( mLayer, attributeTableConfig(), this );
   if ( dialog.exec() == QDialog::Accepted )
   {
-    QgsAttributeTableConfig config = dialog.config();
+    const QgsAttributeTableConfig config = dialog.config();
     setAttributeTableConfig( config );
   }
 }
@@ -837,7 +976,7 @@ void QgsDualView::organizeColumns()
 void QgsDualView::tableColumnResized( int column, int width )
 {
   QgsAttributeTableConfig config = mConfig;
-  int sourceCol = config.mapVisibleColumnToIndex( column );
+  const int sourceCol = config.mapVisibleColumnToIndex( column );
   if ( sourceCol >= 0 && config.columnWidth( sourceCol ) != width )
   {
     config.setColumnWidth( sourceCol, width );
@@ -848,9 +987,9 @@ void QgsDualView::tableColumnResized( int column, int width )
 void QgsDualView::hideColumn()
 {
   QAction *action = qobject_cast<QAction *>( sender() );
-  int col = action->data().toInt();
+  const int col = action->data().toInt();
   QgsAttributeTableConfig config = mConfig;
-  int sourceCol = mConfig.mapVisibleColumnToIndex( col );
+  const int sourceCol = mConfig.mapVisibleColumnToIndex( col );
   if ( sourceCol >= 0 )
   {
     config.setColumnHidden( sourceCol, true );
@@ -861,18 +1000,18 @@ void QgsDualView::hideColumn()
 void QgsDualView::resizeColumn()
 {
   QAction *action = qobject_cast<QAction *>( sender() );
-  int col = action->data().toInt();
+  const int col = action->data().toInt();
   if ( col < 0 )
     return;
 
   QgsAttributeTableConfig config = mConfig;
-  int sourceCol = config.mapVisibleColumnToIndex( col );
+  const int sourceCol = config.mapVisibleColumnToIndex( col );
   if ( sourceCol >= 0 )
   {
     bool ok = false;
-    int width = QInputDialog::getInt( this, tr( "Set column width" ), tr( "Enter column width" ),
-                                      mTableView->columnWidth( col ),
-                                      0, 1000, 10, &ok );
+    const int width = QInputDialog::getInt( this, tr( "Set column width" ), tr( "Enter column width" ),
+                                            mTableView->columnWidth( col ),
+                                            0, 1000, 10, &ok );
     if ( ok )
     {
       config.setColumnWidth( sourceCol, width );
@@ -881,17 +1020,49 @@ void QgsDualView::resizeColumn()
   }
 }
 
+void QgsDualView::resizeAllColumns()
+{
+  QAction *action = qobject_cast<QAction *>( sender() );
+  const int col = action->data().toInt();
+  if ( col < 0 )
+    return;
+
+  QgsAttributeTableConfig config = mConfig;
+
+  bool ok = false;
+  const int width = QInputDialog::getInt( this, tr( "Set Column Width" ), tr( "Enter column width" ),
+                                          mTableView->columnWidth( col ),
+                                          1, 1000, 10, &ok );
+  if ( ok )
+  {
+    const int colCount = mTableView->model()->columnCount();
+    if ( colCount > 0 )
+    {
+      for ( int i = 0; i < colCount; i++ )
+      {
+        config.setColumnWidth( i, width );
+      }
+      setAttributeTableConfig( config );
+    }
+  }
+}
+
 void QgsDualView::autosizeColumn()
 {
   QAction *action = qobject_cast<QAction *>( sender() );
-  int col = action->data().toInt();
+  const int col = action->data().toInt();
   mTableView->resizeColumnToContents( col );
 }
 
-void QgsDualView::modifySort()
+void QgsDualView::autosizeAllColumns()
+{
+  mTableView->resizeColumnsToContents();
+}
+
+bool QgsDualView::modifySort()
 {
   if ( !mLayer )
-    return;
+    return false;
 
   QgsAttributeTableConfig config = mConfig;
 
@@ -911,11 +1082,9 @@ void QgsDualView::modifySort()
   sortingGroupBox->setLayout( new QGridLayout() );
 
   QgsExpressionBuilderWidget *expressionBuilder = new QgsExpressionBuilderWidget();
-  QgsExpressionContext context( QgsExpressionContextUtils::globalProjectLayerScopes( mLayer ) );
-  expressionBuilder->setExpressionContext( context );
-  expressionBuilder->setLayer( mLayer );
-  expressionBuilder->loadFieldNames();
-  expressionBuilder->loadRecent( QStringLiteral( "generic" ) );
+  const QgsExpressionContext context( QgsExpressionContextUtils::globalProjectLayerScopes( mLayer ) );
+
+  expressionBuilder->initWithLayer( mLayer, context, QStringLiteral( "generic" ) );
   expressionBuilder->setExpressionText( sortExpression().isEmpty() ? mLayer->displayExpression() : sortExpression() );
 
   sortingGroupBox->layout()->addWidget( expressionBuilder );
@@ -927,7 +1096,7 @@ void QgsDualView::modifySort()
   layout->addWidget( dialogButtonBox );
   if ( orderByDlg.exec() )
   {
-    Qt::SortOrder sortOrder = cbxSortAscending->isChecked() ? Qt::AscendingOrder : Qt::DescendingOrder;
+    const Qt::SortOrder sortOrder = cbxSortAscending->isChecked() ? Qt::AscendingOrder : Qt::DescendingOrder;
     if ( sortingGroupBox->isChecked() )
     {
       setSortExpression( expressionBuilder->expressionText(), sortOrder );
@@ -941,12 +1110,18 @@ void QgsDualView::modifySort()
     }
 
     setAttributeTableConfig( config );
+    return true;
   }
+  else
+  {
+    return false;
+  }
+
 }
 
 void QgsDualView::zoomToCurrentFeature()
 {
-  QModelIndex currentIndex = mTableView->currentIndex();
+  const QModelIndex currentIndex = mTableView->currentIndex();
   if ( !currentIndex.isValid() )
   {
     return;
@@ -963,7 +1138,7 @@ void QgsDualView::zoomToCurrentFeature()
 
 void QgsDualView::panToCurrentFeature()
 {
-  QModelIndex currentIndex = mTableView->currentIndex();
+  const QModelIndex currentIndex = mTableView->currentIndex();
   if ( !currentIndex.isValid() )
   {
     return;
@@ -980,7 +1155,7 @@ void QgsDualView::panToCurrentFeature()
 
 void QgsDualView::flashCurrentFeature()
 {
-  QModelIndex currentIndex = mTableView->currentIndex();
+  const QModelIndex currentIndex = mTableView->currentIndex();
   if ( !currentIndex.isValid() )
   {
     return;
@@ -1020,20 +1195,10 @@ void QgsDualView::onSortColumnChanged()
   }
 }
 
-void QgsDualView::sortByPreviewExpression()
-{
-  Qt::SortOrder sortOrder = Qt::AscendingOrder;
-  if ( mFeatureListView->displayExpression() == sortExpression() )
-  {
-    sortOrder = mConfig.sortOrder() == Qt::AscendingOrder ? Qt::DescendingOrder : Qt::AscendingOrder;
-  }
-  setSortExpression( mFeatureListView->displayExpression(), sortOrder );
-}
-
 void QgsDualView::updateSelectedFeatures()
 {
   QgsFeatureRequest r = mMasterModel->request();
-  if ( r.filterType() == QgsFeatureRequest::FilterNone && r.filterRect().isNull() )
+  if ( r.filterType() == QgsFeatureRequest::FilterNone && r.spatialFilterType() == Qgis::SpatialFilterType::NoFilter )
     return; // already requested all features
 
   r.setFilterFids( masterModel()->layer()->selectedFeatureIds() );
@@ -1042,12 +1207,24 @@ void QgsDualView::updateSelectedFeatures()
   emit filterChanged();
 }
 
+void QgsDualView::updateEditedAddedFeatures()
+{
+  QgsFeatureRequest r = mMasterModel->request();
+  if ( r.filterType() == QgsFeatureRequest::FilterNone && r.spatialFilterType() == Qgis::SpatialFilterType::NoFilter )
+    return; // already requested all features
+
+  r.setFilterFids( masterModel()->layer()->editBuffer() ? masterModel()->layer()->editBuffer()->allAddedOrEditedFeatures() : QgsFeatureIds() );
+  mMasterModel->setRequest( r );
+  mMasterModel->loadLayer();
+  emit filterChanged();
+}
+
 void QgsDualView::extentChanged()
 {
   QgsFeatureRequest r = mMasterModel->request();
   if ( mFilterModel->mapCanvas() && ( r.filterType() != QgsFeatureRequest::FilterNone || !r.filterRect().isNull() ) )
   {
-    QgsRectangle rect = mFilterModel->mapCanvas()->mapSettings().mapToLayerCoordinates( mLayer, mFilterModel->mapCanvas()->extent() );
+    const QgsRectangle rect = mFilterModel->mapCanvas()->mapSettings().mapToLayerCoordinates( mLayer, mFilterModel->mapCanvas()->extent() );
     r.setFilterRect( rect );
     mMasterModel->setRequest( r );
     mMasterModel->loadLayer();
@@ -1060,7 +1237,10 @@ void QgsDualView::featureFormAttributeChanged( const QString &attribute, const Q
   Q_UNUSED( attribute )
   Q_UNUSED( value )
   if ( attributeChanged )
+  {
     mFeatureListView->setCurrentFeatureEdited( true );
+    mAttributeForm->save();
+  }
 }
 
 void QgsDualView::setFilteredFeatures( const QgsFeatureIds &filteredFeatures )
@@ -1132,7 +1312,7 @@ void QgsDualView::progress( int i, bool &cancel )
     mProgressDlg->show();
   }
 
-  mProgressDlg->setLabelText( tr( "%1 features loaded." ).arg( i ) );
+  mProgressDlg->setLabelText( tr( "%L1 features loaded." ).arg( i ) );
   QCoreApplication::processEvents();
 
   cancel = mProgressDlg && mProgressDlg->wasCanceled();
