@@ -145,7 +145,10 @@ void QgsRelationReferenceWidgetWrapper::showIndeterminateState()
 
 QVariantList QgsRelationReferenceWidgetWrapper::additionalFieldValues() const
 {
-  if ( !mWidget || !mWidget->relation().isValid() )
+  if ( !mWidget )
+    return {};
+
+  if ( !mWidget->relation().isValid() )
   {
     QVariantList values;
     for ( int i = 0; i < mWidget->relation().fieldPairs().count(); i++ )
@@ -190,7 +193,7 @@ QStringList QgsRelationReferenceWidgetWrapper::additionalFields() const
 
 void QgsRelationReferenceWidgetWrapper::updateValues( const QVariant &val, const QVariantList &additionalValues )
 {
-  if ( !mWidget || ( !mIndeterminateState && val == value() && val.isNull() == value().isNull() ) )
+  if ( !mWidget || ( !mIndeterminateState && val == value() && QgsVariantUtils::isNull( val ) == QgsVariantUtils::isNull( value() ) ) )
     return;
 
   mIndeterminateState = false;
