@@ -86,7 +86,7 @@ void QgsLayerTreeView::setModel( QAbstractItemModel *model )
     return;
 
   if ( mMessageBar )
-    connect( treeModel, &QgsLayerTreeModel::messageEmitted,
+    connect( treeModel, &QgsLayerTreeModel::messageEmitted, this,
              [ = ]( const QString & message, Qgis::MessageLevel level = Qgis::MessageLevel::Info, int duration = 5 )
   {
     Q_UNUSED( duration )
@@ -178,9 +178,14 @@ void QgsLayerTreeView::contextMenuEvent( QContextMenuEvent *event )
     setCurrentIndex( QModelIndex() );
 
   QMenu *menu = mMenuProvider->createContextMenu();
-  if ( menu && menu->actions().count() != 0 )
-    menu->exec( mapToGlobal( event->pos() ) );
-  delete menu;
+  if ( menu )
+  {
+    emit contextMenuAboutToShow( menu );
+
+    if ( menu->actions().count() != 0 )
+      menu->exec( mapToGlobal( event->pos() ) );
+    delete menu;
+  }
 }
 
 
@@ -467,8 +472,10 @@ void QgsLayerTreeView::addIndicator( QgsLayerTreeNode *node, QgsLayerTreeViewInd
     connect( indicator, &QgsLayerTreeViewIndicator::changed, this, [ = ]
     {
       update();
+      viewport()->repaint();
     } );
     update();
+    viewport()->repaint(); //update() does not automatically trigger a repaint()
   }
 }
 
@@ -553,7 +560,7 @@ void QgsLayerTreeView::setMessageBar( QgsMessageBar *messageBar )
   mMessageBar = messageBar;
 
   if ( mMessageBar )
-    connect( layerTreeModel(), &QgsLayerTreeModel::messageEmitted,
+    connect( layerTreeModel(), &QgsLayerTreeModel::messageEmitted, this,
              [ = ]( const QString & message, Qgis::MessageLevel level = Qgis::MessageLevel::Info, int duration = 5 )
   {
     Q_UNUSED( duration )
