@@ -16,7 +16,6 @@
  ***************************************************************************/
 
 #include "qgslayoutview.h"
-#include "qgslayout.h"
 #include "qgslayoutframe.h"
 #include "qgslayoutmultiframe.h"
 #include "qgslayoutviewtool.h"
@@ -24,23 +23,22 @@
 #include "qgslayoutviewtooltemporarykeypan.h"
 #include "qgslayoutviewtooltemporarykeyzoom.h"
 #include "qgslayoutviewtooltemporarymousepan.h"
-#include "qgslayoutmousehandles.h"
 #include "qgslayoutruler.h"
 #include "qgslayoutmodel.h"
 #include "qgssettings.h"
 #include "qgsrectangle.h"
-#include "qgsapplication.h"
-#include "qgslayoutitemundocommand.h"
 #include "qgsproject.h"
 #include "qgslayoutitemgroup.h"
 #include "qgslayoutpagecollection.h"
 #include "qgslayoutundostack.h"
 #include "qgslayoutreportsectionlabel.h"
 #include "qgsreadwritecontext.h"
+#include "qgsscreenhelper.h"
+
 #include <memory>
-#include <QDesktopWidget>
 #include <QMenu>
 #include <QClipboard>
+#include <QMimeData>
 
 #define MIN_VIEW_SCALE 0.05
 #define MAX_VIEW_SCALE 1000.0
@@ -66,6 +64,8 @@ QgsLayoutView::QgsLayoutView( QWidget *parent )
   viewport()->setGraphicsEffect( mPreviewEffect );
 
   connect( this, &QgsLayoutView::zoomLevelChanged, this, &QgsLayoutView::invalidateCachedRenders );
+
+  mScreenHelper = new QgsScreenHelper( this );
 }
 
 QgsLayoutView::~QgsLayoutView()
@@ -189,7 +189,7 @@ void QgsLayoutView::scaleSafe( double scale )
 {
   double currentScale = transform().m11();
   scale *= currentScale;
-  scale = qBound( MIN_VIEW_SCALE, scale, MAX_VIEW_SCALE );
+  scale = std::clamp( scale, MIN_VIEW_SCALE, MAX_VIEW_SCALE );
   setTransform( QTransform::fromScale( scale, scale ) );
   emit zoomLevelChanged();
   viewChanged();
@@ -206,13 +206,13 @@ void QgsLayoutView::setZoomLevel( double level )
   }
   else
   {
-    double dpi = QgsApplication::desktop()->logicalDpiX();
+    double dpi = mScreenHelper->screenDpi();
     //monitor dpi is not always correct - so make sure the value is sane
     if ( ( dpi < 60 ) || ( dpi > 1200 ) )
       dpi = 72;
 
     //desired pixel width for 1mm on screen
-    level = qBound( MIN_VIEW_SCALE, level, MAX_VIEW_SCALE );
+    level = std::clamp( level, MIN_VIEW_SCALE, MAX_VIEW_SCALE );
     double mmLevel = currentLayout()->convertFromLayoutUnits( level, QgsUnitTypes::LayoutMillimeters ).length() * dpi / 25.4;
     setTransform( QTransform::fromScale( mmLevel, mmLevel ) );
   }
@@ -364,6 +364,7 @@ void QgsLayoutView::copyItems( const QList<QgsLayoutItem *> &items, QgsLayoutVie
     if ( itemNode.isElement() )
     {
       itemNode.toElement().removeAttribute( QStringLiteral( "uuid" ) );
+      itemNode.toElement().removeAttribute( QStringLiteral( "groupUuid" ) );
     }
   }
   QDomNodeList multiFrameNodes = doc.elementsByTagName( QStringLiteral( "LayoutMultiFrame" ) );
@@ -916,7 +917,7 @@ void QgsLayoutView::ungroupSelectedItems()
 
   if ( !ungroupedItems.empty() )
   {
-    for ( QgsLayoutItem *item : qgis::as_const( ungroupedItems ) )
+    for ( QgsLayoutItem *item : std::as_const( ungroupedItems ) )
     {
       item->setSelected( true );
     }
@@ -941,7 +942,7 @@ void QgsLayoutView::mousePressEvent( QMouseEvent *event )
 
   if ( !mTool || !event->isAccepted() )
   {
-    if ( event->button() == Qt::MidButton )
+    if ( event->button() == Qt::MiddleButton )
     {
       // Pan layout with middle mouse button
       setTool( mMidMouseButtonPanTool );
@@ -1161,7 +1162,7 @@ void QgsLayoutView::invalidateCachedRenders()
   QList< QgsLayoutItem *> items;
   currentLayout()->layoutItems( items );
 
-  for ( QgsLayoutItem *item : qgis::as_const( items ) )
+  for ( QgsLayoutItem *item : std::as_const( items ) )
   {
     item->invalidateCache();
   }
@@ -1223,7 +1224,11 @@ void QgsLayoutView::wheelZoom( QWheelEvent *event )
   QgsRectangle visibleRect = QgsRectangle( mapToScene( viewportRect ).boundingRect() );
 
   //transform the mouse pos to scene coordinates
+#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
   QPointF scenePoint = mapToScene( event->pos() );
+#else
+  QPointF scenePoint = mapToScene( event->position().x(), event->position().y() );
+#endif
 
   //adjust view center
   QgsPointXY oldCenter( visibleRect.center() );
@@ -1263,7 +1268,7 @@ QgsLayoutViewSnapMarker::QgsLayoutViewSnapMarker()
 {
   QFont f;
   QFontMetrics fm( f );
-  mSize = fm.width( QStringLiteral( "X" ) );
+  mSize = fm.horizontalAdvance( 'X' );
   setPen( QPen( Qt::transparent, mSize ) );
 
   setFlags( flags() | QGraphicsItem::ItemIgnoresTransformations );
