@@ -26,26 +26,22 @@
 #include "qgsmapcanvas.h"
 #include "qgsgui.h"
 #include "qgsapplication.h"
+#include "qgsmaplayerutils.h"
 #include <QMessageBox>
 #include <QFileDialog>
 #include <QTextCodec>
 #include <QSpinBox>
 #include <QRegularExpression>
+#include <limits>
 #include "gdal.h"
 #include "qgsdatums.h"
 #include "qgsiconutils.h"
 #include "qgsproviderregistry.h"
 #include "qgsprovidersublayerdetails.h"
 
-static const int COLUMN_IDX_NAME = 0;
-static const int COLUMN_IDX_TYPE = 1;
-static const int COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE = 2;
-
 QgsVectorLayerSaveAsDialog::QgsVectorLayerSaveAsDialog( long srsid, QWidget *parent, Qt::WindowFlags fl )
   : QDialog( parent, fl )
   , mSelectedCrs( QgsCoordinateReferenceSystem::fromSrsId( srsid ) )
-  , mAttributeTableItemChangedSlotEnabled( true )
-  , mReplaceRawFieldValuesStateChangedSlotEnabled( true )
   , mActionOnExistingFile( QgsVectorFileWriter::CreateOrOverwriteFile )
 {
   setup();
@@ -54,8 +50,6 @@ QgsVectorLayerSaveAsDialog::QgsVectorLayerSaveAsDialog( long srsid, QWidget *par
 QgsVectorLayerSaveAsDialog::QgsVectorLayerSaveAsDialog( QgsVectorLayer *layer, Options options, QWidget *parent, Qt::WindowFlags fl )
   : QDialog( parent, fl )
   , mLayer( layer )
-  , mAttributeTableItemChangedSlotEnabled( true )
-  , mReplaceRawFieldValuesStateChangedSlotEnabled( true )
   , mActionOnExistingFile( QgsVectorFileWriter::CreateOrOverwriteFile )
   , mOptions( options )
 {
@@ -66,6 +60,15 @@ QgsVectorLayerSaveAsDialog::QgsVectorLayerSaveAsDialog( QgsVectorLayer *layer, O
   }
   setup();
 
+  if ( layer )
+  {
+    mDefaultOutputLayerNameFromInputLayerName = QgsMapLayerUtils::launderLayerName( layer->name() );
+    leLayername->setDefaultValue( mDefaultOutputLayerNameFromInputLayerName );
+    leLayername->setClearMode( QgsFilterLineEdit::ClearToDefault );
+    if ( leLayername->isEnabled() )
+      leLayername->setText( mDefaultOutputLayerNameFromInputLayerName );
+  }
+
   if ( !( mOptions & Symbology ) )
   {
     mSymbologyExportLabel->hide();
@@ -115,6 +118,7 @@ void QgsVectorLayerSaveAsDialog::setup()
   connect( mGeometryTypeComboBox, static_cast<void ( QComboBox::* )( int )>( &QComboBox::currentIndexChanged ), this, &QgsVectorLayerSaveAsDialog::mGeometryTypeComboBox_currentIndexChanged );
   connect( mSelectAllAttributes, &QPushButton::clicked, this, &QgsVectorLayerSaveAsDialog::mSelectAllAttributes_clicked );
   connect( mDeselectAllAttributes, &QPushButton::clicked, this, &QgsVectorLayerSaveAsDialog::mDeselectAllAttributes_clicked );
+  connect( mUseAliasesForExportedName, &QCheckBox::stateChanged, this, &QgsVectorLayerSaveAsDialog::mUseAliasesForExportedName_stateChanged );
   connect( mReplaceRawFieldValues, &QCheckBox::stateChanged, this, &QgsVectorLayerSaveAsDialog::mReplaceRawFieldValues_stateChanged );
   connect( mAttributeTable, &QTableWidget::itemChanged, this, &QgsVectorLayerSaveAsDialog::mAttributeTable_itemChanged );
 
@@ -195,10 +199,16 @@ void QgsVectorLayerSaveAsDialog::setup()
     QgsSettings settings;
     QFileInfo tmplFileInfo( filePath );
     settings.setValue( QStringLiteral( "UI/lastVectorFileFilterDir" ), tmplFileInfo.absolutePath() );
-    if ( !filePath.isEmpty() && leLayername->isEnabled() )
+
+    const QFileInfo fileInfo( filePath );
+    const QString suggestedLayerName = QgsMapLayerUtils::launderLayerName( fileInfo.completeBaseName() );
+    if ( mDefaultOutputLayerNameFromInputLayerName.isEmpty() )
+      leLayername->setDefaultValue( suggestedLayerName );
+
+    // if no layer name set, then automatically match the output layer name to the file name
+    if ( leLayername->text().isEmpty() && !filePath.isEmpty() && leLayername->isEnabled() )
     {
-      QFileInfo fileInfo( filePath );
-      leLayername->setText( fileInfo.completeBaseName() );
+      leLayername->setText( suggestedLayerName );
     }
     mButtonBox->button( QDialogButtonBox::Ok )->setEnabled( !filePath.isEmpty() );
   } );
@@ -237,6 +247,7 @@ QList<QPair<QLabel *, QWidget *> > QgsVectorLayerSaveAsDialog::createControls( c
         {
           QSpinBox *sb = new QSpinBox();
           sb->setObjectName( it.key() );
+          sb->setMaximum( std::numeric_limits<int>::max() ); // the default is 99
           sb->setValue( opt->defaultValue );
           control = sb;
         }
@@ -483,6 +494,7 @@ void QgsVectorLayerSaveAsDialog::mFormatComboBox_currentIndexChanged( int idx )
                            sFormat == QLatin1String( "XLSX" ) ||
                            sFormat == QLatin1String( "ODS" ) ||
                            sFormat == QLatin1String( "FileGDB" ) ||
+                           sFormat == QLatin1String( "OpenFileGDB" ) ||
                            sFormat == QLatin1String( "SQLite" ) ||
                            sFormat == QLatin1String( "SpatiaLite" ) );
 
@@ -493,10 +505,16 @@ void QgsVectorLayerSaveAsDialog::mFormatComboBox_currentIndexChanged( int idx )
 
   if ( !leLayername->isEnabled() )
     leLayername->setText( QString() );
-  else if ( leLayername->text().isEmpty() &&
-            !mFilename->filePath().isEmpty() )
+  else if ( leLayername->text().isEmpty() )
   {
-    QString layerName = QFileInfo( mFilename->filePath() ).baseName();
+    QString layerName = mDefaultOutputLayerNameFromInputLayerName;
+    if ( layerName.isEmpty() && !mFilename->filePath().isEmpty() )
+    {
+      layerName = QFileInfo( mFilename->filePath() ).baseName();
+      leLayername->setDefaultValue( layerName );
+    }
+    if ( layerName.isEmpty() )
+      layerName = tr( "new_layer" );
     leLayername->setText( layerName );
   }
 
@@ -504,6 +522,10 @@ void QgsVectorLayerSaveAsDialog::mFormatComboBox_currentIndexChanged( int idx )
   {
     mAttributeTable->setRowCount( mLayer->fields().count() );
 
+    QStringList horizontalHeaders = QStringList() << tr( "Name" )  << tr( "Export name" ) << tr( "Type" ) << tr( "Replace with displayed values" );
+    mAttributeTable->setColumnCount( horizontalHeaders.size() );
+    mAttributeTable->setHorizontalHeaderLabels( horizontalHeaders );
+
     bool foundFieldThatCanBeExportedAsDisplayedValue = false;
     for ( int i = 0; i < mLayer->fields().size(); ++i )
     {
@@ -515,69 +537,63 @@ void QgsVectorLayerSaveAsDialog::mFormatComboBox_currentIndexChanged( int idx )
         break;
       }
     }
-    if ( foundFieldThatCanBeExportedAsDisplayedValue )
-    {
-      mAttributeTable->setColumnCount( 3 );
-      mAttributeTable->setHorizontalHeaderLabels( QStringList() << tr( "Name" ) << tr( "Type" ) << tr( "Replace with displayed values" ) );
-    }
-    else
-    {
-      mAttributeTable->setColumnCount( 2 );
-      mAttributeTable->setHorizontalHeaderLabels( QStringList() << tr( "Name" ) << tr( "Type" ) );
-    }
-
-    mAttributeTableItemChangedSlotEnabled = false;
+    mAttributeTable->setColumnHidden( static_cast<int>( ColumnIndex::ExportAsDisplayedValue ),
+                                      ! foundFieldThatCanBeExportedAsDisplayedValue );
 
     bool checkReplaceRawFieldValues = selectAllFields && isFormatForFieldsAsDisplayedValues;
-    for ( int i = 0; i < mLayer->fields().size(); ++i )
+    const QSignalBlocker signalBlockerAttributeTable( mAttributeTable );
     {
-      QgsField fld = mLayer->fields().at( i );
-      Qt::ItemFlags flags = mLayer->providerType() != QLatin1String( "oracle" ) || !fld.typeName().contains( QLatin1String( "SDO_GEOMETRY" ) ) ? Qt::ItemIsEnabled : Qt::NoItemFlags;
-      QTableWidgetItem *item = nullptr;
-      item = new QTableWidgetItem( fld.name() );
-      item->setFlags( flags | Qt::ItemIsUserCheckable );
-      item->setCheckState( ( selectAllFields ) ? Qt::Checked : Qt::Unchecked );
-      mAttributeTable->setItem( i, COLUMN_IDX_NAME, item );
-
-      item = new QTableWidgetItem( fld.typeName() );
-      item->setFlags( flags );
-      mAttributeTable->setItem( i, COLUMN_IDX_TYPE, item );
-
-      if ( foundFieldThatCanBeExportedAsDisplayedValue )
+      for ( int i = 0; i < mLayer->fields().size(); ++i )
       {
-        const QgsEditorWidgetSetup setup = QgsGui::editorWidgetRegistry()->findBest( mLayer, mLayer->fields()[i].name() );
-        QgsEditorWidgetFactory *factory = nullptr;
-        const QString widgetId( setup.type() );
-        if ( flags == Qt::ItemIsEnabled &&
-             widgetId != QLatin1String( "TextEdit" ) &&
-             ( factory = QgsGui::editorWidgetRegistry()->factory( widgetId ) ) )
+        QgsField fld = mLayer->fields().at( i );
+        Qt::ItemFlags flags = mLayer->providerType() != QLatin1String( "oracle" ) || !fld.typeName().contains( QLatin1String( "SDO_GEOMETRY" ) ) ? Qt::ItemIsEnabled : Qt::NoItemFlags;
+        QTableWidgetItem *item = nullptr;
+        item = new QTableWidgetItem( fld.name() );
+        item->setFlags( flags | Qt::ItemIsUserCheckable );
+        item->setCheckState( ( selectAllFields ) ? Qt::Checked : Qt::Unchecked );
+        mAttributeTable->setItem( i, static_cast<int>( ColumnIndex::Name ), item );
+
+        item = new QTableWidgetItem( fld.name() );
+        item->setFlags( flags | Qt::ItemIsEditable );
+        item->setData( Qt::UserRole, fld.displayName() );
+        mAttributeTable->setItem( i, static_cast<int>( ColumnIndex::ExportName ), item );
+
+        item = new QTableWidgetItem( fld.typeName() );
+        item->setFlags( flags );
+        mAttributeTable->setItem( i, static_cast<int>( ColumnIndex::Type ), item );
+
+        if ( foundFieldThatCanBeExportedAsDisplayedValue )
         {
-          item = new QTableWidgetItem( tr( "Use %1" ).arg( factory->name() ) );
-          item->setFlags( ( selectAllFields ) ? ( Qt::ItemIsEnabled | Qt::ItemIsUserCheckable ) : Qt::ItemIsUserCheckable );
-          const bool checkItem = ( selectAllFields && isFormatForFieldsAsDisplayedValues &&
-                                   ( widgetId == QLatin1String( "ValueMap" ) ||
-                                     widgetId == QLatin1String( "ValueRelation" ) ||
-                                     widgetId == QLatin1String( "CheckBox" ) ||
-                                     widgetId == QLatin1String( "RelationReference" ) ) );
-          checkReplaceRawFieldValues &= checkItem;
-          item->setCheckState( checkItem ?
-                               Qt::Checked : Qt::Unchecked );
-          mAttributeTable->setItem( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE, item );
-        }
-        else
-        {
-          item = new QTableWidgetItem();
-          item->setFlags( Qt::NoItemFlags );
-          mAttributeTable->setItem( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE, item );
+          const QgsEditorWidgetSetup setup = QgsGui::editorWidgetRegistry()->findBest( mLayer, mLayer->fields()[i].name() );
+          QgsEditorWidgetFactory *factory = nullptr;
+          const QString widgetId( setup.type() );
+          if ( flags == Qt::ItemIsEnabled &&
+               widgetId != QLatin1String( "TextEdit" ) &&
+               ( factory = QgsGui::editorWidgetRegistry()->factory( widgetId ) ) )
+          {
+            item = new QTableWidgetItem( tr( "Use %1" ).arg( factory->name() ) );
+            item->setFlags( ( selectAllFields ) ? ( Qt::ItemIsEnabled | Qt::ItemIsUserCheckable ) : Qt::ItemIsUserCheckable );
+            const bool checkItem = ( selectAllFields && isFormatForFieldsAsDisplayedValues &&
+                                     ( widgetId == QLatin1String( "ValueMap" ) ||
+                                       widgetId == QLatin1String( "ValueRelation" ) ||
+                                       widgetId == QLatin1String( "CheckBox" ) ||
+                                       widgetId == QLatin1String( "RelationReference" ) ) );
+            checkReplaceRawFieldValues &= checkItem;
+            item->setCheckState( checkItem ?
+                                 Qt::Checked : Qt::Unchecked );
+            mAttributeTable->setItem( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ), item );
+          }
+          else
+          {
+            item = new QTableWidgetItem();
+            item->setFlags( Qt::NoItemFlags );
+            mAttributeTable->setItem( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ), item );
+          }
         }
       }
     }
 
-    mAttributeTableItemChangedSlotEnabled = true;
-
-    mReplaceRawFieldValuesStateChangedSlotEnabled = false;
-    mReplaceRawFieldValues->setChecked( checkReplaceRawFieldValues );
-    mReplaceRawFieldValuesStateChangedSlotEnabled = true;
+    whileBlocking( mReplaceRawFieldValues )->setChecked( checkReplaceRawFieldValues );
     mReplaceRawFieldValues->setEnabled( selectAllFields );
     mReplaceRawFieldValues->setVisible( foundFieldThatCanBeExportedAsDisplayedValue );
 
@@ -671,90 +687,191 @@ void QgsVectorLayerSaveAsDialog::mFormatComboBox_currentIndexChanged( int idx )
   }
 }
 
+void QgsVectorLayerSaveAsDialog::mUseAliasesForExportedName_stateChanged( int state )
+{
+  const QSignalBlocker signalBlocker( mAttributeTable );
+
+  switch ( state )
+  {
+    case Qt::Unchecked:
+    {
+      // Check for modified entries
+      bool modifiedEntries = false;
+      for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
+      {
+        if ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportName ) )->text()
+             != mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportName ) )->data( Qt::UserRole ).toString() )
+        {
+          modifiedEntries = true;
+          break;
+        }
+      }
+
+      if ( modifiedEntries )
+      {
+        if ( QMessageBox::question( this,
+                                    tr( "Modified names" ),
+                                    tr( "Some names were modified and will be overridden. Do you want to continue?" ) )
+             == QMessageBox::No )
+        {
+          whileBlocking( mUseAliasesForExportedName )->setCheckState( Qt::PartiallyChecked );
+          return;
+        }
+      }
+
+      for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
+      {
+        mUseAliasesForExportedName->setTristate( false );
+        mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportName ) )->setText( mAttributeTable->item( i, static_cast<int>( ColumnIndex::Name ) )->text() );
+      }
+    }
+    break;
+    case Qt::Checked:
+    {
+      // Check for modified entries
+      bool modifiedEntries = false;
+      for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
+      {
+        if ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportName ) )->text()
+             != mAttributeTable->item( i, static_cast<int>( ColumnIndex::Name ) )->text() )
+          modifiedEntries = true;
+      }
+
+      if ( modifiedEntries )
+      {
+        if ( QMessageBox::question( this,
+                                    tr( "Modified names" ),
+                                    tr( "Some names were modified and will be overridden. Do you want to continue?" ) )
+             == QMessageBox::No )
+        {
+          whileBlocking( mUseAliasesForExportedName )->setCheckState( Qt::PartiallyChecked );
+          return;
+        }
+      }
+
+      for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
+      {
+        mUseAliasesForExportedName->setTristate( false );
+        const QString alias = mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportName ) )->data( Qt::UserRole ).toString();
+        mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportName ) )->setText( alias );
+      }
+    }
+    break;
+    case Qt::PartiallyChecked:
+      // Do nothing
+      break;
+  }
+}
+
 void QgsVectorLayerSaveAsDialog::mReplaceRawFieldValues_stateChanged( int )
 {
-  if ( !mReplaceRawFieldValuesStateChangedSlotEnabled )
+  if ( mAttributeTable->isColumnHidden( static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) )
     return;
-  if ( mAttributeTable->columnCount() != 3 )
-    return;
-  mReplaceRawFieldValuesStateChangedSlotEnabled = false;
-  mAttributeTableItemChangedSlotEnabled = false;
+
+  const QSignalBlocker signalBlockerAttributeTable( mAttributeTable );
+  const QSignalBlocker signalBlockerReplaceRawFieldValues( mReplaceRawFieldValues );
+
   if ( mReplaceRawFieldValues->checkState() != Qt::PartiallyChecked )
   {
     for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
     {
-      if ( mAttributeTable->item( i, COLUMN_IDX_NAME )->checkState() == Qt::Checked &&
-           mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE ) &&
-           mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->flags() & Qt::ItemIsEnabled )
+      if ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::Name ) )->checkState() == Qt::Checked &&
+           mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) &&
+           mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->flags() & Qt::ItemIsEnabled )
       {
-        mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->setCheckState( mReplaceRawFieldValues->checkState() );
+        mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->setCheckState( mReplaceRawFieldValues->checkState() );
       }
     }
   }
   mReplaceRawFieldValues->setTristate( false );
-  mAttributeTableItemChangedSlotEnabled = true;
-  mReplaceRawFieldValuesStateChangedSlotEnabled = true;
 }
 
 void QgsVectorLayerSaveAsDialog::mAttributeTable_itemChanged( QTableWidgetItem *item )
 {
-  if ( !mAttributeTableItemChangedSlotEnabled )
-    return;
-  mReplaceRawFieldValuesStateChangedSlotEnabled = false;
-  mAttributeTableItemChangedSlotEnabled = false;
+  const QSignalBlocker signalBlockerAttributeTable( mAttributeTable );
+  const QSignalBlocker signalBlockerReplaceRawFieldValues( mReplaceRawFieldValues );
+
   int row = item->row();
   int column = item->column();
-  if ( column == COLUMN_IDX_NAME &&
-       mAttributeTable->item( row, column )->checkState() == Qt::Unchecked &&
-       mAttributeTable->columnCount() == 3 &&
-       mAttributeTable->item( row, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE ) &&
-       ( mAttributeTable->item( row, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->flags() & Qt::ItemIsUserCheckable ) )
+
+  switch ( static_cast<ColumnIndex>( column ) )
   {
-    mAttributeTable->item( row, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->setCheckState( Qt::Unchecked );
-    mAttributeTable->item( row, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->setFlags( Qt::ItemIsUserCheckable );
-    bool checkBoxEnabled = false;
-    for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
+    case ColumnIndex::Name:
     {
-      if ( mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE ) &&
-           mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->flags() & Qt::ItemIsEnabled )
+      if ( mAttributeTable->isColumnHidden( static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) ||
+           ! mAttributeTable->item( row, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) ||
+           !( mAttributeTable->item( row, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->flags() & Qt::ItemIsUserCheckable ) )
+        return;
+
+      if ( mAttributeTable->item( row, column )->checkState() == Qt::Unchecked )
       {
-        checkBoxEnabled = true;
-        break;
+        mAttributeTable->item( row, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->setCheckState( Qt::Unchecked );
+        mAttributeTable->item( row, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->setFlags( Qt::ItemIsUserCheckable );
+        bool checkBoxEnabled = false;
+        for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
+        {
+          if ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) &&
+               mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->flags() & Qt::ItemIsEnabled )
+          {
+            checkBoxEnabled = true;
+            break;
+          }
+        }
+        mReplaceRawFieldValues->setEnabled( checkBoxEnabled );
+        if ( !checkBoxEnabled )
+          mReplaceRawFieldValues->setCheckState( Qt::Unchecked );
+      }
+      else if ( mAttributeTable->item( row, column )->checkState() == Qt::Checked )
+      {
+        mAttributeTable->item( row, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->setFlags( Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
+        mReplaceRawFieldValues->setEnabled( true );
       }
     }
-    mReplaceRawFieldValues->setEnabled( checkBoxEnabled );
-    if ( !checkBoxEnabled )
-      mReplaceRawFieldValues->setCheckState( Qt::Unchecked );
-  }
-  else if ( column == COLUMN_IDX_NAME &&
-            mAttributeTable->item( row, column )->checkState() == Qt::Checked &&
-            mAttributeTable->columnCount() == 3 &&
-            mAttributeTable->item( row, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE ) &&
-            ( mAttributeTable->item( row, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->flags() & Qt::ItemIsUserCheckable ) )
-  {
-    mAttributeTable->item( row, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->setFlags( Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
-    mReplaceRawFieldValues->setEnabled( true );
-  }
-  else if ( column == COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE &&
-            ( mAttributeTable->item( row, column )->flags() & Qt::ItemIsUserCheckable ) )
-  {
-    bool allChecked = true;
-    bool allUnchecked = true;
-    for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
+    break;
+    case ColumnIndex::ExportName:
     {
-      if ( mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE ) &&
-           mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->flags() & Qt::ItemIsEnabled )
+      // Check empty export name
+      if ( item->text().isEmpty() )
       {
-        if ( mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->checkState() == Qt::Unchecked )
-          allChecked = false;
-        else
-          allUnchecked = false;
+        QMessageBox::warning( this,
+                              tr( "Empty export name" ),
+                              tr( "Empty export name are not allowed." ) );
+        return;
+      }
+
+      // Rename eventually duplicated names
+      QStringList names = attributesExportNames();
+      while ( names.count( item->text() ) > 1 )
+        item->setText( QString( "%1_2" ).arg( item->text() ) );
+
+      mUseAliasesForExportedName->setCheckState( Qt::PartiallyChecked );
+    }
+    break;
+    case ColumnIndex::Type:
+      // Nothing to do
+      break;
+    case ColumnIndex::ExportAsDisplayedValue:
+    {
+      if ( mAttributeTable->item( row, column )->flags() & Qt::ItemIsUserCheckable )
+      {
+        bool allChecked = true;
+        bool allUnchecked = true;
+        for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
+        {
+          if ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) &&
+               mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->flags() & Qt::ItemIsEnabled )
+          {
+            if ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->checkState() == Qt::Unchecked )
+              allChecked = false;
+            else
+              allUnchecked = false;
+          }
+        }
+        mReplaceRawFieldValues->setCheckState( ( !allChecked && !allUnchecked ) ? Qt::PartiallyChecked : ( allChecked ) ? Qt::Checked : Qt::Unchecked );
       }
     }
-    mReplaceRawFieldValues->setCheckState( ( !allChecked && !allUnchecked ) ? Qt::PartiallyChecked : ( allChecked ) ? Qt::Checked : Qt::Unchecked );
+    break;
   }
-  mAttributeTableItemChangedSlotEnabled = true;
-  mReplaceRawFieldValuesStateChangedSlotEnabled = true;
 }
 
 void QgsVectorLayerSaveAsDialog::mCrsSelector_crsChanged( const QgsCoordinateReferenceSystem &crs )
@@ -917,7 +1034,7 @@ QgsAttributeList QgsVectorLayerSaveAsDialog::selectedAttributes() const
 
   for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
   {
-    if ( mAttributeTable->item( i, COLUMN_IDX_NAME )->checkState() == Qt::Checked )
+    if ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::Name ) )->checkState() == Qt::Checked )
     {
       attributes.append( i );
     }
@@ -932,9 +1049,9 @@ QgsAttributeList QgsVectorLayerSaveAsDialog::attributesAsDisplayedValues() const
 
   for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
   {
-    if ( mAttributeTable->item( i, COLUMN_IDX_NAME )->checkState() == Qt::Checked &&
-         mAttributeTable->columnCount() == 3 &&
-         mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->checkState() == Qt::Checked )
+    if ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::Name ) )->checkState() == Qt::Checked &&
+         ! mAttributeTable->isColumnHidden( static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) &&
+         mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->checkState() == Qt::Checked )
     {
       attributes.append( i );
     }
@@ -943,6 +1060,15 @@ QgsAttributeList QgsVectorLayerSaveAsDialog::attributesAsDisplayedValues() const
   return attributes;
 }
 
+QStringList QgsVectorLayerSaveAsDialog::attributesExportNames() const
+{
+  QStringList exportNames;
+  for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
+    exportNames.append( mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportName ) )->text() );
+
+  return exportNames;
+}
+
 bool QgsVectorLayerSaveAsDialog::addToCanvas() const
 {
   return mAddToCanvas->isChecked() && mAddToCanvas->isEnabled();
@@ -1070,49 +1196,47 @@ void QgsVectorLayerSaveAsDialog::mGeometryTypeComboBox_currentIndexChanged( int
 
 void QgsVectorLayerSaveAsDialog::mSelectAllAttributes_clicked()
 {
-  mAttributeTableItemChangedSlotEnabled = false;
-  mReplaceRawFieldValuesStateChangedSlotEnabled = false;
+  const QSignalBlocker signalBlockerAttributeTable( mAttributeTable );
+  const QSignalBlocker signalBlockerReplaceRawFieldValues( mReplaceRawFieldValues );
+
   for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
   {
-    if ( mAttributeTable->item( i, COLUMN_IDX_NAME )->flags() & Qt::ItemIsEnabled )
+    if ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::Name ) )->flags() & Qt::ItemIsEnabled )
     {
-      if ( mAttributeTable->columnCount() == 3 &&
-           ( mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->flags() & Qt::ItemIsUserCheckable ) )
+      if ( ! mAttributeTable->isColumnHidden( static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) &&
+           ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->flags() & Qt::ItemIsUserCheckable ) )
       {
-        mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->setFlags( Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
+        mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->setFlags( Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
       }
-      mAttributeTable->item( i, COLUMN_IDX_NAME )->setCheckState( Qt::Checked );
+      mAttributeTable->item( i, static_cast<int>( ColumnIndex::Name ) )->setCheckState( Qt::Checked );
     }
   }
-  if ( mAttributeTable->columnCount() == 3 )
+  if ( ! mAttributeTable->isColumnHidden( static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) )
   {
     mReplaceRawFieldValues->setEnabled( true );
   }
-  mAttributeTableItemChangedSlotEnabled = true;
-  mReplaceRawFieldValuesStateChangedSlotEnabled = true;
 }
 
 void QgsVectorLayerSaveAsDialog::mDeselectAllAttributes_clicked()
 {
-  mAttributeTableItemChangedSlotEnabled = false;
-  mReplaceRawFieldValuesStateChangedSlotEnabled = false;
+  const QSignalBlocker signalBlockerAttributeTable( mAttributeTable );
+  const QSignalBlocker signalBlockerReplaceRawFieldValues( mReplaceRawFieldValues );
+
   for ( int i = 0; i < mAttributeTable->rowCount(); i++ )
   {
-    mAttributeTable->item( i, COLUMN_IDX_NAME )->setCheckState( Qt::Unchecked );
-    if ( mAttributeTable->columnCount() == 3 &&
-         ( mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->flags() & Qt::ItemIsUserCheckable ) )
+    mAttributeTable->item( i, static_cast<int>( ColumnIndex::Name ) )->setCheckState( Qt::Unchecked );
+    if ( ! mAttributeTable->isColumnHidden( static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) &&
+         ( mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->flags() & Qt::ItemIsUserCheckable ) )
     {
-      mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->setFlags( Qt::ItemIsUserCheckable );
-      mAttributeTable->item( i, COLUMN_IDX_EXPORT_AS_DISPLAYED_VALUE )->setCheckState( Qt::Unchecked );
+      mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->setFlags( Qt::ItemIsUserCheckable );
+      mAttributeTable->item( i, static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) )->setCheckState( Qt::Unchecked );
     }
   }
-  if ( mAttributeTable->columnCount() == 3 )
+  if ( ! mAttributeTable->isColumnHidden( static_cast<int>( ColumnIndex::ExportAsDisplayedValue ) ) )
   {
     mReplaceRawFieldValues->setCheckState( Qt::Unchecked );
     mReplaceRawFieldValues->setEnabled( false );
   }
-  mAttributeTableItemChangedSlotEnabled = true;
-  mReplaceRawFieldValuesStateChangedSlotEnabled = true;
 }
 
 void QgsVectorLayerSaveAsDialog::showHelp()
