@@ -34,9 +34,10 @@
 #include "qgsmessagebaritem.h"
 #include "qgspanelwidget.h"
 #include "qgsprocessingmultipleselectiondialog.h"
+#include "qgsprocessinghelpeditorwidget.h"
+#include "qgsscreenhelper.h"
 
 #include <QShortcut>
-#include <QDesktopWidget>
 #include <QKeySequence>
 #include <QFileDialog>
 #include <QPrinter>
@@ -83,6 +84,8 @@ QgsModelDesignerDialog::QgsModelDesignerDialog( QWidget *parent, Qt::WindowFlags
 {
   setupUi( this );
 
+  mScreenHelper = new QgsScreenHelper( this );
+
   setAttribute( Qt::WA_DeleteOnClose );
   setDockOptions( dockOptions() | QMainWindow::GroupedDragging );
   setWindowFlags( Qt::WindowMinimizeButtonHint |
@@ -134,6 +137,7 @@ QgsModelDesignerDialog::QgsModelDesignerDialog( QWidget *parent, Qt::WindowFlags
   QgsSettings settings;
 
   connect( mActionClose, &QAction::triggered, this, &QWidget::close );
+  connect( mActionNew, &QAction::triggered, this, &QgsModelDesignerDialog::newModel );
   connect( mActionZoomIn, &QAction::triggered, this, &QgsModelDesignerDialog::zoomIn );
   connect( mActionZoomOut, &QAction::triggered, this, &QgsModelDesignerDialog::zoomOut );
   connect( mActionZoomActual, &QAction::triggered, this, &QgsModelDesignerDialog::zoomActual );
@@ -148,6 +152,7 @@ QgsModelDesignerDialog::QgsModelDesignerDialog( QWidget *parent, Qt::WindowFlags
   connect( mActionSnapSelected, &QAction::triggered, mView, &QgsModelGraphicsView::snapSelected );
   connect( mActionValidate, &QAction::triggered, this, &QgsModelDesignerDialog::validate );
   connect( mActionReorderInputs, &QAction::triggered, this, &QgsModelDesignerDialog::reorderInputs );
+  connect( mActionEditHelp, &QAction::triggered, this, &QgsModelDesignerDialog::editHelp );
   connect( mReorderInputsButton, &QPushButton::clicked, this, &QgsModelDesignerDialog::reorderInputs );
 
   mActionSnappingEnabled->setChecked( settings.value( QStringLiteral( "/Processing/Modeler/enableSnapToGrid" ), false ).toBool() );
@@ -539,12 +544,19 @@ void QgsModelDesignerDialog::setDirty( bool dirty )
   updateWindowTitle();
 }
 
-bool QgsModelDesignerDialog::validateSave()
+bool QgsModelDesignerDialog::validateSave( SaveAction action )
 {
-  if ( mNameEdit->text().trimmed().isEmpty() )
+  switch ( action )
   {
-    mMessageBar->pushWarning( QString(), tr( "Please enter a model name before saving" ) );
-    return false;
+    case QgsModelDesignerDialog::SaveAction::SaveAsFile:
+      break;
+    case QgsModelDesignerDialog::SaveAction::SaveInProject:
+      if ( mNameEdit->text().trimmed().isEmpty() )
+      {
+        mMessageBar->pushWarning( QString(), tr( "Please enter a model name before saving" ) );
+        return false;
+      }
+      break;
   }
 
   return true;
@@ -589,6 +601,11 @@ void QgsModelDesignerDialog::setLastRunChildAlgorithmInputs( const QVariantMap &
     mScene->setChildAlgorithmInputs( mChildInputs );
 }
 
+void QgsModelDesignerDialog::setModelName( const QString &name )
+{
+  mNameEdit->setText( name );
+}
+
 void QgsModelDesignerDialog::zoomIn()
 {
   mView->setTransformationAnchor( QGraphicsView::NoAnchor );
@@ -613,7 +630,7 @@ void QgsModelDesignerDialog::zoomActual()
 {
   QPointF point = mView->mapToScene( QPoint( mView->viewport()->width() / 2.0, mView->viewport()->height() / 2 ) );
   mView->resetTransform();
-  mView->scale( QgsApplication::desktop()->logicalDpiX() / 96, QgsApplication::desktop()->logicalDpiX() / 96 );
+  mView->scale( mScreenHelper->screenDpi() / 96, mScreenHelper->screenDpi() / 96 );
   mView->centerOn( point );
 }
 
@@ -624,6 +641,16 @@ void QgsModelDesignerDialog::zoomFull()
   mView->fitInView( totalRect, Qt::KeepAspectRatio );
 }
 
+void QgsModelDesignerDialog::newModel()
+{
+  if ( !checkForUnsavedChanges() )
+    return;
+
+  std::unique_ptr< QgsProcessingModelAlgorithm > alg = std::make_unique< QgsProcessingModelAlgorithm >();
+  alg->setProvider( QgsApplication::processingRegistry()->providerById( QStringLiteral( "model" ) ) );
+  setModel( alg.release() );
+}
+
 void QgsModelDesignerDialog::exportToImage()
 {
   QgsSettings settings;
@@ -686,7 +713,15 @@ void QgsModelDesignerDialog::exportToPdf()
   QPrinter printer;
   printer.setOutputFormat( QPrinter::PdfFormat );
   printer.setOutputFileName( filename );
-  printer.setPaperSize( QSizeF( printerRect.width(), printerRect.height() ), QPrinter::DevicePixel );
+
+  const double scaleFactor = 96 / 25.4; // based on 96 dpi sizes
+
+  QPageLayout pageLayout( QPageSize( totalRect.size() / scaleFactor, QPageSize::Millimeter ),
+                          QPageLayout::Portrait,
+                          QMarginsF( 0, 0, 0, 0 ) );
+  pageLayout.setMode( QPageLayout::FullPageMode );
+  printer.setPageLayout( pageLayout );
+
   printer.setFullPage( true );
 
   QPainter painter( &printer );
@@ -935,6 +970,19 @@ void QgsModelDesignerDialog::setPanelVisibility( bool hidden )
   }
 }
 
+void QgsModelDesignerDialog::editHelp()
+{
+  QgsProcessingHelpEditorDialog dialog( this );
+  dialog.setWindowTitle( tr( "Edit Model Help" ) );
+  dialog.setAlgorithm( mModel.get() );
+  if ( dialog.exec() )
+  {
+    beginUndoCommand( tr( "Edit Model Help" ) );
+    mModel->setHelpContent( dialog.helpContent() );
+    endUndoCommand();
+  }
+}
+
 void QgsModelDesignerDialog::validate()
 {
   QStringList issues;
@@ -944,7 +992,7 @@ void QgsModelDesignerDialog::validate()
   }
   else
   {
-    QgsMessageBarItem *messageWidget = mMessageBar->createMessage( QString(), tr( "Model is invalid!" ) );
+    QgsMessageBarItem *messageWidget = QgsMessageBar::createMessage( QString(), tr( "Model is invalid!" ) );
     QPushButton *detailsButton = new QPushButton( tr( "Details" ) );
     connect( detailsButton, &QPushButton::clicked, detailsButton, [ = ]
     {
@@ -1092,7 +1140,7 @@ void QgsModelChildDependenciesWidget::showDialog()
 
 void QgsModelChildDependenciesWidget::updateSummaryText()
 {
-  mLineEdit->setText( tr( "%1 dependencies selected" ).arg( mValue.count() ) );
+  mLineEdit->setText( tr( "%n dependencies selected", nullptr, mValue.count() ) );
 }
 
 ///@endcond
