@@ -147,11 +147,7 @@ void QgsModelGraphicsView::wheelZoom( QWheelEvent *event )
   QgsRectangle visibleRect = QgsRectangle( mapToScene( viewportRect ).boundingRect() );
 
   //transform the mouse pos to scene coordinates
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-  QPointF scenePoint = mapToScene( event->pos().x(), event->pos().y() );
-#else
   QPointF scenePoint = mapToScene( event->position().x(), event->position().y() );
-#endif
 
   //adjust view center
   QgsPointXY oldCenter( visibleRect.center() );
@@ -610,7 +606,6 @@ void QgsModelGraphicsView::pasteItems( QgsModelGraphicsView::PasteMode mode )
   if ( !modelScene() )
     return;
 
-  QList< QgsModelComponentGraphicItem * > pastedItems;
   QDomDocument doc;
   QClipboard *clipboard = QApplication::clipboard();
   if ( doc.setContent( clipboard->mimeData()->data( QStringLiteral( "text/xml" ) ) ) )
