@@ -25,6 +25,7 @@
 #include "qgsstringutils.h"
 #include "qgsapplication.h"
 #include "qgspanelwidget.h"
+#include "qgsjsonutils.h"
 #include <QToolButton>
 #include <QDesktopServices>
 #include <QScrollBar>
@@ -32,6 +33,8 @@
 #include <QClipboard>
 #include <QFileDialog>
 #include <QMimeData>
+#include <QMenu>
+#include <nlohmann/json.hpp>
 
 
 ///@cond NOT_STABLE
@@ -86,8 +89,9 @@ void QgsProcessingAlgorithmDialogFeedback::pushConsoleInfo( const QString &info
 // QgsProcessingAlgorithmDialogBase
 //
 
-QgsProcessingAlgorithmDialogBase::QgsProcessingAlgorithmDialogBase( QWidget *parent, Qt::WindowFlags flags )
+QgsProcessingAlgorithmDialogBase::QgsProcessingAlgorithmDialogBase( QWidget *parent, Qt::WindowFlags flags, DialogMode mode )
   : QDialog( parent, flags )
+  , mMode( mode )
 {
   setupUi( this );
 
@@ -124,6 +128,133 @@ QgsProcessingAlgorithmDialogBase::QgsProcessingAlgorithmDialogBase( QWidget *par
   buttonCancel->setEnabled( false );
   mButtonClose = mButtonBox->button( QDialogButtonBox::Close );
 
+  switch ( mMode )
+  {
+    case DialogMode::Single:
+    {
+      mAdvancedButton = new QPushButton( tr( "Advanced" ) );
+      mAdvancedMenu = new QMenu( this );
+      mAdvancedButton->setMenu( mAdvancedMenu );
+
+      QAction *copyAsPythonCommand = new QAction( tr( "Copy as Python Command" ), mAdvancedMenu );
+      copyAsPythonCommand->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "mIconPythonFile.svg" ) ) );
+
+      mAdvancedMenu->addAction( copyAsPythonCommand );
+      connect( copyAsPythonCommand, &QAction::triggered, this, [this]
+      {
+        if ( const QgsProcessingAlgorithm *alg = algorithm() )
+        {
+          QgsProcessingContext *context = processingContext();
+          if ( !context )
+            return;
+
+          const QString command = alg->asPythonCommand( createProcessingParameters(), *context );
+          QMimeData *m = new QMimeData();
+          m->setText( command );
+          QClipboard *cb = QApplication::clipboard();
+
+#ifdef Q_OS_LINUX
+          cb->setMimeData( m, QClipboard::Selection );
+#endif
+          cb->setMimeData( m, QClipboard::Clipboard );
+        }
+      } );
+
+      mCopyAsQgisProcessCommand = new QAction( tr( "Copy as qgis_process Command" ), mAdvancedMenu );
+      mCopyAsQgisProcessCommand->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "mActionTerminal.svg" ) ) );
+      mAdvancedMenu->addAction( mCopyAsQgisProcessCommand );
+
+      connect( mCopyAsQgisProcessCommand, &QAction::triggered, this, [this]
+      {
+        if ( const QgsProcessingAlgorithm *alg = algorithm() )
+        {
+          QgsProcessingContext *context = processingContext();
+          if ( !context )
+            return;
+
+          bool ok = false;
+          const QString command = alg->asQgisProcessCommand( createProcessingParameters(), *context, ok );
+          if ( ! ok )
+          {
+            mMessageBar->pushMessage( tr( "Current settings cannot be specified as arguments to qgis_process (Pipe parameters as JSON to qgis_process instead)" ), Qgis::MessageLevel::Warning );
+          }
+          else
+          {
+            QMimeData *m = new QMimeData();
+            m->setText( command );
+            QClipboard *cb = QApplication::clipboard();
+
+#ifdef Q_OS_LINUX
+            cb->setMimeData( m, QClipboard::Selection );
+#endif
+            cb->setMimeData( m, QClipboard::Clipboard );
+          }
+        }
+      } );
+
+      mAdvancedMenu->addSeparator();
+
+      QAction *copyAsJson = new QAction( tr( "Copy as JSON" ), mAdvancedMenu );
+      copyAsJson->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "mActionEditCopy.svg" ) ) );
+
+      mAdvancedMenu->addAction( copyAsJson );
+      connect( copyAsJson, &QAction::triggered, this, [this]
+      {
+        if ( const QgsProcessingAlgorithm *alg = algorithm() )
+        {
+          QgsProcessingContext *context = processingContext();
+          if ( !context )
+            return;
+
+          const QVariantMap properties = alg->asMap( createProcessingParameters(), *context );
+          const QString json = QString::fromStdString( QgsJsonUtils::jsonFromVariant( properties ).dump( 2 ) );
+
+          QMimeData *m = new QMimeData();
+          m->setText( json );
+          QClipboard *cb = QApplication::clipboard();
+
+#ifdef Q_OS_LINUX
+          cb->setMimeData( m, QClipboard::Selection );
+#endif
+          cb->setMimeData( m, QClipboard::Clipboard );
+        }
+      } );
+
+      mPasteJsonAction = new QAction( tr( "Paste Settings" ), mAdvancedMenu );
+      mPasteJsonAction->setIcon( QgsApplication::getThemeIcon( QStringLiteral( "mActionEditPaste.svg" ) ) );
+
+      mAdvancedMenu->addAction( mPasteJsonAction );
+      connect( mPasteJsonAction, &QAction::triggered, this, [this]
+      {
+        const QString text = QApplication::clipboard()->text();
+        if ( text.isEmpty() )
+          return;
+
+        const QVariantMap parameterValues = QgsJsonUtils::parseJson( text ).toMap().value( QStringLiteral( "inputs" ) ).toMap();
+        if ( parameterValues.isEmpty() )
+          return;
+
+        setParameters( parameterValues );
+      } );
+
+      mButtonBox->addButton( mAdvancedButton, QDialogButtonBox::ResetRole );
+      break;
+    }
+
+    case DialogMode::Batch:
+      break;
+  }
+
+  if ( mAdvancedMenu )
+  {
+    connect( mAdvancedMenu, &QMenu::aboutToShow, this, [ = ]
+    {
+      mCopyAsQgisProcessCommand->setEnabled( algorithm()
+                                             && !( algorithm()->flags() & QgsProcessingAlgorithm::FlagNotAvailableInStandaloneTool ) );
+      mPasteJsonAction->setEnabled( !QApplication::clipboard()->text().isEmpty() );
+    } );
+  }
+
   connect( mButtonRun, &QPushButton::clicked, this, &QgsProcessingAlgorithmDialogBase::runAlgorithm );
   connect( mButtonChangeParameters, &QPushButton::clicked, this, &QgsProcessingAlgorithmDialogBase::showParameters );
   connect( mButtonBox, &QDialogButtonBox::rejected, this, &QgsProcessingAlgorithmDialogBase::closeClicked );
@@ -146,13 +277,16 @@ QgsProcessingAlgorithmDialogBase::QgsProcessingAlgorithmDialogBase( QWidget *par
 
 QgsProcessingAlgorithmDialogBase::~QgsProcessingAlgorithmDialogBase() = default;
 
+void QgsProcessingAlgorithmDialogBase::setParameters( const QVariantMap & )
+{}
+
 void QgsProcessingAlgorithmDialogBase::setAlgorithm( QgsProcessingAlgorithm *algorithm )
 {
   mAlgorithm.reset( algorithm );
   QString title;
   if ( ( QgsGui::higFlags() & QgsGui::HigDialogTitleIsTitleCase ) && !( algorithm->flags() & QgsProcessingAlgorithm::FlagDisplayNameIsLiteral ) )
   {
-    title = QgsStringUtils::capitalize( mAlgorithm->displayName(), QgsStringUtils::TitleCase );
+    title = QgsStringUtils::capitalize( mAlgorithm->displayName(), Qgis::Capitalization::TitleCase );
   }
   else
   {
@@ -176,12 +310,12 @@ void QgsProcessingAlgorithmDialogBase::setAlgorithm( QgsProcessingAlgorithm *alg
     textShortHelp->show();
   }
 
-  if ( algorithm->helpUrl().isEmpty() && algorithm->provider()->helpId().isEmpty() )
+  if ( algorithm->helpUrl().isEmpty() && ( !algorithm->provider() || algorithm->provider()->helpId().isEmpty() ) )
   {
     mButtonBox->removeButton( mButtonBox->button( QDialogButtonBox::Help ) );
   }
 
-  const QString warning = algorithm->provider()->warningMessage();
+  const QString warning = algorithm->provider() ? algorithm->provider()->warningMessage() : QString();
   if ( !warning.isEmpty() )
   {
     mMessageBar->pushMessage( warning, Qgis::MessageLevel::Warning );
@@ -311,7 +445,7 @@ void QgsProcessingAlgorithmDialogBase::finished( bool, const QVariantMap &, QgsP
 void QgsProcessingAlgorithmDialogBase::openHelp()
 {
   QUrl algHelp = mAlgorithm->helpUrl();
-  if ( algHelp.isEmpty() )
+  if ( algHelp.isEmpty() && mAlgorithm->provider() )
   {
     algHelp = QgsHelp::helpUrl( QStringLiteral( "processing_algs/%1/%2.html#%3" ).arg( mAlgorithm->provider()->helpId(), mAlgorithm->groupId(), QStringLiteral( "%1%2" ).arg( mAlgorithm->provider()->helpId() ).arg( mAlgorithm->name() ) ) );
   }
@@ -376,7 +510,7 @@ void QgsProcessingAlgorithmDialogBase::algExecuted( bool successful, const QVari
   else
   {
     // delete dialog if closed
-    if ( !isVisible() )
+    if ( isFinalized() && !isVisible() )
     {
       deleteLater();
     }
@@ -524,7 +658,7 @@ void QgsProcessingAlgorithmDialogBase::closeEvent( QCloseEvent *e )
 
   QDialog::closeEvent( e );
 
-  if ( !mAlgorithmTask )
+  if ( !mAlgorithmTask && isFinalized() )
   {
     // when running a background task, the dialog is kept around and deleted only when the task
     // completes. But if not running a task, we auto cleanup (later - gotta give callers a chance
@@ -676,6 +810,11 @@ QString QgsProcessingAlgorithmDialogBase::formatStringForLog( const QString &str
   return s;
 }
 
+bool QgsProcessingAlgorithmDialogBase::isFinalized()
+{
+  return true;
+}
+
 void QgsProcessingAlgorithmDialogBase::setInfo( const QString &message, bool isError, bool escapeHtml, bool isWarning )
 {
   constexpr int MESSAGE_COUNT_LIMIT = 10000;
@@ -700,7 +839,7 @@ void QgsProcessingAlgorithmDialogBase::setInfo( const QString &message, bool isE
 
 void QgsProcessingAlgorithmDialogBase::reject()
 {
-  if ( !mAlgorithmTask )
+  if ( !mAlgorithmTask && isFinalized() )
   {
     setAttribute( Qt::WA_DeleteOnClose );
   }
