@@ -20,17 +20,41 @@
 
 #include <QIcon>
 
-QgsOgrDbTableModel::QgsOgrDbTableModel()
+QgsOgrDbTableModel::QgsOgrDbTableModel( QObject *parent )
+  : QgsAbstractDbTableModel( parent )
 {
-  QStringList headerLabels;
-  headerLabels << tr( "Table" );
-  headerLabels << tr( "Type" );
-  headerLabels << tr( "Geometry column" );
-  headerLabels << tr( "Sql" );
-  setHorizontalHeaderLabels( headerLabels );
+  mColumns << tr( "Table" )
+           << tr( "Type" )
+           << tr( "Geometry column" )
+           << tr( "SQL" );
+  setHorizontalHeaderLabels( mColumns );
 }
 
-void QgsOgrDbTableModel::addTableEntry( const Qgis::BrowserLayerType &layerType, const QString &tableName, const QString &uri, const QString &geometryColName, const QString &geometryType, const QString &sql )
+QStringList QgsOgrDbTableModel::columns() const
+{
+  return mColumns;
+}
+
+int QgsOgrDbTableModel::defaultSearchColumn() const
+{
+  return 0;
+}
+
+bool QgsOgrDbTableModel::searchableColumn( int column ) const
+{
+  Columns col = static_cast<Columns>( column );
+  switch ( col )
+  {
+    case QgsOgrDbTableModel::DbtmTable:
+    case QgsOgrDbTableModel::DbtmType:
+    case QgsOgrDbTableModel::DbtmGeomCol:
+    case QgsOgrDbTableModel::DbtmSql:
+      return true;
+  }
+  BUILTIN_UNREACHABLE
+}
+
+void QgsOgrDbTableModel::addTableEntry( Qgis::BrowserLayerType layerType, const QString &tableName, const QString &uri, const QString &geometryColName, const QString &geometryType, const QString &sql )
 {
   //is there already a root item ?
   QStandardItem *dbItem = nullptr;
