@@ -24,6 +24,8 @@
 #include <QWidget>
 #include <QLabel>
 #include <QDialogButtonBox>
+#include <QMultiMap>
+
 #include "qgis_gui.h"
 
 
@@ -268,6 +270,13 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      */
     void flashFeatures( const QString &filter );
 
+    /**
+     * Emitted when the user chooses to open the attribute table dialog with a filtered set of features.
+     * \since QGIS 3.24
+     */
+    void openFilteredFeaturesAttributeTable( const QString &filter );
+
+
   public slots:
 
     /**
@@ -383,6 +392,8 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
     void updateFieldDependenciesVirtualFields( QgsEditorWidgetWrapper *eww );
     void updateRelatedLayerFieldsDependencies( QgsEditorWidgetWrapper *eww = nullptr );
 
+    void setMultiEditFeatureIdsRelations( const QgsFeatureIds &fids );
+
     struct WidgetInfo
     {
       QWidget *widget = nullptr;
@@ -392,12 +403,11 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
       bool labelOnTop = false;
       bool labelAlignRight = false;
       bool showLabel = true;
+      QgsAttributeEditorElement::LabelStyle labelStyle;
     };
 
     WidgetInfo createWidgetFromDef( const QgsAttributeEditorElement *widgetDef, QWidget *parent, QgsVectorLayer *vl, QgsAttributeEditorContext &context );
 
-    void addWidgetWrapper( QgsEditorWidgetWrapper *eww );
-
     /**
      * Creates widget wrappers for all suitable widgets found.
      * Called once maximally.
@@ -453,7 +463,7 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
     QDialogButtonBox *mButtonBox = nullptr;
     QWidget *mSearchButtonBox = nullptr;
     QList<QgsAttributeFormInterface *> mInterfaces;
-    QMap< int, QgsAttributeFormEditorWidget * > mFormEditorWidgets;
+    QMultiMap< int, QgsAttributeFormEditorWidget * > mFormEditorWidgets;
     QList< QgsAttributeFormWidget *> mFormWidgets;
     QMap<const QgsVectorLayerJoinInfo *, QgsFeature> mJoinedFeatures;
     QMap<QLabel *, QgsProperty> mLabelDataDefinedProperties;
@@ -479,10 +489,21 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
         , isVisible( true )
       {}
 
+      ContainerInformation( QWidget *widget, const QgsExpression &visibilityExpression, bool collapsed, const QgsExpression &collapsedExpression )
+        : widget( widget )
+        , expression( visibilityExpression )
+        , isVisible( true )
+        , isCollapsed( collapsed )
+        , collapsedExpression( collapsedExpression )
+      {}
+
+
       QgsTabWidget *tabWidget = nullptr;
       QWidget *widget = nullptr;
       QgsExpression expression;
       bool isVisible;
+      bool isCollapsed = false;
+      QgsExpression collapsedExpression;
 
       void apply( QgsExpressionContext *expressionContext );
     };
@@ -493,8 +514,8 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
 
     void reloadIcon( const QString &file, const QString &tooltip, QSvgWidget *sw );
 
-    // Contains information about tabs and groupboxes, their visibility state visibility conditions
-    QVector<ContainerInformation *> mContainerVisibilityInformation;
+    // Contains information about tabs and groupboxes, their visibility/collapsed state conditions
+    QVector<ContainerInformation *> mContainerVisibilityCollapsedInformation;
     QMap<QString, QVector<ContainerInformation *> > mContainerInformationDependency;
 
     // Variables below are used for Python
@@ -523,13 +544,13 @@ class GUI_EXPORT QgsAttributeForm : public QWidget
      * Dependency map for default values. Attribute index -> widget wrapper.
      * Attribute indexes will be added multiple times if more than one widget depends on them.
      */
-    QMap<int, QgsWidgetWrapper *> mDefaultValueDependencies;
+    QMultiMap<int, QgsWidgetWrapper *> mDefaultValueDependencies;
 
     /**
      * Dependency map for values from virtual fields. Attribute index -> widget wrapper.
      * Attribute indexes will be added multiple times if more than one widget depends on them.
      */
-    QMap<int, QgsWidgetWrapper *> mVirtualFieldsDependencies;
+    QMultiMap<int, QgsWidgetWrapper *> mVirtualFieldsDependencies;
 
     /**
      * Dependency list for values depending on related layers.
