@@ -420,6 +420,25 @@ void QgsColorButton::dropEvent( QDropEvent *e )
   }
 }
 
+void QgsColorButton::wheelEvent( QWheelEvent *event )
+{
+  if ( mAllowOpacity && isEnabled() && !isNull() )
+  {
+    const double increment = ( ( event->modifiers() & Qt::ControlModifier ) ? 0.01 : 0.1 ) *
+                             ( event->angleDelta().y() > 0 ? 1 : -1 );
+    const double alpha = std::min( std::max( 0.0, mColor.alphaF() + increment ), 1.0 );
+    mColor.setAlphaF( alpha );
+
+    setButtonBackground();
+    emit colorChanged( mColor );
+    event->accept();
+  }
+  else
+  {
+    QToolButton::wheelEvent( event );
+  }
+}
+
 void QgsColorButton::setValidColor( const QColor &newColor )
 {
   if ( newColor.isValid() )
