@@ -22,7 +22,7 @@
 
 #include "qgssettings.h"
 #include "qgsmanageconnectionsdialog.h"
-
+#include "qgshttpheaders.h"
 
 QgsManageConnectionsDialog::QgsManageConnectionsDialog( QWidget *parent, Mode mode, Type type, const QString &fileName )
   : QDialog( parent )
@@ -34,11 +34,11 @@ QgsManageConnectionsDialog::QgsManageConnectionsDialog( QWidget *parent, Mode mo
 
   // additional buttons
   QPushButton *pb = nullptr;
-  pb = new QPushButton( tr( "Select all" ) );
+  pb = new QPushButton( tr( "Select All" ) );
   buttonBox->addButton( pb, QDialogButtonBox::ActionRole );
   connect( pb, &QAbstractButton::clicked, this, &QgsManageConnectionsDialog::selectAll );
 
-  pb = new QPushButton( tr( "Clear selection" ) );
+  pb = new QPushButton( tr( "Clear Selection" ) );
   buttonBox->addButton( pb, QDialogButtonBox::ActionRole );
   connect( pb, &QAbstractButton::clicked, this, &QgsManageConnectionsDialog::clearSelection );
 
@@ -127,9 +127,6 @@ void QgsManageConnectionsDialog::doExportImport()
       case Oracle:
         doc = saveOracleConnections( items );
         break;
-      case DB2:
-        doc = saveDb2Connections( items );
-        break;
       case HANA:
         doc = saveHanaConnections( items );
         break;
@@ -210,9 +207,6 @@ void QgsManageConnectionsDialog::doExportImport()
       case Oracle:
         loadOracleConnections( doc, items );
         break;
-      case DB2:
-        loadDb2Connections( doc, items );
-        break;
       case HANA:
         loadHanaConnections( doc, items );
         break;
@@ -266,9 +260,6 @@ bool QgsManageConnectionsDialog::populateConnections()
       case Oracle:
         settings.beginGroup( QStringLiteral( "/Oracle/connections" ) );
         break;
-      case DB2:
-        settings.beginGroup( QStringLiteral( "/DB2/connections" ) );
-        break;
       case HANA:
         settings.beginGroup( QStringLiteral( "/HANA/connections" ) );
         break;
@@ -370,7 +361,7 @@ bool QgsManageConnectionsDialog::populateConnections()
         if ( root.tagName() != QLatin1String( "qgsMssqlConnections" ) )
         {
           QMessageBox::information( this, tr( "Loading Connections" ),
-                                    tr( "The file is not a MSSQL connections exchange file." ) );
+                                    tr( "The file is not a MS SQL Server connections exchange file." ) );
           return false;
         }
         break;
@@ -382,14 +373,6 @@ bool QgsManageConnectionsDialog::populateConnections()
           return false;
         }
         break;
-      case DB2:
-        if ( root.tagName() != QLatin1String( "qgsDb2Connections" ) )
-        {
-          QMessageBox::information( this, tr( "Loading Connections" ),
-                                    tr( "The file is not a DB2 connections exchange file." ) );
-          return false;
-        }
-        break;
       case HANA:
         if ( root.tagName() != QLatin1String( "qgsHanaConnections" ) )
         {
@@ -474,9 +457,11 @@ QDomDocument QgsManageConnectionsDialog::saveOWSConnections( const QStringList &
       el.setAttribute( QStringLiteral( "ignoreGetFeatureInfoURI" ), settings.value( path + connections[i] + "/ignoreGetFeatureInfoURI", false ).toBool() ? "true" : "false" );
       el.setAttribute( QStringLiteral( "ignoreAxisOrientation" ), settings.value( path + connections[i] + "/ignoreAxisOrientation", false ).toBool() ? "true" : "false" );
       el.setAttribute( QStringLiteral( "invertAxisOrientation" ), settings.value( path + connections[i] + "/invertAxisOrientation", false ).toBool() ? "true" : "false" );
-      el.setAttribute( QStringLiteral( "referer" ), settings.value( path + connections[ i ] + "/referer" ).toString() );
       el.setAttribute( QStringLiteral( "smoothPixmapTransform" ), settings.value( path + connections[i] + "/smoothPixmapTransform", false ).toBool() ? "true" : "false" );
       el.setAttribute( QStringLiteral( "dpiMode" ), settings.value( path + connections[i] + "/dpiMode", "7" ).toInt() );
+
+      QgsHttpHeaders httpHeader( path + connections[ i ] );
+      httpHeader.updateDomElement( el );
     }
 
     path = "/qgis/" + service.toUpper() + '/';
@@ -652,47 +637,6 @@ QDomDocument QgsManageConnectionsDialog::saveOracleConnections( const QStringLis
   return doc;
 }
 
-QDomDocument QgsManageConnectionsDialog::saveDb2Connections( const QStringList &connections )
-{
-  QDomDocument doc( QStringLiteral( "connections" ) );
-  QDomElement root = doc.createElement( QStringLiteral( "qgsDb2Connections" ) );
-  root.setAttribute( QStringLiteral( "version" ), QStringLiteral( "1.0" ) );
-  doc.appendChild( root );
-
-  const QgsSettings settings;
-  QString path;
-  for ( int i = 0; i < connections.count(); ++i )
-  {
-    path = "/DB2/connections/" + connections[ i ];
-    QDomElement el = doc.createElement( QStringLiteral( "db2" ) );
-    el.setAttribute( QStringLiteral( "name" ), connections[ i ] );
-    el.setAttribute( QStringLiteral( "host" ), settings.value( path + "/host" ).toString() );
-    el.setAttribute( QStringLiteral( "port" ), settings.value( path + "/port" ).toString() );
-    el.setAttribute( QStringLiteral( "database" ), settings.value( path + "/database" ).toString() );
-    el.setAttribute( QStringLiteral( "service" ), settings.value( path + "/service" ).toString() );
-    el.setAttribute( QStringLiteral( "sslmode" ), settings.value( path + "/sslmode", "1" ).toString() );
-    el.setAttribute( QStringLiteral( "estimatedMetadata" ), settings.value( path + "/estimatedMetadata", "0" ).toString() );
-
-    el.setAttribute( QStringLiteral( "saveUsername" ), settings.value( path + "/saveUsername", "false" ).toString() );
-
-    if ( settings.value( path + "/saveUsername", "false" ).toString() == QLatin1String( "true" ) )
-    {
-      el.setAttribute( QStringLiteral( "username" ), settings.value( path + "/username" ).toString() );
-    }
-
-    el.setAttribute( QStringLiteral( "savePassword" ), settings.value( path + "/savePassword", "false" ).toString() );
-
-    if ( settings.value( path + "/savePassword", "false" ).toString() == QLatin1String( "true" ) )
-    {
-      el.setAttribute( QStringLiteral( "password" ), settings.value( path + "/password" ).toString() );
-    }
-
-    root.appendChild( el );
-  }
-
-  return doc;
-}
-
 QDomDocument QgsManageConnectionsDialog::saveHanaConnections( const QStringList &connections )
 {
   QDomDocument doc( QStringLiteral( "connections" ) );
@@ -788,9 +732,11 @@ QDomDocument QgsManageConnectionsDialog::saveXyzTilesConnections( const QStringL
     el.setAttribute( QStringLiteral( "authcfg" ), settings.value( path + "/authcfg" ).toString() );
     el.setAttribute( QStringLiteral( "username" ), settings.value( path + "/username" ).toString() );
     el.setAttribute( QStringLiteral( "password" ), settings.value( path + "/password" ).toString() );
-    el.setAttribute( QStringLiteral( "referer" ), settings.value( path + "/referer" ).toString() );
     el.setAttribute( QStringLiteral( "tilePixelRatio" ), settings.value( path + "/tilePixelRatio", 0 ).toDouble() );
 
+    QgsHttpHeaders httpHeader( path );
+    httpHeader.updateDomElement( el );
+
     root.appendChild( el );
   }
 
@@ -812,7 +758,9 @@ QDomDocument QgsManageConnectionsDialog::saveArcgisConnections( const QStringLis
     QDomElement el = doc.createElement( service.toLower() );
     el.setAttribute( QStringLiteral( "name" ), connections[ i ] );
     el.setAttribute( QStringLiteral( "url" ), settings.value( path + connections[ i ] + "/url" ).toString() );
-    el.setAttribute( QStringLiteral( "referer" ), settings.value( path + connections[ i ] + "/referer" ).toString() );
+
+    QgsHttpHeaders httpHeader( path + connections[ i ] );
+    httpHeader.updateDomElement( el );
 
     path = "/qgis/" + service.toUpper() + '/';
     el.setAttribute( QStringLiteral( "username" ), settings.value( path + connections[ i ] + "/username" ).toString() );
@@ -846,9 +794,11 @@ QDomDocument QgsManageConnectionsDialog::saveVectorTileConnections( const QStrin
     el.setAttribute( QStringLiteral( "authcfg" ), settings.value( path + "/authcfg" ).toString() );
     el.setAttribute( QStringLiteral( "username" ), settings.value( path + "/username" ).toString() );
     el.setAttribute( QStringLiteral( "password" ), settings.value( path + "/password" ).toString() );
-    el.setAttribute( QStringLiteral( "referer" ), settings.value( path + "/referer" ).toString() );
     el.setAttribute( QStringLiteral( "styleUrl" ), settings.value( path + "/styleUrl" ).toString() );
 
+    QgsHttpHeaders httpHeader( path );
+    httpHeader.updateDomElement( el );
+
     root.appendChild( el );
   }
 
@@ -933,9 +883,12 @@ void QgsManageConnectionsDialog::loadOWSConnections( const QDomDocument &doc, co
     settings.setValue( QString( '/' + connectionName + "/ignoreGetFeatureInfoURI" ), child.attribute( QStringLiteral( "ignoreGetFeatureInfoURI" ) ) == QLatin1String( "true" ) );
     settings.setValue( QString( '/' + connectionName + "/ignoreAxisOrientation" ), child.attribute( QStringLiteral( "ignoreAxisOrientation" ) ) == QLatin1String( "true" ) );
     settings.setValue( QString( '/' + connectionName + "/invertAxisOrientation" ), child.attribute( QStringLiteral( "invertAxisOrientation" ) ) == QLatin1String( "true" ) );
-    settings.setValue( QString( '/' + connectionName + "/referer" ), child.attribute( QStringLiteral( "referer" ) ) );
     settings.setValue( QString( '/' + connectionName + "/smoothPixmapTransform" ), child.attribute( QStringLiteral( "smoothPixmapTransform" ) ) == QLatin1String( "true" ) );
     settings.setValue( QString( '/' + connectionName + "/dpiMode" ), child.attribute( QStringLiteral( "dpiMode" ), QStringLiteral( "7" ) ).toInt() );
+
+    QgsHttpHeaders httpHeader( child );
+    httpHeader.updateSettings( settings, QString( '/' + connectionName ) );
+
     settings.endGroup();
 
     if ( !child.attribute( QStringLiteral( "username" ) ).isEmpty() )
@@ -1152,7 +1105,7 @@ void QgsManageConnectionsDialog::loadMssqlConnections( const QDomDocument &doc,
   {
     QMessageBox::information( this,
                               tr( "Loading Connections" ),
-                              tr( "The file is not a MSSQL connections exchange file." ) );
+                              tr( "The file is not a MS SQL Server connections exchange file." ) );
     return;
   }
 
@@ -1336,103 +1289,6 @@ void QgsManageConnectionsDialog::loadOracleConnections( const QDomDocument &doc,
   }
 }
 
-void QgsManageConnectionsDialog::loadDb2Connections( const QDomDocument &doc, const QStringList &items )
-{
-  const QDomElement root = doc.documentElement();
-  if ( root.tagName() != QLatin1String( "qgsDb2Connections" ) )
-  {
-    QMessageBox::information( this,
-                              tr( "Loading Connections" ),
-                              tr( "The file is not a DB2 connections exchange file." ) );
-    return;
-  }
-
-  QString connectionName;
-  QgsSettings settings;
-  settings.beginGroup( QStringLiteral( "/DB2/connections" ) );
-  QStringList keys = settings.childGroups();
-  settings.endGroup();
-  QDomElement child = root.firstChildElement();
-  bool prompt = true;
-  bool overwrite = true;
-
-  while ( !child.isNull() )
-  {
-    connectionName = child.attribute( QStringLiteral( "name" ) );
-    if ( !items.contains( connectionName ) )
-    {
-      child = child.nextSiblingElement();
-      continue;
-    }
-
-    // check for duplicates
-    if ( keys.contains( connectionName ) && prompt )
-    {
-      const int res = QMessageBox::warning( this,
-                                            tr( "Loading Connections" ),
-                                            tr( "Connection with name '%1' already exists. Overwrite?" )
-                                            .arg( connectionName ),
-                                            QMessageBox::Yes | QMessageBox::YesToAll | QMessageBox::No | QMessageBox::NoToAll | QMessageBox::Cancel );
-      switch ( res )
-      {
-        case QMessageBox::Cancel:
-          return;
-        case QMessageBox::No:
-          child = child.nextSiblingElement();
-          continue;
-        case QMessageBox::Yes:
-          overwrite = true;
-          break;
-        case QMessageBox::YesToAll:
-          prompt = false;
-          overwrite = true;
-          break;
-        case QMessageBox::NoToAll:
-          prompt = false;
-          overwrite = false;
-          break;
-      }
-    }
-
-    if ( keys.contains( connectionName ) )
-    {
-      if ( !overwrite )
-      {
-        child = child.nextSiblingElement();
-        continue;
-      }
-    }
-    else
-    {
-      keys << connectionName;
-    }
-
-    //no dups detected or overwrite is allowed
-    settings.beginGroup( "/DB2/connections/" + connectionName );
-
-    settings.setValue( QStringLiteral( "/host" ), child.attribute( QStringLiteral( "host" ) ) );
-    settings.setValue( QStringLiteral( "/port" ), child.attribute( QStringLiteral( "port" ) ) );
-    settings.setValue( QStringLiteral( "/database" ), child.attribute( QStringLiteral( "database" ) ) );
-    if ( child.hasAttribute( QStringLiteral( "service" ) ) )
-    {
-      settings.setValue( QStringLiteral( "/service" ), child.attribute( QStringLiteral( "service" ) ) );
-    }
-    else
-    {
-      settings.setValue( QStringLiteral( "/service" ), "" );
-    }
-    settings.setValue( QStringLiteral( "/sslmode" ), child.attribute( QStringLiteral( "sslmode" ) ) );
-    settings.setValue( QStringLiteral( "/estimatedMetadata" ), child.attribute( QStringLiteral( "estimatedMetadata" ) ) );
-    settings.setValue( QStringLiteral( "/saveUsername" ), child.attribute( QStringLiteral( "saveUsername" ) ) );
-    settings.setValue( QStringLiteral( "/username" ), child.attribute( QStringLiteral( "username" ) ) );
-    settings.setValue( QStringLiteral( "/savePassword" ), child.attribute( QStringLiteral( "savePassword" ) ) );
-    settings.setValue( QStringLiteral( "/password" ), child.attribute( QStringLiteral( "password" ) ) );
-    settings.endGroup();
-
-    child = child.nextSiblingElement();
-  }
-}
-
 void QgsManageConnectionsDialog::loadHanaConnections( const QDomDocument &doc, const QStringList &items )
 {
   QDomElement root = doc.documentElement();
@@ -1693,8 +1549,11 @@ void QgsManageConnectionsDialog::loadXyzTilesConnections( const QDomDocument &do
     settings.setValue( QStringLiteral( "authcfg" ), child.attribute( QStringLiteral( "authcfg" ) ) );
     settings.setValue( QStringLiteral( "username" ), child.attribute( QStringLiteral( "username" ) ) );
     settings.setValue( QStringLiteral( "password" ), child.attribute( QStringLiteral( "password" ) ) );
-    settings.setValue( QStringLiteral( "referer" ), child.attribute( QStringLiteral( "referer" ) ) );
     settings.setValue( QStringLiteral( "tilePixelRatio" ), child.attribute( QStringLiteral( "tilePixelRatio" ) ) );
+
+    QgsHttpHeaders httpHeader( child );
+    httpHeader.updateSettings( settings );
+
     settings.endGroup();
 
     child = child.nextSiblingElement();
@@ -1775,7 +1634,10 @@ void QgsManageConnectionsDialog::loadArcgisConnections( const QDomDocument &doc,
     // no dups detected or overwrite is allowed
     settings.beginGroup( "/qgis/connections-" + service.toLower() );
     settings.setValue( QString( '/' + connectionName + "/url" ), child.attribute( QStringLiteral( "url" ) ) );
-    settings.setValue( QString( '/' + connectionName + "/referer" ), child.attribute( QStringLiteral( "referer" ) ) );
+
+    QgsHttpHeaders httpHeader( child );
+    httpHeader.updateSettings( settings, QString( '/' + connectionName ) );
+
     settings.endGroup();
 
     settings.beginGroup( "/qgis/" + service.toUpper() + '/' + connectionName );
@@ -1867,9 +1729,11 @@ void QgsManageConnectionsDialog::loadVectorTileConnections( const QDomDocument &
     settings.setValue( QStringLiteral( "authcfg" ), child.attribute( QStringLiteral( "authcfg" ) ) );
     settings.setValue( QStringLiteral( "username" ), child.attribute( QStringLiteral( "username" ) ) );
     settings.setValue( QStringLiteral( "password" ), child.attribute( QStringLiteral( "password" ) ) );
-    settings.setValue( QStringLiteral( "referer" ), child.attribute( QStringLiteral( "referer" ) ) );
     settings.setValue( QStringLiteral( "styleUrl" ), child.attribute( QStringLiteral( "styleUrl" ) ) );
 
+    QgsHttpHeaders httpHeader( child );
+    httpHeader.updateSettings( settings );
+
     settings.endGroup();
 
     child = child.nextSiblingElement();
