@@ -73,6 +73,16 @@ class GUI_EXPORT QgsMapLayerComboBox : public QComboBox
      */
     QStringList excludedProviders() const;
 
+    /**
+     * Sets the \a project from which map layers are shown.
+     *
+     * If \a project is NULLPTR then QgsProject::instance() will be used.
+     *
+     * \since QGIS 3.24
+     */
+    void setProject( QgsProject *project );
+
+
     /**
      * Sets whether an optional empty layer ("not set") option is shown in the combo box.
      *
