@@ -26,10 +26,7 @@
 #include "qgslogger.h"
 #include "qgsmapcanvas.h"
 #include "qgsmaptoolidentify.h"
-#include "qgsmaptopixel.h"
-#include "qgsmessageviewer.h"
 #include "qgsmeshlayer.h"
-#include "qgsmeshlayertemporalproperties.h"
 #include "qgsmaplayer.h"
 #include "qgsrasterdataprovider.h"
 #include "qgsrasterlayer.h"
@@ -53,10 +50,9 @@
 #include "qgsexpressioncontextutils.h"
 #include "qgspointcloudlayer.h"
 #include "qgspointcloudrenderer.h"
-#include "qgspointcloudlayerrenderer.h"
 #include "qgspointcloudlayerelevationproperties.h"
 #include "qgssymbol.h"
-#include "qgsmultilinestring.h"
+#include "qgsguiutils.h"
 
 #include <QMouseEvent>
 #include <QCursor>
@@ -157,23 +153,18 @@ QList<QgsMapToolIdentify::IdentifyResult> QgsMapToolIdentify::identify( const Qg
   {
     QApplication::setOverrideCursor( Qt::WaitCursor );
 
-    int layerCount;
+    QList< QgsMapLayer * > targetLayers;
     if ( layerList.isEmpty() )
-      layerCount = mCanvas->layerCount();
+      targetLayers = mCanvas->layers( true );
     else
-      layerCount = layerList.count();
-
+      targetLayers = layerList;
 
+    const int layerCount = targetLayers.size();
     for ( int i = 0; i < layerCount; i++ )
     {
+      QgsMapLayer *layer = targetLayers.value( i );
 
-      QgsMapLayer *layer = nullptr;
-      if ( layerList.isEmpty() )
-        layer = mCanvas->layer( i );
-      else
-        layer = layerList.value( i );
-
-      emit identifyProgress( i, mCanvas->layerCount() );
+      emit identifyProgress( i, layerCount );
       emit identifyMessage( tr( "Identifying on %1…" ).arg( layer->name() ) );
 
       if ( !layer->flags().testFlag( QgsMapLayer::Identifiable ) )
@@ -186,7 +177,7 @@ QList<QgsMapToolIdentify::IdentifyResult> QgsMapToolIdentify::identify( const Qg
       }
     }
 
-    emit identifyProgress( mCanvas->layerCount(), mCanvas->layerCount() );
+    emit identifyProgress( layerCount, layerCount );
     emit identifyMessage( tr( "Identifying done." ) );
   }
 
@@ -452,9 +443,9 @@ bool QgsMapToolIdentify::identifyVectorTileLayer( QList<QgsMapToolIdentify::Iden
       }
     }
 
-    int tileZoom = QgsVectorTileUtils::scaleToZoomLevel( mCanvas->scale(), layer->sourceMinZoom(), layer->sourceMaxZoom() );
-    QgsTileMatrix tileMatrix = QgsTileMatrix::fromWebMercator( tileZoom );
-    QgsTileRange tileRange = tileMatrix.tileRangeFromExtent( r );
+    const int tileZoom = layer->tileMatrixSet().scaleToZoomLevel( mCanvas->scale() );
+    const QgsTileMatrix tileMatrix = layer->tileMatrixSet().tileMatrix( tileZoom );
+    const QgsTileRange tileRange = tileMatrix.tileRangeFromExtent( r );
 
     for ( int row = tileRange.startRow(); row <= tileRange.endRow(); ++row )
     {
@@ -465,7 +456,7 @@ bool QgsMapToolIdentify::identifyVectorTileLayer( QList<QgsMapToolIdentify::Iden
         if ( data.isEmpty() )
           continue;  // failed to get data
 
-        QgsVectorTileMVTDecoder decoder;
+        QgsVectorTileMVTDecoder decoder( layer->tileMatrixSet() );
         if ( !decoder.decode( tileID, data ) )
           continue;  // failed to decode
 
@@ -530,8 +521,13 @@ bool QgsMapToolIdentify::identifyPointCloudLayer( QList<QgsMapToolIdentify::Iden
 QMap<QString, QString> QgsMapToolIdentify::derivedAttributesForPoint( const QgsPoint &point )
 {
   QMap< QString, QString > derivedAttributes;
-  derivedAttributes.insert( tr( "(clicked coordinate X)" ), formatXCoordinate( point ) );
-  derivedAttributes.insert( tr( "(clicked coordinate Y)" ), formatYCoordinate( point ) );
+
+  QString x;
+  QString y;
+  formatCoordinate( point, x, y );
+
+  derivedAttributes.insert( tr( "(clicked coordinate X)" ), x );
+  derivedAttributes.insert( tr( "(clicked coordinate Y)" ), y );
   if ( point.is3D() )
     derivedAttributes.insert( tr( "(clicked coordinate Z)" ), QLocale().toString( point.z(), 'f' ) );
   return derivedAttributes;
@@ -567,14 +563,14 @@ bool QgsMapToolIdentify::identifyVectorLayer( QList<QgsMapToolIdentify::Identify
 
   QgsGeometry selectionGeom = geometry;
   bool isPointOrRectangle;
-  QgsPointXY point;
+  QgsPoint point;
   bool isSingleClick = selectionGeom.type() == QgsWkbTypes::PointGeometry;
   if ( isSingleClick )
   {
     isPointOrRectangle = true;
-    point = selectionGeom.asPoint();
+    point = *qgsgeometry_cast< const QgsPoint *>( selectionGeom.constGet() );
 
-    commonDerivedAttributes = derivedAttributesForPoint( QgsPoint( point ) );
+    commonDerivedAttributes = derivedAttributesForPoint( point );
   }
   else
   {
@@ -582,8 +578,6 @@ bool QgsMapToolIdentify::identifyVectorLayer( QList<QgsMapToolIdentify::Identify
     isPointOrRectangle = QgsGeometry::fromRect( selectionGeom.boundingBox() ).isGeosEqual( selectionGeom );
   }
 
-  int featureCount = 0;
-
   QgsFeatureList featureList;
   std::unique_ptr<QgsGeometryEngine> selectionGeomPrepared;
 
@@ -647,37 +641,49 @@ bool QgsMapToolIdentify::identifyVectorLayer( QList<QgsMapToolIdentify::Identify
     filter = renderer->capabilities() & QgsFeatureRenderer::Filter;
   }
 
-  for ( const QgsFeature &feature : std::as_const( featureList ) )
+  // When not single click identify, pass an empty point so some derived attributes may still be computed
+  if ( !isSingleClick )
+    point = QgsPoint();
+
+  const int featureCount = identifyVectorLayer( results, layer, featureList, filter ? renderer.get() : nullptr, commonDerivedAttributes,
+                           [point, layer, this]( const QgsFeature & feature )->QMap< QString, QString >
+  {
+    return featureDerivedAttributes( feature, layer, toLayerCoordinates( layer, point ) );
+  }, context );
+
+  if ( renderer )
+  {
+    renderer->stopRender( context );
+  }
+  QApplication::restoreOverrideCursor();
+  return featureCount > 0;
+}
+
+int QgsMapToolIdentify::identifyVectorLayer( QList<IdentifyResult> *results, QgsVectorLayer *layer, const QgsFeatureList &features, QgsFeatureRenderer *renderer, const QMap< QString, QString > &commonDerivedAttributes, const std::function< QMap< QString, QString > ( const QgsFeature & ) > &deriveAttributesForFeature, QgsRenderContext &context )
+{
+  int featureCount = 0;
+  for ( const QgsFeature &feature : std::as_const( features ) )
   {
     QMap< QString, QString > derivedAttributes = commonDerivedAttributes;
 
     QgsFeatureId fid = feature.id();
     context.expressionContext().setFeature( feature );
 
-    if ( filter && !renderer->willRenderFeature( feature, context ) )
+    if ( renderer && !renderer->willRenderFeature( feature, context ) )
       continue;
 
-    featureCount++;
-
-    // When not single click identify, pass an empty point so some derived attributes may still be computed
-    if ( !isSingleClick )
-      point = QgsPointXY();
-    derivedAttributes.unite( featureDerivedAttributes( feature, layer, toLayerCoordinates( layer, point ) ) );
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
+    derivedAttributes.unite( deriveAttributesForFeature( feature ) );
+#else
+    derivedAttributes.insert( deriveAttributesForFeature( feature ) );
+#endif
 
     derivedAttributes.insert( tr( "Feature ID" ), fid < 0 ? tr( "new feature" ) : FID_TO_STRING( fid ) );
 
     results->append( IdentifyResult( qobject_cast<QgsMapLayer *>( layer ), feature, derivedAttributes ) );
+    featureCount++;
   }
-
-  if ( renderer )
-  {
-    renderer->stopRender( context );
-  }
-
-  QgsDebugMsgLevel( "Feature count on identify: " + QString::number( featureCount ), 2 );
-
-  QApplication::restoreOverrideCursor();
-  return featureCount > 0;
+  return featureCount;
 }
 
 void QgsMapToolIdentify::closestVertexAttributes( const QgsAbstractGeometry &geometry, QgsVertexId vId, QgsMapLayer *layer, QMap< QString, QString > &derivedAttributes )
@@ -694,18 +700,22 @@ void QgsMapToolIdentify::closestVertexAttributes( const QgsAbstractGeometry &geo
 
   QgsPoint closestPoint = geometry.vertexAt( vId );
 
-  QgsPointXY closestPointMapCoords = mCanvas->mapSettings().layerToMapCoordinates( layer, QgsPointXY( closestPoint.x(), closestPoint.y() ) );
-  derivedAttributes.insert( tr( "Closest vertex X" ), formatXCoordinate( closestPointMapCoords ) );
-  derivedAttributes.insert( tr( "Closest vertex Y" ), formatYCoordinate( closestPointMapCoords ) );
+  QgsPoint closestPointMapCoords = mCanvas->mapSettings().layerToMapCoordinates( layer, closestPoint );
+
+  QString x;
+  QString y;
+  formatCoordinate( closestPointMapCoords, x, y );
+  derivedAttributes.insert( tr( "Closest vertex X" ), x );
+  derivedAttributes.insert( tr( "Closest vertex Y" ), y );
 
   if ( closestPoint.is3D() )
   {
-    str = QLocale().toString( closestPoint.z(), 'g', 10 );
+    str = QLocale().toString( closestPointMapCoords.z(), 'g', 10 );
     derivedAttributes.insert( tr( "Closest vertex Z" ), str );
   }
   if ( closestPoint.isMeasure() )
   {
-    str = QLocale().toString( closestPoint.m(), 'g', 10 );
+    str = QLocale().toString( closestPointMapCoords.m(), 'g', 10 );
     derivedAttributes.insert( tr( "Closest vertex M" ), str );
   }
 
@@ -726,8 +736,11 @@ void QgsMapToolIdentify::closestPointAttributes( const QgsAbstractGeometry &geom
 {
   QgsPoint closestPoint = QgsGeometryUtils::closestPoint( geometry, QgsPoint( layerPoint ) );
 
-  derivedAttributes.insert( tr( "Closest X" ), formatXCoordinate( closestPoint ) );
-  derivedAttributes.insert( tr( "Closest Y" ), formatYCoordinate( closestPoint ) );
+  QString x;
+  QString y;
+  formatCoordinate( closestPoint, x, y );
+  derivedAttributes.insert( tr( "Closest X" ), x );
+  derivedAttributes.insert( tr( "Closest Y" ), y );
 
   if ( closestPoint.is3D() )
   {
@@ -741,22 +754,10 @@ void QgsMapToolIdentify::closestPointAttributes( const QgsAbstractGeometry &geom
   }
 }
 
-QString QgsMapToolIdentify::formatCoordinate( const QgsPointXY &canvasPoint ) const
+void QgsMapToolIdentify::formatCoordinate( const QgsPointXY &canvasPoint, QString &x, QString &y ) const
 {
-  return QgsCoordinateUtils::formatCoordinateForProject( QgsProject::instance(), canvasPoint, mCanvas->mapSettings().destinationCrs(),
-         mCoordinatePrecision );
-}
-
-QString QgsMapToolIdentify::formatXCoordinate( const QgsPointXY &canvasPoint ) const
-{
-  QString coordinate = formatCoordinate( canvasPoint );
-  return coordinate.split( QgsCoordinateFormatter::separator() ).at( 0 );
-}
-
-QString QgsMapToolIdentify::formatYCoordinate( const QgsPointXY &canvasPoint ) const
-{
-  QString coordinate = formatCoordinate( canvasPoint );
-  return coordinate.split( QgsCoordinateFormatter::separator() ).at( 1 );
+  QgsCoordinateUtils::formatCoordinatePartsForProject( QgsProject::instance(), canvasPoint, mCanvas->mapSettings().destinationCrs(),
+      mCoordinatePrecision, x, y );
 }
 
 QMap< QString, QString > QgsMapToolIdentify::featureDerivedAttributes( const QgsFeature &feature, QgsMapLayer *layer, const QgsPointXY &layerPoint )
@@ -852,15 +853,15 @@ QMap< QString, QString > QgsMapToolIdentify::featureDerivedAttributes( const Qgs
     {
       // Add the start and end points in as derived attributes
       QgsPointXY pnt = mCanvas->mapSettings().layerToMapCoordinates( layer, QgsPointXY( curve->startPoint().x(), curve->startPoint().y() ) );
-      str = formatXCoordinate( pnt );
-      derivedAttributes.insert( tr( "firstX", "attributes get sorted; translation for lastX should be lexically larger than this one" ), str );
-      str = formatYCoordinate( pnt );
-      derivedAttributes.insert( tr( "firstY" ), str );
+      QString x;
+      QString y;
+      formatCoordinate( pnt, x, y );
+      derivedAttributes.insert( tr( "firstX", "attributes get sorted; translation for lastX should be lexically larger than this one" ), x );
+      derivedAttributes.insert( tr( "firstY" ), y );
       pnt = mCanvas->mapSettings().layerToMapCoordinates( layer, QgsPointXY( curve->endPoint().x(), curve->endPoint().y() ) );
-      str = formatXCoordinate( pnt );
-      derivedAttributes.insert( tr( "lastX", "attributes get sorted; translation for firstX should be lexically smaller than this one" ), str );
-      str = formatYCoordinate( pnt );
-      derivedAttributes.insert( tr( "lastY" ), str );
+      formatCoordinate( pnt, x, y );
+      derivedAttributes.insert( tr( "lastX", "attributes get sorted; translation for firstX should be lexically smaller than this one" ), x );
+      derivedAttributes.insert( tr( "lastY" ), y );
     }
   }
   else if ( geometryType == QgsWkbTypes::PolygonGeometry )
@@ -904,19 +905,20 @@ QMap< QString, QString > QgsMapToolIdentify::featureDerivedAttributes( const Qgs
     {
       // Include the x and y coordinates of the point as a derived attribute
       QgsPointXY pnt = mCanvas->mapSettings().layerToMapCoordinates( layer, feature.geometry().asPoint() );
-      QString str = formatXCoordinate( pnt );
-      derivedAttributes.insert( tr( "X" ), str );
-      str = formatYCoordinate( pnt );
-      derivedAttributes.insert( tr( "Y" ), str );
+      QString x;
+      QString y;
+      formatCoordinate( pnt, x, y );
+      derivedAttributes.insert( tr( "X" ), x );
+      derivedAttributes.insert( tr( "Y" ), y );
 
       if ( QgsWkbTypes::hasZ( wkbType ) )
       {
-        str = QLocale().toString( static_cast<const QgsPoint *>( feature.geometry().constGet() )->z(), 'g', 10 );
+        const QString str = QLocale().toString( static_cast<const QgsPoint *>( feature.geometry().constGet() )->z(), 'g', 10 );
         derivedAttributes.insert( tr( "Z" ), str );
       }
       if ( QgsWkbTypes::hasM( wkbType ) )
       {
-        str = QLocale().toString( static_cast<const QgsPoint *>( feature.geometry().constGet() )->m(), 'g', 10 );
+        const QString str = QLocale().toString( static_cast<const QgsPoint *>( feature.geometry().constGet() )->m(), 'g', 10 );
         derivedAttributes.insert( tr( "M" ), str );
       }
     }
@@ -1043,7 +1045,11 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
     identifyResult = dprovider->identify( point, format, viewExtent, width, height );
   }
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
   derivedAttributes.unite( derivedAttributesForPoint( QgsPoint( pointInCanvasCrs ) ) );
+#else
+  derivedAttributes.insert( derivedAttributesForPoint( QgsPoint( pointInCanvasCrs ) ) );
+#endif
 
   if ( identifyResult.isValid() )
   {
@@ -1054,7 +1060,7 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
       for ( auto it = values.constBegin(); it != values.constEnd(); ++it )
       {
         QString valueString;
-        if ( it.value().isNull() )
+        if ( QgsVariantUtils::isNull( it.value() ) )
         {
           valueString = tr( "no data" );
         }
@@ -1128,7 +1134,11 @@ bool QgsMapToolIdentify::identifyRasterLayer( QList<IdentifyResult> *results, Qg
             }
 
             QMap< QString, QString > derAttributes = derivedAttributes;
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
             derAttributes.unite( featureDerivedAttributes( feature, layer, toLayerCoordinates( layer, point ) ) );
+#else
+            derAttributes.insert( featureDerivedAttributes( feature, layer, toLayerCoordinates( layer, point ) ) );
+#endif
 
             IdentifyResult identifyResult( qobject_cast<QgsMapLayer *>( layer ), labels.join( QLatin1String( " / " ) ), featureStore.fields(), feature, derAttributes );
 
@@ -1189,7 +1199,7 @@ QString QgsMapToolIdentify::formatDistance( double distance, QgsUnitTypes::Dista
   QgsSettings settings;
   bool baseUnit = settings.value( QStringLiteral( "qgis/measure/keepbaseunit" ), true ).toBool();
 
-  return QgsDistanceArea::formatDistance( distance, 3, unit, baseUnit );
+  return QgsDistanceArea::formatDistance( distance, mCoordinatePrecision, unit, baseUnit );
 }
 
 QString QgsMapToolIdentify::formatArea( double area, QgsUnitTypes::AreaUnit unit ) const
@@ -1197,7 +1207,7 @@ QString QgsMapToolIdentify::formatArea( double area, QgsUnitTypes::AreaUnit unit
   QgsSettings settings;
   bool baseUnit = settings.value( QStringLiteral( "qgis/measure/keepbaseunit" ), true ).toBool();
 
-  return QgsDistanceArea::formatArea( area, 3, unit, baseUnit );
+  return QgsDistanceArea::formatArea( area, mCoordinatePrecision, unit, baseUnit );
 }
 
 void QgsMapToolIdentify::formatChanged( QgsRasterLayer *layer )
@@ -1241,3 +1251,98 @@ void QgsMapToolIdentify::fromPointCloudIdentificationToIdentifyResults( QgsPoint
     ++id;
   }
 }
+
+void QgsMapToolIdentify::fromElevationProfileLayerIdentificationToIdentifyResults( QgsMapLayer *layer, const QVector<QVariantMap> &identified, QList<IdentifyResult> &results )
+{
+  if ( !layer )
+    return;
+
+  if ( identified.empty() )
+    return;
+
+  switch ( layer->type() )
+  {
+    case QgsMapLayerType::VectorLayer:
+    {
+      QgsVectorLayer *vl = qobject_cast< QgsVectorLayer * >( layer );
+
+      QgsFeatureList features;
+      QHash< QgsFeatureId, QVariant > featureDistances;
+      QHash< QgsFeatureId, QVariant > featureElevations;
+
+      QgsFeatureIds filterIds;
+      for ( const QVariantMap &map : identified )
+      {
+        if ( !map.contains( QStringLiteral( "id" ) ) )
+        {
+          QMap< QString, QString > attributes;
+          if ( map.value( QStringLiteral( "distance" ) ).isValid() )
+            attributes.insert( tr( "Distance along curve" ), QString::number( map.value( QStringLiteral( "distance" ) ).toDouble() ) );
+          if ( map.value( QStringLiteral( "elevation" ) ).isValid() )
+            attributes.insert( tr( "Elevation" ), QString::number( map.value( QStringLiteral( "elevation" ) ).toDouble() ) );
+
+          results.append( IdentifyResult( layer, layer->name(), {}, attributes ) );
+        }
+        else
+        {
+          const QgsFeatureId id = map.value( QStringLiteral( "id" ) ).toLongLong();
+          filterIds.insert( id );
+
+          featureDistances.insert( id, map.value( QStringLiteral( "distance" ) ) );
+          featureElevations.insert( id, map.value( QStringLiteral( "elevation" ) ) );
+        }
+      }
+
+      QgsFeatureRequest request;
+      request.setFilterFids( filterIds );
+      QgsFeatureIterator it = vl->getFeatures( request );
+      QgsFeature f;
+      while ( it.nextFeature( f ) )
+        features << f;
+
+      QgsRenderContext context;
+      identifyVectorLayer( &results, vl, features, nullptr, QMap< QString, QString >(), [this, vl, &featureDistances, &featureElevations]( const QgsFeature & feature )->QMap< QString, QString >
+      {
+        QMap< QString, QString > attributes = featureDerivedAttributes( feature, vl, QgsPointXY() );
+
+        if ( featureDistances.value( feature.id() ).isValid() )
+          attributes.insert( tr( "Distance along curve" ), QString::number( featureDistances.value( feature.id() ).toDouble() ) );
+        if ( featureElevations.value( feature.id() ).isValid() )
+          attributes.insert( tr( "Elevation" ), QString::number( featureElevations.value( feature.id() ).toDouble() ) );
+
+        return attributes;
+      }, context );
+      break;
+    }
+
+    case QgsMapLayerType::RasterLayer:
+    case QgsMapLayerType::MeshLayer:
+    {
+      for ( const QVariantMap &map : identified )
+      {
+        QMap< QString, QString > attributes;
+        if ( map.value( QStringLiteral( "distance" ) ).isValid() )
+          attributes.insert( tr( "Distance along curve" ), QString::number( map.value( QStringLiteral( "distance" ) ).toDouble() ) );
+        if ( map.value( QStringLiteral( "elevation" ) ).isValid() )
+          attributes.insert( tr( "Elevation" ), QString::number( map.value( QStringLiteral( "elevation" ) ).toDouble() ) );
+
+        results.append( IdentifyResult( layer, layer->name(), {}, attributes ) );
+      }
+
+      break;
+    }
+
+    case QgsMapLayerType::PointCloudLayer:
+    {
+      QgsPointCloudLayer *pcLayer = qobject_cast< QgsPointCloudLayer * >( layer );
+      fromPointCloudIdentificationToIdentifyResults( pcLayer, identified, results );
+      break;
+    }
+
+    case QgsMapLayerType::PluginLayer:
+    case QgsMapLayerType::VectorTileLayer:
+    case QgsMapLayerType::AnnotationLayer:
+    case QgsMapLayerType::GroupLayer:
+      break;
+  }
+}
