@@ -39,7 +39,7 @@ QgsNewVectorTableDialog::QgsNewVectorTableDialog( QgsAbstractDatabaseProviderCon
   {
     QMessageBox::critical( nullptr, tr( "Cannot Create New Tables" ), tr( "Error retrieving native types from the data provider: creation of new tables is not possible.\n"
                            "Error message: %1" ).arg( ex.what() ) );
-    QTimer::singleShot( 0, [ = ] { reject(); } );
+    QTimer::singleShot( 0, this, [ = ] { reject(); } );
     return;
   }
 
@@ -140,7 +140,9 @@ QgsNewVectorTableDialog::QgsNewVectorTableDialog( QgsAbstractDatabaseProviderCon
   mCrs->setShowAccuracyWarnings( true );
 
   // geometry types
+  Q_NOWARN_DEPRECATED_PUSH
   const bool hasSinglePart { conn->geometryColumnCapabilities().testFlag( QgsAbstractDatabaseProviderConnection::GeometryColumnCapability::SinglePart ) };
+  Q_NOWARN_DEPRECATED_POP
 
   const auto addGeomItem = [this]( QgsWkbTypes::Type type )
   {
@@ -148,13 +150,13 @@ QgsNewVectorTableDialog::QgsNewVectorTableDialog( QgsAbstractDatabaseProviderCon
   };
 
   mGeomTypeCbo->addItem( QgsApplication::getThemeIcon( QStringLiteral( "mIconTableLayer.svg" ) ), tr( "No Geometry" ), QgsWkbTypes::Type::NoGeometry );
-  if ( hasSinglePart )
+  if ( hasSinglePart || conn->geometryColumnCapabilities().testFlag( QgsAbstractDatabaseProviderConnection::GeometryColumnCapability::SinglePoint ) )
     addGeomItem( QgsWkbTypes::Type::Point );
   addGeomItem( QgsWkbTypes::Type::MultiPoint );
-  if ( hasSinglePart )
+  if ( hasSinglePart || conn->geometryColumnCapabilities().testFlag( QgsAbstractDatabaseProviderConnection::GeometryColumnCapability::SingleLineString ) )
     addGeomItem( QgsWkbTypes::Type::LineString );
   addGeomItem( QgsWkbTypes::Type::MultiLineString );
-  if ( hasSinglePart )
+  if ( hasSinglePart || conn->geometryColumnCapabilities().testFlag( QgsAbstractDatabaseProviderConnection::GeometryColumnCapability::SinglePolygon ) )
     addGeomItem( QgsWkbTypes::Type::Polygon );
   addGeomItem( QgsWkbTypes::Type::MultiPolygon );
 
@@ -432,7 +434,7 @@ QWidget *QgsNewVectorTableDialogFieldsDelegate::createEditor( QWidget *parent, c
       connect( cbo, qOverload<int>( &QComboBox::currentIndexChanged ), this, &QgsNewVectorTableDialogFieldsDelegate::onFieldTypeChanged );
       for ( const auto &f : std::as_const( mTypeList ) )
       {
-        cbo->addItem( f.mTypeDesc, f.mTypeName );
+        cbo->addItem( QgsFields::iconForFieldType( f.mType, f.mSubType ), f.mTypeDesc, f.mTypeName );
       }
       return cbo;
     }
