@@ -33,6 +33,10 @@
 #include "qgseptproviderguimetadata.h"
 #endif
 
+#ifdef HAVE_COPC
+#include "qgscopcproviderguimetadata.h"
+#endif
+
 #ifdef HAVE_STATIC_PROVIDERS
 #include "qgswmsprovidergui.h"
 #include "qgswcsprovidergui.h"
@@ -92,6 +96,11 @@ void QgsProviderGuiRegistry::loadStaticProviders( )
   mProviders[ ept->key() ] = ept;
 #endif
 
+#ifdef HAVE_COPC
+  QgsProviderGuiMetadata *copc = new QgsCopcProviderGuiMetadata();
+  mProviders[ copc->key() ] = copc;
+#endif
+
   // only show point cloud option if we have at least one point cloud provider available!
   if ( !QgsProviderRegistry::instance()->filePointCloudFilters().isEmpty() )
   {
@@ -210,11 +219,9 @@ QgsProviderGuiRegistry::~QgsProviderGuiRegistry()
 
 void QgsProviderGuiRegistry::registerGuis( QMainWindow *parent )
 {
-  GuiProviders::const_iterator it = mProviders.begin();
-  while ( it != mProviders.end() )
+  for ( auto it = mProviders.begin(); it != mProviders.end(); ++it )
   {
     it->second->registerGui( parent );
-    ++it;
   }
 }
 
