@@ -53,17 +53,47 @@ QgsSymbolButton::QgsSymbolButton( QWidget *parent, const QString &dialogTitle )
   setMenu( mMenu );
   setPopupMode( QToolButton::MenuButtonPopup );
 
+  updateSizeHint();
+}
+
+void QgsSymbolButton::updateSizeHint()
+{
   //make sure height of button looks good under different platforms
   const QSize size = QToolButton::minimumSizeHint();
   const int fontHeight = static_cast< int >( Qgis::UI_SCALE_FACTOR * fontMetrics().height() * 1.4 );
-  mSizeHint = QSize( size.width(), std::max( size.height(), fontHeight ) );
+  switch ( mType )
+  {
+    case Qgis::SymbolType::Marker:
+      if ( mSymbol )
+      {
+        mSizeHint = QSize( size.width(), std::max( size.height(), fontHeight * 3 ) );
+        setMaximumWidth( mSizeHint.height() * 1.5 );
+        setMinimumWidth( maximumWidth() );
+      }
+      else
+      {
+        mSizeHint = QSize( size.width(), fontHeight );
+        setMaximumWidth( 999999 );
+        mSizeHint.setWidth( QToolButton::sizeHint().width() );
+      }
+      break;
+
+    case Qgis::SymbolType::Line:
+    case Qgis::SymbolType::Fill:
+    case Qgis::SymbolType::Hybrid:
+      mSizeHint = QSize( size.width(), std::max( size.height(), fontHeight ) );
+      break;
+  }
+
+  setMinimumHeight( mSizeHint.height( ) );
+
+  updateGeometry();
 }
 
 QgsSymbolButton::~QgsSymbolButton() = default;
 
 QSize QgsSymbolButton::minimumSizeHint() const
 {
-
   return mSizeHint;
 }
 
@@ -94,8 +124,9 @@ void QgsSymbolButton::setSymbolType( Qgis::SymbolType type )
         break;
     }
   }
-  updatePreview();
   mType = type;
+  updateSizeHint();
+  updatePreview();
 }
 
 void QgsSymbolButton::showSettingsDialog()
@@ -107,7 +138,29 @@ void QgsSymbolButton::showSettingsDialog()
   {
     context.appendScopes( QgsExpressionContextUtils::globalProjectLayerScopes( mLayer.data() ) );
   }
-  QgsSymbol *newSymbol = mSymbol->clone();
+
+  QgsSymbol *newSymbol = nullptr;
+  if ( mSymbol )
+  {
+    newSymbol = mSymbol->clone();
+  }
+  else
+  {
+    switch ( mType )
+    {
+      case Qgis::SymbolType::Marker:
+        newSymbol = QgsSymbol::defaultSymbol( QgsWkbTypes::PointGeometry );
+        break;
+      case Qgis::SymbolType::Line:
+        newSymbol = QgsSymbol::defaultSymbol( QgsWkbTypes::LineGeometry );
+        break;
+      case Qgis::SymbolType::Fill:
+        newSymbol = QgsSymbol::defaultSymbol( QgsWkbTypes::PolygonGeometry );
+        break;
+      case Qgis::SymbolType::Hybrid:
+        break;
+    }
+  }
 
   QgsSymbolWidgetContext symbolContext;
   symbolContext.setExpressionContext( &context );
@@ -196,12 +249,16 @@ void QgsSymbolButton::registerExpressionContextGenerator( QgsExpressionContextGe
 void QgsSymbolButton::setSymbol( QgsSymbol *symbol )
 {
   mSymbol.reset( symbol );
+  updateSizeHint();
   updatePreview();
   emit changed();
 }
 
 void QgsSymbolButton::setColor( const QColor &color )
 {
+  if ( !mSymbol )
+    return;
+
   QColor opaque = color;
   opaque.setAlphaF( 1.0 );
 
@@ -363,6 +420,58 @@ void QgsSymbolButton::dropEvent( QDropEvent *e )
   updatePreview();
 }
 
+void QgsSymbolButton::wheelEvent( QWheelEvent *event )
+{
+  if ( isEnabled() && mSymbol )
+  {
+    bool symbolChanged = false;
+    const double increment = ( ( event->modifiers() & Qt::ControlModifier ) ? 0.1 : 1 ) *
+                             ( event->angleDelta().y() > 0 ? 1 : -1 );
+    switch ( mSymbol->type() )
+    {
+      case Qgis::SymbolType::Marker:
+      {
+        QgsMarkerSymbol *marker = qgis::down_cast<QgsMarkerSymbol *>( mSymbol.get() );
+        if ( marker )
+        {
+          const double size = std::max( 0.0, marker->size() + increment );
+          marker->setSize( size );
+          symbolChanged = true;
+        }
+        break;
+      }
+
+      case Qgis::SymbolType::Line:
+      {
+        QgsLineSymbol *line = qgis::down_cast<QgsLineSymbol *>( mSymbol.get() );
+        if ( line )
+        {
+          const double width = std::max( 0.0, line->width() + increment );
+          line->setWidth( width );
+          symbolChanged = true;
+        }
+        break;
+      }
+
+      case Qgis::SymbolType::Fill:
+      case Qgis::SymbolType::Hybrid:
+        break;
+    }
+
+    if ( symbolChanged )
+    {
+      updatePreview();
+      emit changed();
+    }
+
+    event->accept();
+  }
+  else
+  {
+    QToolButton::wheelEvent( event );
+  }
+}
+
 void QgsSymbolButton::prepareMenu()
 {
   //we need to tear down and rebuild this menu every time it is shown. Otherwise the space allocated to any
@@ -376,8 +485,10 @@ void QgsSymbolButton::prepareMenu()
   connect( configureAction, &QAction::triggered, this, &QgsSymbolButton::showSettingsDialog );
 
   QAction *copySymbolAction = new QAction( tr( "Copy Symbol" ), this );
+  copySymbolAction->setEnabled( !isNull() );
   mMenu->addAction( copySymbolAction );
   connect( copySymbolAction, &QAction::triggered, this, &QgsSymbolButton::copySymbol );
+
   QAction *pasteSymbolAction = new QAction( tr( "Paste Symbol" ), this );
   //enable or disable paste action based on current clipboard contents. We always show the paste
   //action, even if it's disabled, to give hint to the user that pasting symbols is possible
@@ -394,73 +505,84 @@ void QgsSymbolButton::prepareMenu()
   mMenu->addAction( pasteSymbolAction );
   connect( pasteSymbolAction, &QAction::triggered, this, &QgsSymbolButton::pasteSymbol );
 
-  mMenu->addSeparator();
-
-  QgsColorWheel *colorWheel = new QgsColorWheel( mMenu );
-  colorWheel->setColor( mSymbol->color() );
-  QgsColorWidgetAction *colorAction = new QgsColorWidgetAction( colorWheel, mMenu, mMenu );
-  colorAction->setDismissOnColorSelection( false );
-  connect( colorAction, &QgsColorWidgetAction::colorChanged, this, &QgsSymbolButton::setColor );
-  mMenu->addAction( colorAction );
-
-  QgsColorRampWidget *alphaRamp = new QgsColorRampWidget( mMenu, QgsColorWidget::Alpha, QgsColorRampWidget::Horizontal );
-  QColor alphaColor = mSymbol->color();
-  alphaColor.setAlphaF( mSymbol->opacity() );
-  alphaRamp->setColor( alphaColor );
-  QgsColorWidgetAction *alphaAction = new QgsColorWidgetAction( alphaRamp, mMenu, mMenu );
-  alphaAction->setDismissOnColorSelection( false );
-  connect( alphaAction, &QgsColorWidgetAction::colorChanged, this, [ = ]( const QColor & color )
+  if ( mShowNull )
   {
-    const double opacity = color.alphaF();
-    mSymbol->setOpacity( opacity );
-    updatePreview();
-    emit changed();
-  } );
-  connect( colorAction, &QgsColorWidgetAction::colorChanged, alphaRamp, [alphaRamp]( const QColor & color ) { alphaRamp->setColor( color, false ); }
-         );
-  mMenu->addAction( alphaAction );
+    QAction *nullAction = new QAction( tr( "Clear Current Symbol" ), this );
+    nullAction->setEnabled( !isNull() );
+    mMenu->addAction( nullAction );
+    connect( nullAction, &QAction::triggered, this, &QgsSymbolButton::setToNull );
+  }
 
-  //get schemes with ShowInColorButtonMenu flag set
-  QList< QgsColorScheme * > schemeList = QgsApplication::colorSchemeRegistry()->schemes( QgsColorScheme::ShowInColorButtonMenu );
-  QList< QgsColorScheme * >::iterator it = schemeList.begin();
-  for ( ; it != schemeList.end(); ++it )
+  if ( mSymbol )
   {
-    QgsColorSwatchGridAction *colorAction = new QgsColorSwatchGridAction( *it, mMenu, QStringLiteral( "symbology" ), this );
-    colorAction->setBaseColor( mSymbol->color() );
+    mMenu->addSeparator();
+
+    QgsColorWheel *colorWheel = new QgsColorWheel( mMenu );
+    colorWheel->setColor( mSymbol->color() );
+    QgsColorWidgetAction *colorAction = new QgsColorWidgetAction( colorWheel, mMenu, mMenu );
+    colorAction->setDismissOnColorSelection( false );
+    connect( colorAction, &QgsColorWidgetAction::colorChanged, this, &QgsSymbolButton::setColor );
     mMenu->addAction( colorAction );
-    connect( colorAction, &QgsColorSwatchGridAction::colorChanged, this, &QgsSymbolButton::setColor );
-    connect( colorAction, &QgsColorSwatchGridAction::colorChanged, this, &QgsSymbolButton::addRecentColor );
+
+    QgsColorRampWidget *alphaRamp = new QgsColorRampWidget( mMenu, QgsColorWidget::Alpha, QgsColorRampWidget::Horizontal );
+    QColor alphaColor = mSymbol->color();
+    alphaColor.setAlphaF( mSymbol->opacity() );
+    alphaRamp->setColor( alphaColor );
+    QgsColorWidgetAction *alphaAction = new QgsColorWidgetAction( alphaRamp, mMenu, mMenu );
+    alphaAction->setDismissOnColorSelection( false );
+    connect( alphaAction, &QgsColorWidgetAction::colorChanged, this, [ = ]( const QColor & color )
+    {
+      const double opacity = color.alphaF();
+      mSymbol->setOpacity( opacity );
+      updatePreview();
+      emit changed();
+    } );
+    connect( colorAction, &QgsColorWidgetAction::colorChanged, alphaRamp, [alphaRamp]( const QColor & color ) { alphaRamp->setColor( color, false ); }
+           );
+    mMenu->addAction( alphaAction );
+
+    //get schemes with ShowInColorButtonMenu flag set
+    QList< QgsColorScheme * > schemeList = QgsApplication::colorSchemeRegistry()->schemes( QgsColorScheme::ShowInColorButtonMenu );
+    QList< QgsColorScheme * >::iterator it = schemeList.begin();
+    for ( ; it != schemeList.end(); ++it )
+    {
+      QgsColorSwatchGridAction *colorAction = new QgsColorSwatchGridAction( *it, mMenu, QStringLiteral( "symbology" ), this );
+      colorAction->setBaseColor( mSymbol->color() );
+      mMenu->addAction( colorAction );
+      connect( colorAction, &QgsColorSwatchGridAction::colorChanged, this, &QgsSymbolButton::setColor );
+      connect( colorAction, &QgsColorSwatchGridAction::colorChanged, this, &QgsSymbolButton::addRecentColor );
+    }
+
+    mMenu->addSeparator();
+
+    QAction *copyColorAction = new QAction( tr( "Copy Color" ), this );
+    mMenu->addAction( copyColorAction );
+    connect( copyColorAction, &QAction::triggered, this, &QgsSymbolButton::copyColor );
+
+    QAction *pasteColorAction = new QAction( tr( "Paste Color" ), this );
+    //enable or disable paste action based on current clipboard contents. We always show the paste
+    //action, even if it's disabled, to give hint to the user that pasting colors is possible
+    QColor clipColor;
+    bool hasAlpha = false;
+    if ( colorFromMimeData( QApplication::clipboard()->mimeData(), clipColor, hasAlpha ) )
+    {
+      pasteColorAction->setIcon( createColorIcon( clipColor ) );
+    }
+    else
+    {
+      pasteColorAction->setEnabled( false );
+    }
+    mMenu->addAction( pasteColorAction );
+    connect( pasteColorAction, &QAction::triggered, this, &QgsSymbolButton::pasteColor );
+
+    QAction *pickColorAction = new QAction( tr( "Pick Color" ), this );
+    mMenu->addAction( pickColorAction );
+    connect( pickColorAction, &QAction::triggered, this, &QgsSymbolButton::activatePicker );
+
+    QAction *chooseColorAction = new QAction( tr( "Choose Color…" ), this );
+    mMenu->addAction( chooseColorAction );
+    connect( chooseColorAction, &QAction::triggered, this, &QgsSymbolButton::showColorDialog );
   }
-
-  mMenu->addSeparator();
-
-  QAction *copyColorAction = new QAction( tr( "Copy Color" ), this );
-  mMenu->addAction( copyColorAction );
-  connect( copyColorAction, &QAction::triggered, this, &QgsSymbolButton::copyColor );
-
-  QAction *pasteColorAction = new QAction( tr( "Paste Color" ), this );
-  //enable or disable paste action based on current clipboard contents. We always show the paste
-  //action, even if it's disabled, to give hint to the user that pasting colors is possible
-  QColor clipColor;
-  bool hasAlpha = false;
-  if ( colorFromMimeData( QApplication::clipboard()->mimeData(), clipColor, hasAlpha ) )
-  {
-    pasteColorAction->setIcon( createColorIcon( clipColor ) );
-  }
-  else
-  {
-    pasteColorAction->setEnabled( false );
-  }
-  mMenu->addAction( pasteColorAction );
-  connect( pasteColorAction, &QAction::triggered, this, &QgsSymbolButton::pasteColor );
-
-  QAction *pickColorAction = new QAction( tr( "Pick Color" ), this );
-  mMenu->addAction( pickColorAction );
-  connect( pickColorAction, &QAction::triggered, this, &QgsSymbolButton::activatePicker );
-
-  QAction *chooseColorAction = new QAction( tr( "Choose Color…" ), this );
-  mMenu->addAction( chooseColorAction );
-  connect( chooseColorAction, &QAction::triggered, this, &QgsSymbolButton::showColorDialog );
 }
 
 void QgsSymbolButton::addRecentColor( const QColor &color )
@@ -504,16 +626,6 @@ void QgsSymbolButton::resizeEvent( QResizeEvent *event )
 
 void QgsSymbolButton::updatePreview( const QColor &color, QgsSymbol *tempSymbol )
 {
-  std::unique_ptr< QgsSymbol > previewSymbol;
-
-  if ( tempSymbol )
-    previewSymbol.reset( tempSymbol->clone() );
-  else
-    previewSymbol.reset( mSymbol->clone() );
-
-  if ( color.isValid() )
-    previewSymbol->setColor( color );
-
   QSize currentIconSize;
   //icon size is button size with a small margin
   if ( menu() )
@@ -549,6 +661,27 @@ void QgsSymbolButton::updatePreview( const QColor &color, QgsSymbol *tempSymbol
     return;
   }
 
+  std::unique_ptr< QgsSymbol > previewSymbol;
+
+  if ( tempSymbol )
+  {
+    previewSymbol.reset( tempSymbol->clone() );
+  }
+  else if ( mSymbol )
+  {
+    previewSymbol.reset( mSymbol->clone() );
+  }
+  else
+  {
+    setIconSize( currentIconSize );
+    setIcon( QIcon() );
+    setToolTip( QString( ) );
+    return;
+  }
+
+  if ( color.isValid() )
+    previewSymbol->setColor( color );
+
   //create an icon pixmap
   const QIcon icon = QgsSymbolLayerUtils::symbolPreviewIcon( previewSymbol.get(), currentIconSize );
   setIconSize( currentIconSize );
@@ -556,7 +689,6 @@ void QgsSymbolButton::updatePreview( const QColor &color, QgsSymbol *tempSymbol
 
   // set tooltip
   // create very large preview image
-
   const int width = static_cast< int >( Qgis::UI_SCALE_FACTOR * fontMetrics().horizontalAdvance( 'X' ) * 23 );
   const int height = static_cast< int >( width / 1.61803398875 ); // golden ratio
 
@@ -625,6 +757,9 @@ void QgsSymbolButton::stopPicking( QPoint eventPos, bool samplingColor )
 
 void QgsSymbolButton::showColorDialog()
 {
+  if ( !mSymbol )
+    return;
+
   QgsPanelWidget *panel = QgsPanelWidget::findParentPanel( this );
   if ( panel && panel->dockMode() )
   {
@@ -676,3 +811,23 @@ QgsSymbol *QgsSymbolButton::symbol()
 {
   return mSymbol.get();
 }
+
+void QgsSymbolButton::setShowNull( bool showNull )
+{
+  mShowNull = showNull;
+}
+
+bool QgsSymbolButton::showNull() const
+{
+  return mShowNull;
+}
+
+bool QgsSymbolButton::isNull() const
+{
+  return !mSymbol;
+}
+
+void QgsSymbolButton::setToNull()
+{
+  setSymbol( nullptr );
+}
