@@ -204,6 +204,39 @@ class GUI_EXPORT QgsSymbolButton : public QToolButton
      */
     void pasteColor();
 
+    /**
+     * Sets whether a set to null (clear) option is shown in the button's drop-down menu.
+     * \param showNull set to TRUE to show a null option
+     * \see showNull()
+     * \see isNull()
+     * \since QGIS 3.26
+     */
+    void setShowNull( bool showNull );
+
+    /**
+     * Returns whether the set to null (clear) option is shown in the button's drop-down menu.
+     * \see setShowNull()
+     * \see isNull()
+     * \since QGIS 3.26
+     */
+    bool showNull() const;
+
+    /**
+     * Returns TRUE if the current symbol is null.
+     * \see setShowNull()
+     * \see showNull()
+     * \since QGIS 3.26
+     */
+    bool isNull() const;
+
+    /**
+     * Sets symbol to to null.
+     * \see setShowNull()
+     * \see showNull()
+     * \since QGIS 3.26
+     */
+    void setToNull();
+
   signals:
 
     /**
@@ -234,6 +267,8 @@ class GUI_EXPORT QgsSymbolButton : public QToolButton
     // Reimplemented to accept dropped colors
     void dropEvent( QDropEvent *e ) override;
 
+    void wheelEvent( QWheelEvent *event ) override;
+
   private slots:
 
     void showSettingsDialog();
@@ -277,6 +312,8 @@ class GUI_EXPORT QgsSymbolButton : public QToolButton
 
     bool mPickingColor = false;
 
+    bool mShowNull = false;
+
     /**
      * Regenerates the text preview. If \a color is specified, a temporary color preview
      * is shown instead.
@@ -309,6 +346,8 @@ class GUI_EXPORT QgsSymbolButton : public QToolButton
 
     void showColorDialog();
 
+    void updateSizeHint();
+
 };
 
 #endif // QGSSYMBOLBUTTON_H
