@@ -14,20 +14,27 @@
  ***************************************************************************/
 
 #include "qgstextpreview.h"
-#include "qgsapplication.h"
 #include "qgstextrenderer.h"
-#include <QDesktopWidget>
+#include "qgsscreenhelper.h"
+
 #include <QPainter>
 
 QgsTextPreview::QgsTextPreview( QWidget *parent )
   : QLabel( parent )
 {
+  mScreenHelper = new QgsScreenHelper( this );
+  connect( mScreenHelper, &QgsScreenHelper::screenDpiChanged, this, [ = ]( double dpi )
+  {
+    mContext.setScaleFactor( dpi / 25.4 );
+    updateContext();
+  } );
+
   // initially use a basic transform with no scale
   QgsMapToPixel newCoordXForm;
   newCoordXForm.setParameters( 1, 0, 0, 0, 0, 0 );
   mContext.setMapToPixel( newCoordXForm );
 
-  mContext.setScaleFactor( QgsApplication::desktop()->logicalDpiX() / 25.4 );
+  mContext.setScaleFactor( mScreenHelper->screenDpi() / 25.4 );
   mContext.setUseAdvancedEffects( true );
 
   mContext.setFlag( Qgis::RenderContextFlag::Antialiasing, true );
@@ -35,7 +42,6 @@ QgsTextPreview::QgsTextPreview( QWidget *parent )
   mContext.setIsGuiPreview( true );
 }
 
-
 void QgsTextPreview::paintEvent( QPaintEvent *e )
 {
   Q_UNUSED( e )
@@ -44,16 +50,21 @@ void QgsTextPreview::paintEvent( QPaintEvent *e )
   p.setRenderHint( QPainter::Antialiasing );
 
   // slightly inset text
+  const double fontSize = mContext.convertToPainterUnits( mFormat.size(), mFormat.sizeUnit(), mFormat.sizeMapUnitScale() );
   double xtrans = 0;
   if ( mFormat.buffer().enabled() )
-    xtrans = mContext.convertToPainterUnits( mFormat.buffer().size(), mFormat.buffer().sizeUnit(), mFormat.buffer().sizeMapUnitScale() );
+    xtrans = mFormat.buffer().sizeUnit() == QgsUnitTypes::RenderPercentage
+             ? fontSize * mFormat.buffer().size() / 100
+             : mContext.convertToPainterUnits( mFormat.buffer().size(), mFormat.buffer().sizeUnit(), mFormat.buffer().sizeMapUnitScale() );
   if ( mFormat.background().enabled() && mFormat.background().sizeType() != QgsTextBackgroundSettings::SizeFixed )
     xtrans = std::max( xtrans, mContext.convertToPainterUnits( mFormat.background().size().width(), mFormat.background().sizeUnit(), mFormat.background().sizeMapUnitScale() ) );
   xtrans += 4;
 
   double ytrans = 0.0;
   if ( mFormat.buffer().enabled() )
-    ytrans = std::max( ytrans, mContext.convertToPainterUnits( mFormat.buffer().size(), mFormat.buffer().sizeUnit(), mFormat.buffer().sizeMapUnitScale() ) );
+    ytrans = std::max( ytrans, mFormat.buffer().sizeUnit() == QgsUnitTypes::RenderPercentage
+                       ? fontSize * mFormat.buffer().size() / 100
+                       : mContext.convertToPainterUnits( mFormat.buffer().size(), mFormat.buffer().sizeUnit(), mFormat.buffer().sizeMapUnitScale() ) );
   if ( mFormat.background().enabled() )
     ytrans = std::max( ytrans, mContext.convertToPainterUnits( mFormat.background().size().height(), mFormat.background().sizeUnit(), mFormat.background().sizeMapUnitScale() ) );
   ytrans += 4;
@@ -68,7 +79,7 @@ void QgsTextPreview::paintEvent( QPaintEvent *e )
     textRect.setWidth( 2000 );
 
   mContext.setPainter( &p );
-  QgsTextRenderer::drawText( textRect, 0, QgsTextRenderer::AlignLeft, QStringList() << text(),
+  QgsTextRenderer::drawText( textRect, 0, Qgis::TextHorizontalAlignment::Left, QStringList() << text(),
                              mContext, mFormat );
 }
 
@@ -82,7 +93,7 @@ void QgsTextPreview::updateContext()
 {
   if ( mScale >= 0 )
   {
-    const QgsMapToPixel newCoordXForm = QgsMapToPixel::fromScale( mScale, mMapUnits, QgsApplication::desktop()->logicalDpiX() );
+    const QgsMapToPixel newCoordXForm = QgsMapToPixel::fromScale( mScale, mMapUnits, mScreenHelper->screenDpi() );
     mContext.setMapToPixel( newCoordXForm );
   }
   update();
