@@ -28,6 +28,7 @@
 
 #include <QStandardItemModel>
 #include <QDialogButtonBox>
+#include <QPointer>
 #include "qgis_gui.h"
 
 class QgsStyle;
@@ -82,7 +83,7 @@ class QgsSymbolSelectorDialog;
 
 /**
  * \ingroup gui
- * Symbol selector widget that can be used to select and build a symbol
+ * \brief Symbol selector widget that can be used to select and build a symbol
  */
 class GUI_EXPORT QgsSymbolSelectorWidget: public QgsPanelWidget, private Ui::QgsSymbolSelectorDialogBase
 {
@@ -199,7 +200,7 @@ class GUI_EXPORT QgsSymbolSelectorWidget: public QgsPanelWidget, private Ui::Qgs
   signals:
 
     /**
-     * Emiited when a symbol is modified in the widget.
+     * Emitted when a symbol is modified in the widget.
      */
     void symbolModified();
 
@@ -256,9 +257,9 @@ class GUI_EXPORT QgsSymbolSelectorWidget: public QgsPanelWidget, private Ui::Qgs
     QgsStyle *mStyle = nullptr;
     QgsSymbol *mSymbol = nullptr;
     QMenu *mAdvancedMenu = nullptr;
-    QgsVectorLayer *mVectorLayer = nullptr;
+    QPointer< QgsVectorLayer > mVectorLayer;
 
-    QStandardItemModel *model = nullptr;
+    QStandardItemModel *mSymbolLayersModel = nullptr;
     QWidget *mPresentWidget = nullptr;
 
     std::unique_ptr<DataDefinedRestorer> mDataDefineRestorer;
