@@ -90,8 +90,7 @@ QgsGrassModule::QgsGrassModule( QgsGrassTools *tools, QString moduleName, QgisIn
   , mSuccess( false )
   , mDirect( direct )
 {
-  Q_UNUSED( f )
-  QgsDebugMsg( "called" );
+  QgsDebugMsgLevel( "called", 4 );
 
   setupUi( this );
   connect( mRunButton, &QPushButton::clicked, this, &QgsGrassModule::mRunButton_clicked );
@@ -109,7 +108,7 @@ QgsGrassModule::QgsGrassModule( QgsGrassTools *tools, QString moduleName, QgisIn
 
   // Open QGIS module description
   QString mpath = QgsGrass::modulesConfigDirPath() + "/" + moduleName + ".qgm";
-  QgsDebugMsg( QString( "mpath = %1" ).arg( mpath ) );
+  QgsDebugMsgLevel( QString( "mpath = %1" ).arg( mpath ), 2 );
   QFile qFile( mpath );
   if ( !qFile.exists() )
   {
@@ -218,7 +217,7 @@ QgsGrassModule::QgsGrassModule( QgsGrassTools *tools, QString moduleName, QgisIn
 
 QgsGrassModule::Description QgsGrassModule::description( QString path )
 {
-  QgsDebugMsg( "called." );
+  QgsDebugMsgLevel( "called.", 4 );
 
   // Open QGIS module description
   path.append( ".qgm" );
@@ -258,7 +257,7 @@ QString QgsGrassModule::label( QString path )
 
 QPixmap QgsGrassModule::pixmap( QString path, int height )
 {
-  //QgsDebugMsg( QString( "path = %1" ).arg( path ) );
+  //QgsDebugMsgLevel( QString( "path = %1" ).arg( path ), 2 );
 
   QList<QPixmap> pixmaps;
 
@@ -451,7 +450,7 @@ QPixmap QgsGrassModule::pixmap( QString path, int height )
 
 void QgsGrassModule::run()
 {
-  QgsDebugMsg( "called." );
+  QgsDebugMsgLevel( "called.", 4 );
 
   if ( mProcess.state() == QProcess::Running )
   {
@@ -499,7 +498,7 @@ void QgsGrassModule::run()
       if ( outsideRegion.size() > 0 )
       {
         QMessageBox questionBox( QMessageBox::Question, tr( "Warning" ),
-                                 tr( "Input %1 outside current region!" ).arg( outsideRegion.join( QStringLiteral( "," ) ) ),
+                                 tr( "Input %1 outside current region!" ).arg( outsideRegion.join( QLatin1Char( ',' ) ) ),
                                  QMessageBox::Ok | QMessageBox::Cancel );
         QPushButton *resetButton = nullptr;
         if ( QgsGrass::versionMajor() > 6 || ( QgsGrass::versionMajor() == 6 && QgsGrass::versionMinor() >= 1 ) )
@@ -532,7 +531,7 @@ void QgsGrassModule::run()
       if ( outputExists.size() > 0 )
       {
         QMessageBox::StandardButton ret = QMessageBox::question( nullptr, QStringLiteral( "Warning" ),
-                                          tr( "Output %1 exists! Overwrite?" ).arg( outputExists.join( QStringLiteral( "," ) ) ),
+                                          tr( "Output %1 exists! Overwrite?" ).arg( outputExists.join( QLatin1Char( ',' ) ) ),
                                           QMessageBox::Ok | QMessageBox::Cancel );
 
         if ( ret == QMessageBox::Cancel )
@@ -544,9 +543,9 @@ void QgsGrassModule::run()
 
     // Remember output maps
     mOutputVector = mOptions->output( QgsGrassModuleOption::Vector );
-    QgsDebugMsg( QString( "mOutputVector.size() = %1" ).arg( mOutputVector.size() ) );
+    QgsDebugMsgLevel( QString( "mOutputVector.size() = %1" ).arg( mOutputVector.size() ), 2 );
     mOutputRaster = mOptions->output( QgsGrassModuleOption::Raster );
-    QgsDebugMsg( QString( "mOutputRaster.size() = %1" ).arg( mOutputRaster.size() ) );
+    QgsDebugMsgLevel( QString( "mOutputRaster.size() = %1" ).arg( mOutputRaster.size() ), 2 );
     mSuccess = false;
     mViewButton->setEnabled( false );
 
@@ -556,7 +555,7 @@ void QgsGrassModule::run()
     QStringList argumentsHtml;
     for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
     {
-      QgsDebugMsg( "option: " + ( *it ) );
+      QgsDebugMsgLevel( "option: " + ( *it ), 2 );
       //command.append ( " " + *it );
       arguments.append( *it );
       //mProcess.addArgument( *it );
@@ -579,7 +578,7 @@ void QgsGrassModule::run()
      * if  G_get_gisrc_mode() == G_GISRC_MODE_FILE. Because QGIS GRASS provider starts drivers in
      * G_GISRC_MODE_MEMORY mode, the variable remains set in variable when a module is run
      * -> unset GISRC_MODE_MEMORY. Remove later once 6.1.x / 6.0.1 is widespread.
-    *   */
+    */
     putenv( ( char * ) "GISRC_MODE_MEMORY" ); // unset
 
     mOutputTextBrowser->clear();
@@ -595,7 +594,7 @@ void QgsGrassModule::run()
     if ( resetRegion )
     {
       QString reg = QgsGrass::regionString( &tempWindow );
-      QgsDebugMsg( "reg: " + reg );
+      QgsDebugMsgLevel( "reg: " + reg, 2 );
       environment.insert( QStringLiteral( "GRASS_REGION" ), reg );
     }
 
@@ -620,15 +619,15 @@ void QgsGrassModule::run()
 
       // Print some important variables
       variables << QStringLiteral( "QGIS_PREFIX_PATH" ) << QStringLiteral( "QGIS_GRASS_CRS" ) << QStringLiteral( "GRASS_REGION" );
-      Q_FOREACH ( const QString &v, variables )
+      for ( const QString &v : variables )
       {
         mOutputTextBrowser->append( v + "=" + environment.value( v ) + "<BR>" );
       }
     }
 
-    QString commandHtml = mXName + " " + argumentsHtml.join( QStringLiteral( " " ) );
+    QString commandHtml = mXName + " " + argumentsHtml.join( QLatin1Char( ' ' ) );
 
-    QgsDebugMsg( "command: " + commandHtml );
+    QgsDebugMsgLevel( "command: " + commandHtml, 2 );
     commandHtml.replace( QLatin1String( "&" ), QLatin1String( "&amp;" ) );
     commandHtml.replace( QLatin1String( "<" ), QLatin1String( "&lt;" ) );
     commandHtml.replace( QLatin1String( ">" ), QLatin1String( "&gt;" ) );
@@ -697,7 +696,7 @@ void QgsGrassModule::run()
         //QString env = "GIS_FLAG_"
         //              + QString( allFlagNames.at( i ).toUpper() )
         //              + "=0";
-        //QgsDebugMsg( "set: " + env );
+        //QgsDebugMsgLevel( "set: " + env, 2 );
         //environment.append( env );
         environment.insert( "GIS_FLAG_" + QString( allFlagNames.at( i ).toUpper() ), "0" );
       }
@@ -731,9 +730,9 @@ void QgsGrassModule::run()
 
 void QgsGrassModule::finished( int exitCode, QProcess::ExitStatus exitStatus )
 {
-  QgsDebugMsg( "called." );
+  QgsDebugMsgLevel( "called.", 4 );
 
-  QgsDebugMsg( QString( "exitCode = %1" ).arg( exitCode ) );
+  QgsDebugMsgLevel( QString( "exitCode = %1" ).arg( exitCode ), 2 );
   if ( exitStatus == QProcess::NormalExit )
   {
     if ( exitCode == 0 )
@@ -761,7 +760,7 @@ void QgsGrassModule::finished( int exitCode, QProcess::ExitStatus exitStatus )
 
 void QgsGrassModule::readStdout()
 {
-  QgsDebugMsg( "called." );
+  QgsDebugMsgLevel( "called.", 4 );
 
   QString line;
   QRegExp rxpercent( "GRASS_INFO_PERCENT: (\\d+)" );
@@ -788,7 +787,7 @@ void QgsGrassModule::readStdout()
 
 void QgsGrassModule::readStderr()
 {
-  QgsDebugMsg( "called." );
+  QgsDebugMsgLevel( "called.", 4 );
 
   QString line;
 
@@ -832,7 +831,7 @@ void QgsGrassModule::close()
 
 void QgsGrassModule::viewOutput()
 {
-  QgsDebugMsg( "called." );
+  QgsDebugMsgLevel( "called.", 4 );
 
   if ( !mSuccess )
     return;
@@ -921,7 +920,7 @@ QgisInterface *QgsGrassModule::qgisIface()
 
 QgsGrassModule::~QgsGrassModule()
 {
-  QgsDebugMsg( "called." );
+  QgsDebugMsgLevel( "called.", 4 );
   if ( mProcess.state() == QProcess::Running )
   {
     mProcess.kill();
@@ -958,6 +957,6 @@ void QgsGrassModule::setDirectLibraryPath( QProcessEnvironment &environment )
   QString lp = environment.value( pathVariable );
   lp = QgsApplication::pluginPath() + separator + lp;
   environment.insert( pathVariable, lp );
-  QgsDebugMsg( pathVariable + "=" + lp );
+  QgsDebugMsgLevel( pathVariable + "=" + lp, 2 );
 }
 
