@@ -36,6 +36,7 @@
 #endif
 
 #include <iostream>
+#include <string>
 #include <QObject>
 #include <QLibrary>
 
@@ -254,6 +255,13 @@ int QgsProcessingExec::run( const QStringList &constArgs )
     args.removeAt( verboseIndex );
   }
 
+  const int noPythonIndex = args.indexOf( QLatin1String( "--no-python" ) );
+  if ( noPythonIndex >= 0 )
+  {
+    mSkipPython = true;
+    args.removeAt( noPythonIndex );
+  }
+
   if ( args.size() == 1 )
   {
     showUsage( args.at( 0 ) );
@@ -267,14 +275,17 @@ int QgsProcessingExec::run( const QStringList &constArgs )
 #endif
 
 #ifdef WITH_BINDINGS
-
-  // give Python plugins a chance to load providers
-  mPythonUtils = loadPythonSupport();
-  if ( !mPythonUtils )
+  if ( !mSkipPython )
   {
-    QCoreApplication::exit( 1 );
-    return 1;
+    // give Python plugins a chance to load providers
+    mPythonUtils = loadPythonSupport();
+    if ( !mPythonUtils )
+    {
+      QCoreApplication::exit( 1 );
+      return 1;
+    }
   }
+#endif
 
   const QString command = args.at( 1 );
   if ( command == QLatin1String( "plugins" ) )
@@ -342,42 +353,138 @@ int QgsProcessingExec::run( const QStringList &constArgs )
     QgsUnitTypes::AreaUnit areaUnit = QgsUnitTypes::AreaUnknownUnit;
     QString projectPath;
     QVariantMap params;
-    int i = 3;
-    for ( ; i < args.count(); i++ )
-    {
-      QString arg = args.at( i );
 
-      if ( arg == QLatin1String( "--" ) )
+    if ( args.size() == 4 && args.at( 3 ) == '-' )
+    {
+      // read arguments as JSON value from stdin
+      std::string stdinJson;
+      for ( std::string line; std::getline( std::cin, line ); )
       {
-        break;
+        stdinJson.append( line + '\n' );
       }
 
-      if ( arg.startsWith( QLatin1String( "--" ) ) )
-        arg = arg.mid( 2 );
-
-      const QStringList parts = arg.split( '=' );
-      if ( parts.count() >= 2 )
+      QString error;
+      const QVariantMap json = QgsJsonUtils::parseJson( stdinJson, error ).toMap();
+      if ( !error.isEmpty() )
       {
-        const QString name = parts.at( 0 );
+        std::cerr << QStringLiteral( "Could not parse JSON parameters: %1" ).arg( error ).toLocal8Bit().constData() << std::endl;
+        return 1;
+      }
+      if ( !json.contains( QStringLiteral( "inputs" ) ) )
+      {
+        std::cerr << QStringLiteral( "JSON parameters object must contain an \"inputs\" key." ).toLocal8Bit().constData() << std::endl;
+        return 1;
+      }
 
-        if ( name.compare( QLatin1String( "ellipsoid" ), Qt::CaseInsensitive ) == 0 )
+      params = json.value( QStringLiteral( "inputs" ) ).toMap();
+
+      // JSON format for input parameters implies JSON output format
+      useJson = true;
+
+      ellipsoid = json.value( QStringLiteral( "ellipsoid" ) ).toString();
+      projectPath = json.value( QStringLiteral( "project_path" ) ).toString();
+      if ( json.contains( "distance_units" ) )
+      {
+        bool ok = false;
+        const QString distanceUnitsString = json.value( QStringLiteral( "distance_units" ) ).toString();
+        distanceUnit = QgsUnitTypes::decodeDistanceUnit( distanceUnitsString, &ok );
+        if ( !ok )
         {
-          ellipsoid = parts.mid( 1 ).join( '=' );
+          std::cerr << QStringLiteral( "%1 is not a valid distance unit value." ).arg( distanceUnitsString ).toLocal8Bit().constData() << std::endl;
+          return 1;
         }
-        else if ( name.compare( QLatin1String( "distance_units" ), Qt::CaseInsensitive ) == 0 )
+      }
+
+      if ( json.contains( "area_units" ) )
+      {
+        bool ok = false;
+        const QString areaUnitsString = json.value( QStringLiteral( "area_units" ) ).toString();
+        areaUnit = QgsUnitTypes::decodeAreaUnit( areaUnitsString, &ok );
+        if ( !ok )
         {
-          distanceUnit = QgsUnitTypes::decodeDistanceUnit( parts.mid( 1 ).join( '=' ) );
+          std::cerr << QStringLiteral( "%1 is not a valid area unit value." ).arg( areaUnitsString ).toLocal8Bit().constData() << std::endl;
+          return 1;
         }
-        else if ( name.compare( QLatin1String( "area_units" ), Qt::CaseInsensitive ) == 0 )
+      }
+    }
+    else
+    {
+      int i = 3;
+      for ( ; i < args.count(); i++ )
+      {
+        QString arg = args.at( i );
+
+        if ( arg == QLatin1String( "--" ) )
         {
-          areaUnit = QgsUnitTypes::decodeAreaUnit( parts.mid( 1 ).join( '=' ) );
+          break;
         }
-        else if ( name.compare( QLatin1String( "project_path" ), Qt::CaseInsensitive ) == 0 )
+
+        if ( arg.startsWith( QLatin1String( "--" ) ) )
+          arg = arg.mid( 2 );
+
+        const QStringList parts = arg.split( '=' );
+        if ( parts.count() >= 2 )
         {
-          projectPath = parts.mid( 1 ).join( '=' );
+          const QString name = parts.at( 0 );
+
+          if ( name.compare( QLatin1String( "ellipsoid" ), Qt::CaseInsensitive ) == 0 )
+          {
+            ellipsoid = parts.mid( 1 ).join( '=' );
+          }
+          else if ( name.compare( QLatin1String( "distance_units" ), Qt::CaseInsensitive ) == 0 )
+          {
+            distanceUnit = QgsUnitTypes::decodeDistanceUnit( parts.mid( 1 ).join( '=' ) );
+          }
+          else if ( name.compare( QLatin1String( "area_units" ), Qt::CaseInsensitive ) == 0 )
+          {
+            areaUnit = QgsUnitTypes::decodeAreaUnit( parts.mid( 1 ).join( '=' ) );
+          }
+          else if ( name.compare( QLatin1String( "project_path" ), Qt::CaseInsensitive ) == 0 )
+          {
+            projectPath = parts.mid( 1 ).join( '=' );
+          }
+          else
+          {
+            const QString value = parts.mid( 1 ).join( '=' );
+            if ( params.contains( name ) )
+            {
+              // parameter specified multiple times, store all of them in a list...
+              if ( params.value( name ).type() == QVariant::StringList )
+              {
+                // append to existing list
+                QStringList listValue = params.value( name ).toStringList();
+                listValue << value;
+                params.insert( name, listValue );
+              }
+              else
+              {
+                // upgrade previous value to list
+                QStringList listValue = QStringList() << params.value( name ).toString()
+                                        << value;
+                params.insert( name, listValue );
+              }
+            }
+            else
+            {
+              params.insert( name, value );
+            }
+          }
         }
         else
         {
+          std::cerr << QStringLiteral( "Invalid parameter value %1. Parameter values must be entered after \"--\" e.g.\n  Example:\n    qgis_process run algorithm_name -- PARAM1=VALUE PARAM2=42\"\n" ).arg( arg ).toLocal8Bit().constData();
+          return 1;
+        }
+      }
+
+      // After '--' we only have params
+      for ( ; i < args.count(); i++ )
+      {
+        const QString arg = args.at( i );
+        const QStringList parts = arg.split( '=' );
+        if ( parts.count() >= 2 )
+        {
+          const QString name = parts.first();
           const QString value = parts.mid( 1 ).join( '=' );
           if ( params.contains( name ) )
           {
@@ -403,45 +510,6 @@ int QgsProcessingExec::run( const QStringList &constArgs )
           }
         }
       }
-      else
-      {
-        std::cerr << QStringLiteral( "Invalid parameter value %1. Parameter values must be entered after \"--\" e.g.\n  Example:\n    qgis_process run algorithm_name -- PARAM1=VALUE PARAM2=42\"\n" ).arg( arg ).toLocal8Bit().constData();
-        return 1;
-      }
-    }
-
-    // After '--' we only have params
-    for ( ; i < args.count(); i++ )
-    {
-      const QString arg = args.at( i );
-      const QStringList parts = arg.split( '=' );
-      if ( parts.count() >= 2 )
-      {
-        const QString name = parts.first();
-        const QString value = parts.mid( 1 ).join( '=' );
-        if ( params.contains( name ) )
-        {
-          // parameter specified multiple times, store all of them in a list...
-          if ( params.value( name ).type() == QVariant::StringList )
-          {
-            // append to existing list
-            QStringList listValue = params.value( name ).toStringList();
-            listValue << value;
-            params.insert( name, listValue );
-          }
-          else
-          {
-            // upgrade previous value to list
-            QStringList listValue = QStringList() << params.value( name ).toString()
-                                    << value;
-            params.insert( name, listValue );
-          }
-        }
-        else
-        {
-          params.insert( name, value );
-        }
-      }
     }
 
     return execute( algId, params, ellipsoid, distanceUnit, areaUnit, logLevel, useJson, projectPath );
@@ -450,7 +518,6 @@ int QgsProcessingExec::run( const QStringList &constArgs )
   {
     std::cerr << QStringLiteral( "Command %1 not known!\n" ).arg( command ).toLocal8Bit().constData();
   }
-#endif
   return 1;
 }
 
@@ -460,12 +527,13 @@ void QgsProcessingExec::showUsage( const QString &appName )
 
   msg << "QGIS Processing Executor - " << VERSION << " '" << RELEASE_NAME << "' ("
       << Qgis::version() << ")\n"
-      << "Usage: " << appName <<  " [--help] [--version] [--json] [--verbose] [command] [algorithm id or path to model file] [parameters]\n"
+      << "Usage: " << appName <<  " [--help] [--version] [--json] [--verbose] [--no-python] [command] [algorithm id, path to model file, or path to Python script] [parameters]\n"
       << "\nOptions:\n"
       << "\t--help or -h\t\tOutput the help\n"
       << "\t--version or -v\t\tOutput all versions related to QGIS Process\n"
       << "\t--json\t\tOutput results as JSON objects\n"
       << "\t--verbose\tOutput verbose logs\n"
+      << "\t--no-python\tDisable Python support (results in faster startup)"
       << "\nAvailable commands:\n"
       << "\tplugins\t\tlist available and active plugins\n"
       << "\tplugins enable\tenables an installed plugin. The plugin name must be specified, e.g. \"plugins enable cartography_tools\"\n"
@@ -473,15 +541,20 @@ void QgsProcessingExec::showUsage( const QString &appName )
       << "\tlist\t\tlist all available processing algorithms\n"
       << "\thelp\t\tshow help for an algorithm. The algorithm id or a path to a model file must be specified.\n"
       << "\trun\t\truns an algorithm. The algorithm id or a path to a model file and parameter values must be specified. Parameter values are specified after -- with PARAMETER=VALUE syntax. Ordered list values for a parameter can be created by specifying the parameter multiple times, e.g. --LAYERS=layer1.shp --LAYERS=layer2.shp\n"
+      << "\t\t\tAlternatively, a '-' character in place of the parameters argument indicates that the parameters should be read from STDIN as a JSON object. The JSON should be structured as a map containing at least the \"inputs\" key specifying a map of input parameter values. This implies the --json option for output as a JSON object.\n"
       << "\t\t\tIf required, the ellipsoid to use for distance and area calculations can be specified via the \"--ELLIPSOID=name\" argument.\n"
-      << "\t\t\tIf required, an existing QGIS project to use during the algorithm execution can be specified via the \"--PROJECT_PATH=path\" argument.\n";
+      << "\t\t\tIf required, an existing QGIS project to use during the algorithm execution can be specified via the \"--PROJECT_PATH=path\" argument.\n"
+      << "\t\t\tWhen passing parameters as a JSON object from STDIN, these extra arguments can be provided as an \"ellipsoid\" and a \"project_path\" key respectively.\n";
 
   std::cout << msg.join( QString() ).toLocal8Bit().constData();
 }
 
-#ifdef WITH_BINDINGS
 void QgsProcessingExec::loadPlugins()
 {
+#ifdef WITH_BINDINGS
+  if ( !mPythonUtils )
+    return;
+
   QgsSettings settings;
   // load plugins
   const QStringList plugins = mPythonUtils->pluginList();
@@ -499,8 +572,8 @@ void QgsProcessingExec::loadPlugins()
       }
     }
   }
-}
 #endif
+}
 
 void QgsProcessingExec::listAlgorithms( bool useJson )
 {
@@ -586,25 +659,28 @@ void QgsProcessingExec::listPlugins( bool useJson, bool showLoaded )
 
 #ifdef WITH_BINDINGS
   QVariantMap jsonPlugins;
-  const QStringList plugins = mPythonUtils->pluginList();
-  for ( const QString &plugin : plugins )
+  if ( mPythonUtils )
   {
-    if ( !mPythonUtils->pluginHasProcessingProvider( plugin ) )
-      continue;
+    const QStringList plugins = mPythonUtils->pluginList();
+    for ( const QString &plugin : plugins )
+    {
+      if ( !mPythonUtils->pluginHasProcessingProvider( plugin ) )
+        continue;
 
-    if ( !useJson )
-    {
-      if ( showLoaded ? mPythonUtils->isPluginLoaded( plugin ) : mPythonUtils->isPluginEnabled( plugin ) )
-        std::cout << "* ";
+      if ( !useJson )
+      {
+        if ( showLoaded ? mPythonUtils->isPluginLoaded( plugin ) : mPythonUtils->isPluginEnabled( plugin ) )
+          std::cout << "* ";
+        else
+          std::cout << "  ";
+        std::cout << plugin.toLocal8Bit().constData() << "\n";
+      }
       else
-        std::cout << "  ";
-      std::cout << plugin.toLocal8Bit().constData() << "\n";
-    }
-    else
-    {
-      QVariantMap jsonPlugin;
-      jsonPlugin.insert( QStringLiteral( "loaded" ), showLoaded ? mPythonUtils->isPluginLoaded( plugin ) : mPythonUtils->isPluginEnabled( plugin ) );
-      jsonPlugins.insert( plugin, jsonPlugin );
+      {
+        QVariantMap jsonPlugin;
+        jsonPlugin.insert( QStringLiteral( "loaded" ), showLoaded ? mPythonUtils->isPluginLoaded( plugin ) : mPythonUtils->isPluginEnabled( plugin ) );
+        jsonPlugins.insert( plugin, jsonPlugin );
+      }
     }
   }
 
@@ -624,6 +700,12 @@ int QgsProcessingExec::enablePlugin( const QString &name, bool enabled )
     std::cout << QStringLiteral( "Disabling plugin: \"%1\"\n" ).arg( name ).toLocal8Bit().constData();
 
 #ifdef WITH_BINDINGS
+  if ( !mPythonUtils )
+  {
+    std::cerr << "\nPython not available!";
+    return 1;
+  }
+
   const QStringList plugins = mPythonUtils->pluginList();
   if ( !plugins.contains( name ) )
   {
@@ -679,8 +761,10 @@ int QgsProcessingExec::enablePlugin( const QString &name, bool enabled )
 #endif
 }
 
-int QgsProcessingExec::showAlgorithmHelp( const QString &id, bool useJson )
+int QgsProcessingExec::showAlgorithmHelp( const QString &inputId, bool useJson )
 {
+  QString id = inputId;
+
   std::unique_ptr< QgsProcessingModelAlgorithm > model;
   const QgsProcessingAlgorithm *alg = nullptr;
   if ( QFile::exists( id ) && QFileInfo( id ).suffix() == QLatin1String( "model3" ) )
@@ -694,7 +778,21 @@ int QgsProcessingExec::showAlgorithmHelp( const QString &id, bool useJson )
 
     alg = model.get();
   }
-  else
+#ifdef WITH_BINDINGS
+  else if ( mPythonUtils && QFile::exists( id ) && QFileInfo( id ).suffix() == QLatin1String( "py" ) )
+  {
+    QString res;
+    if ( !mPythonUtils->evalString( QStringLiteral( "qgis.utils.import_script_algorithm(\"%1\")" ).arg( id ), res ) || res.isEmpty() )
+    {
+      std::cerr << QStringLiteral( "File %1 is not a valid Processing script!\n" ).arg( id ).toLocal8Bit().constData();
+      return 1;
+    }
+
+    id = res;
+  }
+#endif
+
+  if ( !alg )
   {
     alg = QgsApplication::processingRegistry()->algorithmById( id );
     if ( ! alg )
@@ -708,13 +806,41 @@ int QgsProcessingExec::showAlgorithmHelp( const QString &id, bool useJson )
   if ( !useJson )
   {
     std::cout << QStringLiteral( "%1 (%2)\n" ).arg( alg->displayName(), alg->id() ).toLocal8Bit().constData();
+
     std::cout << "\n----------------\n";
     std::cout << "Description\n";
     std::cout << "----------------\n";
-    if ( !alg->shortDescription().isEmpty() )
-      std::cout << alg->shortDescription().toLocal8Bit().constData() << '\n';
-    if ( !alg->shortHelpString().isEmpty() && alg->shortHelpString() != alg->shortDescription() )
-      std::cout << alg->shortHelpString().toLocal8Bit().constData() << '\n';
+
+    if ( const QgsProcessingModelAlgorithm *model = dynamic_cast< const QgsProcessingModelAlgorithm * >( alg ) )
+    {
+      // show finer help content for models
+      const QVariantMap help = model->helpContent();
+      std::cout << help.value( QStringLiteral( "ALG_DESC" ) ).toString().toLocal8Bit().constData() << '\n';
+
+      if ( !help.value( QStringLiteral( "ALG_CREATOR" ) ).toString().isEmpty() ||
+           !help.value( QStringLiteral( "ALG_VERSION" ) ).toString().isEmpty() )
+        std::cout << '\n';
+
+      if ( !help.value( QStringLiteral( "ALG_CREATOR" ) ).toString().isEmpty() )
+        std::cout << "Algorithm author:\t" << help.value( QStringLiteral( "ALG_CREATOR" ) ).toString().toLocal8Bit().constData() << '\n';
+      if ( !help.value( QStringLiteral( "ALG_VERSION" ) ).toString().isEmpty() )
+        std::cout << "Algorithm version:\t" << help.value( QStringLiteral( "ALG_VERSION" ) ).toString().toLocal8Bit().constData() << '\n';
+
+      if ( !help.value( QStringLiteral( "EXAMPLES" ) ).toString().isEmpty() )
+      {
+        std::cout << "\n----------------\n";
+        std::cout << "Examples\n";
+        std::cout << "----------------\n";
+        std::cout << help.value( QStringLiteral( "EXAMPLES" ) ).toString().toLocal8Bit().constData() << '\n';
+      }
+    }
+    else
+    {
+      if ( !alg->shortDescription().isEmpty() )
+        std::cout << alg->shortDescription().toLocal8Bit().constData() << '\n';
+      if ( !alg->shortHelpString().isEmpty() && alg->shortHelpString() != alg->shortDescription() )
+        std::cout << alg->shortHelpString().toLocal8Bit().constData() << '\n';
+    }
 
     std::cout << "\n----------------\n";
     std::cout << "Arguments\n";
@@ -729,11 +855,12 @@ int QgsProcessingExec::showAlgorithmHelp( const QString &id, bool useJson )
     addAlgorithmInformation( algorithmDetails, alg );
     json.insert( QStringLiteral( "algorithm_details" ), algorithmDetails );
     QVariantMap providerJson;
-    addProviderInformation( providerJson, alg->provider() );
+    if ( alg->provider() )
+      addProviderInformation( providerJson, alg->provider() );
     json.insert( QStringLiteral( "provider_details" ), providerJson );
   }
 
-
+  QgsProcessingContext context;
   QVariantMap parametersJson;
   const QgsProcessingParameterDefinitions defs = alg->parameterDefinitions();
   for ( const QgsProcessingParameterDefinition *p : defs )
@@ -744,7 +871,18 @@ int QgsProcessingExec::showAlgorithmHelp( const QString &id, bool useJson )
     QVariantMap parameterJson;
 
     if ( !useJson )
-      std::cout << QStringLiteral( "%1: %2\n" ).arg( p->name(), p->description() ).toLocal8Bit().constData();
+    {
+      QString line = QStringLiteral( "%1: %2" ).arg( p->name(), p->description() );
+      if ( p->flags() & QgsProcessingParameterDefinition::FlagOptional )
+        line += QLatin1String( " (optional)" );
+      std::cout << QStringLiteral( "%1\n" ).arg( line ).toLocal8Bit().constData();
+
+      if ( p->defaultValue().isValid() )
+      {
+        bool ok = false;
+        std::cout << QStringLiteral( "\tDefault value:\t%1\n" ).arg( p->valueAsString( p->defaultValue(), context, ok ) ).toLocal8Bit().constData();
+      }
+    }
     else
     {
       parameterJson.insert( QStringLiteral( "name" ), p->name() );
@@ -861,7 +999,7 @@ int QgsProcessingExec::showAlgorithmHelp( const QString &id, bool useJson )
   return 0;
 }
 
-int QgsProcessingExec::execute( const QString &id, const QVariantMap &params, const QString &ellipsoid, QgsUnitTypes::DistanceUnit distanceUnit, QgsUnitTypes::AreaUnit areaUnit, QgsProcessingContext::LogLevel logLevel, bool useJson, const QString &projectPath )
+int QgsProcessingExec::execute( const QString &inputId, const QVariantMap &params, const QString &ellipsoid, QgsUnitTypes::DistanceUnit distanceUnit, QgsUnitTypes::AreaUnit areaUnit, QgsProcessingContext::LogLevel logLevel, bool useJson, const QString &projectPath )
 {
   QVariantMap json;
   if ( useJson )
@@ -869,6 +1007,8 @@ int QgsProcessingExec::execute( const QString &id, const QVariantMap &params, co
     addVersionInformation( json );
   }
 
+  QString id = inputId;
+
   std::unique_ptr< QgsProcessingModelAlgorithm > model;
   const QgsProcessingAlgorithm *alg = nullptr;
   if ( QFile::exists( id ) && QFileInfo( id ).suffix() == QLatin1String( "model3" ) )
@@ -882,7 +1022,21 @@ int QgsProcessingExec::execute( const QString &id, const QVariantMap &params, co
 
     alg = model.get();
   }
-  else
+#ifdef WITH_BINDINGS
+  else if ( mPythonUtils && QFile::exists( id ) && QFileInfo( id ).suffix() == QLatin1String( "py" ) )
+  {
+    QString res;
+    if ( !mPythonUtils->evalString( QStringLiteral( "qgis.utils.import_script_algorithm(\"%1\")" ).arg( id ), res ) || res.isEmpty() )
+    {
+      std::cerr << QStringLiteral( "File %1 is not a valid Processing script!\n" ).arg( id ).toLocal8Bit().constData();
+      return 1;
+    }
+
+    id = res;
+  }
+#endif
+
+  if ( !alg )
   {
     alg = QgsApplication::processingRegistry()->algorithmById( id );
     if ( ! alg )
@@ -942,6 +1096,7 @@ int QgsProcessingExec::execute( const QString &id, const QVariantMap &params, co
       std::cerr << QStringLiteral( "Could not load the QGIS project \"%1\"\n" ).arg( projectPath ).toLocal8Bit().constData();
       return 1;
     }
+    json.insert( QStringLiteral( "project_path" ), projectPath );
   }
 
   if ( !useJson )
@@ -985,6 +1140,7 @@ int QgsProcessingExec::execute( const QString &id, const QVariantMap &params, co
       json.insert( QStringLiteral( "area_unit" ), QgsUnitTypes::toString( areaUnit ) );
   }
 
+
   QgsProcessingContext context;
   context.setEllipsoid( ellipsoid );
   context.setDistanceUnit( distanceUnit );
