@@ -92,17 +92,17 @@ void QgsGrassVectorMapLayer::load()
     return;
   }
 
-  QgsDebugMsg( QString( "cidxFieldIndex() = %1 cidxFieldNumCats() = %2" ).arg( cidxFieldIndex() ).arg( cidxFieldNumCats() ) );
+  QgsDebugMsgLevel( QString( "cidxFieldIndex() = %1 cidxFieldNumCats() = %2" ).arg( cidxFieldIndex() ).arg( cidxFieldNumCats() ), 2 );
 
   mFieldInfo = Vect_get_field( mMap->map(), mField ); // should work also with field = 0
 
   if ( !mFieldInfo )
   {
-    QgsDebugMsg( "No field info -> no attribute table" );
+    QgsDebugMsgLevel( "No field info -> no attribute table", 2 );
   }
   else
   {
-    QgsDebugMsg( "Field info found -> open database" );
+    QgsDebugMsgLevel( "Field info found -> open database", 2 );
 
     QFileInfo di( mMap->grassObject().mapsetPath() + "/vector/" + mMap->grassObject().name() + "/dbln" );
     mLastLoaded = di.lastModified();
@@ -116,14 +116,14 @@ void QgsGrassVectorMapLayer::load()
     }
     else
     {
-      QgsDebugMsg( "Database opened -> open select cursor" );
+      QgsDebugMsgLevel( "Database opened -> open select cursor", 2 );
       QgsGrass::lock(); // not sure if lock is necessary
       dbString dbstr;
       db_init_string( &dbstr );
       db_set_string( &dbstr, ( char * )"select * from " );
       db_append_string( &dbstr, mFieldInfo->table );
 
-      QgsDebugMsg( QString( "SQL: %1" ).arg( db_get_string( &dbstr ) ) );
+      QgsDebugMsgLevel( QString( "SQL: %1" ).arg( db_get_string( &dbstr ) ), 2 );
       dbCursor databaseCursor;
       if ( db_open_select_cursor( databaseDriver, &dbstr, &databaseCursor, DB_SCROLL ) != DB_OK )
       {
