@@ -50,6 +50,7 @@ enum QgsOraclePrimaryKeyType
   PktFidMap
 };
 
+
 /**
   * \class QgsOracleProvider
   * \brief Data provider for oracle layers.
@@ -77,6 +78,15 @@ class QgsOracleProvider final: public QgsVectorDataProvider
       const QMap<QString, QVariant> *options = nullptr
     );
 
+    enum Relkind
+    {
+      NotSet,
+      Unknown,
+      Table,
+      View,
+    };
+    Q_ENUM( Relkind )
+
     /**
      * Constructor for the provider. The uri must be in the following format:
      * host=localhost user=gsherman dbname=test password=xxx table=test.alaska (the_geom)
@@ -173,7 +183,7 @@ class QgsOracleProvider final: public QgsVectorDataProvider
     QString description() const override;
     QgsFeatureIterator getFeatures( const QgsFeatureRequest &request = QgsFeatureRequest() ) const override;
 
-    static bool exec( QSqlQuery &qry, QString sql, const QVariantList &args );
+    static bool execLoggedStatic( QSqlQuery &qry, const QString &sql, const QVariantList &args, const QString &uri, const QString &originatorClass = QString(), const QString &queryOrigin = QString() );
 
     bool isSaveAndLoadStyleToDatabaseSupported() const override { return true; }
     void setTransaction( QgsTransaction *transaction ) override;
@@ -189,7 +199,17 @@ class QgsOracleProvider final: public QgsVectorDataProvider
      */
     QString getWorkspace() const;
 
+    Qgis::VectorLayerTypeFlags vectorLayerTypeFlags() const override;
+
+    void handlePostCloneOperations( QgsVectorDataProvider *source ) override;
+
   private:
+
+    /**
+     * \returns relation kind
+     */
+    Relkind relkind() const;
+
     QString whereClause( QgsFeatureId featureId, QVariantList &args ) const;
     QString pkParamWhereClause() const;
 
@@ -263,6 +283,11 @@ class QgsOracleProvider final: public QgsVectorDataProvider
      */
     QString mSqlWhereClause;
 
+    /**
+     * Kind of relation
+     */
+    mutable Relkind mKind = Relkind::NotSet;
+
     /**
      * Data type for the primary key
      */
@@ -414,12 +439,15 @@ class QgsOracleProviderMetadata final: public QgsProviderMetadata
 
   public:
     QgsOracleProviderMetadata();
-    QString getStyleById( const QString &uri, QString styleId, QString &errCause ) override;
+    QIcon icon() const override;
+    QString getStyleById( const QString &uri, const QString &styleId, QString &errCause ) override;
     int listStyles( const QString &uri, QStringList &ids, QStringList &names, QStringList &descriptions, QString &errCause ) override;
     QString loadStyle( const QString &uri, QString &errCause ) override;
+    bool styleExists( const QString &uri, const QString &styleId, QString &errorCause ) override;
     bool saveStyle( const QString &uri, const QString &qmlStyle, const QString &sldStyle, const QString &styleName,
                     const QString &styleDescription, const QString &uiFileContent, bool useAsDefault, QString &errCause ) override;
     void cleanupProvider() override;
+    void initProvider() override;
     Qgis::VectorExportResult createEmptyLayer( const QString &uri,
         const QgsFields &fields, QgsWkbTypes::Type wkbType,
         const QgsCoordinateReferenceSystem &srs, bool overwrite,
@@ -438,6 +466,13 @@ class QgsOracleProviderMetadata final: public QgsProviderMetadata
 
     QVariantMap decodeUri( const QString &uri ) const override;
     QString encodeUri( const QVariantMap &parts ) const override;
+    QList< QgsMapLayerType > supportedLayerTypes() const override;
+
+  private:
+
+    // helper method to check if LAYER_STYLES table exists
+    bool layerStylesTableExists( QgsOracleConn *conn, const QgsDataSourceUri &dsUri, QString &errCause );
+
 };
 
 #ifdef HAVE_GUI
@@ -446,6 +481,7 @@ class QgsOracleProviderGuiMetadata final: public QgsProviderGuiMetadata
   public:
     QgsOracleProviderGuiMetadata();
     QList<QgsSourceSelectProvider *> sourceSelectProviders() override;
+    QList<QgsProjectStorageGuiProvider *> projectStorageGuiProviders() override;
     void registerGui( QMainWindow *mainWindow ) override;
 };
 #endif
