@@ -35,6 +35,7 @@ const QStringList CONFIGURATION_PARAMETERS
   QStringLiteral( "disableInvalidGeometryHandling" ),
   QStringLiteral( "onlyExistingTypes" ),
   QStringLiteral( "includeGeoAttributes" ),
+  QStringLiteral( "projectsInDatabase" ),
   QStringLiteral( "saveUsername" ),
   QStringLiteral( "savePassword" ),
   QStringLiteral( "schema" )
@@ -209,8 +210,7 @@ QgsVectorLayer *QgsOracleProviderConnection::createSqlVectorLayer( const QgsAbst
       const int type { row[ 0 ].toInt( &ok ) };
       if ( ok )
       {
-        const int srid { row[ 0 ].toInt( &ok ) };
-
+        const int srid { row[ 1 ].toInt( &ok ) };
         if ( ok )
         {
 
@@ -239,23 +239,23 @@ QgsVectorLayer *QgsOracleProviderConnection::createSqlVectorLayer( const QgsAbst
               break;
             // 3K...
             case 3001:
-              geomType = QgsWkbTypes::Point25D;
+              geomType = QgsWkbTypes::PointZ;
               break;
             case 3002:
-              geomType = QgsWkbTypes::LineString25D;
+              geomType = QgsWkbTypes::LineStringZ;
               break;
             case 3003:
-              geomType = QgsWkbTypes::Polygon25D;
+              geomType = QgsWkbTypes::PolygonZ;
               break;
             // Note: 3004 is missing
             case 3005:
-              geomType = QgsWkbTypes::MultiPoint25D;
+              geomType = QgsWkbTypes::MultiPointZ;
               break;
             case 3006:
-              geomType = QgsWkbTypes::MultiLineString25D;
+              geomType = QgsWkbTypes::MultiLineStringZ;
               break;
             case 3007:
-              geomType = QgsWkbTypes::MultiPolygon25D;
+              geomType = QgsWkbTypes::MultiPolygonZ;
               break;
             default:
               geomType = QgsWkbTypes::Type::Unknown;
@@ -1368,15 +1368,22 @@ QgsAbstractDatabaseProviderConnection::QueryResult QgsOracleProviderConnection::
 
   QSqlQuery qry( *pconn.get() );
   std::chrono::steady_clock::time_point begin = std::chrono::steady_clock::now();
+
+  QgsDatabaseQueryLogWrapper logWrapper { sql, uri(), providerKey(), QStringLiteral( "QgsAbstractDatabaseProviderConnection" ), QGS_QUERY_LOG_ORIGIN };
+
   if ( !qry.exec( sql ) )
   {
+    logWrapper.setError( qry.lastError().text() );
     throw QgsProviderConnectionException( QObject::tr( "SQL error: %1 returned %2" )
                                           .arg( qry.lastQuery(),
                                               qry.lastError().text() ) );
   }
 
   if ( feedback && feedback->isCanceled() )
+  {
+    logWrapper.setCanceled();
     return QgsAbstractDatabaseProviderConnection::QueryResult();
+  }
 
   if ( qry.isActive() )
   {
