@@ -200,18 +200,46 @@ class QgsPoolPostgresConn
     class QgsPostgresConn *get() const { return mPgConn; }
 };
 
+#include "qgsconfig.h"
+constexpr int sPostgresConQueryLogFilePrefixLength = CMAKE_SOURCE_DIR[sizeof( CMAKE_SOURCE_DIR ) - 1] == '/' ? sizeof( CMAKE_SOURCE_DIR ) + 1 : sizeof( CMAKE_SOURCE_DIR );
+#define LoggedPQexecNR(_class, query) PQexecNR( query, _class, QString(QString( __FILE__ ).mid( sPostgresConQueryLogFilePrefixLength ) + ':' + QString::number( __LINE__ ) + " (" + __FUNCTION__ + ")") )
+#define LoggedPQexec(_class, query) PQexec( query, true, true, _class, QString(QString( __FILE__ ).mid( sPostgresConQueryLogFilePrefixLength ) + ':' + QString::number( __LINE__ ) + " (" + __FUNCTION__ + ")") )
+#define LoggedPQexecNoLogError(_class, query ) PQexec( query, false, true, _class, QString(QString( __FILE__ ).mid( sPostgresConQueryLogFilePrefixLength ) + ':' + QString::number( __LINE__ ) + " (" + __FUNCTION__ + ")") )
+
 class QgsPostgresConn : public QObject
 {
     Q_OBJECT
 
   public:
-    /*
+
+    /**
+     * Get a new PostgreSQL connection
+     *
+     * \param connInfo the QgsDataSourceUri connection info with username / password
+     * \param readOnly is the connection read only ?
      * \param shared allow using a shared connection. Should never be
      *        called from a thread other than the main one.
      *        An assertion guards against such programmatic error.
+     * \param transaction is the connection a transaction ?
+     *
+     * \returns the PostgreSQL connection
      */
     static QgsPostgresConn *connectDb( const QString &connInfo, bool readOnly, bool shared = true, bool transaction = false );
 
+    /**
+     * Get a new PostgreSQL connection
+     *
+     * \param uri the QgsDataSourceUri with username / password
+     * \param readOnly is the connection read only ?
+     * \param shared allow using a shared connection. Should never be
+     *        called from a thread other than the main one.
+     *        An assertion guards against such programmatic error.
+     * \param transaction is the connection a transaction ?
+     *
+     * \returns the PostgreSQL connection
+     */
+    static QgsPostgresConn *connectDb( const QgsDataSourceUri &uri, bool readOnly, bool shared = true, bool transaction = false );
+
 
     QgsPostgresConn( const QString &conninfo, bool readOnly, bool shared, bool transaction );
     ~QgsPostgresConn() override;
@@ -246,8 +274,29 @@ class QgsPostgresConn : public QObject
     //! PostgreSQL version
     int pgVersion() const { return mPostgresqlVersion; }
 
+    /**
+     * Sets the current user identifier of the current PostgreSQL session
+     *
+     * \param sessionRole the PostgreSQL ROLE for the session, it must be a role that the current session user is a member of.
+     *
+     * \returns TRUE if successful
+     *
+     * \since QGIS 3.28.0
+     */
+    bool setSessionRole( const QString &sessionRole );
+
+    /**
+     * Resets the current user identifier of the current PostgreSQL session
+     * to the current session user identifier (user used to log in)
+     *
+     * \returns TRUE if successful
+     *
+     * \since QGIS 3.28.0
+     */
+    bool resetSessionRole();
+
     //! run a query and free result buffer
-    bool PQexecNR( const QString &query );
+    bool PQexecNR( const QString &query, const QString &originatorClass = QString(), const QString &queryOrigin = QString() );
 
     //! cursor handling
     bool openCursor( const QString &cursorName, const QString &declare );
@@ -264,13 +313,13 @@ class QgsPostgresConn : public QObject
     //
 
     // run a query and check for errors, thread-safe
-    PGresult *PQexec( const QString &query, bool logError = true, bool retry = true ) const;
+    PGresult *PQexec( const QString &query, bool logError = true, bool retry = true, const QString &originatorClass = QString(), const QString &queryOrigin = QString() ) const;
     int PQCancel();
     void PQfinish();
     QString PQerrorMessage() const;
     int PQstatus() const;
-    PGresult *PQprepare( const QString &stmtName, const QString &query, int nParams, const Oid *paramTypes );
-    PGresult *PQexecPrepared( const QString &stmtName, const QStringList &params );
+    PGresult *PQprepare( const QString &stmtName, const QString &query, int nParams, const Oid *paramTypes, const QString &originatorClass = QString(), const QString &queryOrigin = QString() );
+    PGresult *PQexecPrepared( const QString &stmtName, const QStringList &params, const QString &originatorClass = QString(), const QString &queryOrigin = QString() );
 
     /**
      * PQsendQuery is used for asynchronous queries (with PQgetResult)
@@ -401,11 +450,16 @@ class QgsPostgresConn : public QObject
     static bool allowGeometrylessTables( const QString &connName );
     static bool allowProjectsInDatabase( const QString &connName );
     static void deleteConnection( const QString &connName );
+    static bool allowMetadataInDatabase( const QString &connName );
 
     //! A connection needs to be locked when it uses transactions, see QgsPostgresConn::{begin,commit,rollback}
     void lock() { mLock.lock(); }
     void unlock() { mLock.unlock(); }
 
+    QgsCoordinateReferenceSystem sridToCrs( int srsId );
+
+    int crsToSrid( const QgsCoordinateReferenceSystem &crs );
+
   private:
 
     int mRef;
@@ -478,11 +532,15 @@ class QgsPostgresConn : public QObject
 
     bool mTransaction;
 
-#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
-    mutable QMutex mLock { QMutex::Recursive };
-#else
+    QString mCurrentSessionRole;
+
     mutable QRecursiveMutex mLock;
-#endif
+
+    /* Mutex protecting sCrsCache */
+    QMutex mCrsCacheMutex;
+
+    /* Cache of SRID to CRS */
+    mutable QMap<int, QgsCoordinateReferenceSystem> mCrsCache;
 };
 
 // clazy:excludeall=qstring-allocations
