@@ -32,12 +32,43 @@ class QMainWindow;
 class QgsEmbeddedLayerSelectDialog;
 class QgsLayerTreeView;
 
+class QgsVirtualLayerSourceWidget : public QWidget
+{
+    Q_OBJECT
+
+  public:
+
+    QgsVirtualLayerSourceWidget( QWidget *parent = nullptr );
+    void setBrowserModel( QgsBrowserModel *model );
+
+    void setSource( const QString &source, const QString &provider );
+    QString source() const;
+    QString provider() const;
+
+  signals:
+
+    void sourceChanged( const QString &source, const QString &provider );
+
+  public slots:
+
+    void browseForLayer();
+  private:
+
+    QLineEdit *mLineEdit = nullptr;
+    QString mProvider;
+    QgsBrowserModel *mBrowserModel = nullptr;
+};
+
+
 class QgsVirtualLayerSourceSelect : public QgsAbstractDataSourceWidget, private Ui::QgsVirtualLayerSourceSelectBase
 {
     Q_OBJECT
 
   public:
     QgsVirtualLayerSourceSelect( QWidget *parent = nullptr, Qt::WindowFlags fl = QgsGuiUtils::ModalDialogFlags, QgsProviderRegistry::WidgetMode widgetMode = QgsProviderRegistry::WidgetMode::None );
+    ~QgsVirtualLayerSourceSelect() override;
+
+    void setBrowserModel( QgsBrowserModel *model ) override;
 
   public slots:
     //! Triggered when the provider's connections need to be refreshed
@@ -49,21 +80,29 @@ class QgsVirtualLayerSourceSelect : public QgsAbstractDataSourceWidget, private
     void testQuery();
     void browseCRS();
     void layerComboChanged( int );
-    void addLayer();
+    void addLayer( bool browseForLayer = false );
     void removeLayer();
     void importLayer();
     void tableRowChanged( const QModelIndex &current, const QModelIndex &previous );
     void updateLayersList();
     void showHelp();
-
+    void rowSourceChanged();
 
   private:
+
+    enum LayerColumn
+    {
+      Name = 0,
+      Source = 1,
+      Provider = 2,
+      Encoding = 3,
+    };
+
     QgsVirtualLayerDefinition getVirtualLayerDef();
     long mSrid = 0;
     QStringList mProviderList;
-    QgsEmbeddedLayerSelectDialog *mEmbeddedSelectionDialog = nullptr;
     void addEmbeddedLayer( const QString &name, const QString &provider, const QString &encoding, const QString &source );
-    QgsLayerTreeView *mTreeView  = nullptr;
+    QgsLayerTreeView *mTreeView = nullptr;
     bool preFlight();
 };
 
