@@ -49,17 +49,11 @@ struct QgsWFSConstants
   static const QString URI_PARAM_PAGING_ENABLED;
   static const QString URI_PARAM_PAGE_SIZE;
   static const QString URI_PARAM_WFST_1_1_PREFER_COORDINATES;
+  static const QString URI_PARAM_SKIP_INITIAL_GET_FEATURE;
+  static const QString URI_PARAM_SQL;
 
   //
   static const QString VERSION_AUTO;
-
-  // Settings
-  static const QString CONNECTIONS_WFS;
-  static const QString SETTINGS_VERSION;
-  static const QString SETTINGS_MAXNUMFEATURES;
-  static const QString SETTINGS_PAGING_ENABLED;
-  static const QString SETTINGS_PAGE_SIZE;
-  static const QString SETTINGS_WFST_1_1_PREFER_COORDINATES;
 };
 
 #endif // QGSWFSCONSTANTS_H
