@@ -71,6 +71,28 @@ QgsWFSProvider::QgsWFSProvider( const QString &uri, const ProviderOptions &optio
     return;
   }
 
+  if ( mShared->mURI.typeName().isEmpty() )
+  {
+    QgsMessageLog::logMessage( tr( "Missing or empty 'typename' URI parameter" ), tr( "WFS" ) );
+    mValid = false;
+    return;
+  }
+
+  const QSet<QString> &unknownParamKeys = mShared->mURI.unknownParamKeys();
+  if ( !unknownParamKeys.isEmpty() )
+  {
+    QString msg = tr( "The following unknown parameter(s) have been found in the URI: " );
+    bool firstOne = true;
+    for ( const QString &key : unknownParamKeys )
+    {
+      if ( !firstOne )
+        msg += QLatin1String( ", " );
+      firstOne = false;
+      msg += key;
+    }
+    QgsMessageLog::logMessage( msg, tr( "WFS" ) );
+  }
+
   //create mSourceCrs from url if possible [WBC 111221] refactored from GetFeatureGET()
   QString srsname = mShared->mURI.SRSName();
   if ( !srsname.isEmpty() )
@@ -143,7 +165,7 @@ QgsWFSProvider::QgsWFSProvider( const QString &uri, const ProviderOptions &optio
     {
       auto processEvents = []()
       {
-        QApplication::instance()->processEvents();
+        QApplication::processEvents();
       };
       connect( downloader.get(), &QgsFeatureDownloader::resumeMainThread,
                this, processEvents );
@@ -154,24 +176,89 @@ QgsWFSProvider::QgsWFSProvider( const QString &uri, const ProviderOptions &optio
     mShared->setCurrentRect( QgsRectangle() );
   };
 
-  //Failed to detect feature type from describeFeatureType -> get first feature from layer to detect type
-  if ( mShared->mWKBType == QgsWkbTypes::Unknown )
+  const auto TryToDetectGeometryType = [&]()
   {
-    GetGeometryTypeFromOneFeature( true );
+    const QgsWkbTypes::Type initialGeometryType = mShared->mWKBType;
 
-    // If we still didn't get the geometry type, and have a filter, temporarily
-    // disable the filter.
-    // See https://github.com/qgis/QGIS/issues/43950
-    if ( mShared->mWKBType == QgsWkbTypes::Unknown && !mSubsetString.isEmpty() )
+    // try first without a BBOX, because some servers exhibit very poor
+    // performance when being requested on a large extent
+    GetGeometryTypeFromOneFeature( false );
+    if ( initialGeometryType == QgsWkbTypes::Unknown )
     {
-      const QString oldFilter = mShared->setWFSFilter( QString() );
-      GetGeometryTypeFromOneFeature( true );
-      if ( mShared->mWKBType == QgsWkbTypes::Unknown )
+      bool noGeometryFound = ( mShared->mWKBType == QgsWkbTypes::NoGeometry );
+      if ( noGeometryFound )
+        mShared->mWKBType = QgsWkbTypes::Unknown;
+
+      // If we still didn't get the geometry type, and have a filter, temporarily
+      // disable the filter.
+      // See https://github.com/qgis/QGIS/issues/43950
+      if ( mShared->mWKBType == QgsWkbTypes::Unknown && !mSubsetString.isEmpty() )
+      {
+        const QString oldFilter = mShared->setWFSFilter( QString() );
         GetGeometryTypeFromOneFeature( false );
-      mShared->setWFSFilter( oldFilter );
+        if ( mShared->mWKBType == QgsWkbTypes::NoGeometry )
+        {
+          noGeometryFound = true;
+          mShared->mWKBType = QgsWkbTypes::Unknown;
+        }
+        if ( mShared->mWKBType == QgsWkbTypes::Unknown )
+          GetGeometryTypeFromOneFeature( true );
+        mShared->setWFSFilter( oldFilter );
+      }
+      else if ( mShared->mWKBType == QgsWkbTypes::Unknown )
+        GetGeometryTypeFromOneFeature( true );
+
+      if ( noGeometryFound && mShared->mWKBType == QgsWkbTypes::Unknown )
+        mShared->mWKBType = QgsWkbTypes::NoGeometry;
+    }
+    else
+    {
+      mShared->mWKBType = initialGeometryType;
+    }
+  };
+
+  if ( !mShared->mURI.skipInitialGetFeature() )
+  {
+    // For WFS = 1.0, issue a GetFeature on one feature to check
+    // if we do not known the exact geometry type from
+    // describeFeatureType()
+    if ( mShared->mWFSVersion.startsWith( QLatin1String( "1.0" ) ) &&
+         mShared->mWKBType == QgsWkbTypes::Unknown )
+    {
+      TryToDetectGeometryType();
+    }
+    // For WFS >= 1.1, by default, issue a GetFeature on one feature to check
+    // if gml:description, gml:identifier, gml:name attributes are
+    // present (unless mShared->mURI.skipInitialGetFeature() returns false)
+    // Another reason to issue it if we do not known the exact geometry type
+    // from describeFeatureType()
+    else if ( !mShared->mWFSVersion.startsWith( QLatin1String( "1.0" ) ) )
+    {
+      // Try to see if gml:description, gml:identifier, gml:name attributes are
+      // present. So insert them temporarily in mShared->mFields so that the
+      // GML parser can detect them.
+      const auto addGMLFields = [ = ]( bool forceAdd )
+      {
+        if ( mShared->mFields.indexOf( QLatin1String( "description" ) ) < 0  && ( forceAdd || mSampleFeatureHasDescription ) )
+          mShared->mFields.append( QgsField( QStringLiteral( "description" ), QVariant::String, QStringLiteral( "xsd:string" ) ) );
+        if ( mShared->mFields.indexOf( QLatin1String( "identifier" ) ) < 0  && ( forceAdd || mSampleFeatureHasIdentifier ) )
+          mShared->mFields.append( QgsField( QStringLiteral( "identifier" ), QVariant::String, QStringLiteral( "xsd:string" ) ) );
+        if ( mShared->mFields.indexOf( QLatin1String( "name" ) ) < 0  && ( forceAdd || mSampleFeatureHasName ) )
+          mShared->mFields.append( QgsField( QStringLiteral( "name" ), QVariant::String, QStringLiteral( "xsd:string" ) ) );
+      };
+
+      const QgsFields fieldsBackup = mShared->mFields;
+      addGMLFields( true );
+
+      TryToDetectGeometryType();
+
+      // Edit the final mFields to add the description, identifier, name fields
+      // if they were actually found.
+      mShared->mFields.clear();
+      addGMLFields( false );
+      for ( const QgsField &field : fieldsBackup )
+        mShared->mFields.append( field );
     }
-    else if ( mShared->mWKBType == QgsWkbTypes::Unknown )
-      GetGeometryTypeFromOneFeature( false );
   }
 }
 
@@ -722,7 +809,7 @@ void QgsWFSProvider::pushErrorSlot( const QString &errorMsg )
 
 void QgsWFSProvider::featureReceivedAnalyzeOneFeature( QVector<QgsFeatureUniqueIdPair> list )
 {
-  if ( list.size() != 0 )
+  if ( list.size() != 0 && mShared->mWKBType == QgsWkbTypes::Unknown )
   {
     QgsFeature feat = list[0].first;
     QgsGeometry geometry = feat.geometry();
@@ -781,6 +868,14 @@ void QgsWFSProvider::featureReceivedAnalyzeOneFeature( QVector<QgsFeatureUniqueI
       }
     }
   }
+  if ( list.size() != 0 )
+  {
+    QgsFeature feat = list[0].first;
+    feat.padAttributes( mShared->mFields.count() );
+    mSampleFeatureHasDescription = !feat.attribute( "description" ).isNull();
+    mSampleFeatureHasIdentifier = !feat.attribute( "identifier" ).isNull();
+    mSampleFeatureHasName = !feat.attribute( "name" ).isNull();
+  }
 }
 
 QString QgsWFSProvider::subsetString() const
@@ -797,9 +892,14 @@ bool QgsWFSProvider::setSubsetString( const QString &theSQL, bool updateFeatureC
   if ( theSQL == mSubsetString )
     return true;
 
-  // Invalid and cancel current download before altering fields, etc...
-  // (crashes might happen if not done at the beginning)
-  mShared->invalidateCache();
+  disconnect( mShared.get(), &QgsWFSSharedData::raiseError, this, &QgsWFSProvider::pushErrorSlot );
+  disconnect( mShared.get(), &QgsWFSSharedData::extentUpdated, this, &QgsWFSProvider::fullExtentCalculated );
+
+  // We must not change the subset string of the shared data used in another iterator/data provider ...
+  mShared.reset( mShared->clone() );
+
+  connect( mShared.get(), &QgsWFSSharedData::raiseError, this, &QgsWFSProvider::pushErrorSlot );
+  connect( mShared.get(), &QgsWFSSharedData::extentUpdated, this, &QgsWFSProvider::fullExtentCalculated );
 
   mSubsetString = theSQL;
   clearMinMaxCache();
@@ -967,7 +1067,7 @@ bool QgsWFSProvider::addFeatures( QgsFeatureList &flist, Flags flags )
     for ( int i = 0; i < nAttrs; ++i )
     {
       const QVariant &value = featureAttributes.at( i );
-      if ( value.isValid() && !value.isNull() )
+      if ( value.isValid() && !QgsVariantUtils::isNull( value ) )
       {
         QDomElement fieldElem = transactionDoc.createElementNS( mApplicationNamespace, mShared->mFields.at( i ).name() );
         QDomText fieldText = transactionDoc.createTextNode( convertToXML( value ) );
@@ -1254,7 +1354,7 @@ bool QgsWFSProvider::changeAttributeValues( const QgsChangedAttributesMap &attr_
 
       QDomElement valueElem = transactionDoc.createElementNS( QgsWFSConstants::WFS_NAMESPACE, QStringLiteral( "Value" ) );
 
-      if ( attMapIt.value().isValid() && !attMapIt.value().isNull() )
+      if ( attMapIt.value().isValid() && !QgsVariantUtils::isNull( attMapIt.value() ) )
       {
         // WFS does not support :nil='true'
         // if value is NULL, do not add value element
@@ -1356,6 +1456,11 @@ bool QgsWFSProvider::empty() const
 #endif
   return !getFeatures( request ).nextFeature( f );
 
+}
+
+void QgsWFSProvider::handlePostCloneOperations( QgsVectorDataProvider *source )
+{
+  mShared = qobject_cast<QgsWFSProvider *>( source )->mShared;
 };
 
 bool QgsWFSProvider::describeFeatureType( QString &geometryAttribute, QgsFields &fields, QgsWkbTypes::Type &geomType )
@@ -2075,6 +2180,11 @@ QgsWFSProvider *QgsWfsProviderMetadata::createProvider( const QString &uri, cons
   return new QgsWFSProvider( uri, options );
 }
 
+QList<QgsMapLayerType> QgsWfsProviderMetadata::supportedLayerTypes() const
+{
+  return { QgsMapLayerType::VectorLayer };
+}
+
 QList<QgsDataItemProvider *> QgsWfsProviderMetadata::dataItemProviders() const
 {
   QList<QgsDataItemProvider *> providers;
@@ -2086,6 +2196,11 @@ QList<QgsDataItemProvider *> QgsWfsProviderMetadata::dataItemProviders() const
 QgsWfsProviderMetadata::QgsWfsProviderMetadata():
   QgsProviderMetadata( QgsWFSProvider::WFS_PROVIDER_KEY, QgsWFSProvider::WFS_PROVIDER_DESCRIPTION ) {}
 
+QIcon QgsWfsProviderMetadata::icon() const
+{
+  return QgsApplication::getThemeIcon( QStringLiteral( "mIconWfs.svg" ) );
+}
+
 
 #ifndef HAVE_STATIC_PROVIDERS
 QGISEXTERN void *multipleProviderMetadataFactory()
