@@ -77,6 +77,9 @@ QgsWMSSourceSelect::QgsWMSSourceSelect( QWidget *parent, Qt::WindowFlags fl, Qgs
 
   connect( mLayersFilterLineEdit, &QgsFilterLineEdit::textChanged, this, &QgsWMSSourceSelect::filterLayers );
   connect( mTilesetsFilterLineEdit, &QgsFilterLineEdit::textChanged, this, &QgsWMSSourceSelect::filterTiles );
+  connect( mLoadLayersIndividuallyCheckBox, &QCheckBox::toggled, leLayerName, &QLineEdit::setDisabled );
+
+  leLayerName->setDisabled( mLoadLayersIndividuallyCheckBox->isChecked() );
 
   // Creates and connects standard ok/apply buttons
   setupButtons( buttonBox );
@@ -87,7 +90,7 @@ QgsWMSSourceSelect::QgsWMSSourceSelect( QWidget *parent, Qt::WindowFlags fl, Qgs
   mStepHeight->setValidator( new QIntValidator( 0, 999999, this ) );
   mFeatureCount->setValidator( new QIntValidator( 0, 9999, this ) );
 
-  mImageFormatGroup = new QButtonGroup;
+  mImageFormatGroup = new QButtonGroup( this );
 
   if ( widgetMode() != QgsProviderRegistry::WidgetMode::Manager )
   {
@@ -143,6 +146,10 @@ QgsWMSSourceSelect::QgsWMSSourceSelect( QWidget *parent, Qt::WindowFlags fl, Qgs
     tabLayers->layout()->removeWidget( gbCRS );
   }
 
+
+  mInterpretationCombo = new QgsWmsInterpretationComboBox( this );
+  mInterpretationLayout->addWidget( mInterpretationCombo );
+
   clear();
 
   // set up the WMS connections we already know about
@@ -175,7 +182,7 @@ void QgsWMSSourceSelect::populateConnectionList()
 
 void QgsWMSSourceSelect::btnNew_clicked()
 {
-  QgsNewHttpConnection *nc = new QgsNewHttpConnection( this, QgsNewHttpConnection::ConnectionWms, QStringLiteral( "qgis/connections-wms/" ), QString(), QgsNewHttpConnection::FlagShowHttpSettings );
+  QgsNewHttpConnection *nc = new QgsNewHttpConnection( this, QgsNewHttpConnection::ConnectionWms, QStringLiteral( "WMS" ), QString(), QgsNewHttpConnection::FlagShowHttpSettings );
 
 
   if ( nc->exec() )
@@ -189,7 +196,7 @@ void QgsWMSSourceSelect::btnNew_clicked()
 
 void QgsWMSSourceSelect::btnEdit_clicked()
 {
-  QgsNewHttpConnection *nc = new QgsNewHttpConnection( this, QgsNewHttpConnection::ConnectionWms, QStringLiteral( "qgis/connections-wms/" ), cmbConnections->currentText(), QgsNewHttpConnection::FlagShowHttpSettings );
+  QgsNewHttpConnection *nc = new QgsNewHttpConnection( this, QgsNewHttpConnection::ConnectionWms, QStringLiteral( "WMS" ), cmbConnections->currentText(), QgsNewHttpConnection::FlagShowHttpSettings );
 
   if ( nc->exec() )
   {
@@ -284,6 +291,8 @@ void QgsWMSSourceSelect::clear()
   }
 
   mFeatureCount->setEnabled( false );
+
+  mInterpretationCombo->setInterpretation( QString() );
 }
 
 bool QgsWMSSourceSelect::populateLayerList( const QgsWmsCapabilities &capabilities )
@@ -390,6 +399,7 @@ bool QgsWMSSourceSelect::populateLayerList( const QgsWmsCapabilities &capabiliti
           for ( const QString &format : l.formats )
           {
             QTableWidgetItem *item = new QTableWidgetItem( l.identifier );
+            item->setIcon( QgsApplication::getThemeIcon( l.timeDimensionIdentifier.isEmpty() ? QStringLiteral( "/mIconRaster.svg" ) : QStringLiteral( "/mIconTemporalRaster.svg" ) ) );
             item->setData( Qt::UserRole + 0, l.identifier );
             item->setData( Qt::UserRole + 1, format );
             item->setData( Qt::UserRole + 2, style.identifier );
@@ -482,7 +492,7 @@ void QgsWMSSourceSelect::btnConnect_clicked()
     QMessageBox::warning(
       this,
       tr( "WMS Provider" ),
-      tr( "Failed to download capabilities:\n" ) + capDownload.lastError()
+      capDownload.lastError()
     );
     return;
   }
@@ -559,7 +569,7 @@ void QgsWMSSourceSelect::addButtonClicked()
     if ( !layer )
       return;
 
-    if ( !layer->dimensions.isEmpty() )
+    if ( layer->dimensions.size() > 1 || ( layer->dimensions.size() == 1 && layer->dimensions.constBegin()->identifier != layer->timeDimensionIdentifier ) )
     {
       QgsWmtsDimensions *dlg = new QgsWmtsDimensions( *layer, this );
       if ( dlg->exec() != QDialog::Accepted )
@@ -568,8 +578,7 @@ void QgsWMSSourceSelect::addButtonClicked()
         return;
       }
 
-      QHash<QString, QString> dims;
-      dlg->selectedDimensions( dims );
+      const QHash<QString, QString> dims = dlg->selectedDimensions();
 
       QString dimString;
       QString delim;
@@ -587,9 +596,6 @@ void QgsWMSSourceSelect::addButtonClicked()
       uri.setParam( QStringLiteral( "tileDimensions" ), dimString );
     }
   }
-
-  uri.setParam( QStringLiteral( "layers" ), layers );
-  uri.setParam( QStringLiteral( "styles" ), styles );
   uri.setParam( QStringLiteral( "format" ), format );
   uri.setParam( QStringLiteral( "crs" ), crs );
   QgsDebugMsgLevel( QStringLiteral( "crs=%2 " ).arg( crs ), 2 );
@@ -599,11 +605,34 @@ void QgsWMSSourceSelect::addButtonClicked()
     uri.setParam( QStringLiteral( "featureCount" ), mFeatureCount->text() );
   }
 
-  uri.setParam( QStringLiteral( "contextualWMSLegend" ), mContextualLegendCheckbox->isChecked() ? "1" : "0" );
+  if ( tabTilesets->isEnabled() && !mInterpretationCombo->interpretation().isEmpty() )
+    uri.setParam( QStringLiteral( "interpretation" ), mInterpretationCombo->interpretation() );
 
-  emit addRasterLayer( uri.encodedUri(),
-                       leLayerName->text().isEmpty() ? titles.join( QLatin1Char( '/' ) ) : leLayerName->text(),
-                       QStringLiteral( "wms" ) );
+  uri.setParam( QStringLiteral( "contextualWMSLegend" ), mContextualLegendCheckbox->isChecked() ? "1" : "0" );
+  if ( mLoadLayersIndividuallyCheckBox->isChecked() )
+  {
+    QgsDebugMsgLevel( QStringLiteral( "layers=%1 " ).arg( layers.join( ", " ) ), 2 );
+    for ( int i = 0; i < layers.count(); i++ )
+    {
+      QgsDataSourceUri individualUri( uri );
+      individualUri.setParam( QStringLiteral( "layers" ), layers.at( i ) );
+      individualUri.setParam( QStringLiteral( "styles" ), styles.at( i ) );
+
+      emit addRasterLayer( individualUri.encodedUri(),
+                           titles.at( i ),
+                           QStringLiteral( "wms" ) );
+    }
+
+  }
+  else
+  {
+    uri.setParam( QStringLiteral( "layers" ), layers );
+    uri.setParam( QStringLiteral( "styles" ), styles );
+
+    emit addRasterLayer( uri.encodedUri(),
+                         leLayerName->text().isEmpty() ? titles.join( QLatin1Char( '/' ) ) : leLayerName->text(),
+                         QStringLiteral( "wms" ) );
+  }
 }
 
 void QgsWMSSourceSelect::reset()
@@ -843,6 +872,15 @@ void QgsWMSSourceSelect::lstLayers_itemSelectionChanged()
   labelCoordRefSys->setDisabled( mCRSs.isEmpty() );
   mCrsSelector->setDisabled( mCRSs.isEmpty() );
 
+  QList< QgsCoordinateReferenceSystem > crsFilter;
+  crsFilter.reserve( mCRSs.size() );
+  for ( const QString &crs : std::as_const( mCRSs ) )
+  {
+    crsFilter << QgsCoordinateReferenceSystem( crs );
+  }
+  if ( !crsFilter.isEmpty() )
+    mCrsSelector->setFilter( crsFilter );
+
   if ( !layers.isEmpty() && !mCRSs.isEmpty() )
   {
 
@@ -977,32 +1015,27 @@ void QgsWMSSourceSelect::updateButtons()
     }
   }
 
-  if ( leLayerName->text().isEmpty() || leLayerName->text() == mLastLayerName )
+  if ( addButton()->isEnabled() )
   {
-    if ( addButton()->isEnabled() )
+    if ( !lstTilesets->selectedItems().isEmpty() )
     {
-      if ( !lstTilesets->selectedItems().isEmpty() )
-      {
-        QTableWidgetItem *item = lstTilesets->selectedItems().first();
-        mLastLayerName = item->data( Qt::UserRole + 5 ).toString();
-        if ( mLastLayerName.isEmpty() )
-          mLastLayerName = item->data( Qt::UserRole + 0 ).toString();
-        leLayerName->setText( mLastLayerName );
-      }
-      else
-      {
-        QStringList layers, styles, titles;
-        collectSelectedLayers( layers, styles, titles );
-        mLastLayerName = titles.join( QLatin1Char( '/' ) );
-        leLayerName->setText( mLastLayerName );
-      }
+      QTableWidgetItem *item = lstTilesets->selectedItems().first();
+      QString tileLayerName = item->data( Qt::UserRole + 5 ).toString();
+      if ( tileLayerName.isEmpty() )
+        tileLayerName = item->data( Qt::UserRole + 0 ).toString();
+      leLayerName->setText( tileLayerName );
     }
     else
     {
-      mLastLayerName.clear();
-      leLayerName->setText( mLastLayerName );
+      QStringList layers, styles, titles;
+      collectSelectedLayers( layers, styles, titles );
+      leLayerName->setText( titles.join( QLatin1Char( '/' ) ) );
     }
   }
+  else
+  {
+    leLayerName->setText( "" );
+  }
 }
 
 
@@ -1308,3 +1341,27 @@ void QgsWMSSourceSelect::showHelp()
 {
   QgsHelp::openHelp( QStringLiteral( "working_with_ogc/ogc_client_support.html" ) );
 }
+
+QgsWmsInterpretationComboBox::QgsWmsInterpretationComboBox( QWidget *parent ): QComboBox( parent )
+{
+  addItem( tr( "Default" ), QString() );
+  addItem( QgsWmsInterpretationConverterMapTilerTerrainRGB::displayName(), QgsWmsInterpretationConverterMapTilerTerrainRGB::interpretationKey() );
+  addItem( QgsWmsInterpretationConverterTerrariumRGB::displayName(), QgsWmsInterpretationConverterTerrariumRGB::interpretationKey() );
+}
+
+void QgsWmsInterpretationComboBox::setInterpretation( const QString &interpretationKey )
+{
+  if ( ! interpretationKey.isEmpty() )
+  {
+    int index = findData( interpretationKey );
+    if ( index == -1 )
+      setCurrentIndex( 0 );
+    else
+      setCurrentIndex( index );
+  }
+}
+
+QString QgsWmsInterpretationComboBox::interpretation() const
+{
+  return currentData().toString();
+}
