@@ -20,6 +20,7 @@
 
 //for CMAKE_INSTALL_PREFIX
 #include "qgsconfig.h"
+#include "qgsversion.h"
 #include "qgsserver.h"
 #include "qgsauthmanager.h"
 #include "qgscapabilitiescache.h"
@@ -307,6 +308,8 @@ bool QgsServer::init()
   // Configure locale
   initLocale();
 
+  QgsMessageLog::logMessage( QStringLiteral( "QGIS Server Starting : %1 (%2)" ).arg( _QGIS_VERSION, QGSVERSION ), "Server", Qgis::MessageLevel::Info );
+
   // log settings currently used
   sSettings()->logSummary();
 
@@ -362,6 +365,9 @@ bool QgsServer::init()
   // qDebug() << QStringLiteral( "Initializing server modules from: %1" ).arg( modulePath );
   sServiceRegistry->init( modulePath,  sServerInterface );
 
+  // Initialize config cache
+  QgsConfigCache::initialize( sSettings );
+
   sInitialized = true;
   QgsMessageLog::logMessage( QStringLiteral( "Server initialized" ), QStringLiteral( "Server" ), Qgis::MessageLevel::Info );
   return true;
@@ -460,13 +466,14 @@ void QgsServer::handleRequest( QgsServerRequest &request, QgsServerResponse &res
         printRequestParameters( params.toMap(), logLevel );
 
         // Setup project (config file path)
-        if ( ! project )
+        if ( !project )
         {
           const QString configFilePath = configPath( *sConfigFilePath, params.map() );
 
           // load the project if needed and not empty
           if ( ! configFilePath.isEmpty() )
           {
+            // Note that  QgsConfigCache::project( ... ) call QgsProject::setInstance(...)
             project = mConfigCache->project( configFilePath, sServerInterface->serverSettings() );
           }
         }
@@ -490,6 +497,7 @@ void QgsServer::handleRequest( QgsServerRequest &request, QgsServerResponse &res
         // Dispatcher: if SERVICE is set, we assume a OWS service, if not, let's try an API
         // TODO: QGIS 4 fix the OWS services and treat them as APIs
         QgsServerApi *api = nullptr;
+
         if ( params.service().isEmpty() && ( api = sServiceRegistry->apiForRequest( request ) ) )
         {
           const QgsServerApiContext context { api->rootPath(), &request, &responseDecorator, project, sServerInterface };
@@ -497,7 +505,6 @@ void QgsServer::handleRequest( QgsServerRequest &request, QgsServerResponse &res
         }
         else
         {
-
           // Project is mandatory for OWS at this point
           if ( ! project )
           {
