@@ -84,6 +84,14 @@ class SERVER_EXPORT QgsServerApiUtils
      */
     static QgsDateTimeRange parseTemporalDateTimeInterval( const QString &interval ) SIP_THROW( QgsServerApiBadRequestException );
 
+
+    /**
+     * Given a field \a name (or display name) and a \a layer returns the actual name of the field.
+     * \throws QgsServerApiBadRequestException if \a name is neither a field name or a display name.
+     * \since QGIS 3.28
+     */
+    static QString fieldName( const QString &name, const QgsVectorLayer *layer ) SIP_THROW( QgsServerApiBadRequestException );
+
 ///@cond PRIVATE
     // T is TemporalDateInterval|TemporalDateTimeInterval, T2 is QDate|QdateTime
     template<typename T, class T2> static T parseTemporalInterval( const QString &interval ) SIP_SKIP;
