@@ -40,14 +40,14 @@ using namespace inja;
 QVariantMap QgsServerOgcApiHandler::values( const QgsServerApiContext &context ) const
 {
   QVariantMap result ;
-  QVariantList positional;
   const auto constParameters { parameters( context ) };
   for ( const auto &p : constParameters )
   {
     // value() calls the validators and throws an exception if validation fails
     result[p.name()] = p.value( context );
   }
-  const auto match { path().match( context.request()->url().toString() ) };
+  const auto sanitizedPath { QgsServerOgcApi::sanitizeUrl( context.handlerPath( ) ).path() };
+  const auto match { path().match( sanitizedPath ) };
   if ( match.hasMatch() )
   {
     const auto constNamed { path().namedCaptureGroups() };
@@ -68,7 +68,8 @@ QgsServerOgcApiHandler::~QgsServerOgcApiHandler()
 
 QgsServerOgcApi::ContentType QgsServerOgcApiHandler::defaultContentType() const
 {
-  return contentTypes().size() > 0 ? contentTypes().first() : QgsServerOgcApi::ContentType::JSON;
+  const auto constContentTypes( contentTypes() );
+  return constContentTypes.size() > 0 ? constContentTypes.first() : QgsServerOgcApi::ContentType::JSON;
 }
 
 QList<QgsServerOgcApi::ContentType> QgsServerOgcApiHandler::contentTypes() const
@@ -84,8 +85,9 @@ void QgsServerOgcApiHandler::handleRequest( const QgsServerApiContext &context )
 
 QString QgsServerOgcApiHandler::contentTypeForAccept( const QString &accept ) const
 {
-  for ( auto it = QgsServerOgcApi::contentTypeMimes().constBegin();
-        it != QgsServerOgcApi::contentTypeMimes().constEnd(); ++it )
+  const auto constContentTypes( QgsServerOgcApi::contentTypeMimes() );
+  for ( auto it = constContentTypes.constBegin();
+        it != constContentTypes.constEnd(); ++it )
   {
     const auto constValues = it.value();
     for ( const auto &value : constValues )
@@ -99,7 +101,7 @@ QString QgsServerOgcApiHandler::contentTypeForAccept( const QString &accept ) co
   // Log level info because this is not completely unexpected
   QgsMessageLog::logMessage( QStringLiteral( "Content type for accept %1 not found!" ).arg( accept ),
                              QStringLiteral( "Server" ),
-                             Qgis::Info );
+                             Qgis::MessageLevel::Info );
 
   return QString();
 }
@@ -122,6 +124,9 @@ void QgsServerOgcApiHandler::write( json &data, const QgsServerApiContext &conte
     case QgsServerOgcApi::ContentType::OPENAPI3:
       jsonDump( data, context, QgsServerOgcApi::contentTypeMimes().value( contentType ).first() );
       break;
+    case QgsServerOgcApi::ContentType::XML:
+      // Not handled yet
+      break;
   }
 }
 
@@ -136,7 +141,7 @@ std::string QgsServerOgcApiHandler::href( const QgsServerApiContext &context, co
 {
   QUrl url { context.request()->url() };
   QString urlBasePath { context.matchedPath() };
-  const auto match { path().match( url.path() ) };
+  const auto match { path().match( QgsServerOgcApi::sanitizeUrl( context.handlerPath( ) ).path( ) ) };
   if ( match.captured().count() > 0 )
   {
     url.setPath( urlBasePath + match.captured( 0 ) );
@@ -147,7 +152,7 @@ std::string QgsServerOgcApiHandler::href( const QgsServerApiContext &context, co
   }
 
   // Remove any existing extension
-  const auto suffixLength { QFileInfo( url.path() ).completeSuffix().length() };
+  const auto suffixLength { QFileInfo( url.path() ).suffix().length() };
   if ( suffixLength > 0 )
   {
     auto path {url.path()};
@@ -255,11 +260,11 @@ const QString QgsServerOgcApiHandler::templatePath( const QgsServerApiContext &c
 {
   // resources/server/api + /ogc/templates/ + operationId + .html
   QString path { context.serverInterface()->serverSettings()->apiResourcesDirectory() };
-  path += QStringLiteral( "/ogc/templates" );
+  path += QLatin1String( "/ogc/templates" );
   path += context.apiRootPath();
   path += '/';
   path += QString::fromStdString( operationId() );
-  path += QStringLiteral( ".html" );
+  path += QLatin1String( ".html" );
   return path;
 }
 
@@ -270,14 +275,14 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
   auto path { templatePath( context ) };
   if ( ! QFile::exists( path ) )
   {
-    QgsMessageLog::logMessage( QStringLiteral( "Template not found error: %1" ).arg( path ), QStringLiteral( "Server" ), Qgis::Critical );
+    QgsMessageLog::logMessage( QStringLiteral( "Template not found error: %1" ).arg( path ), QStringLiteral( "Server" ), Qgis::MessageLevel::Critical );
     throw QgsServerApiBadRequestException( QStringLiteral( "Template not found: %1" ).arg( QFileInfo( path ).fileName() ) );
   }
 
   QFile f( path );
   if ( ! f.open( QFile::ReadOnly | QFile::Text ) )
   {
-    QgsMessageLog::logMessage( QStringLiteral( "Could not open template file: %1" ).arg( path ), QStringLiteral( "Server" ), Qgis::Critical );
+    QgsMessageLog::logMessage( QStringLiteral( "Could not open template file: %1" ).arg( path ), QStringLiteral( "Server" ), Qgis::MessageLevel::Critical );
     throw QgsServerApiInternalServerError( QStringLiteral( "Could not open template file: %1" ).arg( QFileInfo( path ).fileName() ) );
   }
 
@@ -285,7 +290,7 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
   {
     // Get the template directory and the file name
     QFileInfo pathInfo { path };
-    Environment env { ( pathInfo.dir().path() + QDir::separator() ).toStdString() };
+    Environment env { QString( pathInfo.dir().path() + QDir::separator() ).toStdString() };
 
     // For template debugging:
     env.add_callback( "json_dump", 0, [ = ]( Arguments & )
@@ -300,11 +305,16 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
       QFileInfo fi{ url.path() };
       auto suffix { fi.suffix() };
       auto fName { fi.filePath()};
-      if ( suffix.length() != 0 )
+      if ( !suffix.isEmpty() )
       {
         fName.chop( suffix.length() + 1 );
       }
-      fName += '/' + QString::number( args.at( 0 )->get<QgsFeatureId>( ) );
+      // Chop any ending slashes
+      while ( fName.endsWith( '/' ) )
+      {
+        fName.chop( 1 );
+      }
+      fName += '/' + QString::fromStdString( args.at( 0 )->get<std::string>( ) );
       if ( !suffix.isEmpty() )
       {
         fName += '.' + suffix;
@@ -362,19 +372,63 @@ void QgsServerOgcApiHandler::htmlDump( const json &data, const QgsServerApiConte
       return QgsServerOgcApi::contentTypeToStdString( ct );
     } );
 
+    // Replace newlines with <br>
+    env.add_callback( "nl2br", 1, [ = ]( Arguments & args )
+    {
+      QString text { QString::fromStdString( args.at( 0 )->get<std::string>( ) ) };
+      return text.replace( '\n', QLatin1String( "<br>" ) ).toStdString();
+    } );
+
+
+    // Returns a list of parameter component data from components -> parameters by ref name
+    // parameter( <ref object> )
+    env.add_callback( "component_parameter", 1, [ = ]( Arguments & args )
+    {
+      json ret = json::array();
+      json ref = args.at( 0 )->get<json>( );
+      if ( ! ref.is_object() )
+      {
+        return ret;
+      }
+      try
+      {
+        QString name = QString::fromStdString( ref["$ref"] );
+        name = name.split( '/' ).last();
+        ret.push_back( data["components"]["parameters"][name.toStdString()] );
+      }
+      catch ( std::exception & )
+      {
+        // Do nothing
+      }
+      return ret;
+    } );
+
 
     // Static: returns the full URL to the specified static <path>
     env.add_callback( "static", 1, [ = ]( Arguments & args )
     {
       auto asset( args.at( 0 )->get<std::string>( ) );
-      return context.matchedPath().toStdString() + "/static/" + asset;
+      QString matchedPath { context.matchedPath() };
+      // If its the root path '/' strip it!
+      if ( matchedPath == '/' )
+      {
+        matchedPath.clear();
+      }
+      return matchedPath.toStdString() + "/static/" + asset;
+    } );
+
+
+    // Returns true if a string begins with the provided string prefix, false otherwise
+    env.add_callback( "starts_with", 2, [ ]( Arguments & args )
+    {
+      return string_view::starts_with( args.at( 0 )->get<std::string_view>( ), args.at( 1 )->get<std::string_view>( ) );
     } );
 
     context.response()->write( env.render_file( pathInfo.fileName().toStdString(), data ) );
   }
   catch ( std::exception &e )
   {
-    QgsMessageLog::logMessage( QStringLiteral( "Error parsing template file: %1 - %2" ).arg( path, e.what() ), QStringLiteral( "Server" ), Qgis::Critical );
+    QgsMessageLog::logMessage( QStringLiteral( "Error parsing template file: %1 - %2" ).arg( path, e.what() ), QStringLiteral( "Server" ), Qgis::MessageLevel::Critical );
     throw QgsServerApiInternalServerError( QStringLiteral( "Error parsing template file: %1" ).arg( e.what() ) );
   }
 }
@@ -385,7 +439,7 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
   QgsServerOgcApi::ContentType result { defaultContentType() };
   bool found { false };
   // First file extension ...
-  const QString extension { QFileInfo( request->url().path() ).completeSuffix().toUpper() };
+  const QString extension { QFileInfo( request->url().path() ).suffix().toUpper() };
   if ( ! extension.isEmpty() )
   {
     static QMetaEnum metaEnum { QMetaEnum::fromType<QgsServerOgcApi::ContentType>() };
@@ -398,7 +452,7 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
     }
     else
     {
-      QgsMessageLog::logMessage( QStringLiteral( "The client requested an unsupported extension: %1" ).arg( extension ), QStringLiteral( "Server" ), Qgis::Warning );
+      QgsMessageLog::logMessage( QStringLiteral( "The client requested an unsupported extension: %1" ).arg( extension ), QStringLiteral( "Server" ), Qgis::MessageLevel::Warning );
     }
   }
   // ... then "Accept"
@@ -408,8 +462,9 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
     const QString ctFromAccept { contentTypeForAccept( accept ) };
     if ( ! ctFromAccept.isEmpty() )
     {
-      auto it = QgsServerOgcApi::contentTypeMimes().constBegin();
-      while ( ! found && it != QgsServerOgcApi::contentTypeMimes().constEnd() )
+      const auto constContentTypes( QgsServerOgcApi::contentTypeMimes() );
+      auto it = constContentTypes.constBegin();
+      while ( ! found && it != constContentTypes.constEnd() )
       {
         int idx = it.value().indexOf( ctFromAccept );
         if ( idx >= 0 )
@@ -422,7 +477,7 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
     }
     else
     {
-      QgsMessageLog::logMessage( QStringLiteral( "The client requested an unsupported content type in Accept header: %1" ).arg( accept ), QStringLiteral( "Server" ), Qgis::Warning );
+      QgsMessageLog::logMessage( QStringLiteral( "The client requested an unsupported content type in Accept header: %1" ).arg( accept ), QStringLiteral( "Server" ), Qgis::MessageLevel::Warning );
     }
   }
   // Validation: check if the requested content type (or an alias) is supported by the handler
@@ -430,7 +485,7 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
   {
     // Check aliases
     bool found { false };
-    if ( QgsServerOgcApi::contentTypeAliases().keys().contains( result ) )
+    if ( QgsServerOgcApi::contentTypeAliases().contains( result ) )
     {
       const QList<QgsServerOgcApi::ContentType> constCt { contentTypes() };
       for ( const auto &ct : constCt )
@@ -446,7 +501,7 @@ QgsServerOgcApi::ContentType QgsServerOgcApiHandler::contentTypeFromRequest( con
 
     if ( ! found )
     {
-      QgsMessageLog::logMessage( QStringLiteral( "Unsupported Content-Type: %1" ).arg( QgsServerOgcApi::contentTypeToString( result ) ), QStringLiteral( "Server" ), Qgis::Info );
+      QgsMessageLog::logMessage( QStringLiteral( "Unsupported Content-Type: %1" ).arg( QgsServerOgcApi::contentTypeToString( result ) ), QStringLiteral( "Server" ), Qgis::MessageLevel::Info );
       throw QgsServerApiBadRequestException( QStringLiteral( "Unsupported Content-Type: %1" ).arg( QgsServerOgcApi::contentTypeToString( result ) ) );
     }
   }
@@ -472,8 +527,9 @@ QString QgsServerOgcApiHandler::parentLink( const QUrl &url, int levels )
     path = path.replace( re, QString() );
   }
   QUrl result( url );
+  QUrlQuery query( result );
   QList<QPair<QString, QString> > qi;
-  const auto constItems { result.queryItems( ) };
+  const auto constItems { query.queryItems( ) };
   for ( const auto &i : constItems )
   {
     if ( i.first.compare( QStringLiteral( "MAP" ), Qt::CaseSensitivity::CaseInsensitive ) == 0 )
@@ -481,7 +537,14 @@ QString QgsServerOgcApiHandler::parentLink( const QUrl &url, int levels )
       qi.push_back( i );
     }
   }
-  result.setQueryItems( qi );
+  // Make sure the parent link ends with a slash
+  if ( ! path.endsWith( '/' ) )
+  {
+    path.append( '/' );
+  }
+  QUrlQuery resultQuery;
+  resultQuery.setQueryItems( qi );
+  result.setQuery( resultQuery );
   result.setPath( path );
   return result.toString();
 }
@@ -535,7 +598,7 @@ json QgsServerOgcApiHandler::jsonTags() const
 void QgsServerOgcApiHandler::setContentTypesInt( const QList<int> &contentTypes )
 {
   mContentTypes.clear();
-  for ( const int &i : qgis::as_const( contentTypes ) )
+  for ( const int &i : std::as_const( contentTypes ) )
   {
     mContentTypes.push_back( static_cast<QgsServerOgcApi::ContentType>( i ) );
   }
