@@ -74,9 +74,6 @@ namespace QgsWfs
   const QString OGC_NAMESPACE = QStringLiteral( "http://www.opengis.net/ogc" );
   const QString QGS_NAMESPACE = QStringLiteral( "http://www.qgis.org/gml" );
 
-  // Define clean tagName regExp
-  const QRegExp cleanTagNameRegExp( "(?![\\w\\d\\.-])." );
-
 } // namespace QgsWfs
 
 #endif
