@@ -40,6 +40,7 @@ class TestNineCellFilters : public QObject
   private slots:
 
     void initTestCase();
+    void cleanupTestCase();
     void init();
 
     void testHillshade();
@@ -88,6 +89,11 @@ void TestNineCellFilters::initTestCase()
   QgsApplication::init(); // needed for CRS database
 }
 
+void TestNineCellFilters::cleanupTestCase()
+{
+  QgsApplication::exitQgis();
+}
+
 template <class T>
 void TestNineCellFilters::_testAlg( const QString &name, bool useOpenCl )
 {
