@@ -61,6 +61,7 @@ class TestQgsAttributeTable : public QObject
     void testSortByDisplayExpression();
     void testOrderColumn();
     void testFilteredFeatures();
+    void testOpenWithFilterExpression();
     void testVisibleTemporal();
     void testCopySelectedRows();
     void testSortNumbers();
@@ -757,5 +758,40 @@ void TestQgsAttributeTable::testCopySelectedRows()
   QCOMPARE( clipboard->crs().authid(), "EPSG:3111" );
 }
 
+void TestQgsAttributeTable::testOpenWithFilterExpression()
+{
+  // test attribute table opening in show feature visible mode
+  std::unique_ptr< QgsVectorLayer> tempLayer( new QgsVectorLayer( QStringLiteral( "LineString?crs=epsg:4326&field=pk:int&field=col1:date" ), QStringLiteral( "vl" ), QStringLiteral( "memory" ) ) );
+  QVERIFY( tempLayer->isValid() );
+
+  QgsPolylineXY line;
+  line << QgsPointXY( 0, 0 ) << QgsPointXY( 1, 1 );
+  QgsGeometry geometry = QgsGeometry::fromPolylineXY( line ) ;
+  QgsFeature f1( tempLayer->dataProvider()->fields(), 1 );
+  f1.setGeometry( geometry );
+  f1.setAttributes( QgsAttributes() << 1 << QDate( 2020, 1, 1 ) );
+  QgsFeature f2( tempLayer->dataProvider()->fields(), 2 );
+  f2.setGeometry( geometry );
+  f2.setAttributes( QgsAttributes() << 2 << QDate( 2020, 3, 1 ) );
+  QgsFeature f3( tempLayer->dataProvider()->fields(), 3 );
+  line.clear();
+  line << QgsPointXY( -3, -3 ) << QgsPointXY( -2, -2 );
+  geometry = QgsGeometry::fromPolylineXY( line );
+  f3.setGeometry( geometry );
+  f3.setAttributes( QgsAttributes() << 3 << QDate( 2020, 1, 1 ) );
+  QVERIFY( tempLayer->dataProvider()->addFeatures( QgsFeatureList() << f1 << f2 << f3 ) );
+
+  const QString filterExpression = QStringLiteral( "col1 < to_date('2020-02-03')" );
+  std::unique_ptr< QgsAttributeTableDialog > dlg( new QgsAttributeTableDialog( tempLayer.get(),
+      QgsAttributeTableFilterModel::ShowFilteredList,
+      nullptr,
+      Qt::Window,
+      nullptr,
+      filterExpression ) );
+
+  // feature id 2 is filtered out due not matching the provided filter expression
+  QCOMPARE( dlg->mMainView->filteredFeatures(), QgsFeatureIds() << 1 << 3 );
+}
+
 QGSTEST_MAIN( TestQgsAttributeTable )
 #include "testqgsattributetable.moc"
