@@ -21,10 +21,10 @@
 
 #include <memory>
 
-#include "qgssettings.h"
 #include "qgsfield.h"
 #include "qgsapplication.h"
 #include "qgstest.h"
+#include "qgsreferencedgeometry.h"
 
 class TestQgsField: public QObject
 {
@@ -47,6 +47,9 @@ class TestQgsField: public QObject
     void convertCompatible();
     void dataStream();
     void displayName();
+    void displayNameWithAlias();
+    void displayType();
+    void friendlyTypeString();
     void editorWidgetSetup();
     void collection();
 
@@ -85,6 +88,7 @@ void TestQgsField::create()
   QCOMPARE( field->length(), 5 );
   QCOMPARE( field->precision(), 2 );
   QCOMPARE( field->comment(), QString( "comment" ) );
+  QCOMPARE( field->isReadOnly(), false );
 }
 
 void TestQgsField::copy()
@@ -95,6 +99,7 @@ void TestQgsField::copy()
   constraints.setConstraintExpression( QStringLiteral( "constraint expression" ), QStringLiteral( "description" ) );
   constraints.setConstraintStrength( QgsFieldConstraints::ConstraintExpression, QgsFieldConstraints::ConstraintStrengthSoft );
   original.setConstraints( constraints );
+  original.setReadOnly( true );
   QgsField copy( original );
   QVERIFY( copy == original );
 
@@ -111,6 +116,7 @@ void TestQgsField::assignment()
   constraints.setConstraintExpression( QStringLiteral( "constraint expression" ), QStringLiteral( "description" ) );
   constraints.setConstraintStrength( QgsFieldConstraints::ConstraintExpression, QgsFieldConstraints::ConstraintStrengthSoft );
   original.setConstraints( constraints );
+  original.setReadOnly( true );
   QgsField copy;
   copy = original;
   QVERIFY( copy == original );
@@ -141,10 +147,12 @@ void TestQgsField::gettersSetters()
   QCOMPARE( field.defaultValueDefinition().expression(), QString( "1+2" ) );
   QgsFieldConstraints constraints;
   constraints.setConstraint( QgsFieldConstraints::ConstraintNotNull, QgsFieldConstraints::ConstraintOriginProvider );
+  constraints.setDomainName( QStringLiteral( "domain" ) );
   field.setConstraints( constraints );
   QCOMPARE( field.constraints().constraints(), QgsFieldConstraints::ConstraintNotNull );
   QCOMPARE( field.constraints().constraintOrigin( QgsFieldConstraints::ConstraintNotNull ), QgsFieldConstraints::ConstraintOriginProvider );
   QCOMPARE( field.constraints().constraintOrigin( QgsFieldConstraints::ConstraintUnique ), QgsFieldConstraints::ConstraintOriginNotSet );
+  QCOMPARE( field.constraints().domainName(), QStringLiteral( "domain" ) );
   constraints.setConstraint( QgsFieldConstraints::ConstraintNotNull, QgsFieldConstraints::ConstraintOriginNotSet );
   field.setConstraints( constraints );
   QCOMPARE( field.constraints().constraints(), 0 );
@@ -175,6 +183,9 @@ void TestQgsField::gettersSetters()
   constraints.setConstraintStrength( QgsFieldConstraints::ConstraintUnique, QgsFieldConstraints::ConstraintStrengthSoft );
   field.setConstraints( constraints );
   QCOMPARE( field.constraints().constraintStrength( QgsFieldConstraints::ConstraintUnique ), QgsFieldConstraints::ConstraintStrengthSoft );
+
+  field.setReadOnly( true );
+  QCOMPARE( field.isReadOnly(), true );
 }
 
 void TestQgsField::isNumeric()
@@ -273,6 +284,10 @@ void TestQgsField::equality()
   QVERIFY( !( field1 == field2 ) );
   QVERIFY( field1 != field2 );
   field2.setAlias( QString() );
+  field2.setReadOnly( true );
+  QVERIFY( !( field1 == field2 ) );
+  QVERIFY( field1 != field2 );
+  field2.setReadOnly( false );
   field2.setDefaultValueDefinition( QgsDefaultValue( QStringLiteral( "1+2" ) ) );
   QVERIFY( !( field1 == field2 ) );
   QVERIFY( field1 != field2 );
@@ -306,6 +321,13 @@ void TestQgsField::equality()
   field1.setConstraints( constraints );
   QVERIFY( !( field1 == field2 ) );
   QVERIFY( field1 != field2 );
+
+  QgsFieldConstraints constraints1;
+  QgsFieldConstraints constraints2;
+  constraints1.setDomainName( QStringLiteral( "d" ) );
+  QVERIFY( !( constraints1 == constraints2 ) );
+  constraints2.setDomainName( QStringLiteral( "d" ) );
+  QVERIFY( constraints1 == constraints2 );
 }
 
 void TestQgsField::asVariant()
@@ -316,70 +338,98 @@ void TestQgsField::asVariant()
   original.setConstraints( constraints );
 
   //convert to and from a QVariant
-  QVariant var = QVariant::fromValue( original );
+  const QVariant var = QVariant::fromValue( original );
   QVERIFY( var.isValid() );
 
-  QgsField fromVar = qvariant_cast<QgsField>( var );
+  const QgsField fromVar = qvariant_cast<QgsField>( var );
   QCOMPARE( fromVar, original );
 }
 
 void TestQgsField::displayString()
 {
-  QgsField stringField( QStringLiteral( "string" ), QVariant::String, QStringLiteral( "string" ) );
+  const QgsField stringField( QStringLiteral( "string" ), QVariant::String, QStringLiteral( "string" ) );
 
   //test string value
-  QString test( QStringLiteral( "test string" ) );
+  const QString test( QStringLiteral( "test string" ) );
   QCOMPARE( stringField.displayString( test ), test );
 
   //test NULL
   QgsApplication::setNullRepresentation( QStringLiteral( "TEST NULL" ) );
-  QVariant nullString = QVariant( QVariant::String );
+  const QVariant nullString = QVariant( QVariant::String );
   QCOMPARE( stringField.displayString( nullString ), QString( "TEST NULL" ) );
 
   //test int value in string type
-  QgsField intField( QStringLiteral( "int" ), QVariant::String, QStringLiteral( "int" ) );
+  const QgsField intField( QStringLiteral( "int" ), QVariant::String, QStringLiteral( "int" ) );
   QCOMPARE( intField.displayString( 5 ), QString( "5" ) );
   QCOMPARE( intField.displayString( 599999898999LL ), QString( "599999898999" ) );
 
   //test int value in int type
-  QgsField intField2( QStringLiteral( "int" ), QVariant::Int, QStringLiteral( "int" ) );
+  const QgsField intField2( QStringLiteral( "int" ), QVariant::Int, QStringLiteral( "int" ) );
   QCOMPARE( intField2.displayString( 5 ), QString( "5" ) );
   QCOMPARE( intField2.displayString( 599999898999LL ), QString( "599,999,898,999" ) );
 
   //test long type
-  QgsField longField( QStringLiteral( "long" ), QVariant::LongLong, QStringLiteral( "longlong" ) );
+  const QgsField longField( QStringLiteral( "long" ), QVariant::LongLong, QStringLiteral( "longlong" ) );
   QCOMPARE( longField.displayString( 5 ), QString( "5" ) );
   QCOMPARE( longField.displayString( 599999898999LL ), QString( "599,999,898,999" ) );
 
   //test NULL int
-  QVariant nullInt = QVariant( QVariant::Int );
+  const QVariant nullInt = QVariant( QVariant::Int );
   QCOMPARE( intField.displayString( nullInt ), QString( "TEST NULL" ) );
 
   //test double value
-  QgsField doubleField( QStringLiteral( "double" ), QVariant::Double, QStringLiteral( "double" ), 10, 3 );
+  const QgsField doubleField( QStringLiteral( "double" ), QVariant::Double, QStringLiteral( "double" ), 10, 3 );
   QCOMPARE( doubleField.displayString( 5.005005 ), QString( "5.005" ) );
-  QCOMPARE( doubleField.displayString( 4.5e-09 ), QString( "4.5e-09" ) );
+  QCOMPARE( doubleField.displayString( 4.5e-09 ).toLower(), QString( "4.5e-09" ) );
   QCOMPARE( doubleField.displayString( 1e-04 ), QString( "0.0001" ) );
-  QgsField doubleFieldNoPrec( QStringLiteral( "double" ), QVariant::Double, QStringLiteral( "double" ), 10 );
+  QCOMPARE( doubleField.displayString( -5.005005 ), QString( "-5.005" ) );
+  QCOMPARE( doubleField.displayString( -4.5e-09 ).toLower(), QString( "-4.5e-09" ) );
+  QCOMPARE( doubleField.displayString( -1e-04 ), QString( "-0.0001" ) );
+  const QgsField doubleFieldNoPrec( QStringLiteral( "double" ), QVariant::Double, QStringLiteral( "double" ), 10 );
   QCOMPARE( doubleFieldNoPrec.displayString( 5.005005 ), QString( "5.005005" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5.005005005 ), QString( "5.005005005" ) );
-  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-09 ), QString( "4.5e-09" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-09 ).toLower(), QString( "4.5e-09" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-08 ).toLower(), QString( "4.5e-08" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-07 ).toLower(), QString( "4.5e-07" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-06 ).toLower(), QString( "4.5e-06" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-05 ).toLower(), QString( "4.5e-05" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 1e-05 ).toLower(), QString( "1e-05" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 5e-05 ).toLower(), QString( "5e-05" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 1e-04 ), QString( "0.0001" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 1e-03 ), QString( "0.001" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 1e-02 ), QString( "0.01" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 1e-01 ), QString( "0.1" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -5.005005 ), QString( "-5.005005" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -5.005005005 ), QString( "-5.005005005" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -4.5e-09 ).toLower(), QString( "-4.5e-09" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -4.5e-08 ).toLower(), QString( "-4.5e-08" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -4.5e-07 ).toLower(), QString( "-4.5e-07" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -4.5e-06 ).toLower(), QString( "-4.5e-06" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -4.5e-05 ).toLower(), QString( "-4.5e-05" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -1e-05 ).toLower(), QString( "-1e-05" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -5e-05 ).toLower(), QString( "-5e-05" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -1e-04 ), QString( "-0.0001" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -1e-03 ), QString( "-0.001" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -1e-02 ), QString( "-0.01" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( -1e-01 ), QString( "-0.1" ) );
   QCOMPARE( QLocale().numberOptions() & QLocale::NumberOption::OmitGroupSeparator, QLocale::NumberOption::DefaultNumberOptions );
   QCOMPARE( doubleFieldNoPrec.displayString( 599999898999.0 ), QString( "599,999,898,999" ) );
 
+  // no conversion when this is default value expression
+  QCOMPARE( doubleFieldNoPrec.displayString( ( "(1+2)" ) ), QString( "(1+2)" ) );
+
   //test NULL double
-  QVariant nullDouble = QVariant( QVariant::Double );
+  const QVariant nullDouble = QVariant( QVariant::Double );
   QCOMPARE( doubleField.displayString( nullDouble ), QString( "TEST NULL" ) );
 
   //test double value with German locale
   QLocale::setDefault( QLocale::German );
   QCOMPARE( doubleField.displayString( 5.005005 ), QString( "5,005" ) );
-  QCOMPARE( doubleField.displayString( 4.5e-09 ), QString( "4,5e-09" ) );
+  QCOMPARE( doubleField.displayString( 4.5e-09 ).toLower(), QString( "4,5e-09" ) );
   QCOMPARE( doubleField.displayString( 1e-04 ), QString( "0,0001" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5.005005 ), QString( "5,005005" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5.005005005 ), QString( "5,005005005" ) );
-  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-09 ), QString( "4,5e-09" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-09 ).toLower(), QString( "4,5e-09" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 1e-04 ), QString( "0,0001" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 599999898999.0 ), QString( "599.999.898.999" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5999.123456 ), QString( "5.999,123456" ) );
@@ -389,11 +439,11 @@ void TestQgsField::displayString()
   customGerman.setNumberOptions( QLocale::NumberOption::OmitGroupSeparator );
   QLocale::setDefault( customGerman );
   QCOMPARE( doubleField.displayString( 5.005005 ), QString( "5,005" ) );
-  QCOMPARE( doubleField.displayString( 4.5e-09 ), QString( "4,5e-09" ) );
+  QCOMPARE( doubleField.displayString( 4.5e-09 ).toLower(), QString( "4,5e-09" ) );
   QCOMPARE( doubleField.displayString( 1e-04 ), QString( "0,0001" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5.005005 ), QString( "5,005005" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5.005005005 ), QString( "5,005005005" ) );
-  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-09 ), QString( "4,5e-09" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-09 ).toLower(), QString( "4,5e-09" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 1e-04 ), QString( "0,0001" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 599999898999.0 ), QString( "599999898999" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5999.123456 ), QString( "5999,123456" ) );
@@ -411,20 +461,20 @@ void TestQgsField::displayString()
   customEnglish.setNumberOptions( QLocale::NumberOption::OmitGroupSeparator );
   QLocale::setDefault( customEnglish );
   QCOMPARE( doubleField.displayString( 5.005005 ), QString( "5.005" ) );
-#if QT_VERSION >= QT_VERSION_CHECK( 5, 11, 0 )
-  QCOMPARE( doubleField.displayString( 4.5e-09 ), QString( "4.5e-09" ) );
-#else
-  QCOMPARE( doubleField.displayString( 4.5e-09 ), QString( "4.5e-9" ) );
-#endif
+  QCOMPARE( doubleField.displayString( 4.5e-09 ).toLower(), QString( "4.5e-09" ) );
   QCOMPARE( doubleField.displayString( 1e-04 ), QString( "0.0001" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5.005005 ), QString( "5.005005" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5.005005005 ), QString( "5.005005005" ) );
-#if QT_VERSION >= QT_VERSION_CHECK( 5, 11, 0 )
-  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-09 ), QString( "4.5e-09" ) );
-#else
-  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-09 ), QString( "4.5e-9" ) );
-#endif
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-09 ).toLower(), QString( "4.5e-09" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-08 ).toLower(), QString( "4.5e-08" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-07 ).toLower(), QString( "4.5e-07" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-06 ).toLower(), QString( "4.5e-06" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 4.5e-05 ).toLower(), QString( "4.5e-05" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 1e-05 ).toLower(), QString( "1e-05" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 1e-04 ), QString( "0.0001" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 1e-03 ), QString( "0.001" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 1e-02 ), QString( "0.01" ) );
+  QCOMPARE( doubleFieldNoPrec.displayString( 1e-01 ), QString( "0.1" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 599999898999.0 ), QString( "599999898999" ) );
   QCOMPARE( doubleFieldNoPrec.displayString( 5999.123456 ), QString( "5999.123456" ) );
 
@@ -437,16 +487,22 @@ void TestQgsField::displayString()
   QCOMPARE( longField.displayString( 599999898999LL ), QString( "599999898999" ) );
 
   // binary field
-  QgsField binaryField( QStringLiteral( "binary" ), QVariant::ByteArray, QStringLiteral( "Binary" ) );
-  QString testBAString( QStringLiteral( "test string" ) );
-  QByteArray testBA( testBAString.toLocal8Bit() );
+  const QgsField binaryField( QStringLiteral( "binary" ), QVariant::ByteArray, QStringLiteral( "Binary" ) );
+  const QString testBAString( QStringLiteral( "test string" ) );
+  const QByteArray testBA( testBAString.toLocal8Bit() );
   QCOMPARE( binaryField.displayString( testBA ), QStringLiteral( "BLOB" ) );
+
+  // array field
+  const QgsField stringArrayField( QStringLiteral( "stringArray" ), QVariant::StringList, QStringLiteral( "StringArray" ) );
+  QCOMPARE( stringArrayField.displayString( QStringList() << "A" << "B" << "C" ), QStringLiteral( "A, B, C" ) );
+  const QgsField intArrayField( QStringLiteral( "intArray" ), QVariant::List, QStringLiteral( "IntArray" ) );
+  QCOMPARE( intArrayField.displayString( QVariantList() << 1 << 2 << 3 ), QStringLiteral( "1, 2, 3" ) );
 }
 
 void TestQgsField::convertCompatible()
 {
   //test string field
-  QgsField stringField( QStringLiteral( "string" ), QVariant::String, QStringLiteral( "string" ) );
+  const QgsField stringField( QStringLiteral( "string" ), QVariant::String, QStringLiteral( "string" ) );
 
   QVariant stringVar( "test string" );
   QVERIFY( stringField.convertCompatible( stringVar ) );
@@ -473,14 +529,19 @@ void TestQgsField::convertCompatible()
   QVERIFY( nullDouble.isNull() );
 
   //test double
-  QgsField doubleField( QStringLiteral( "double" ), QVariant::Double, QStringLiteral( "double" ) );
+  const QgsField doubleField( QStringLiteral( "double" ), QVariant::Double, QStringLiteral( "double" ) );
 
   stringVar = QVariant( "test string" );
-  QVERIFY( !doubleField.convertCompatible( stringVar ) );
+  QString error;
+  QVERIFY( !doubleField.convertCompatible( stringVar, &error ) );
   QCOMPARE( stringVar.type(), QVariant::Double );
+  QCOMPARE( error, QStringLiteral( "Could not convert value \"test string\" to target type \"double\"" ) );
+  stringVar = QVariant( "test string" );
+  QVERIFY( !doubleField.convertCompatible( stringVar ) );
   QVERIFY( stringVar.isNull() );
   nullString = QVariant( QVariant::String );
-  QVERIFY( doubleField.convertCompatible( nullString ) );
+  QVERIFY( doubleField.convertCompatible( nullString, &error ) );
+  QVERIFY( error.isEmpty() );
   QCOMPARE( nullString.type(), QVariant::Double );
   QVERIFY( nullString.isNull() );
   intVar = QVariant( 5 );
@@ -515,12 +576,18 @@ void TestQgsField::convertCompatible()
   QCOMPARE( negativeSmallDouble, QVariant( -9346 ) );
   //large double, cannot be converted
   QVariant largeDouble( 9999999999.99 );
+  QVERIFY( !intField.convertCompatible( largeDouble, &error ) );
+  QCOMPARE( error, QStringLiteral( "Value \"10000000000\" is too large for integer field" ) );
+  largeDouble = QVariant( 9999999999.99 );
   QVERIFY( !intField.convertCompatible( largeDouble ) );
   QCOMPARE( largeDouble.type(), QVariant::Int );
   QVERIFY( largeDouble.isNull() );
 
   //conversion of string double value to int
   QVariant notNumberString( "notanumber" );
+  QVERIFY( !intField.convertCompatible( notNumberString, &error ) );
+  QCOMPARE( error, QStringLiteral( "Value \"notanumber\" is not a number" ) );
+  notNumberString = QVariant( "notanumber" );
   QVERIFY( !intField.convertCompatible( notNumberString ) );
   QCOMPARE( notNumberString.type(), QVariant::Int );
   QVERIFY( notNumberString.isNull() );
@@ -535,19 +602,34 @@ void TestQgsField::convertCompatible()
   QCOMPARE( negativeSmallDoubleString, QVariant( -9346 ) );
   //large double, cannot be converted
   QVariant largeDoubleString( "9999999999.99" );
+  QVERIFY( !intField.convertCompatible( largeDoubleString, &error ) );
+  QCOMPARE( error, QStringLiteral( "Value \"1e+10\" is too large for integer field" ) );
+  largeDoubleString = QVariant( "9999999999.99" );
   QVERIFY( !intField.convertCompatible( largeDoubleString ) );
   QCOMPARE( largeDoubleString.type(), QVariant::Int );
   QVERIFY( largeDoubleString.isNull() );
 
   //conversion of longlong to int
   QVariant longlong( 99999999999999999LL );
-  QVERIFY( !intField.convertCompatible( longlong ) );
+  QVERIFY( !intField.convertCompatible( longlong, &error ) );
+  QCOMPARE( error, QStringLiteral( "Value \"99999999999999999\" is too large for integer field" ) );
   QCOMPARE( longlong.type(), QVariant::Int );
   QVERIFY( longlong.isNull() );
   QVariant smallLonglong( 99LL );
   QVERIFY( intField.convertCompatible( smallLonglong ) );
   QCOMPARE( smallLonglong.type(), QVariant::Int );
   QCOMPARE( smallLonglong, QVariant( 99 ) );
+  // negative longlong to int
+  QVariant negativeLonglong( -99999999999999999LL );
+  QVERIFY( !intField.convertCompatible( negativeLonglong, &error ) );
+  QCOMPARE( error, QStringLiteral( "Value \"-99999999999999999\" is too large for integer field" ) );
+  QCOMPARE( negativeLonglong.type(), QVariant::Int );
+  QVERIFY( negativeLonglong.isNull() );
+  // small negative longlong to int
+  QVariant smallNegativeLonglong( -99LL );
+  QVERIFY( intField.convertCompatible( smallNegativeLonglong ) );
+  QCOMPARE( smallNegativeLonglong.type(), QVariant::Int );
+  QCOMPARE( smallNegativeLonglong, QVariant( -99 ) );
 
   //string representation of an int
   QVariant stringInt( "123456" );
@@ -561,7 +643,7 @@ void TestQgsField::convertCompatible()
   QCOMPARE( stringInt, QVariant( "123456" ) );
 
   //conversion of longlong to longlong field
-  QgsField longlongField( QStringLiteral( "long" ), QVariant::LongLong, QStringLiteral( "longlong" ) );
+  const QgsField longlongField( QStringLiteral( "long" ), QVariant::LongLong, QStringLiteral( "longlong" ) );
   longlong = QVariant( 99999999999999999LL );
   QVERIFY( longlongField.convertCompatible( longlong ) );
   QCOMPARE( longlong.type(), QVariant::LongLong );
@@ -580,7 +662,8 @@ void TestQgsField::convertCompatible()
 
   //conversion of string double value to longlong
   notNumberString = QVariant( "notanumber" );
-  QVERIFY( !longlongField.convertCompatible( notNumberString ) );
+  QVERIFY( !longlongField.convertCompatible( notNumberString, &error ) );
+  QCOMPARE( error, QStringLiteral( "Value \"notanumber\" is not a number" ) );
   QCOMPARE( notNumberString.type(), QVariant::LongLong );
   QVERIFY( notNumberString.isNull() );
   //small double, should be rounded
@@ -599,7 +682,8 @@ void TestQgsField::convertCompatible()
   QCOMPARE( largeDoubleString, QVariant( 10000000000LL ) );
   //extra large double, cannot be converted
   largeDoubleString = QVariant( "999999999999999999999.99" );
-  QVERIFY( !longlongField.convertCompatible( largeDoubleString ) );
+  QVERIFY( !longlongField.convertCompatible( largeDoubleString, &error ) );
+  QCOMPARE( error, QStringLiteral( "Value \"1e+21\" is too large for long long field" ) );
   QCOMPARE( largeDoubleString.type(), QVariant::LongLong );
   QVERIFY( largeDoubleString.isNull() );
 
@@ -615,11 +699,11 @@ void TestQgsField::convertCompatible()
   QCOMPARE( stringDouble, QVariant( 1223456.012345 ) );
   // This should not convert
   stringDouble = QVariant( "1.223.456,012345" );
-  QVERIFY( ! doubleField.convertCompatible( stringDouble ) );
-
+  QVERIFY( ! doubleField.convertCompatible( stringDouble, &error ) );
+  QCOMPARE( error, QStringLiteral( "Could not convert value \"1.223.456,012345\" to target type \"double\"" ) );
 
   //double with precision
-  QgsField doubleWithPrecField( QStringLiteral( "double" ), QVariant::Double, QStringLiteral( "double" ), 10, 3 );
+  const QgsField doubleWithPrecField( QStringLiteral( "double" ), QVariant::Double, QStringLiteral( "double" ), 10, 3 );
   doubleVar = QVariant( 10.12345678 );
   //note - this returns true!
   QVERIFY( doubleWithPrecField.convertCompatible( doubleVar ) );
@@ -627,9 +711,10 @@ void TestQgsField::convertCompatible()
   QCOMPARE( doubleVar.toDouble(), 10.123 );
 
   //truncating string length
-  QgsField stringWithLen( QStringLiteral( "string" ), QVariant::String, QStringLiteral( "string" ), 3 );
+  const QgsField stringWithLen( QStringLiteral( "string" ), QVariant::String, QStringLiteral( "string" ), 3 );
   stringVar = QVariant( "longstring" );
-  QVERIFY( !stringWithLen.convertCompatible( stringVar ) );
+  QVERIFY( !stringWithLen.convertCompatible( stringVar, &error ) );
+  QCOMPARE( error, QStringLiteral( "String of length 10 exceeds maximum field length (3)" ) );
   QCOMPARE( stringVar.type(), QVariant::String );
   QCOMPARE( stringVar.toString(), QString( "lon" ) );
 
@@ -694,6 +779,59 @@ void TestQgsField::convertCompatible()
   QCOMPARE( stringDouble.type(), QVariant::Double );
   QCOMPARE( stringDouble, QVariant( 1223456.012345 ) );
 
+  // Test 0 on int fields
+  intField = QgsField( QStringLiteral( "int" ), QVariant::Int, QStringLiteral( "Integer" ), 10 );
+  QVariant vZero = 0;
+  QVERIFY( intField.convertCompatible( vZero ) );
+
+  // Test string-based json field conversion
+  {
+    const QgsField jsonField( QStringLiteral( "json" ), QVariant::String, QStringLiteral( "json" ) );
+    QVariant jsonValue = QVariant::fromValue( QVariantList() << 1 << 5 << 8 );
+    QVERIFY( jsonField.convertCompatible( jsonValue ) );
+    QCOMPARE( jsonValue.type(), QVariant::String );
+    QCOMPARE( jsonValue, QString( "[1,5,8]" ) );
+    QVariantMap variantMap;
+    variantMap.insert( QStringLiteral( "a" ), 1 );
+    variantMap.insert( QStringLiteral( "c" ), 3 );
+    jsonValue = QVariant::fromValue( variantMap );
+    QVERIFY( jsonField.convertCompatible( jsonValue ) );
+    QCOMPARE( jsonValue.type(), QVariant::String );
+    QCOMPARE( jsonValue, QString( "{\"a\":1,\"c\":3}" ) );
+  }
+
+  // Test map-based json field (i.e. OGR geopackage JSON fields) conversion
+  {
+    const QgsField jsonField( QStringLiteral( "json" ), QVariant::Map, QStringLiteral( "json" ) );
+    QVariant jsonValue = QVariant::fromValue( QVariantList() << 1 << 5 << 8 );
+    QVERIFY( jsonField.convertCompatible( jsonValue ) );
+    QCOMPARE( jsonValue.type(), QVariant::List );
+    QCOMPARE( jsonValue, QVariantList() << 1 << 5 << 8 );
+    QVariantMap variantMap;
+    variantMap.insert( QStringLiteral( "a" ), 1 );
+    variantMap.insert( QStringLiteral( "c" ), 3 );
+    jsonValue = QVariant::fromValue( variantMap );
+    QVERIFY( jsonField.convertCompatible( jsonValue ) );
+    QCOMPARE( jsonValue.type(), QVariant::Map );
+    QCOMPARE( jsonValue, variantMap );
+  }
+
+  // geometry field conversion
+  const QgsField geometryField( QStringLiteral( "geometry" ), QVariant::UserType, QStringLiteral( "geometry" ) );
+  QVariant geometryValue;
+  QVERIFY( geometryField.convertCompatible( geometryValue ) );
+  QVERIFY( geometryValue.isNull() );
+  geometryValue = QVariant::fromValue( QgsGeometry::fromWkt( QStringLiteral( "Point( 1 2 )" ) ) );
+  QVERIFY( geometryField.convertCompatible( geometryValue ) );
+  QCOMPARE( geometryValue.userType(), QMetaType::type( "QgsGeometry" ) );
+
+  geometryValue = QVariant::fromValue( QgsReferencedGeometry( QgsGeometry::fromWkt( QStringLiteral( "Point( 1 2 )" ) ), QgsCoordinateReferenceSystem( QStringLiteral( "EPSG:3111" ) ) ) );
+  QVERIFY( geometryField.convertCompatible( geometryValue ) );
+  QCOMPARE( geometryValue.userType(), QMetaType::type( "QgsReferencedGeometry" ) );
+
+  geometryValue = QStringLiteral( "LineString( 1 2, 3 4 )" );
+  QVERIFY( geometryField.convertCompatible( geometryValue ) );
+  QCOMPARE( geometryValue.userType(), QMetaType::type( "QgsGeometry" ) );
 }
 
 void TestQgsField::dataStream()
@@ -738,6 +876,52 @@ void TestQgsField::displayName()
   QCOMPARE( field.displayName(), QString( "name" ) );
 }
 
+
+void TestQgsField::displayNameWithAlias()
+{
+  QgsField field;
+  field.setName( QStringLiteral( "name" ) );
+  QCOMPARE( field.displayNameWithAlias(), QString( "name" ) );
+  field.setAlias( QStringLiteral( "alias" ) );
+  QCOMPARE( field.displayNameWithAlias(), QString( "name (alias)" ) );
+  field.setAlias( QString() );
+  QCOMPARE( field.displayNameWithAlias(), QString( "name" ) );
+}
+
+
+void TestQgsField::displayType()
+{
+  QgsField field;
+  field.setTypeName( QStringLiteral( "numeric" ) );
+  QCOMPARE( field.displayType(), QString( "numeric" ) );
+  field.setLength( 20 );
+  QCOMPARE( field.displayType(), QString( "numeric(20)" ) );
+  field.setPrecision( 10 );
+  field.setPrecision( 10 );
+  QCOMPARE( field.displayType(), QString( "numeric(20, 10)" ) );
+  QCOMPARE( field.displayType( true ), QString( "numeric(20, 10) NULL" ) );
+  QgsFieldConstraints constraints;
+  constraints.setConstraint( QgsFieldConstraints::ConstraintUnique );
+  field.setConstraints( constraints );
+  QCOMPARE( field.displayType( true ), QString( "numeric(20, 10) NULL UNIQUE" ) );
+}
+
+void TestQgsField::friendlyTypeString()
+{
+  QgsField field;
+  field.setType( QVariant::String );
+  QCOMPARE( field.friendlyTypeString(), QStringLiteral( "Text (string)" ) );
+  field.setType( QVariant::Double );
+  field.setLength( 20 );
+  QCOMPARE( field.friendlyTypeString(), QStringLiteral( "Decimal (double)" ) );
+  field.setType( QVariant::List );
+  field.setSubType( QVariant::String );
+  QCOMPARE( field.friendlyTypeString(), QStringLiteral( "List" ) );
+  field.setType( QVariant::UserType );
+  field.setTypeName( QStringLiteral( "geometry" ) );
+  QCOMPARE( field.friendlyTypeString(), QStringLiteral( "Geometry" ) );
+}
+
 void TestQgsField::editorWidgetSetup()
 {
   QgsField field;
@@ -759,6 +943,27 @@ void TestQgsField::collection()
 
   QVariant str( "hello" );
   QVERIFY( !field.convertCompatible( str ) );
+
+  QVariant intList = QVariantList( {1, 2, 3 } );
+  QVERIFY( field.convertCompatible( intList ) );
+  QCOMPARE( intList.toList(), QVariantList( {1, 2, 3} ) );
+
+  QVariant doubleList = QVariantList( {1.1, 2.2, 3.3 } );
+  QVERIFY( field.convertCompatible( doubleList ) );
+  QCOMPARE( doubleList.toList(), QVariantList( {1.1, 2.2, 3.3 } ) );
+
+  QgsField stringListField( QStringLiteral( "collection" ), QVariant::StringList );
+  str = QVariant( "hello" );
+  QVERIFY( stringListField.convertCompatible( str ) );
+  QCOMPARE( str, QStringList{ QStringLiteral( "hello" )} );
+
+  QVariant strList = QVariant( QStringList( { "hello", "there" } ) );
+  QVERIFY( stringListField.convertCompatible( strList ) );
+  QCOMPARE( strList, QVariant( QStringList( { "hello", "there" } ) ) );
+
+  QVariant strInVariantList = QVariant( QVariantList( { "hello", "there" } ) );
+  QVERIFY( stringListField.convertCompatible( strInVariantList ) );
+  QCOMPARE( strInVariantList, QVariant( QStringList( { "hello", "there" } ) ) );
 }
 
 QGSTEST_MAIN( TestQgsField )
