@@ -42,18 +42,16 @@ class TestGeoPdfExporter : public QgsAbstractGeoPdfExporter
 
 };
 
-class TestQgsGeoPdfExport : public QObject
+class TestQgsGeoPdfExport : public QgsTest
 {
     Q_OBJECT
 
   public:
-    TestQgsGeoPdfExport() = default;
+    TestQgsGeoPdfExport() : QgsTest( QStringLiteral( "GeoPDF Export Testss" ) ) {}
 
   private slots:
     void initTestCase();// will be called before the first testfunction is executed.
     void cleanupTestCase();// will be called after the last testfunction was executed.
-    void init();// will be called before each testfunction is executed.
-    void cleanup();// will be called after every testfunction.
     void testCollectingFeatures();
     void testComposition();
     void testMetadata();
@@ -62,42 +60,19 @@ class TestQgsGeoPdfExport : public QObject
     void testGroups();
     void testCustomGroups();
     void compositionMode();
-
-  private:
-
-    QString mReport;
 };
 
 void TestQgsGeoPdfExport::initTestCase()
 {
   QgsApplication::init();
   QgsApplication::initQgis();
-
-  mReport = QStringLiteral( "<h1>GeoPDF Export Tests</h1>\n" );
 }
 
 void TestQgsGeoPdfExport::cleanupTestCase()
 {
-  QString myReportFile = QDir::tempPath() + "/qgistest.html";
-  QFile myFile( myReportFile );
-  if ( myFile.open( QIODevice::WriteOnly | QIODevice::Append ) )
-  {
-    QTextStream myQTextStream( &myFile );
-    myQTextStream << mReport;
-    myFile.close();
-  }
-
   QgsApplication::exitQgis();
 }
 
-void TestQgsGeoPdfExport::init()
-{
-}
-
-void TestQgsGeoPdfExport::cleanup()
-{
-}
-
 void TestQgsGeoPdfExport::testCollectingFeatures()
 {
   if ( !QgsAbstractGeoPdfExporter::geoPDFCreationAvailable() )
